define(["@jupyter-widgets/base"],function(t){return function(t){var e={};function n(r){if(e[r])return e[r].exports;var i=e[r]={i:r,l:!1,exports:{}};return t[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=t,n.c=e,n.d=function(t,e,r){n.o(t,e)||Object.defineProperty(t,e,{enumerable:!0,get:r})},n.r=function(t){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0})},n.t=function(t,e){if(1&e&&(t=n(t)),8&e)return t;if(4&e&&"object"==typeof t&&t&&t.__esModule)return t;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:t}),2&e&&"string"!=typeof t)for(var i in t)n.d(r,i,function(e){return t[e]}.bind(null,i));return r},n.n=function(t){var e=t&&t.__esModule?function(){return t.default}:function(){return t};return n.d(e,"a",e),e},n.o=function(t,e){return Object.prototype.hasOwnProperty.call(t,e)},n.p="",n(n.s=4)}([function(t,e,n){(function(e,n){
/*! lineupjs - v3.1.7 - 2018
* https://lineup.js.org
* Copyright (c) 2018 Samuel Gratzl; Licensed BSD-3-Clause*/
window,t.exports=function(t){var e={};function n(r){if(e[r])return e[r].exports;var i=e[r]={i:r,l:!1,exports:{}};return t[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=t,n.c=e,n.d=function(t,e,r){n.o(t,e)||Object.defineProperty(t,e,{enumerable:!0,get:r})},n.r=function(t){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0})},n.t=function(t,e){if(1&e&&(t=n(t)),8&e)return t;if(4&e&&"object"==typeof t&&t&&t.__esModule)return t;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:t}),2&e&&"string"!=typeof t)for(var i in t)n.d(r,i,function(e){return t[e]}.bind(null,i));return r},n.n=function(t){var e=t&&t.__esModule?function(){return t.default}:function(){return t};return n.d(e,"a",e),e},n.o=function(t,e){return Object.prototype.hasOwnProperty.call(t,e)},n.p="",n(n.s=99)}([function(t,e,n){"use strict";n.d(e,"c",function(){return i}),n.d(e,"a",function(){return o}),n.d(e,"b",function(){return a});
/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
var r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)};function i(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)}var o=function(){return(o=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)};function a(t,e,n,r){var i,o=arguments.length,a=o<3?e:null===r?r=Object.getOwnPropertyDescriptor(e,n):r;if("object"==typeof Reflect&&"function"==typeof Reflect.decorate)a=Reflect.decorate(t,e,n,r);else for(var u=t.length-1;u>=0;u--)(i=t[u])&&(a=(o<3?i(a):o>3?i(e,n,a):i(e,n))||a);return o>3&&a&&Object.defineProperty(e,n,a),a}},function(t,e,n){"use strict";n.d(e,"d",function(){return a}),n.d(e,"c",function(){return u}),n.d(e,"j",function(){return s}),n.d(e,"a",function(){return c}),n.d(e,"b",function(){return d}),n.d(e,"l",function(){return p}),n.d(e,"g",function(){return f}),n.d(e,"k",function(){return g}),n.d(e,"e",function(){return m}),n.d(e,"f",function(){return b}),n.d(e,"h",function(){return y}),n.d(e,"i",function(){return v}),n(100);var r=n(2),i=Symbol.for("SupportType"),o=Symbol.for("Category");function a(){return Reflect.metadata(i,!0)}function u(t){return void 0===t&&(t="ascending"),"descending"===t?Reflect.metadata(Symbol.for("sortDescendingByDefault"),!0):function(t){return t}}function s(t){var e=t.constructor;return!Reflect.hasMetadata(Symbol.for("sortDescendingByDefault"),e)}var c=function(){this.string={label:"label",order:1,name:"string"},this.categorical={label:"categorical",order:2,name:"categorical"},this.number={label:"numerical",order:3,name:"number"},this.date={label:"date",order:4,name:"date"},this.array={label:"matrix",order:5,name:"array"},this.map={label:"map",order:6,name:"map"},this.composite={label:"combined",order:7,name:"composite"},this.support={label:"support",order:8,name:"support"},this.other={label:"others",order:9,name:"other"}},l=new c;function d(t){return Reflect.metadata(o,t)}function p(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return Reflect.metadata(Symbol.for("toolbarIcon"),t)}function f(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return Reflect.metadata(Symbol.for("toolbarDialogAddon"+t),e)}var h=new Map;function g(t){var e=t.constructor;return Reflect.hasMetadata(i,e)}function m(t){var e=Reflect.getMetadata(o,t instanceof r.e?Object.getPrototypeOf(t).constructor:t)||"other";return l[e]||l.other}function b(t,e){var n=e["string"==typeof t?t:t.type];return n?m(n):l.other}function y(t){if(h.has(t.desc.type))return h.get(t.desc.type);var e=[],n=t,r=Symbol.for("toolbarIcon");do{var i=Reflect.getOwnMetadata(r,n.constructor);i&&e.push.apply(e,i),n=Object.getPrototypeOf(n)}while(n);return h.set(t.desc.type,e),e}function v(t,e){var n=t.desc.type+"@"+e;if(h.has(n))return h.get(n);var r=[],i=t,o=Symbol.for("toolbarDialogAddon"+e);do{var a=Reflect.getOwnMetadata(o,i.constructor);a&&r.push.apply(r,a),i=Object.getPrototypeOf(i)}while(i);return h.set(n,r),r}},function(t,e,n){"use strict";var r=n(0),i=n(16),o=n(9),a=n(74),u=n(50),s=n(7),c=n(1),l=n(73);n.d(e,"a",function(){return l.IColumnDesc});var d=function(t){function e(n,r){var i=t.call(this)||this;return i.desc=r,i.width=100,i.parent=null,i.uid=Object(a.d)(n),i.renderer=i.desc.renderer||i.desc.type,i.groupRenderer=i.desc.groupRenderer||i.desc.type,i.summaryRenderer=i.desc.summaryRenderer||i.desc.type,i.width=null!=i.desc.width&&i.desc.width>0?i.desc.width:100,i.visible=!1!==i.desc.visible,i.metadata={label:r.label||i.id,description:r.description||"",color:r.color||e.DEFAULT_COLOR},i}return r.c(e,t),Object.defineProperty(e.prototype,"fixed",{get:function(){return Boolean(this.desc.fixed)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"frozen",{get:function(){return Boolean(this.desc.frozen)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"id",{get:function(){return this.uid},enumerable:!0,configurable:!0}),e.prototype.assignNewId=function(t){this.uid=Object(a.d)(t())},Object.defineProperty(e.prototype,"label",{get:function(){return this.metadata.label},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"description",{get:function(){return this.metadata.description},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"color",{get:function(){return this.metadata.color},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"fqid",{get:function(){return this.parent?this.parent.fqid+"_"+this.id:this.id},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"fqpath",{get:function(){return this.parent?this.parent.fqpath+"@"+this.parent.indexOf(this):""},enumerable:!0,configurable:!0}),e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([e.EVENT_WIDTH_CHANGED,e.EVENT_LABEL_CHANGED,e.EVENT_METADATA_CHANGED,e.EVENT_VISIBILITY_CHANGED,e.EVENT_SUMMARY_RENDERER_TYPE_CHANGED,e.EVENT_RENDERER_TYPE_CHANGED,e.EVENT_GROUP_RENDERER_TYPE_CHANGED,e.EVENT_DIRTY,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getWidth=function(){return this.width},e.prototype.hide=function(){this.setVisible(!1)},e.prototype.show=function(){this.setVisible(!0)},e.prototype.isVisible=function(){return this.visible},e.prototype.getVisible=function(){return this.isVisible()},e.prototype.setVisible=function(t){this.visible!==t&&this.fire([e.EVENT_VISIBILITY_CHANGED,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],this.visible,this.visible=t)},e.prototype.flatten=function(t,e,n,r){void 0===n&&(n=0),void 0===r&&(r=0);var i=this.getWidth();return t.push({col:this,offset:e,width:i}),i},e.prototype.setWidth=function(t){Object(o.l)(this.width,t,.5)||this.fire([e.EVENT_WIDTH_CHANGED,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],this.width,this.width=t)},e.prototype.setWidthImpl=function(t){this.width=t},e.prototype.setMetaData=function(t){if(t.label!==this.label||this.color!==t.color||this.description!==t.description){var n=this.color===t.color?[e.EVENT_LABEL_CHANGED,e.EVENT_METADATA_CHANGED,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY]:[e.EVENT_LABEL_CHANGED,e.EVENT_METADATA_CHANGED,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],r=this.getMetaData();this.metadata={label:t.label,color:t.color,description:t.description},this.fire(n,r,this.getMetaData())}},e.prototype.getMetaData=function(){return Object.assign({},this.metadata)},e.prototype.sortByMe=function(t,e){void 0===t&&(t=Object(c.j)(this)),void 0===e&&(e=0);var n=this.findMyRanker();return!!n&&n.sortBy(this,t,e)},e.prototype.groupByMe=function(){var t=this.findMyRanker();return!!t&&t.toggleGrouping(this)},e.prototype.isGroupedBy=function(){var t=this.findMyRanker();return t?t.getGroupCriteria().indexOf(this):-1},e.prototype.toggleMySorting=function(){var t=this.findMyRanker();return!!t&&t.toggleSorting(this)},e.prototype.isSortedByMeImpl=function(t){var e=this,n=this.findMyRanker();if(!n)return{asc:void 0,priority:void 0};var r=t(n),i=r.findIndex(function(t){return t.col===e});return i<0?{asc:void 0,priority:void 0}:{asc:r[i].asc?"asc":"desc",priority:i}},e.prototype.isSortedByMe=function(){return this.isSortedByMeImpl(function(t){return t.getSortCriteria()})},e.prototype.groupSortByMe=function(t,e){void 0===t&&(t=Object(c.j)(this)),void 0===e&&(e=0);var n=this.findMyRanker();return!!n&&n.groupSortBy(this,t,e)},e.prototype.toggleMyGroupSorting=function(){var t=this.findMyRanker();return!!t&&t.toggleGroupSorting(this)},e.prototype.isGroupSortedByMe=function(){return this.isSortedByMeImpl(function(t){return t.getGroupSortCriteria()})},e.prototype.removeMe=function(){return!this.fixed&&!!this.parent&&this.parent.remove(this)},e.prototype.attach=function(t){this.parent=t},e.prototype.detach=function(){this.parent=null},e.prototype.insertAfterMe=function(t){return!!this.parent&&null!=this.parent.insertAfter(t,this)},e.prototype.findMyRanker=function(){return this.parent?this.parent.findMyRanker():null},e.prototype.dump=function(t){var n={id:this.id,desc:t(this.desc),width:this.width};return this.label!==(this.desc.label||this.id)&&(n.label=this.label),this.color!==(this.desc.color||e.DEFAULT_COLOR)&&this.color&&(n.color=this.color),this.getRenderer()!==this.desc.type&&(n.renderer=this.getRenderer()),this.getGroupRenderer()!==this.desc.type&&(n.groupRenderer=this.getGroupRenderer()),this.getSummaryRenderer()!==this.desc.type&&(n.summaryRenderer=this.getSummaryRenderer()),n},e.prototype.restore=function(t,e){this.width=t.width||this.width,this.metadata={label:t.label||this.label,color:t.color||this.color,description:this.description},(t.renderer||t.rendererType)&&(this.renderer=t.renderer||t.rendererType),t.groupRenderer&&(this.groupRenderer=t.groupRenderer),t.summaryRenderer&&(this.summaryRenderer=t.summaryRenderer)},e.prototype.getLabel=function(t){return String(this.getValue(t))},e.prototype.getValue=function(t){return""},e.prototype.getExportValue=function(t,e){return"text"===e?this.getLabel(t):this.getValue(t)},e.prototype.getColor=function(t){return this.color},e.prototype.isMissing=function(t){return Object(s.c)(this.getValue(t))},e.prototype.compare=function(t,e){return 0},e.prototype.group=function(t){return u.a},e.prototype.groupCompare=function(t,e){return t.name.toLowerCase().localeCompare(e.name.toLowerCase())},e.prototype.isFiltered=function(){return!1},e.prototype.filter=function(t){return null!=t},e.prototype.getRenderer=function(){return this.renderer},e.prototype.getGroupRenderer=function(){return this.groupRenderer},e.prototype.getSummaryRenderer=function(){return this.summaryRenderer},e.prototype.setRenderer=function(t){t!==this.renderer&&this.fire([e.EVENT_RENDERER_TYPE_CHANGED,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],this.renderer,this.renderer=t)},e.prototype.setDefaultRenderer=function(t){if(this.renderer===this.desc.type&&!this.desc.renderer)return this.setRenderer(t)},e.prototype.setGroupRenderer=function(t){t!==this.groupRenderer&&this.fire([e.EVENT_GROUP_RENDERER_TYPE_CHANGED,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],this.groupRenderer,this.groupRenderer=t)},e.prototype.setDefaultGroupRenderer=function(t){if(this.groupRenderer===this.desc.type&&!this.desc.groupRenderer)return this.setGroupRenderer(t)},e.prototype.setSummaryRenderer=function(t){t!==this.summaryRenderer&&this.fire([e.EVENT_SUMMARY_RENDERER_TYPE_CHANGED,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY],this.summaryRenderer,this.summaryRenderer=t)},e.prototype.setDefaultSummaryRenderer=function(t){if(this.summaryRenderer===this.desc.type&&!this.desc.summaryRenderer)return this.setSummaryRenderer(t)},e.prototype.setDefaultWidth=function(t){if(100===this.width&&!this.desc.width)return this.setWidthImpl(t)},e.prototype.markDirty=function(t){switch(void 0===t&&(t="all"),t){case"header":return this.fire([e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY]);case"values":return this.fire([e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY]);default:return this.fire([e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY])}},e.DEFAULT_COLOR="#C1C1C1",e.FLAT_ALL_COLUMNS=-1,e.EVENT_WIDTH_CHANGED="widthChanged",e.EVENT_LABEL_CHANGED="labelChanged",e.EVENT_METADATA_CHANGED="metaDataChanged",e.EVENT_DIRTY="dirty",e.EVENT_DIRTY_HEADER="dirtyHeader",e.EVENT_DIRTY_VALUES="dirtyValues",e.EVENT_RENDERER_TYPE_CHANGED="rendererTypeChanged",e.EVENT_GROUP_RENDERER_TYPE_CHANGED="groupRendererChanged",e.EVENT_SUMMARY_RENDERER_TYPE_CHANGED="summaryRendererChanged",e.EVENT_VISIBILITY_CHANGED="visibilityChanged",e}(i.a);e.e=d},function(t,e,n){"use strict";n.d(e,"c",function(){return o}),n.d(e,"h",function(){return a}),n.d(e,"g",function(){return u}),n.d(e,"i",function(){return s}),n.d(e,"d",function(){return c}),n.d(e,"e",function(){return l}),n.d(e,"f",function(){return d}),n.d(e,"k",function(){return p}),n.d(e,"b",function(){return f}),n.d(e,"a",function(){return h}),n.d(e,"j",function(){return g});var r=n(25),i=n(24);function o(t,e,n,r){return void 0===e&&(e={}),void 0===n&&(n={}),Object.keys(e).forEach(function(n){var r=String(e[n]);t.getAttribute(n)!==r&&t.setAttribute(n,r)}),Object.keys(n).forEach(function(e){var r=n[e];t.style.getPropertyValue(e)!==r&&t.style.setProperty(e,r)}),s(t,r)}function a(){}var u={template:"<div></div>",update:a,render:a};function s(t,e){return void 0===e?t:(t.textContent!==e&&(t.textContent=e),t)}function c(t,e,n){Array.from(t.querySelectorAll(e)).forEach(n)}function l(t,e){Array.from(t.children).forEach(e)}function d(t,e){if(0!==t.childElementCount){if(!e.every(function(e,n){var r=t.children[n];return null!=r&&r.dataset.columnId===e.column.id&&r.dataset.renderer===e.rendererId})){var n=new Set(e.map(function(t){return t.column.id+"@"+t.rendererId}));l(t,function(e){var r=e.dataset.columnId+"@"+e.dataset.renderer;n.has(r)||t.removeChild(e)}),e.forEach(function(e){var n=t.querySelector('[data-column-id="'+e.column.id+'"]');n||(t.insertAdjacentHTML("beforeend",e.template),(n=t.lastElementChild).dataset.columnId=e.column.id,n.dataset.renderer=e.rendererId),t.appendChild(n)})}}else{t.innerHTML=e.map(function(t){return t.template}).join("");var r=Array.from(t.children);e.forEach(function(t,e){var n=r[e];n.dataset.columnId=t.column.id,n.dataset.renderer=t.rendererId})}}function p(t,e){return void 0===e&&(e=t.labels.length),t.getWidth()/e>r.e}function f(t){return Object(i.d)(t).l>.5?"black":"white"}function h(t,e,n,r){var i=f(e);if(r<=.05||"black"===i||r>.9)t.style.color="black"===i||r<=.05?null:i;else{t.style.color=null,t.innerText=n;var o=t.ownerDocument.createElement("span");o.classList.add("lu-gradient-text"),o.style.color=i,o.innerText=n,t.appendChild(o)}}var g=function(){var t=0;return function(e){return""+e+(t++).toString(36)}}()},function(t,e,n){"use strict";n.d(e,"a",function(){return c}),n.d(e,"i",function(){return l}),n.d(e,"d",function(){return d}),n.d(e,"f",function(){return p}),n.d(e,"c",function(){return r}),n.d(e,"g",function(){return f}),n.d(e,"b",function(){return i}),n.d(e,"k",function(){return h}),n.d(e,"m",function(){return g}),n.d(e,"l",function(){return m}),n.d(e,"h",function(){return b}),n.d(e,"j",function(){return y});var r,i,o=n(48),a=n(9),u=n(2),s=n(7),c=Object(o.a)(".3n");function l(t){return t instanceof u.e&&"function"==typeof t.getNumber||!(t instanceof u.e)&&null!=t.type.match(/(number|stack|ordinal)/)}function d(t,e,n){var r=t.getBoxPlotData(e),i=t.getBoxPlotData(n);if(null==r)return null==i?0:s.b;if(null==i)return-1*s.b;var o=t.getSortMethod();return g(r[o],i[o])}function p(t,e,n){var r="normalized"===n?t.getBoxPlotData(e):t.getRawBoxPlotData(e);return null==r?NaN:r[t.getSortMethod()]}function f(t){return"function"==typeof t.getBoxPlotData}function h(t){return f(t)&&"function"==typeof t.getNumbers}function g(t,e,n,r){return void 0===n&&(n=!1),void 0===r&&(r=!1),n=n||null==t||isNaN(t),r=r||null==e||isNaN(e),n?r?0:s.b:r?-1*s.b:t-e}function m(){return{min:-1/0,max:1/0,filterMissing:!1}}function b(t,e){return Object(a.l)(t.min,e.min,.001)&&Object(a.l)(t.max,e.max,.001)&&t.filterMissing===e.filterMissing}function y(t,e){return!(t&&(isNaN(e)?t.filterMissing:isFinite(t.min)&&e<t.min||isFinite(t.max)&&e>t.max))}!function(t){t.min="min",t.max="max",t.median="median",t.q1="q1",t.q3="q3"}(r||(r={})),function(t){t.min="min",t.max="max",t.median="median",t.q1="q1",t.q3="q3",t.mean="mean"}(i||(i={}))},function(t,e,n){"use strict";n.d(e,"d",function(){return c}),n.d(e,"e",function(){return l}),n.d(e,"h",function(){return d}),n.d(e,"f",function(){return p}),n.d(e,"b",function(){return f}),n.d(e,"a",function(){return h}),n.d(e,"g",function(){return g}),n.d(e,"c",function(){return m});for(var r=n(114),i=new Map,o=/^[$]([\w]+): ([\w #.()'\/,-]+)( !default)?;/gim,a=String(r),u=a.match(o);null!=u;)i.set(u[1],u[2]),u=a.match(o);function s(t,e){return void 0===e&&(e=""),"$"===t[0]&&(t=t.slice(1)),i.has(t)?i.get(t):e}var c=parseInt(s("lu_engine_grip_gap","5px"),10),l={width:parseInt(s("lu_missing_dash_width","3px"),10),height:parseInt(s("lu_missing_dash_height","10px"),10),color:s("lu_missing_dash_color","gray")},d={circle:s("lu_renderer_upset_circle_color"),inactive:parseFloat(s("lu_renderer_upset_inactive_opacity","0.1")),stroke:s("lu_renderer_upset_stroke")},p={color:s("lu_renderer_dot_color","gray"),size:parseInt(s("lu_renderer_dot_size","5px"),10),opacity:parseFloat(s("lu_renderer_dot_opacity","0.7"))},f={box:s("lu_renderer_boxplot_box","#e0e0e0"),stroke:s("lu_renderer_boxplot_stroke","black"),sort:s("lu_renderer_boxplot_sort_indicator","#ffa500"),outlier:s("lu_renderer_boxplot_outlier","#e0e0e0")},h={width:parseInt(s("lu_aggregate_square_bracket_width","4px"),10),strokeWidth:parseInt(s("lu_aggregate_square_bracket_stroke_width","1px"),10),color:s("lu_aggregate_square_bracket_stroke_color","#000")},g=parseInt(s("lu_slopw_width","200px"),10),m=4},function(t,e,n){"use strict";var r;n.d(e,"a",function(){return r}),function(t){t[t.CELL=0]="CELL",t[t.GROUP=1]="GROUP",t[t.SUMMARY=2]="SUMMARY"}(r||(r={}))},function(t,e,n){"use strict";function r(t){return void 0===t||null==t||"number"==typeof t&&isNaN(t)||""===t||"NA"===t||"NaN"===t||"string"==typeof t&&"na"===t.toLowerCase()||Array.isArray(t)&&t.every(function(t){return r(t)})}function i(t){return null==t||void 0===t||isNaN(t)}n.d(e,"c",function(){return r}),n.d(e,"d",function(){return i}),n.d(e,"b",function(){return o}),n.d(e,"a",function(){return a}),n.d(e,"e",function(){return u});var o=-1,a=1,u={name:"Missing values",color:"gray"}},function(t,e,n){"use strict";var r=n(0),i=n(2),o=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.accessor=n.accessor||function(){return null},r.loaded=!0!==n.lazyLoaded,r}return r.c(e,t),e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([e.EVENT_DATA_LOADED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getLabel=function(t){return this.isLoaded()?String(this.getValue(t)):""},e.prototype.getRaw=function(t){return this.isLoaded()?this.accessor(t,this.id,this.desc,this.findMyRanker()):null},e.prototype.getValue=function(t){return this.getRaw(t)},e.prototype.isLoaded=function(){return this.loaded},e.prototype.setLoaded=function(t){this.loaded!==t&&this.fire([e.EVENT_DATA_LOADED,i.e.EVENT_DIRTY_HEADER,i.e.EVENT_DIRTY_VALUES,i.e.EVENT_DIRTY],this.loaded,this.loaded=t)},e.prototype.getRenderer=function(){return this.isLoaded()?t.prototype.getRenderer.call(this):e.RENDERER_LOADING},e.prototype.dump=function(n){var r=t.prototype.dump.call(this,n);return r.loaded=this.loaded,this.loaded||r.rendererType!==e.RENDERER_LOADING||delete r.rendererType,r},e.prototype.restore=function(e,n){void 0!==e.loaded&&(this.loaded=e.loaded),t.prototype.restore.call(this,e,n)},e.EVENT_DATA_LOADED="dataLoaded",e.RENDERER_LOADING="loading",e}(i.e);e.b=o},function(t,e,n){"use strict";n.d(e,"j",function(){return a}),n.d(e,"g",function(){return u}),n.d(e,"i",function(){return s}),n.d(e,"h",function(){return c}),n.d(e,"k",function(){return l}),n.d(e,"l",function(){return d});var r=n(0),i=n(18),o=n(7);function a(t){return Math.ceil(Math.log(t)/Math.LN2)+1}var u=function(){function t(t,e,n){this.scale=e,this.histGen=n,this.values=t.filter(function(t){return!Object(o.c)(t)}),this.missing=t.length-this.values.length}return Object.defineProperty(t.prototype,"count",{get:function(){return this.values.length+this.missing},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"sorted",{get:function(){return this.values.slice().sort(i.a)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"sortedMapped",{get:function(){var t=this;return this.scale?this.sorted.map(function(e){return t.scale.apply(e)}):this.sorted},enumerable:!0,configurable:!0}),t.prototype.map=function(t){return this.scale&&null!=t?this.scale.apply(t):t},Object.defineProperty(t.prototype,"hist",{get:function(){return this.histGen?this.histGen(this.values):[]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"maxBin",{get:function(){return Math.max.apply(Math,this.hist.map(function(t){return t.length}))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"min",{get:function(){return this.map(Math.min.apply(Math,this.values))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"max",{get:function(){return this.map(Math.max.apply(Math,this.values))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"median",{get:function(){return this.map(Object(i.h)(this.sorted))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"q1",{get:function(){return this.map(Object(i.j)(this.sorted,.25))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"q3",{get:function(){return this.map(Object(i.j)(this.sorted,.75))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"mean",{get:function(){return this.map(Object(i.g)(this.values))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"whiskerLow",{get:function(){for(var t=this.q1,e=t-1.5*(this.q3-t),n=e,r=0,i=this.sortedMapped;r<i.length;r++){var o=i[r];if(e<o){n=o;break}}return n},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"whiskerHigh",{get:function(){for(var t=this.q1,e=this.q3,n=e+1.5*(e-t),r=n,i=0,o=this.sortedMapped.slice().reverse();i<o.length;i++){var a=o[i];if(a<n){r=a;break}}return r},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"outlier",{get:function(){var t=this.whiskerLow,e=this.whiskerHigh;return this.sortedMapped.filter(function(n){return(n<t||n>e)&&!Object(o.c)(n)})},enumerable:!0,configurable:!0}),r.b([function(t,e,n){var r=n.get,i="__"+e;return n.get=function(){if(this.hasOwnProperty(i))return this[i];var t=r.call(this);return this[i]=t,t},n}],t.prototype,"sorted",null),r.b([function(t,e,n){var r=n.get,i="__"+e;return n.get=function(){if(this.hasOwnProperty(i))return this[i];var t=r.call(this);return this[i]=t,t},n}],t.prototype,"sortedMapped",null),r.b([function(t,e,n){var r=n.get,i="__"+e;return n.get=function(){if(this.hasOwnProperty(i))return this[i];var t=r.call(this);return this[i]=t,t},n}],t.prototype,"hist",null),r.b([function(t,e,n){var r=n.get,i="__"+e;return n.get=function(){if(this.hasOwnProperty(i))return this[i];var t=r.call(this);return this[i]=t,t},n}],t.prototype,"maxBin",null),r.b([function(t,e,n){var r=n.get,i="__"+e;return n.get=function(){if(this.hasOwnProperty(i))return this[i];var t=r.call(this);return this[i]=t,t},n}],t.prototype,"min",null),r.b([function(t,e,n){var r=n.get,i="__"+e;return n.get=function(){if(this.hasOwnProperty(i))return this[i];var t=r.call(this);return this[i]=t,t},n}],t.prototype,"max",null),r.b([function(t,e,n){var r=n.get,i="__"+e;return n.get=function(){if(this.hasOwnProperty(i))return this[i];var t=r.call(this);return this[i]=t,t},n}],t.prototype,"median",null),r.b([function(t,e,n){var r=n.get,i="__"+e;return n.get=function(){if(this.hasOwnProperty(i))return this[i];var t=r.call(this);return this[i]=t,t},n}],t.prototype,"q1",null),r.b([function(t,e,n){var r=n.get,i="__"+e;return n.get=function(){if(this.hasOwnProperty(i))return this[i];var t=r.call(this);return this[i]=t,t},n}],t.prototype,"q3",null),r.b([function(t,e,n){var r=n.get,i="__"+e;return n.get=function(){if(this.hasOwnProperty(i))return this[i];var t=r.call(this);return this[i]=t,t},n}],t.prototype,"mean",null),r.b([function(t,e,n){var r=n.get,i="__"+e;return n.get=function(){if(this.hasOwnProperty(i))return this[i];var t=r.call(this);return this[i]=t,t},n}],t.prototype,"whiskerLow",null),r.b([function(t,e,n){var r=n.get,i="__"+e;return n.get=function(){if(this.hasOwnProperty(i))return this[i];var t=r.call(this);return this[i]=t,t},n}],t.prototype,"whiskerHigh",null),r.b([function(t,e,n){var r=n.get,i="__"+e;return n.get=function(){if(this.hasOwnProperty(i))return this[i];var t=r.call(this);return this[i]=t,t},n}],t.prototype,"outlier",null),t}();function s(t,e,n,r,o){if(0===t.length)return{min:NaN,max:NaN,mean:NaN,q1:NaN,q3:NaN,outlier:[],median:NaN,count:0,maxBin:0,hist:[],missing:0};var s=Object(i.e)();r&&s.domain(r),o?s.thresholds(o):s.thresholds(a(t.length));var c=t.map(function(t){return n(t)?NaN:e(t)});return new u(c,void 0,s)}function c(t,e,n){var r=new Map,i=0;n.forEach(function(t){return r.set(t.name,0)}),t.forEach(function(t){var n=e(t);null!=n?r.set(n.name,(r.get(n.name)||0)+1):i+=1});var o=[];return r.forEach(function(t,e){return o.push({cat:e,y:t})}),{maxBin:Math.max.apply(Math,o.map(function(t){return t.y})),hist:o,missing:i}}function l(t,e){if(void 0===e&&(e=0),0===e)return Math.round(t);var n=Math.pow(10,e);return Math.round(t*n)/n}function d(t,e,n){return void 0===n&&(n=.5),t===e||Math.abs(t-e)<n}},function(t,e,n){"use strict";var r=n(0),i=n(48),o=n(74),a=n(1),u=n(2),s=n(23),c=n(4),l=n(12),d=n(7),p=n(8),f=n(27);n.d(e,"a",function(){return c.default}),n.d(e,"d",function(){return c.i});var h=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.currentFilter=Object(c.l)(),r.numberFormat=Object(i.a)(".2f"),r.currentGroupThresholds=[],r.groupSortMethod=c.b.median,r.mapping=Object(l.e)(n),r.original=r.mapping.clone(),r.colorMapping=Object(f.h)(r.color,n),n.numberFormat&&(r.numberFormat=Object(i.a)(n.numberFormat)),r.missingValue=null!=n.missingValue?n.missingValue:NaN,r.setGroupRenderer("boxplot"),r.setDefaultSummaryRenderer("histogram"),r}var n;return r.c(e,t),n=e,e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return n.map=this.mapping.dump(),n.colorMapping=this.colorMapping.dump(),n.filter=Object(s.c)(this.currentFilter)?null:this.currentFilter,n.groupSortMethod=this.groupSortMethod,this.currentGroupThresholds&&(n.stratifyThreshholds=this.currentGroupThresholds),n},e.prototype.restore=function(e,n){t.prototype.restore.call(this,e,n),e.map?this.mapping=Object(l.c)(e.map):e.domain&&(this.mapping=new l.a(e.domain,"linear",e.range||[0,1])),e.colorMapping&&(this.colorMapping=Object(f.e)(this.color,e.colorMapping)),e.groupSortMethod&&(this.groupSortMethod=e.groupSortMethod),e.filter&&(this.currentFilter=Object(s.g)(e.filter)),e.stratifyThreshholds&&(this.currentGroupThresholds=e.stratifyThresholds),e.numberFormat&&(this.numberFormat=Object(i.a)(e.numberFormat))},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_MAPPING_CHANGED,n.EVENT_COLOR_MAPPING_CHANGED,n.EVENT_FILTER_CHANGED,n.EVENT_SORTMETHOD_CHANGED,n.EVENT_GROUPING_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getLabel=function(e){if(this.desc.numberFormat){var n=this.getRawValue(e);return isNaN(n)?"NaN":isFinite(n)?this.numberFormat(n):n.toString()}var r=t.prototype.getValue.call(this,e);return"number"==typeof r?this.numberFormat(+r):String(r)},e.prototype.getRange=function(){return this.mapping.getRange(this.numberFormat)},e.prototype.getRawValue=function(e,n){void 0===n&&(n=this.missingValue);var r=t.prototype.getValue.call(this,e);return Object(d.c)(r)?n:+r},e.prototype.getExportValue=function(e,n){return"json"===n?this.getRawValue(e):t.prototype.getExportValue.call(this,e,n)},e.prototype.isMissing=function(e){return Object(d.c)(t.prototype.getValue.call(this,e))},e.prototype.getValue=function(t){var e=this.getRawValue(t);return isNaN(e)?e:this.mapping.apply(e)},e.prototype.getNumber=function(t){return this.getValue(t)},e.prototype.getRawNumber=function(t,e){return void 0===e&&(e=this.missingValue),this.getRawValue(t,e)},e.prototype.compare=function(t,e){return Object(c.m)(this.getNumber(t),this.getNumber(e),this.isMissing(t),this.isMissing(e))},e.prototype.groupCompare=function(t,e){return Object(s.b)(t.rows,e.rows,this,this.groupSortMethod)},e.prototype.getOriginalMapping=function(){return this.original.clone()},e.prototype.getMapping=function(){return this.mapping.clone()},e.prototype.setMapping=function(t){this.mapping.eq(t)||this.fire([n.EVENT_MAPPING_CHANGED,u.e.EVENT_DIRTY_VALUES,u.e.EVENT_DIRTY_HEADER,u.e.EVENT_DIRTY],this.mapping.clone(),this.mapping=t)},e.prototype.getColor=function(t){var e=this.getNumber(t);return isNaN(e)?this.color:this.colorMapping.apply(e)},e.prototype.getColorMapping=function(){return this.colorMapping.clone()},e.prototype.setColorMapping=function(t){this.colorMapping.eq(t)||this.fire([n.EVENT_COLOR_MAPPING_CHANGED,u.e.EVENT_DIRTY_VALUES,u.e.EVENT_DIRTY_HEADER,u.e.EVENT_DIRTY],this.colorMapping.clone(),this.colorMapping=t)},e.prototype.isFiltered=function(){return!Object(s.c)(this.currentFilter)},e.prototype.getFilter=function(){return Object.assign({},this.currentFilter)},e.prototype.setFilter=function(t){if(void 0===t&&(t={min:-1/0,max:1/0,filterMissing:!1}),!Object(c.h)(t,this.currentFilter)){var e=this.getFilter();this.currentFilter.min=Object(d.d)(t.min)?-1/0:t.min,this.currentFilter.max=Object(d.d)(t.max)?1/0:t.max,this.currentFilter.filterMissing=t.filterMissing,this.fire([n.EVENT_FILTER_CHANGED,u.e.EVENT_DIRTY_VALUES,u.e.EVENT_DIRTY],e,this.getFilter())}},e.prototype.filter=function(t){return Object(c.j)(this.currentFilter,this.getRawNumber(t,NaN))},e.prototype.getGroupThresholds=function(){return this.currentGroupThresholds.slice()},e.prototype.setGroupThresholds=function(t){if(!Object(o.b)(this.currentGroupThresholds,t)){var e=this.getGroupThresholds();this.currentGroupThresholds=t.slice(),this.fire([n.EVENT_GROUPING_CHANGED,u.e.EVENT_DIRTY_VALUES,u.e.EVENT_DIRTY],e,t)}},e.prototype.group=function(t){if(this.isMissing(t))return d.e;var e=this.currentGroupThresholds;if(0===e.length){var n=this.mapping.domain;e=[(n[1]-n[0])/2]}var r=this.getRawNumber(t),i=e.findIndex(function(t){return r<=t});switch(i){case-1:return{name:this.label+" > "+e[e.length-1],color:this.colorMapping.apply(1)};case 0:return{name:this.label+" <= "+e[0],color:this.colorMapping.apply(0)};default:return{name:e[i-1]+" <= "+this.label+" <= "+e[i],color:this.colorMapping.apply(this.mapping.apply((e[i-1]+e[i])/2))}}},e.prototype.getSortMethod=function(){return this.groupSortMethod},e.prototype.setSortMethod=function(t){this.groupSortMethod!==t&&(this.fire([n.EVENT_SORTMETHOD_CHANGED],this.groupSortMethod,this.groupSortMethod=t),this.isGroupSortedByMe().asc||this.toggleMyGroupSorting())},e.EVENT_MAPPING_CHANGED="mappingChanged",e.EVENT_COLOR_MAPPING_CHANGED="colorMappingChanged",e.EVENT_FILTER_CHANGED="filterChanged",e.EVENT_SORTMETHOD_CHANGED="sortMethodChanged",e.EVENT_GROUPING_CHANGED="groupingChanged",e=n=r.b([Object(a.l)("groupBy","sortGroupBy","filterNumber","colorMapped","editMapping"),Object(a.g)("sortGroup","sortNumber"),Object(a.g)("group","groupNumber"),Object(a.b)("number"),Object(a.c)("descending")],e)}(p.b);e.c=h},function(t,e,n){"use strict";n.d(e,"f",function(){return u}),n.d(e,"h",function(){return s}),n.d(e,"a",function(){return c}),n.d(e,"b",function(){return d}),n.d(e,"g",function(){return p}),n.d(e,"c",function(){return f}),n.d(e,"e",function(){return g}),n.d(e,"d",function(){return m});var r=n(2),i=n(71),o=n(23),a=n(7);function u(t){return Object(i.a)(t)&&Array.isArray(t.categories)}function s(t,e,n){if(void 0===n&&(n=function(){return r.e.DEFAULT_COLOR}),"string"==typeof t)return{name:t,label:t,color:n(),value:e};var i=null==t.name?String(t.value):t.name;return{name:i,label:t.label||i,color:t.color||n(),value:null!=t.value?t.value:e}}function c(t,e){var n=null==t||isNaN(t.value),r=null==e||isNaN(e.value);return n||null==t?r?0:a.a:r||null==e?-a.a:t.value===e.value?t.label.toLowerCase().localeCompare(e.label.toLowerCase()):t.value-e.value}function l(t,e){for(var n=new Map,r=0,i=t;r<i.length;r++){var o=i[r],a=e.getCategory(o);n.set(a,(n.get(a)||0)+1)}if(0===n.size)return{cat:null,count:0};var u=null,s=0;return n.forEach(function(t,e){t>s&&(u=e,s=t)}),{cat:u,count:s}}function d(t,e,n){if(0===t.length)return 0===e.length?0:a.a;if(0===e.length)return-a.a;var r=l(t,n),i=l(e,n);return null===r.cat?null===i.cat?0:a.a:null===i.cat?-a.a:r.cat===i.cat?i.count-r.count:r.cat.value-i.cat.value}function p(t){if(!t.categories)return[];var e=Object(o.a)(),n=t.categories.length-1;return t.categories.map(function(t,r){return s(t,r/n,e)}).sort(c)}function f(t){return t instanceof r.e&&"function"==typeof t.getCategory||!(t instanceof r.e)&&null!=t.type.match(/(categorical|ordinal|hierarchy)/)}function h(t){return null==t||!t.filterMissing&&(null==t.filter||""===t.filter)}function g(t,e){return t===e||(null==t||null==e?h(t)===h(e):t.filterMissing===e.filterMissing&&typeof t.filter==typeof e.filter&&(Array.isArray(t.filter)?function(t,e){var n=null!=t?t.length:0,r=null!=e?e.length:0;return n===r&&(0===n||t.every(function(t,n){return t===e[n]}))}(t.filter,e.filter):String(t.filter)===String(e.filter)))}function m(t,e){if(null==t)return!0;if(null==e||isNaN(e.value))return!t.filterMissing;var n=t.filter;return Array.isArray(n)?n.includes(e.name):"string"==typeof n&&n.length>0?e.name.toLowerCase().includes(n.toLowerCase()):!(n instanceof RegExp)||n.test(e.name)}},function(t,e,n){"use strict";n.d(e,"d",function(){return a}),n.d(e,"a",function(){return c}),n.d(e,"b",function(){return l}),n.d(e,"c",function(){return d}),n.d(e,"e",function(){return p});var r=n(59),i=n(9);function o(t){switch(void 0===t&&(t="linear"),t){case"log":return Object(r.b)().clamp(!0);case"sqrt":return Object(r.d)().clamp(!0);case"pow1.1":return Object(r.c)().exponent(1.1).clamp(!0);case"pow2":return Object(r.c)().exponent(2).clamp(!0);case"pow3":return Object(r.c)().exponent(3).clamp(!0);default:return Object(r.a)().clamp(!0)}}function a(t){return"function"==typeof t.getMapping}function u(t,e){return t.length===e.length&&t.every(function(t,n){return Object(i.l)(t,e[n],1e-4)})}function s(t,e){return"log"===e&&0===t[0]&&(t[0]=1e-7),t}var c=function(){function t(t,e,n){void 0===t&&(t=[0,1]),void 0===e&&(e="linear"),void 0===n&&(n=[0,1]),this.type=e,this.s=o(e).domain(s(t,this.type)).range(n)}return Object.defineProperty(t.prototype,"domain",{get:function(){return this.s.domain()},set:function(t){this.s.domain(s(t,this.type))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"range",{get:function(){return this.s.range()},set:function(t){this.s.range(t)},enumerable:!0,configurable:!0}),t.prototype.getRange=function(t){return[t(this.invert(0)),t(this.invert(1))]},t.prototype.apply=function(t){return this.s(t)},t.prototype.invert=function(t){return this.s.invert(t)},Object.defineProperty(t.prototype,"scaleType",{get:function(){return this.type},enumerable:!0,configurable:!0}),t.prototype.dump=function(){return{type:this.type,domain:this.domain,range:this.range}},t.prototype.eq=function(e){if(!(e instanceof t))return!1;var n=e;return n.type===this.type&&u(this.domain,n.domain)&&u(this.range,n.range)},t.prototype.restore=function(t){this.type=t.type,this.s=o(t.type).domain(t.domain).range(t.range)},t.prototype.clone=function(){return new t(this.domain,this.type,this.range)},t}(),l=function(){function t(t,e){void 0===t&&(t=[0,1]),void 0===e&&(e="return this.linear(value,this.value_min,this.value_max);"),this.domain=t,this._code=e,this.f=new Function("value",e)}return Object.defineProperty(t.prototype,"code",{get:function(){return this._code},set:function(t){this._code!==t&&(this._code=t,this.f=new Function("value",t))},enumerable:!0,configurable:!0}),t.prototype.getRange=function(){return["?","?"]},t.prototype.apply=function(t){var e=this.domain[0],n=this.domain[this.domain.length-1],r=this.f.call({value_min:e,value_max:n,value_range:n-e,value_domain:this.domain.slice(),linear:function(t,e,n){return(t-e)/(n-e)}},t);return"number"==typeof r?Math.max(Math.min(r,1),0):NaN},t.prototype.dump=function(){return{type:"script",code:this.code,domain:this.domain}},t.prototype.eq=function(e){return e instanceof t&&e.code===this.code},t.prototype.restore=function(t){this.code=t.code,this.domain=t.domain},t.prototype.clone=function(){return new t(this.domain,this.code)},t}();function d(t){if("script"===t.type){var e=new l;return e.restore(t),e}var n=new c;return n.restore(t),n}function p(t){return t.map?d(t.map):t.domain?new c(t.domain,"linear",t.range||[0,1]):new c([0,1],"linear",[0,1])}},function(t,e,n){"use strict";n.d(e,"c",function(){return c});var r=n(0),i=n(16),o=n(2),a=n(1),u=n(4),s=n(8);function c(t){return"function"==typeof t.getCollapsed}var l=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._children=[],e}var n;return r.c(e,t),n=e,e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_FILTER_CHANGED,n.EVENT_ADD_COLUMN,n.EVENT_MOVE_COLUMN,n.EVENT_REMOVE_COLUMN])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.assignNewId=function(e){t.prototype.assignNewId.call(this,e),this._children.forEach(function(t){return t.assignNewId(e)})},Object.defineProperty(e.prototype,"children",{get:function(){return this._children.slice()},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"length",{get:function(){return this._children.length},enumerable:!0,configurable:!0}),e.prototype.flatten=function(t,e,n,r){void 0===n&&(n=0),void 0===r&&(r=0);var i=0;return(0===n||n<=o.e.FLAT_ALL_COLUMNS)&&(i=this.getWidth(),t.push({col:this,offset:e,width:i}),0===n)?i:(this._children.forEach(function(i){i.isVisible()&&n<=o.e.FLAT_ALL_COLUMNS&&i.flatten(t,e,n-1,r)}),i)},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return n.children=this._children.map(function(t){return t.dump(e)}),n},e.prototype.restore=function(e,n){var r=this;e.children.map(function(t){var e=n(t);e&&r.push(e)}),t.prototype.restore.call(this,e,n)},e.prototype.insert=function(t,e){return!Object(u.i)(t)&&this.canJustAddNumbers?null:(this._children.splice(e,0,t),this.insertImpl(t,e))},e.prototype.move=function(t,e){if(t.parent!==this)return null;var n=this._children.indexOf(t);return e===n?t:(this._children.splice(n,1),this._children.splice(n<e?e-1:e,0,t),this.moveImpl(t,e,n))},e.prototype.insertImpl=function(t,e){return t.attach(this),this.forward.apply(this,[t].concat(Object(i.b)(".combine",o.e.EVENT_DIRTY_HEADER,o.e.EVENT_DIRTY_VALUES,o.e.EVENT_DIRTY,n.EVENT_FILTER_CHANGED,o.e.EVENT_RENDERER_TYPE_CHANGED,o.e.EVENT_GROUP_RENDERER_TYPE_CHANGED))),this.fire([n.EVENT_ADD_COLUMN,o.e.EVENT_DIRTY_HEADER,o.e.EVENT_DIRTY_VALUES,o.e.EVENT_DIRTY],t,e),t},e.prototype.moveImpl=function(t,e,r){return this.fire([n.EVENT_MOVE_COLUMN,o.e.EVENT_DIRTY_HEADER,o.e.EVENT_DIRTY_VALUES,o.e.EVENT_DIRTY,o.e.EVENT_RENDERER_TYPE_CHANGED,o.e.EVENT_GROUP_RENDERER_TYPE_CHANGED],t,e,r),t},e.prototype.push=function(t){return this.insert(t,this._children.length)},e.prototype.at=function(t){return this._children[t]},e.prototype.indexOf=function(t){return this._children.indexOf(t)},e.prototype.insertAfter=function(t,e){var n=this.indexOf(e);return n<0?null:this.insert(t,n+1)},e.prototype.moveAfter=function(t,e){var n=this.indexOf(e);return n<0?null:this.move(t,n+1)},e.prototype.remove=function(t){var e=this._children.indexOf(t);return!(e<0)&&(this._children.splice(e,1),this.removeImpl(t,e))},e.prototype.removeImpl=function(t,e){return t.detach(),this.unforward.apply(this,[t].concat(Object(i.b)(".combine",o.e.EVENT_DIRTY_HEADER,o.e.EVENT_DIRTY_VALUES,o.e.EVENT_DIRTY,n.EVENT_FILTER_CHANGED))),this.fire([n.EVENT_REMOVE_COLUMN,o.e.EVENT_DIRTY_HEADER,o.e.EVENT_DIRTY_VALUES,o.e.EVENT_DIRTY],t,e),!0},e.prototype.getColor=function(t){return this.color},e.prototype.isFiltered=function(){return this._children.some(function(t){return t.isFiltered()})},e.prototype.filter=function(t){return this._children.every(function(e){return e.filter(t)})},e.prototype.isLoaded=function(){return this._children.every(function(t){return!(t instanceof s.b||t instanceof n)||t.isLoaded()})},Object.defineProperty(e.prototype,"canJustAddNumbers",{get:function(){return!1},enumerable:!0,configurable:!0}),e.EVENT_FILTER_CHANGED="filterChanged",e.EVENT_ADD_COLUMN="addColumn",e.EVENT_MOVE_COLUMN="moveColumn",e.EVENT_REMOVE_COLUMN="removeColumn",e=n=r.b([Object(a.l)("compositeContained","splitCombined"),Object(a.b)("composite")],e)}(o.e);e.b=l},function(t,e,n){"use strict";var r=n(0),i=n(74),o=n(16),a=n(1),u=n(2),s=n(50),c=n(23),l=n(10),d=n(13),p=function(t){function e(n,r,o){void 0===r&&(r=2),void 0===o&&(o=1);var a=t.call(this)||this;return a.id=n,a.maxSortCriteria=r,a.maxGroupColumns=o,a.sortCriteria=[],a.groupSortCriteria=[],a.groupColumns=[],a.columns=[],a.comparator=function(t,e){if(0===a.sortCriteria.length)return t.i-e.i;for(var n=0,r=a.sortCriteria;n<r.length;n++){var i=r[n],o=i.col.compare(t,e);if(0!==o)return i.asc?o:-o}return t.i-e.i},a.groupComparator=function(t,e){if(0===a.groupSortCriteria.length)return t.name.toLowerCase().localeCompare(e.name.toLowerCase());for(var n=0,r=a.groupSortCriteria;n<r.length;n++){var i=r[n],o=i.col.groupCompare(t,e);if(0!==o)return i.asc?o:-o}return t.name.localeCompare(e.name)},a.grouper=function(t){var e=a.groupColumns;switch(e.length){case 0:return s.a;case 1:return e[0].group(t);default:var n=e.map(function(e){return e.group(t)});return Object(c.d)(n)}},a.dirtyOrder=function(){a.fire([e.EVENT_DIRTY_ORDER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],a.getSortCriteria(),a.getGroupSortCriteria(),a.getGroupCriteria())},a.groups=[Object.assign({order:[]},s.a)],a.id=Object(i.d)(n),a.label="Ranking "+(n.startsWith("rank")?n.slice(4):n),a}return r.c(e,t),e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([e.EVENT_WIDTH_CHANGED,e.EVENT_FILTER_CHANGED,e.EVENT_LABEL_CHANGED,e.EVENT_GROUPS_CHANGED,e.EVENT_ADD_COLUMN,e.EVENT_REMOVE_COLUMN,e.EVENT_GROUP_CRITERIA_CHANGED,e.EVENT_MOVE_COLUMN,e.EVENT_DIRTY,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_GROUP_SORT_CRITERIA_CHANGED,e.EVENT_COLUMN_VISIBILITY_CHANGED,e.EVENT_SORT_CRITERIA_CHANGED,e.EVENT_DIRTY_ORDER,e.EVENT_ORDER_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.assignNewId=function(t){this.id=Object(i.d)(t()),this.columns.forEach(function(e){return e.assignNewId(t)})},e.prototype.getLabel=function(){return this.label},e.prototype.setLabel=function(t){t!==this.label&&this.fire(e.EVENT_LABEL_CHANGED,this.label,this.label=t)},e.prototype.setOrder=function(t){this.setGroups([Object.assign({order:t},s.a)])},e.prototype.setGroups=function(t){var n=this.getOrder(),r=this.groups;this.groups=t,this.fire([e.EVENT_ORDER_CHANGED,e.EVENT_GROUPS_CHANGED,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],n,this.getOrder(),r,t)},e.prototype.getOrder=function(){var t;switch(this.groups.length){case 0:return[];case 1:return this.groups[0].order;default:return(t=[]).concat.apply(t,this.groups.map(function(t){return t.order}))}},e.prototype.getGroups=function(){return this.groups.slice()},e.prototype.dump=function(t){var e={};return e.columns=this.columns.map(function(e){return e.dump(t)}),e.sortCriteria=this.sortCriteria.map(function(t){return{asc:t.asc,sortBy:t.col.id}}),e.groupSortCriteria=this.groupSortCriteria.map(function(t){return{asc:t.asc,sortBy:t.col.id}}),e.groupColumns=this.groupColumns.map(function(t){return t.id}),e},e.prototype.restore=function(t,e){var n=this;if(this.clear(),t.columns.map(function(t){var r=e(t);r&&n.push(r)}),t.sortColumn&&t.sortColumn.sortBy){var r=this.columns.find(function(e){return e.id===t.sortColumn.sortBy});r&&this.sortBy(r,t.sortColumn.asc)}if(t.groupColumns){var i=t.groupColumns.map(function(t){return n.columns.find(function(e){return e.id===t})});this.groupBy(i)}var o=function(t){return t.map(function(t){return{asc:t.asc,col:n.columns.find(function(e){return e.id===t.sortBy})||null}}).filter(function(t){return t.col})};t.sortCriteria&&this.setSortCriteria(o(t.sortCriteria)),t.groupSortCriteria&&this.setGroupSortCriteria(o(t.groupSortCriteria))},e.prototype.flatten=function(t,e,n,r){void 0===n&&(n=0),void 0===r&&(r=0);var i=e;return(n>0||n<=u.e.FLAT_ALL_COLUMNS)&&this.columns.forEach(function(e){e.getVisible()&&n<=u.e.FLAT_ALL_COLUMNS&&(i+=e.flatten(t,i,n-1,r)+r)}),i-e},e.prototype.getPrimarySortCriteria=function(){return 0===this.sortCriteria.length?null:this.sortCriteria[0]},e.prototype.getSortCriteria=function(){return this.sortCriteria.map(function(t){return Object.assign({},t)})},e.prototype.getGroupSortCriteria=function(){return this.groupSortCriteria.map(function(t){return Object.assign({},t)})},e.prototype.toggleSorting=function(t){return this.setSortCriteria(this.toggleSortingLogic(t,this.sortCriteria))},e.prototype.toggleSortingLogic=function(t,e){var n=e.slice(),r=n.findIndex(function(e){return e.col===t}),i=Object(a.j)(t);return r<0?n.splice(0,n.length,{col:t,asc:i}):n[r].asc===i?n.splice(r,1,{col:t,asc:!i}):n.splice(r,1),n},e.prototype.toggleGrouping=function(t){var e=this.groupColumns.indexOf(t);if(e>=0){var n=this.groupColumns.slice();return n.splice(e,1),this.setGroupCriteria(n)}return this.setGroupCriteria([t])},e.prototype.getGroupCriteria=function(){return this.groupColumns.slice()},e.prototype.sortBy=function(t,e,n){return void 0===e&&(e=!1),void 0===n&&(n=0),t.findMyRanker()===this&&this.setSortCriteria(this.hierarchyLogic(this.sortCriteria,this.sortCriteria.findIndex(function(e){return e.col===t}),{col:t,asc:e},n))},e.prototype.groupSortBy=function(t,e,n){return void 0===e&&(e=!1),void 0===n&&(n=0),t.findMyRanker()===this&&this.setGroupSortCriteria(this.hierarchyLogic(this.groupSortCriteria,this.groupSortCriteria.findIndex(function(e){return e.col===t}),{col:t,asc:e},n))},e.prototype.hierarchyLogic=function(t,e,n,r){return t=t.slice(),e>=0?(t.splice(e,1),r>=0&&t.splice(Math.min(r,t.length),0,n)):r>=0&&(t[Math.min(r,t.length)]=n),t},e.prototype.groupBy=function(t,e){return void 0===e&&(e=0),t.findMyRanker()===this&&this.setGroupCriteria(this.hierarchyLogic(this.groupColumns,this.groupColumns.indexOf(t),t,e))},e.prototype.setSortCriteria=function(t){var e,n=this,r=Array.isArray(t)?t.slice():[t];r.length>this.maxSortCriteria&&r.splice(this.maxSortCriteria,r.length-this.maxSortCriteria);var i=this.sortCriteria.slice();return!f(r,i)&&(i.forEach(function(t){t.col.on(u.e.EVENT_DIRTY_VALUES+".order",null),t.col.on(l.c.EVENT_SORTMETHOD_CHANGED+".order",null)}),r.forEach(function(t){t.col.on(u.e.EVENT_DIRTY_VALUES+".order",n.dirtyOrder),t.col.on(l.c.EVENT_SORTMETHOD_CHANGED+".order",n.dirtyOrder)}),(e=this.sortCriteria).splice.apply(e,[0,this.sortCriteria.length].concat(r.slice())),this.triggerResort(i),!0)},e.prototype.setGroupCriteria=function(t){var n,r=this,a=Array.isArray(t)?t:[t];if(a.length>this.maxGroupColumns&&(a=a.slice(0,this.maxGroupColumns)),Object(i.b)(this.groupColumns,a))return!0;this.groupColumns.forEach(function(t){t.on(Object(o.b)(".group",u.e.EVENT_DIRTY_VALUES,l.c.EVENT_SORTMETHOD_CHANGED,l.c.EVENT_GROUPING_CHANGED),null)});var s=this.groupColumns.slice();return(n=this.groupColumns).splice.apply(n,[0,this.groupColumns.length].concat(a)),this.groupColumns.forEach(function(t){t.on(Object(o.b)(".group",u.e.EVENT_DIRTY_VALUES,l.c.EVENT_SORTMETHOD_CHANGED,l.c.EVENT_GROUPING_CHANGED),r.dirtyOrder)}),this.fire([e.EVENT_GROUP_CRITERIA_CHANGED,e.EVENT_DIRTY_ORDER,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],s,this.getGroupCriteria()),!0},e.prototype.toggleGroupSorting=function(t){return this.setGroupSortCriteria(this.toggleSortingLogic(t,this.groupSortCriteria))},e.prototype.setMaxSortCriteria=function(t){var e=this.maxSortCriteria;e!==t&&(this.maxSortCriteria=t,e<t||this.sortCriteria.length<t||this.setSortCriteria(this.sortCriteria.slice(0,t)))},e.prototype.getMaxSortCriteria=function(){return this.maxSortCriteria},e.prototype.setMaxGroupColumns=function(t){var e=this.maxGroupColumns;e!==t&&(this.maxGroupColumns=t,e<t||this.groupColumns.length<t||this.setGroupCriteria(this.groupColumns.slice(0,t)))},e.prototype.getMaxGroupColumns=function(){return this.maxGroupColumns},e.prototype.setGroupSortCriteria=function(t){var e,n=this,r=Array.isArray(t)?t.slice():[t];r.length>this.maxSortCriteria&&r.splice(this.maxSortCriteria,r.length-this.maxSortCriteria);var i=this.groupSortCriteria.slice();return!f(r,i)&&(i.forEach(function(t){t.col.on(u.e.EVENT_DIRTY_VALUES+".groupOrder",null),t.col.on(l.c.EVENT_SORTMETHOD_CHANGED+".groupOrder",null)}),r.forEach(function(t){t.col.on(u.e.EVENT_DIRTY_VALUES+".groupOrder",n.dirtyOrder),t.col.on(l.c.EVENT_SORTMETHOD_CHANGED+".groupOrder",n.dirtyOrder)}),(e=this.groupSortCriteria).splice.apply(e,[0,this.groupSortCriteria.length].concat(r.slice())),this.triggerGroupResort(i),!0)},e.prototype.triggerGroupResort=function(t){var n=this.getGroupSortCriteria(),r=Array.isArray(t)?t:n;this.fire([e.EVENT_GROUP_SORT_CRITERIA_CHANGED,e.EVENT_DIRTY_ORDER,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],r,n)},e.prototype.triggerResort=function(t){var n=this.getSortCriteria(),r=Array.isArray(t)?t:n;this.fire([e.EVENT_SORT_CRITERIA_CHANGED,e.EVENT_DIRTY_ORDER,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],r,n)},Object.defineProperty(e.prototype,"children",{get:function(){return this.columns.slice()},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"length",{get:function(){return this.columns.length},enumerable:!0,configurable:!0}),e.prototype.insert=function(t,n){var r=this;return void 0===n&&(n=this.columns.length),this.columns.splice(n,0,t),t.attach(this),this.forward.apply(this,[t].concat(Object(o.b)(".ranking",u.e.EVENT_DIRTY_VALUES,u.e.EVENT_DIRTY_HEADER,u.e.EVENT_DIRTY,l.c.EVENT_FILTER_CHANGED))),t.on(e.EVENT_FILTER_CHANGED+".order",this.dirtyOrder),t.on(u.e.EVENT_VISIBILITY_CHANGED+".ranking",function(n,i){return r.fire([e.EVENT_COLUMN_VISIBILITY_CHANGED,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],t,n,i)}),this.fire([e.EVENT_ADD_COLUMN,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],t,n),t},e.prototype.move=function(t,n){if(void 0===n&&(n=this.columns.length),t.parent!==this)return console.error("invalid move operation: ",t),null;var r=this.columns.indexOf(t);return n===r?t:(this.columns.splice(r,1),this.columns.splice(r<n?n-1:n,0,t),this.fire([e.EVENT_MOVE_COLUMN,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],t,n,r),t)},e.prototype.moveAfter=function(t,e){var n=this.columns.indexOf(e);return n<0?null:this.move(t,n+1)},Object.defineProperty(e.prototype,"fqpath",{get:function(){return""},enumerable:!0,configurable:!0}),e.prototype.findByPath=function(t){for(var e=this,n=t.split("@").map(Number).slice(1);n.length>0;){var r=n.shift();e=e.at(r)}return e},e.prototype.indexOf=function(t){return this.columns.indexOf(t)},e.prototype.at=function(t){return this.columns[t]},e.prototype.insertAfter=function(t,e){var n=this.columns.indexOf(e);return n<0?null:this.insert(t,n+1)},e.prototype.push=function(t){return this.insert(t)},e.prototype.remove=function(t){var n=this.columns.indexOf(t);if(n<0)return!1;this.unforward.apply(this,[t].concat(Object(o.b)(".ranking",u.e.EVENT_VISIBILITY_CHANGED,u.e.EVENT_DIRTY_VALUES,u.e.EVENT_DIRTY_HEADER,u.e.EVENT_DIRTY,l.c.EVENT_FILTER_CHANGED)));var r=this.sortCriteria.findIndex(function(e){return e.col===t}),i=r>=0;i&&this.sortCriteria.splice(r,1);var a=this.groupSortCriteria.findIndex(function(e){return e.col===t}),s=a>=0;s&&this.groupSortCriteria.splice(a,1);var c=null,d=this.groupColumns.indexOf(t);return d>=0&&(c=this.groupColumns.slice()).splice(d,1),t.detach(),this.columns.splice(n,1),this.fire([e.EVENT_REMOVE_COLUMN,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],t,n),c?this.setGroupCriteria(c):i?this.triggerResort(null):s&&this.triggerGroupResort(null),!0},e.prototype.clear=function(){var t=this;if(0!==this.columns.length){this.sortCriteria.splice(0,this.sortCriteria.length),this.groupSortCriteria.splice(0,this.groupSortCriteria.length),this.groupColumns.forEach(function(t){t.on(u.e.EVENT_DIRTY_VALUES+".group",null),t.on(l.c.EVENT_SORTMETHOD_CHANGED+".group",null)}),this.groupColumns.splice(0,this.groupColumns.length),this.columns.forEach(function(e){t.unforward.apply(t,[e].concat(Object(o.b)(".ranking",u.e.EVENT_DIRTY_VALUES,u.e.EVENT_DIRTY_HEADER,u.e.EVENT_DIRTY,l.c.EVENT_FILTER_CHANGED))),e.detach()});var n=this.columns.splice(0,this.columns.length);this.fire([e.EVENT_REMOVE_COLUMN,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],n)}},Object.defineProperty(e.prototype,"flatColumns",{get:function(){var t=[];return this.flatten(t,0,u.e.FLAT_ALL_COLUMNS),t.map(function(t){return t.col})},enumerable:!0,configurable:!0}),e.prototype.find=function(t){for(var e="string"==typeof t?function(e){return e.id===t}:t,n=0,r=this.flatColumns;n<r.length;n++){var i=r[n];if(e(i))return i}return null},e.prototype.isFiltered=function(){return this.columns.some(function(t){return t.isFiltered()})},e.prototype.filter=function(t){return this.columns.every(function(e){return e.filter(t)})},e.prototype.findMyRanker=function(){return this},Object.defineProperty(e.prototype,"fqid",{get:function(){return this.id},enumerable:!0,configurable:!0}),e.prototype.markDirty=function(t){switch(void 0===t&&(t="all"),t){case"header":return this.fire([u.e.EVENT_DIRTY_HEADER,u.e.EVENT_DIRTY]);case"values":return this.fire([u.e.EVENT_DIRTY_VALUES,u.e.EVENT_DIRTY]);default:return this.fire([u.e.EVENT_DIRTY_HEADER,u.e.EVENT_DIRTY_VALUES,u.e.EVENT_DIRTY])}},e.EVENT_WIDTH_CHANGED=u.e.EVENT_WIDTH_CHANGED,e.EVENT_FILTER_CHANGED=l.c.EVENT_FILTER_CHANGED,e.EVENT_LABEL_CHANGED=u.e.EVENT_LABEL_CHANGED,e.EVENT_ADD_COLUMN=d.b.EVENT_ADD_COLUMN,e.EVENT_MOVE_COLUMN=d.b.EVENT_MOVE_COLUMN,e.EVENT_REMOVE_COLUMN=d.b.EVENT_REMOVE_COLUMN,e.EVENT_DIRTY=u.e.EVENT_DIRTY,e.EVENT_DIRTY_HEADER=u.e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES=u.e.EVENT_DIRTY_VALUES,e.EVENT_COLUMN_VISIBILITY_CHANGED=u.e.EVENT_VISIBILITY_CHANGED,e.EVENT_SORT_CRITERIA_CHANGED="sortCriteriaChanged",e.EVENT_GROUP_CRITERIA_CHANGED="groupCriteriaChanged",e.EVENT_GROUP_SORT_CRITERIA_CHANGED="groupSortCriteriaChanged",e.EVENT_DIRTY_ORDER="dirtyOrder",e.EVENT_ORDER_CHANGED="orderChanged",e.EVENT_GROUPS_CHANGED="groupsChanged",e}(o.a);function f(t,e){return t.length===e.length&&t.every(function(t,n){var r=e[n];return t.col===r.col&&t.asc===r.asc})}e.b=p},function(t,e,n){"use strict";n.d(e,"c",function(){return i}),n.d(e,"b",function(){return o}),n.d(e,"a",function(){return a});var r=n(5);function i(t,e,n,i,o){void 0===i&&(i=0),void 0===o&&(o=0);var a=Math.max(0,i+(e-r.e.width)/2),u=Math.max(0,o+(n-r.e.height)/2);t.fillStyle=r.e.color,t.fillRect(a,u,Math.min(e,r.e.width),Math.min(n,r.e.height))}function o(t,e,n){var r=e.isMissing(n);return t.classList.toggle("lu-missing",r),r}function a(t,e,n,o,a,u){void 0===a&&(a=0),void 0===u&&(u=0);var s=e.isMissing(n);return s&&i(t,o,r.c,a,u),s}},function(t,e,n){"use strict";var r={value:function(){}};function i(){for(var t,e=0,n=arguments.length,r={};e<n;++e){if(!(t=arguments[e]+"")||t in r)throw new Error("illegal type: "+t);r[t]=[]}return new o(r)}function o(t){this._=t}function a(t,e){for(var n,r=0,i=t.length;r<i;++r)if((n=t[r]).name===e)return n.value}function u(t,e,n){for(var i=0,o=t.length;i<o;++i)if(t[i].name===e){t[i]=r,t=t.slice(0,i).concat(t.slice(i+1));break}return null!=n&&t.push({name:e,value:n}),t}o.prototype=i.prototype={constructor:o,on:function(t,e){var n,r=this._,i=function(t,e){return t.trim().split(/^|\s+/).map(function(t){var n="",r=t.indexOf(".");if(r>=0&&(n=t.slice(r+1),t=t.slice(0,r)),t&&!e.hasOwnProperty(t))throw new Error("unknown type: "+t);return{type:t,name:n}})}(t+"",r),o=-1,s=i.length;if(!(arguments.length<2)){if(null!=e&&"function"!=typeof e)throw new Error("invalid callback: "+e);for(;++o<s;)if(n=(t=i[o]).type)r[n]=u(r[n],t.name,e);else if(null==e)for(n in r)r[n]=u(r[n],t.name,null);return this}for(;++o<s;)if((n=(t=i[o]).type)&&(n=a(r[n],t.name)))return n},copy:function(){var t={},e=this._;for(var n in e)t[n]=e[n].slice();return new o(t)},call:function(t,e){if((n=arguments.length-2)>0)for(var n,r,i=new Array(n),o=0;o<n;++o)i[o]=arguments[o+2];if(!this._.hasOwnProperty(t))throw new Error("unknown type: "+t);for(o=0,n=(r=this._[t]).length;o<n;++o)r[o].value.apply(e,i)},apply:function(t,e,n){if(!this._.hasOwnProperty(t))throw new Error("unknown type: "+t);for(var r=this._[t],i=0,o=r.length;i<o;++i)r[i].value.apply(e,n)}};var s=i;function c(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return e.map(function(e){return""+e+t})}n.d(e,"b",function(){return c});var l=function(){function t(){var t=this.createEventList();this.listenerEvents=new Set(t),this.listeners=s.apply(void 0,t);var e=this;this.forwarder=function(){for(var t=[],n=0;n<arguments.length;n++)t[n]=arguments[n];e.fireImpl.apply(e,[this.type,this.primaryType,this.origin].concat(t))}}return t.prototype.on=function(t,e){var n=this;return Array.isArray(t)?t.forEach(function(t){n.listenerEvents.has(t.split(".")[0])&&(n.listenersChanged(t,Boolean(e)),n.listeners.on(t,e))}):this.listenerEvents.has(t.split(".")[0])&&(this.listenersChanged(t,Boolean(e)),this.listeners.on(t,e)),this},t.prototype.listenersChanged=function(t,e){},t.prototype.createEventList=function(){return[]},t.prototype.fire=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];var r=Array.isArray(t)?t[0]:t;this.fireImpl.apply(this,[t,r,this].concat(e))},t.prototype.fireImpl=function(t,e,n){for(var r=this,i=[],o=3;o<arguments.length;o++)i[o-3]=arguments[o];var a=function(t){if(r.listenerEvents.has(t)){var o={source:r,origin:n,type:t,primaryType:e,args:i};r.listeners.apply(t,o,i)}};Array.isArray(t)?t.forEach(a.bind(this)):a(t)},t.prototype.forward=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];t.on(e,this.forwarder)},t.prototype.unforward=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];t.on(e,null)},t}();e.a=l},function(t,n,r){"use strict";
/**!
 * @fileOverview Kickass library to create and place poppers near their reference elements.
 * @version 1.14.4
 * @license
 * Copyright (c) 2016 Federico Zivolo and contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */for(var i="undefined"!=typeof window&&"undefined"!=typeof document,o=["Edge","Trident","Firefox"],a=0,u=0;u<o.length;u+=1)if(i&&navigator.userAgent.indexOf(o[u])>=0){a=1;break}var s=i&&window.Promise?function(t){var e=!1;return function(){e||(e=!0,window.Promise.resolve().then(function(){e=!1,t()}))}}:function(t){var e=!1;return function(){e||(e=!0,setTimeout(function(){e=!1,t()},a))}};function c(t){return t&&"[object Function]"==={}.toString.call(t)}function l(t,e){if(1!==t.nodeType)return[];var n=getComputedStyle(t,null);return e?n[e]:n}function d(t){return"HTML"===t.nodeName?t:t.parentNode||t.host}function p(t){if(!t)return document.body;switch(t.nodeName){case"HTML":case"BODY":return t.ownerDocument.body;case"#document":return t.body}var e=l(t),n=e.overflow,r=e.overflowX,i=e.overflowY;return/(auto|scroll|overlay)/.test(n+i+r)?t:p(d(t))}var f=i&&!(!window.MSInputMethodContext||!document.documentMode),h=i&&/MSIE 10/.test(navigator.userAgent);function g(t){return 11===t?f:10===t?h:f||h}function m(t){if(!t)return document.documentElement;for(var e=g(10)?document.body:null,n=t.offsetParent;n===e&&t.nextElementSibling;)n=(t=t.nextElementSibling).offsetParent;var r=n&&n.nodeName;return r&&"BODY"!==r&&"HTML"!==r?-1!==["TD","TABLE"].indexOf(n.nodeName)&&"static"===l(n,"position")?m(n):n:t?t.ownerDocument.documentElement:document.documentElement}function b(t){return null!==t.parentNode?b(t.parentNode):t}function y(t,e){if(!(t&&t.nodeType&&e&&e.nodeType))return document.documentElement;var n=t.compareDocumentPosition(e)&Node.DOCUMENT_POSITION_FOLLOWING,r=n?t:e,i=n?e:t,o=document.createRange();o.setStart(r,0),o.setEnd(i,0);var a=o.commonAncestorContainer;if(t!==a&&e!==a||r.contains(i))return function(t){var e=t.nodeName;return"BODY"!==e&&("HTML"===e||m(t.firstElementChild)===t)}(a)?a:m(a);var u=b(t);return u.host?y(u.host,e):y(t,b(e).host)}function v(t){var e="top"===(arguments.length>1&&void 0!==arguments[1]?arguments[1]:"top")?"scrollTop":"scrollLeft",n=t.nodeName;if("BODY"===n||"HTML"===n){var r=t.ownerDocument.documentElement;return(t.ownerDocument.scrollingElement||r)[e]}return t[e]}function E(t,e){var n="x"===e?"Left":"Top",r="Left"===n?"Right":"Bottom";return parseFloat(t["border"+n+"Width"],10)+parseFloat(t["border"+r+"Width"],10)}function C(t,e,n,r){return Math.max(e["offset"+t],e["scroll"+t],n["client"+t],n["offset"+t],n["scroll"+t],g(10)?parseInt(n["offset"+t])+parseInt(r["margin"+("Height"===t?"Top":"Left")])+parseInt(r["margin"+("Height"===t?"Bottom":"Right")]):0)}function w(t){var e=t.body,n=t.documentElement,r=g(10)&&getComputedStyle(n);return{height:C("Height",e,n,r),width:C("Width",e,n,r)}}var A=function(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")},x=function(){function t(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r)}}return function(e,n,r){return n&&t(e.prototype,n),r&&t(e,r),e}}(),N=function(t,e,n){return e in t?Object.defineProperty(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t},T=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t};function O(t){return T({},t,{right:t.left+t.width,bottom:t.top+t.height})}function R(t){var e={};try{if(g(10)){e=t.getBoundingClientRect();var n=v(t,"top"),r=v(t,"left");e.top+=n,e.left+=r,e.bottom+=n,e.right+=r}else e=t.getBoundingClientRect()}catch(t){}var i={left:e.left,top:e.top,width:e.right-e.left,height:e.bottom-e.top},o="HTML"===t.nodeName?w(t.ownerDocument):{},a=o.width||t.clientWidth||i.right-i.left,u=o.height||t.clientHeight||i.bottom-i.top,s=t.offsetWidth-a,c=t.offsetHeight-u;if(s||c){var d=l(t);s-=E(d,"x"),c-=E(d,"y"),i.width-=s,i.height-=c}return O(i)}function M(t,e){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=g(10),i="HTML"===e.nodeName,o=R(t),a=R(e),u=p(t),s=l(e),c=parseFloat(s.borderTopWidth,10),d=parseFloat(s.borderLeftWidth,10);n&&i&&(a.top=Math.max(a.top,0),a.left=Math.max(a.left,0));var f=O({top:o.top-a.top-c,left:o.left-a.left-d,width:o.width,height:o.height});if(f.marginTop=0,f.marginLeft=0,!r&&i){var h=parseFloat(s.marginTop,10),m=parseFloat(s.marginLeft,10);f.top-=c-h,f.bottom-=c-h,f.left-=d-m,f.right-=d-m,f.marginTop=h,f.marginLeft=m}return(r&&!n?e.contains(u):e===u&&"BODY"!==u.nodeName)&&(f=function(t,e){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=v(e,"top"),i=v(e,"left"),o=n?-1:1;return t.top+=r*o,t.bottom+=r*o,t.left+=i*o,t.right+=i*o,t}(f,e)),f}function D(t){if(!t||!t.parentElement||g())return document.documentElement;for(var e=t.parentElement;e&&"none"===l(e,"transform");)e=e.parentElement;return e||document.documentElement}function _(t,e,n,r){var i=arguments.length>4&&void 0!==arguments[4]&&arguments[4],o={top:0,left:0},a=i?D(t):y(t,e);if("viewport"===r)o=function(t){var e=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=t.ownerDocument.documentElement,r=M(t,n),i=Math.max(n.clientWidth,window.innerWidth||0),o=Math.max(n.clientHeight,window.innerHeight||0),a=e?0:v(n),u=e?0:v(n,"left");return O({top:a-r.top+r.marginTop,left:u-r.left+r.marginLeft,width:i,height:o})}(a,i);else{var u=void 0;"scrollParent"===r?"BODY"===(u=p(d(e))).nodeName&&(u=t.ownerDocument.documentElement):u="window"===r?t.ownerDocument.documentElement:r;var s=M(u,a,i);if("HTML"!==u.nodeName||function t(e){var n=e.nodeName;return"BODY"!==n&&"HTML"!==n&&("fixed"===l(e,"position")||t(d(e)))}(a))o=s;else{var c=w(t.ownerDocument),f=c.height,h=c.width;o.top+=s.top-s.marginTop,o.bottom=f+s.top,o.left+=s.left-s.marginLeft,o.right=h+s.left}}var g="number"==typeof(n=n||0);return o.left+=g?n:n.left||0,o.top+=g?n:n.top||0,o.right-=g?n:n.right||0,o.bottom-=g?n:n.bottom||0,o}function L(t,e,n,r,i){var o=arguments.length>5&&void 0!==arguments[5]?arguments[5]:0;if(-1===t.indexOf("auto"))return t;var a=_(n,r,o,i),u={top:{width:a.width,height:e.top-a.top},right:{width:a.right-e.right,height:a.height},bottom:{width:a.width,height:a.bottom-e.bottom},left:{width:e.left-a.left,height:a.height}},s=Object.keys(u).map(function(t){return T({key:t},u[t],{area:function(t){return t.width*t.height}(u[t])})}).sort(function(t,e){return e.area-t.area}),c=s.filter(function(t){var e=t.width,r=t.height;return e>=n.clientWidth&&r>=n.clientHeight}),l=c.length>0?c[0].key:s[0].key,d=t.split("-")[1];return l+(d?"-"+d:"")}function S(t,e,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:null;return M(n,r?D(e):y(e,n),r)}function I(t){var e=getComputedStyle(t),n=parseFloat(e.marginTop)+parseFloat(e.marginBottom),r=parseFloat(e.marginLeft)+parseFloat(e.marginRight);return{width:t.offsetWidth+r,height:t.offsetHeight+n}}function j(t){var e={left:"right",right:"left",bottom:"top",top:"bottom"};return t.replace(/left|right|bottom|top/g,function(t){return e[t]})}function k(t,e,n){n=n.split("-")[0];var r=I(t),i={width:r.width,height:r.height},o=-1!==["right","left"].indexOf(n),a=o?"top":"left",u=o?"left":"top",s=o?"height":"width",c=o?"width":"height";return i[a]=e[a]+e[s]/2-r[s]/2,i[u]=n===u?e[u]-r[c]:e[j(u)],i}function V(t,e){return Array.prototype.find?t.find(e):t.filter(e)[0]}function H(t,e,n){return(void 0===n?t:t.slice(0,function(t,e,n){if(Array.prototype.findIndex)return t.findIndex(function(t){return t[e]===n});var r=V(t,function(t){return t[e]===n});return t.indexOf(r)}(t,"name",n))).forEach(function(t){t.function&&console.warn("`modifier.function` is deprecated, use `modifier.fn`!");var n=t.function||t.fn;t.enabled&&c(n)&&(e.offsets.popper=O(e.offsets.popper),e.offsets.reference=O(e.offsets.reference),e=n(e,t))}),e}function P(t,e){return t.some(function(t){var n=t.name;return t.enabled&&n===e})}function G(t){for(var e=[!1,"ms","Webkit","Moz","O"],n=t.charAt(0).toUpperCase()+t.slice(1),r=0;r<e.length;r++){var i=e[r],o=i?""+i+n:t;if(void 0!==document.body.style[o])return o}return null}function F(t){var e=t.ownerDocument;return e?e.defaultView:window}function U(t){return""!==t&&!isNaN(parseFloat(t))&&isFinite(t)}function B(t,e){Object.keys(e).forEach(function(n){var r="";-1!==["width","height","top","right","bottom","left"].indexOf(n)&&U(e[n])&&(r="px"),t.style[n]=e[n]+r})}function z(t,e,n){var r=V(t,function(t){return t.name===e}),i=!!r&&t.some(function(t){return t.name===n&&t.enabled&&t.order<r.order});if(!i){var o="`"+e+"`",a="`"+n+"`";console.warn(a+" modifier is required by "+o+" modifier in order to work, be sure to include it before "+o+"!")}return i}var Y=["auto-start","auto","auto-end","top-start","top","top-end","right-start","right","right-end","bottom-end","bottom","bottom-start","left-end","left","left-start"],W=Y.slice(3);function q(t){var e=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=W.indexOf(t),r=W.slice(n+1).concat(W.slice(0,n));return e?r.reverse():r}var K={FLIP:"flip",CLOCKWISE:"clockwise",COUNTERCLOCKWISE:"counterclockwise"},J={placement:"bottom",positionFixed:!1,eventsEnabled:!0,removeOnDestroy:!1,onCreate:function(){},onUpdate:function(){},modifiers:{shift:{order:100,enabled:!0,fn:function(t){var e=t.placement,n=e.split("-")[0],r=e.split("-")[1];if(r){var i=t.offsets,o=i.reference,a=i.popper,u=-1!==["bottom","top"].indexOf(n),s=u?"left":"top",c=u?"width":"height",l={start:N({},s,o[s]),end:N({},s,o[s]+o[c]-a[c])};t.offsets.popper=T({},a,l[r])}return t}},offset:{order:200,enabled:!0,fn:function(t,e){var n=e.offset,r=t.placement,i=t.offsets,o=i.popper,a=i.reference,u=r.split("-")[0],s=void 0;return s=U(+n)?[+n,0]:function(t,e,n,r){var i=[0,0],o=-1!==["right","left"].indexOf(r),a=t.split(/(\+|\-)/).map(function(t){return t.trim()}),u=a.indexOf(V(a,function(t){return-1!==t.search(/,|\s/)}));a[u]&&-1===a[u].indexOf(",")&&console.warn("Offsets separated by white space(s) are deprecated, use a comma (,) instead.");var s=/\s*,\s*|\s+/,c=-1!==u?[a.slice(0,u).concat([a[u].split(s)[0]]),[a[u].split(s)[1]].concat(a.slice(u+1))]:[a];return(c=c.map(function(t,r){var i=(1===r?!o:o)?"height":"width",a=!1;return t.reduce(function(t,e){return""===t[t.length-1]&&-1!==["+","-"].indexOf(e)?(t[t.length-1]=e,a=!0,t):a?(t[t.length-1]+=e,a=!1,t):t.concat(e)},[]).map(function(t){return function(t,e,n,r){var i=t.match(/((?:\-|\+)?\d*\.?\d*)(.*)/),o=+i[1],a=i[2];if(!o)return t;if(0===a.indexOf("%")){var u=void 0;switch(a){case"%p":u=n;break;case"%":case"%r":default:u=r}return O(u)[e]/100*o}return"vh"===a||"vw"===a?("vh"===a?Math.max(document.documentElement.clientHeight,window.innerHeight||0):Math.max(document.documentElement.clientWidth,window.innerWidth||0))/100*o:o}(t,i,e,n)})})).forEach(function(t,e){t.forEach(function(n,r){U(n)&&(i[e]+=n*("-"===t[r-1]?-1:1))})}),i}(n,o,a,u),"left"===u?(o.top+=s[0],o.left-=s[1]):"right"===u?(o.top+=s[0],o.left+=s[1]):"top"===u?(o.left+=s[0],o.top-=s[1]):"bottom"===u&&(o.left+=s[0],o.top+=s[1]),t.popper=o,t},offset:0},preventOverflow:{order:300,enabled:!0,fn:function(t,e){var n=e.boundariesElement||m(t.instance.popper);t.instance.reference===n&&(n=m(n));var r=G("transform"),i=t.instance.popper.style,o=i.top,a=i.left,u=i[r];i.top="",i.left="",i[r]="";var s=_(t.instance.popper,t.instance.reference,e.padding,n,t.positionFixed);i.top=o,i.left=a,i[r]=u,e.boundaries=s;var c=e.priority,l=t.offsets.popper,d={primary:function(t){var n=l[t];return l[t]<s[t]&&!e.escapeWithReference&&(n=Math.max(l[t],s[t])),N({},t,n)},secondary:function(t){var n="right"===t?"left":"top",r=l[n];return l[t]>s[t]&&!e.escapeWithReference&&(r=Math.min(l[n],s[t]-("right"===t?l.width:l.height))),N({},n,r)}};return c.forEach(function(t){var e=-1!==["left","top"].indexOf(t)?"primary":"secondary";l=T({},l,d[e](t))}),t.offsets.popper=l,t},priority:["left","right","top","bottom"],padding:5,boundariesElement:"scrollParent"},keepTogether:{order:400,enabled:!0,fn:function(t){var e=t.offsets,n=e.popper,r=e.reference,i=t.placement.split("-")[0],o=Math.floor,a=-1!==["top","bottom"].indexOf(i),u=a?"right":"bottom",s=a?"left":"top",c=a?"width":"height";return n[u]<o(r[s])&&(t.offsets.popper[s]=o(r[s])-n[c]),n[s]>o(r[u])&&(t.offsets.popper[s]=o(r[u])),t}},arrow:{order:500,enabled:!0,fn:function(t,e){var n;if(!z(t.instance.modifiers,"arrow","keepTogether"))return t;var r=e.element;if("string"==typeof r){if(!(r=t.instance.popper.querySelector(r)))return t}else if(!t.instance.popper.contains(r))return console.warn("WARNING: `arrow.element` must be child of its popper element!"),t;var i=t.placement.split("-")[0],o=t.offsets,a=o.popper,u=o.reference,s=-1!==["left","right"].indexOf(i),c=s?"height":"width",d=s?"Top":"Left",p=d.toLowerCase(),f=s?"left":"top",h=s?"bottom":"right",g=I(r)[c];u[h]-g<a[p]&&(t.offsets.popper[p]-=a[p]-(u[h]-g)),u[p]+g>a[h]&&(t.offsets.popper[p]+=u[p]+g-a[h]),t.offsets.popper=O(t.offsets.popper);var m=u[p]+u[c]/2-g/2,b=l(t.instance.popper),y=parseFloat(b["margin"+d],10),v=parseFloat(b["border"+d+"Width"],10),E=m-t.offsets.popper[p]-y-v;return E=Math.max(Math.min(a[c]-g,E),0),t.arrowElement=r,t.offsets.arrow=(N(n={},p,Math.round(E)),N(n,f,""),n),t},element:"[x-arrow]"},flip:{order:600,enabled:!0,fn:function(t,e){if(P(t.instance.modifiers,"inner"))return t;if(t.flipped&&t.placement===t.originalPlacement)return t;var n=_(t.instance.popper,t.instance.reference,e.padding,e.boundariesElement,t.positionFixed),r=t.placement.split("-")[0],i=j(r),o=t.placement.split("-")[1]||"",a=[];switch(e.behavior){case K.FLIP:a=[r,i];break;case K.CLOCKWISE:a=q(r);break;case K.COUNTERCLOCKWISE:a=q(r,!0);break;default:a=e.behavior}return a.forEach(function(u,s){if(r!==u||a.length===s+1)return t;r=t.placement.split("-")[0],i=j(r);var c=t.offsets.popper,l=t.offsets.reference,d=Math.floor,p="left"===r&&d(c.right)>d(l.left)||"right"===r&&d(c.left)<d(l.right)||"top"===r&&d(c.bottom)>d(l.top)||"bottom"===r&&d(c.top)<d(l.bottom),f=d(c.left)<d(n.left),h=d(c.right)>d(n.right),g=d(c.top)<d(n.top),m=d(c.bottom)>d(n.bottom),b="left"===r&&f||"right"===r&&h||"top"===r&&g||"bottom"===r&&m,y=-1!==["top","bottom"].indexOf(r),v=!!e.flipVariations&&(y&&"start"===o&&f||y&&"end"===o&&h||!y&&"start"===o&&g||!y&&"end"===o&&m);(p||b||v)&&(t.flipped=!0,(p||b)&&(r=a[s+1]),v&&(o=function(t){return t}(o)),t.placement=r+(o?"-"+o:""),t.offsets.popper=T({},t.offsets.popper,k(t.instance.popper,t.offsets.reference,t.placement)),t=H(t.instance.modifiers,t,"flip"))}),t},behavior:"flip",padding:5,boundariesElement:"viewport"},inner:{order:700,enabled:!1,fn:function(t){var e=t.placement,n=e.split("-")[0],r=t.offsets,i=r.popper,o=r.reference,a=-1!==["left","right"].indexOf(n),u=-1===["top","left"].indexOf(n);return i[a?"left":"top"]=o[n]-(u?i[a?"width":"height"]:0),t.placement=j(e),t.offsets.popper=O(i),t}},hide:{order:800,enabled:!0,fn:function(t){if(!z(t.instance.modifiers,"hide","preventOverflow"))return t;var e=t.offsets.reference,n=V(t.instance.modifiers,function(t){return"preventOverflow"===t.name}).boundaries;if(e.bottom<n.top||e.left>n.right||e.top>n.bottom||e.right<n.left){if(!0===t.hide)return t;t.hide=!0,t.attributes["x-out-of-boundaries"]=""}else{if(!1===t.hide)return t;t.hide=!1,t.attributes["x-out-of-boundaries"]=!1}return t}},computeStyle:{order:850,enabled:!0,fn:function(t,e){var n=e.x,r=e.y,i=t.offsets.popper,o=V(t.instance.modifiers,function(t){return"applyStyle"===t.name}).gpuAcceleration;void 0!==o&&console.warn("WARNING: `gpuAcceleration` option moved to `computeStyle` modifier and will not be supported in future versions of Popper.js!");var a=void 0!==o?o:e.gpuAcceleration,u=m(t.instance.popper),s=R(u),c={position:i.position},l={left:Math.floor(i.left),top:Math.round(i.top),bottom:Math.round(i.bottom),right:Math.floor(i.right)},d="bottom"===n?"top":"bottom",p="right"===r?"left":"right",f=G("transform"),h=void 0,g=void 0;if(g="bottom"===d?"HTML"===u.nodeName?-u.clientHeight+l.bottom:-s.height+l.bottom:l.top,h="right"===p?"HTML"===u.nodeName?-u.clientWidth+l.right:-s.width+l.right:l.left,a&&f)c[f]="translate3d("+h+"px, "+g+"px, 0)",c[d]=0,c[p]=0,c.willChange="transform";else{var b="bottom"===d?-1:1,y="right"===p?-1:1;c[d]=g*b,c[p]=h*y,c.willChange=d+", "+p}var v={"x-placement":t.placement};return t.attributes=T({},v,t.attributes),t.styles=T({},c,t.styles),t.arrowStyles=T({},t.offsets.arrow,t.arrowStyles),t},gpuAcceleration:!0,x:"bottom",y:"right"},applyStyle:{order:900,enabled:!0,fn:function(t){return B(t.instance.popper,t.styles),function(t,e){Object.keys(e).forEach(function(n){!1!==e[n]?t.setAttribute(n,e[n]):t.removeAttribute(n)})}(t.instance.popper,t.attributes),t.arrowElement&&Object.keys(t.arrowStyles).length&&B(t.arrowElement,t.arrowStyles),t},onLoad:function(t,e,n,r,i){var o=S(i,e,t,n.positionFixed),a=L(n.placement,o,e,t,n.modifiers.flip.boundariesElement,n.modifiers.flip.padding);return e.setAttribute("x-placement",a),B(e,{position:n.positionFixed?"fixed":"absolute"}),n},gpuAcceleration:void 0}}},Z=function(){function t(e,n){var r=this,i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};A(this,t),this.scheduleUpdate=function(){return requestAnimationFrame(r.update)},this.update=s(this.update.bind(this)),this.options=T({},t.Defaults,i),this.state={isDestroyed:!1,isCreated:!1,scrollParents:[]},this.reference=e&&e.jquery?e[0]:e,this.popper=n&&n.jquery?n[0]:n,this.options.modifiers={},Object.keys(T({},t.Defaults.modifiers,i.modifiers)).forEach(function(e){r.options.modifiers[e]=T({},t.Defaults.modifiers[e]||{},i.modifiers?i.modifiers[e]:{})}),this.modifiers=Object.keys(this.options.modifiers).map(function(t){return T({name:t},r.options.modifiers[t])}).sort(function(t,e){return t.order-e.order}),this.modifiers.forEach(function(t){t.enabled&&c(t.onLoad)&&t.onLoad(r.reference,r.popper,r.options,t,r.state)}),this.update();var o=this.options.eventsEnabled;o&&this.enableEventListeners(),this.state.eventsEnabled=o}return x(t,[{key:"update",value:function(){return function(){if(!this.state.isDestroyed){var t={instance:this,styles:{},arrowStyles:{},attributes:{},flipped:!1,offsets:{}};t.offsets.reference=S(this.state,this.popper,this.reference,this.options.positionFixed),t.placement=L(this.options.placement,t.offsets.reference,this.popper,this.reference,this.options.modifiers.flip.boundariesElement,this.options.modifiers.flip.padding),t.originalPlacement=t.placement,t.positionFixed=this.options.positionFixed,t.offsets.popper=k(this.popper,t.offsets.reference,t.placement),t.offsets.popper.position=this.options.positionFixed?"fixed":"absolute",t=H(this.modifiers,t),this.state.isCreated?this.options.onUpdate(t):(this.state.isCreated=!0,this.options.onCreate(t))}}.call(this)}},{key:"destroy",value:function(){return function(){return this.state.isDestroyed=!0,P(this.modifiers,"applyStyle")&&(this.popper.removeAttribute("x-placement"),this.popper.style.position="",this.popper.style.top="",this.popper.style.left="",this.popper.style.right="",this.popper.style.bottom="",this.popper.style.willChange="",this.popper.style[G("transform")]=""),this.disableEventListeners(),this.options.removeOnDestroy&&this.popper.parentNode.removeChild(this.popper),this}.call(this)}},{key:"enableEventListeners",value:function(){return function(){this.state.eventsEnabled||(this.state=function(t,e,n,r){n.updateBound=r,F(t).addEventListener("resize",n.updateBound,{passive:!0});var i=p(t);return function t(e,n,r,i){var o="BODY"===e.nodeName,a=o?e.ownerDocument.defaultView:e;a.addEventListener(n,r,{passive:!0}),o||t(p(a.parentNode),n,r,i),i.push(a)}(i,"scroll",n.updateBound,n.scrollParents),n.scrollElement=i,n.eventsEnabled=!0,n}(this.reference,this.options,this.state,this.scheduleUpdate))}.call(this)}},{key:"disableEventListeners",value:function(){return function(){this.state.eventsEnabled&&(cancelAnimationFrame(this.scheduleUpdate),this.state=function(t,e){return F(t).removeEventListener("resize",e.updateBound),e.scrollParents.forEach(function(t){t.removeEventListener("scroll",e.updateBound)}),e.updateBound=null,e.scrollParents=[],e.scrollElement=null,e.eventsEnabled=!1,e}(this.reference,this.state))}.call(this)}}]),t}();Z.Utils=("undefined"!=typeof window?window:e).PopperUtils,Z.placements=Y,Z.Defaults=J;var Q=Z,X=r(81),$=function(){function t(t,e){void 0===e&&(e={}),this.dialog=t,this.options={title:"",fullDialog:!1,resetPossible:!0,placement:"bottom-start",toggleDialog:!0,modifiers:{}},this.popper=null,Object.assign(this.options,e),this.node=t.attachment.ownerDocument.createElement("form"),this.node.classList.add("lu-dialog")}return Object.defineProperty(t.prototype,"attachment",{get:function(){return this.dialog.attachment},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"level",{get:function(){return this.dialog.level},enumerable:!0,configurable:!0}),t.prototype.equals=function(t){return this.dialog.level===t.dialog.level&&this.dialog.attachment===t.dialog.attachment},t.prototype.open=function(){var t=this;if(!(this.options.toggleDialog&&this.dialog.manager.removeLike(this)||!1===this.build(this.node))){var e=this.attachment.closest(".lu");this.options.title&&this.node.insertAdjacentHTML("afterbegin","<strong>"+this.options.title+"</strong>"),this.options.fullDialog&&this.node.insertAdjacentHTML("beforeend",'<div>\n        <button type="submit" title="Apply"></button>\n        <button type="button" title="Cancel"></button>\n        <button type="reset" title="Reset to default values" '+(this.options.resetPossible?"":'style="visibility: hidden"')+"></button>\n      </div>"),e.appendChild(this.node),this.popper=new Q(this.attachment,this.node,Object(X.a)({modifiers:{preventOverflow:{boundariesElement:e}}},this.options));var n=this.find("input[autofocus]");n&&self.setTimeout(function(){return n.focus()});var r=this.find("button[type=reset]");r&&(r.onclick=function(e){e.stopPropagation(),e.preventDefault(),t.reset()}),this.node.onsubmit=function(e){return e.stopPropagation(),e.preventDefault(),!!t.node.checkValidity()&&(t.submit()&&t.destroy(),!1)};var i=this.find("button[title=Cancel]");i&&(i.onclick=function(e){e.stopPropagation(),e.preventDefault(),t.destroy()}),this.dialog.manager.push(this)}},t.prototype.find=function(t){return this.node.querySelector(t)},t.prototype.findInput=function(t){return this.find(t)},t.prototype.forEach=function(t,e){return Array.from(this.node.querySelectorAll(t)).map(e)},t.prototype.reset=function(){},t.prototype.submit=function(){return!0},t.prototype.destroy=function(){this.dialog.manager.remove(this),this.popper&&this.popper.destroy(),this.node.remove()},t}();n.c=$},function(t,e,n){"use strict";var r=function(t,e){return t<e?-1:t>e?1:t>=e?0:NaN},i=function(t){return 1===t.length&&(t=function(t){return function(e,n){return r(t(e),n)}}(t)),{left:function(e,n,r,i){for(null==r&&(r=0),null==i&&(i=e.length);r<i;){var o=r+i>>>1;t(e[o],n)<0?r=o+1:i=o}return r},right:function(e,n,r,i){for(null==r&&(r=0),null==i&&(i=e.length);r<i;){var o=r+i>>>1;t(e[o],n)>0?i=o:r=o+1}return r}}},o=i(r),a=o.right,u=(o.left,a),s=function(t){return null===t?NaN:+t},c=function(t,e){var n,r,i,o=t.length,a=-1;if(null==e){for(;++a<o;)if(null!=(n=t[a])&&n>=n)for(r=i=n;++a<o;)null!=(n=t[a])&&(r>n&&(r=n),i<n&&(i=n))}else for(;++a<o;)if(null!=(n=e(t[a],a,t))&&n>=n)for(r=i=n;++a<o;)null!=(n=e(t[a],a,t))&&(r>n&&(r=n),i<n&&(i=n));return[r,i]},l=Array.prototype,d=l.slice,p=(l.map,function(t){return function(){return t}}),f=function(t){return t},h=function(t,e,n){t=+t,e=+e,n=(i=arguments.length)<2?(e=t,t=0,1):i<3?1:+n;for(var r=-1,i=0|Math.max(0,Math.ceil((e-t)/n)),o=new Array(i);++r<i;)o[r]=t+r*n;return o},g=Math.sqrt(50),m=Math.sqrt(10),b=Math.sqrt(2),y=function(t,e,n){var r,i,o,a,u=-1;if(n=+n,(t=+t)==(e=+e)&&n>0)return[t];if((r=e<t)&&(i=t,t=e,e=i),0===(a=v(t,e,n))||!isFinite(a))return[];if(a>0)for(t=Math.ceil(t/a),e=Math.floor(e/a),o=new Array(i=Math.ceil(e-t+1));++u<i;)o[u]=(t+u)*a;else for(t=Math.floor(t*a),e=Math.ceil(e*a),o=new Array(i=Math.ceil(t-e+1));++u<i;)o[u]=(t-u)/a;return r&&o.reverse(),o};function v(t,e,n){var r=(e-t)/Math.max(0,n),i=Math.floor(Math.log(r)/Math.LN10),o=r/Math.pow(10,i);return i>=0?(o>=g?10:o>=m?5:o>=b?2:1)*Math.pow(10,i):-Math.pow(10,-i)/(o>=g?10:o>=m?5:o>=b?2:1)}function E(t,e,n){var r=Math.abs(e-t)/Math.max(0,n),i=Math.pow(10,Math.floor(Math.log(r)/Math.LN10)),o=r/i;return o>=g?i*=10:o>=m?i*=5:o>=b&&(i*=2),e<t?-i:i}var C=function(t){return Math.ceil(Math.log(t.length)/Math.LN2)+1},w=function(){var t=f,e=c,n=C;function r(r){var i,o,a=r.length,s=new Array(a);for(i=0;i<a;++i)s[i]=t(r[i],i,r);var c=e(s),l=c[0],d=c[1],p=n(s,l,d);Array.isArray(p)||(p=E(l,d,p),p=h(Math.ceil(l/p)*p,d,p));for(var f=p.length;p[0]<=l;)p.shift(),--f;for(;p[f-1]>d;)p.pop(),--f;var g,m=new Array(f+1);for(i=0;i<=f;++i)(g=m[i]=[]).x0=i>0?p[i-1]:l,g.x1=i<f?p[i]:d;for(i=0;i<a;++i)l<=(o=s[i])&&o<=d&&m[u(p,o,0,f)].push(r[i]);return m}return r.value=function(e){return arguments.length?(t="function"==typeof e?e:p(e),r):t},r.domain=function(t){return arguments.length?(e="function"==typeof t?t:p([t[0],t[1]]),r):e},r.thresholds=function(t){return arguments.length?(n="function"==typeof t?t:Array.isArray(t)?p(d.call(t)):p(t),r):n},r},A=function(t,e,n){if(null==n&&(n=s),r=t.length){if((e=+e)<=0||r<2)return+n(t[0],0,t);if(e>=1)return+n(t[r-1],r-1,t);var r,i=(r-1)*e,o=Math.floor(i),a=+n(t[o],o,t);return a+(+n(t[o+1],o+1,t)-a)*(i-o)}},x=function(t,e){var n,r,i=t.length,o=-1;if(null==e){for(;++o<i;)if(null!=(n=t[o])&&n>=n)for(r=n;++o<i;)null!=(n=t[o])&&n>r&&(r=n)}else for(;++o<i;)if(null!=(n=e(t[o],o,t))&&n>=n)for(r=n;++o<i;)null!=(n=e(t[o],o,t))&&n>r&&(r=n);return r},N=function(t,e){var n,r=t.length,i=r,o=-1,a=0;if(null==e)for(;++o<r;)isNaN(n=s(t[o]))?--i:a+=n;else for(;++o<r;)isNaN(n=s(e(t[o],o,t)))?--i:a+=n;if(i)return a/i},T=function(t,e){var n,i=t.length,o=-1,a=[];if(null==e)for(;++o<i;)isNaN(n=s(t[o]))||a.push(n);else for(;++o<i;)isNaN(n=s(e(t[o],o,t)))||a.push(n);return A(a.sort(r),.5)},O=function(t,e){var n,r,i=t.length,o=-1;if(null==e){for(;++o<i;)if(null!=(n=t[o])&&n>=n)for(r=n;++o<i;)null!=(n=t[o])&&r>n&&(r=n)}else for(;++o<i;)if(null!=(n=e(t[o],o,t))&&n>=n)for(r=n;++o<i;)null!=(n=e(t[o],o,t))&&r>n&&(r=n);return r};n.d(e,"b",function(){return u}),n.d(e,"a",function(){return r}),n.d(e,"c",function(){return i}),n.d(e,"d",function(){return c}),n.d(e,"e",function(){return w}),n.d(e,"f",function(){return x}),n.d(e,"g",function(){return N}),n.d(e,"h",function(){return T}),n.d(e,"i",function(){return O}),n.d(e,"j",function(){return A}),n.d(e,"k",function(){return h}),n.d(e,"n",function(){return y}),n.d(e,"l",function(){return v}),n.d(e,"m",function(){return E})},function(t,e,n){"use strict";var r=n(0),i=n(16),o=n(54),a=n(88),u=n(22),s=n(39),c=n(68),l=n(61),d=n(26),p=n(56),f=n(65),h=n(23),g=n(14),m=n(44),b=n(1);n.d(e,"c",function(){return m.IExportOptions});var y=function(t){function e(e){void 0===e&&(e={});var n=t.call(this)||this;return n.rankings=[],n.selection=new a.a,n.aggregations=new Set,n.uid=0,n.createHelper=function(t){var e=n.fromDescRef(t.desc),r=null;return e&&e.type&&(n.fixDesc(t.desc),(r=new(0,n.columnTypes[e.type])(t.id,e)).restore(t,n.createHelper)),r},n.columnTypes=Object.assign(Object(u.Dc)(),e.columnTypes||{}),n.multiSelections=!1!==e.multiSelection,n}return r.c(e,t),e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([e.EVENT_ADD_COLUMN,e.EVENT_REMOVE_COLUMN,e.EVENT_ADD_RANKING,e.EVENT_REMOVE_RANKING,e.EVENT_DIRTY,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_ORDER_CHANGED,e.EVENT_SELECTION_CHANGED,e.EVENT_ADD_DESC,e.EVENT_CLEAR_DESC,e.EVENT_JUMP_TO_NEAREST,e.EVENT_GROUP_AGGREGATION_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.pushRanking=function(t){var e=this.cloneRanking(t);return this.insertRanking(e),e},e.prototype.takeSnapshot=function(t){var e=this,n=this.cloneRanking(),r=t.findMyRanker(),i="string"===t.desc.type,o=!1,a=r?r.children.filter(function(e){return e===t?(o=!1,!0):i||"string"!==e.desc.type?Object(b.k)(e):(i=!0,!0)}):[t];return o||a.push(t),a.forEach(function(r){var i=e.clone(r);n.push(i),r===t&&i.sortByMe()}),this.insertRanking(n),n},e.prototype.insertRanking=function(t,n){void 0===n&&(n=this.rankings.length),this.rankings.splice(n,0,t),this.forward.apply(this,[t].concat(Object(i.b)(".provider",g.b.EVENT_ADD_COLUMN,g.b.EVENT_REMOVE_COLUMN,g.b.EVENT_DIRTY,g.b.EVENT_DIRTY_HEADER,g.b.EVENT_ORDER_CHANGED,g.b.EVENT_DIRTY_VALUES)));var r=this;t.on(g.b.EVENT_DIRTY_ORDER+".provider",Object(o.a)(function(){r.triggerReorder(this.source)},100)),this.fire([e.EVENT_ADD_RANKING,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],t,n),this.triggerReorder(t)},e.prototype.triggerReorder=function(t){Promise.resolve(this.sort(t)).then(function(e){Object(h.i)(e),t.setGroups(e)})},e.prototype.removeRanking=function(t){var n=this.rankings.indexOf(t);return!(n<0||(this.unforward.apply(this,[t].concat(Object(i.b)(".provider",g.b.EVENT_ADD_COLUMN,g.b.EVENT_REMOVE_COLUMN,g.b.EVENT_DIRTY,g.b.EVENT_DIRTY_HEADER,g.b.EVENT_ORDER_CHANGED,g.b.EVENT_DIRTY_VALUES))),this.rankings.splice(n,1),t.on(g.b.EVENT_DIRTY_ORDER+".provider",null),this.cleanUpRanking(t),this.fire([e.EVENT_REMOVE_RANKING,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],t,n),0))},e.prototype.clearRankings=function(){var t=this;this.rankings.forEach(function(e){t.unforward.apply(t,[e].concat(Object(i.b)(".provider",g.b.EVENT_ADD_COLUMN,g.b.EVENT_REMOVE_COLUMN,g.b.EVENT_DIRTY,g.b.EVENT_DIRTY_HEADER,g.b.EVENT_ORDER_CHANGED,g.b.EVENT_DIRTY_VALUES))),e.on(g.b.EVENT_DIRTY_ORDER+".provider",null),t.cleanUpRanking(e)}),this.rankings=[],this.fire([e.EVENT_REMOVE_RANKING,e.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],null,-1)},e.prototype.getRankings=function(){return this.rankings.slice()},e.prototype.getFirstRanking=function(){return this.rankings[0]||null},e.prototype.getLastRanking=function(){return this.rankings[this.rankings.length-1]},e.prototype.ensureOneRanking=function(){if(0===this.rankings.length){var t=this.pushRanking();this.push(t,Object(s.a)())}},e.prototype.cleanUpRanking=function(t){},e.prototype.push=function(t,e){var n=this.create(e);return n?(t.push(n),n):null},e.prototype.insert=function(t,e,n){var r=this.create(n);return r?(t.insert(r,e),r):null},e.prototype.nextId=function(){return"col"+this.uid++},e.prototype.fixDesc=function(t){var e=this;"rank"===t.type?t.accessor=this.rankAccessor.bind(this):"selection"===t.type?(t.accessor=function(t){return e.isSelected(t.i)},t.setter=function(t,n){return n?e.select(t.i):e.deselect(t.i)},t.setterAll=function(t,n){return n?e.selectAll(t.map(function(t){return t.i})):e.deselectAll(t.map(function(t){return t.i}))}):"aggregate"===t.type&&(t.isAggregated=function(t,n){return e.isAggregated(t,n)},t.setAggregated=function(t,n,r){return e.setAggregated(t,n,r)})},e.prototype.create=function(t){this.fixDesc(t);var e=this.columnTypes[t.type];return e?new e(this.nextId(),t):null},e.prototype.clone=function(t){var e=this.dumpColumn(t);return this.restoreColumn(e)},e.prototype.restoreColumn=function(t){var e=this,n=function(t){var r=e.fromDescRef(t.desc),i=e.columnTypes[r.type];e.fixDesc(r);var o=new i("",r);return o.restore(t,n),o.assignNewId(e.nextId.bind(e)),o};return n(t)},e.prototype.find=function(t){for(var e="string"==typeof t?function(e){return e.id===t}:t,n=0,r=this.rankings;n<r.length;n++){var i=r[n].find(e);if(i)return i}return null},e.prototype.dump=function(){var t=this;return{uid:this.uid,selection:this.getSelection(),aggregations:Array.from(this.aggregations),rankings:this.rankings.map(function(e){return e.dump(t.toDescRef)})}},e.prototype.dumpColumn=function(t){return t.dump(this.toDescRef)},e.prototype.toDescRef=function(t){return t},e.prototype.fromDescRef=function(t){return t},e.prototype.restoreRanking=function(t){var e=this.cloneRanking();e.restore(t,this.createHelper),e.children.some(function(t){return t instanceof s.b})||e.insert(this.create(Object(s.a)()),0);var n=this.nextId.bind(this);return e.children.forEach(function(t){return t.assignNewId(n)}),e},e.prototype.restore=function(t){var e=this;this.clearRankings(),this.uid=t.uid||0,t.selection&&t.selection.forEach(function(t){return e.selection.add(t)}),t.aggregations&&(this.aggregations.clear(),t.aggregations.forEach(function(t){return e.aggregations.add(t)})),t.rankings&&t.rankings.forEach(function(t){var n=e.cloneRanking();n.restore(t,e.createHelper),n.children.some(function(t){return t instanceof s.b})||n.insert(e.create(Object(s.a)()),0),e.insertRanking(n)}),t.layout&&Object.keys(t.layout).forEach(function(n){e.deriveRanking(t.layout[n])});var n=this.nextId.bind(this);this.rankings.forEach(function(t){t.children.forEach(function(t){return t.assignNewId(n)})})},e.prototype.deriveRanking=function(t){var e=this,n=this.cloneRanking();n.clear();var r=function(t){switch(t.type){case"rank":return e.create(Object(s.a)());case"selection":return e.create(Object(c.b)());case"group":return e.create(Object(l.b)());case"aggregate":return e.create(Object(d.b)());case"actions":var n=e.create(Object(p.a)(t.label||"actions"));return n.restore(t,e.createHelper),n;case"stacked":var i=e.create(Object(f.a)(t.label||"Combined"));return(t.children||[]).forEach(function(t){var e=r(t);e&&i.push(e)}),i;default:var o=e.findDesc(t.column);if(o){var a=e.create(o);return t.label=t.label||o.label||o.column,a&&a.restore(t,e.createHelper),a}return null}};return t.forEach(function(t){var e=r(t);e&&n.push(e)}),n.children.some(function(t){return t instanceof s.b})||n.insert(this.create(Object(s.a)()),0),this.insertRanking(n),n},e.prototype.isAggregated=function(t,e){for(var n=e;n;){var r=t.id+"@"+Object(h.h)(n);if(this.aggregations.has(r))return this.aggregations.add(t.id+"@"+Object(h.h)(e)),!0;n=n.parent}return!1},e.prototype.unaggregateParents=function(t,e){for(var n=e.parent;n;)this.aggregations.delete(t.id+"@"+Object(h.h)(n)),n=n.parent},e.prototype.setAggregated=function(t,n,r){this.unaggregateParents(t,n);var i=t.id+"@"+Object(h.h)(n);this.isAggregated(t,n)!==r&&(r?this.aggregations.add(i):this.aggregations.delete(i),this.fire([e.EVENT_GROUP_AGGREGATION_CHANGED,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],t,n,r))},e.prototype.aggregateAllOf=function(t,n){var r=this,i=t.getGroups();i.forEach(function(e){if(r.unaggregateParents(t,e),r.isAggregated(t,e)!==n){var i=t.id+"@"+Object(h.h)(e);n?r.aggregations.add(i):r.aggregations.delete(i)}}),this.fire([e.EVENT_GROUP_AGGREGATION_CHANGED,e.EVENT_DIRTY_VALUES,e.EVENT_DIRTY],t,i,n)},e.prototype.isSelected=function(t){return this.selection.has(t)},e.prototype.select=function(t){this.selection.has(t)||(!this.multiSelections&&this.selection.size>0&&this.selection.clear(),this.selection.add(t),this.fire(e.EVENT_SELECTION_CHANGED,this.getSelection()))},e.prototype.jumpToNearest=function(t){0!==t.length&&this.fire(e.EVENT_JUMP_TO_NEAREST,t)},e.prototype.selectAll=function(t){var n=this;t.every(function(t){return n.selection.has(t)})||(this.multiSelections||(this.selection.clear(),t=t.slice(0,1)),t.forEach(function(t){n.selection.add(t)}),this.fire(e.EVENT_SELECTION_CHANGED,this.getSelection()))},e.prototype.selectAllOf=function(t){this.setSelection(t.getOrder())},e.prototype.setSelection=function(t){var e=this;if(0===t.length)return this.clearSelection();this.selection.size===t.length&&t.every(function(t){return e.selection.has(t)})||(this.selection.clear(),this.selectAll(t))},e.prototype.toggleSelection=function(t,e){return void 0===e&&(e=!1),this.isSelected(t)?(e?this.deselect(t):this.clearSelection(),!1):(e?this.select(t):this.setSelection([t]),!0)},e.prototype.deselect=function(t){this.selection.has(t)&&(this.selection.delete(t),this.fire(e.EVENT_SELECTION_CHANGED,this.getSelection()))},e.prototype.deselectAll=function(t){var n=this;t.every(function(t){return!n.selection.has(t)})||(t.forEach(function(t){n.selection.delete(t)}),this.fire(e.EVENT_SELECTION_CHANGED,this.getSelection()))},e.prototype.selectedRows=function(){return 0===this.selection.size?[]:this.view(this.getSelection())},e.prototype.getSelection=function(){return Array.from(this.selection)},e.prototype.clearSelection=function(){0!==this.selection.size&&(this.selection.clear(),this.fire(e.EVENT_SELECTION_CHANGED,[],!1))},e.prototype.exportTable=function(t,e){return void 0===e&&(e={}),Promise.resolve(this.view(t.getOrder())).then(function(n){return Object(m.e)(t,n,e)})},e.EVENT_SELECTION_CHANGED="selectionChanged",e.EVENT_ADD_COLUMN=g.b.EVENT_ADD_COLUMN,e.EVENT_REMOVE_COLUMN=g.b.EVENT_REMOVE_COLUMN,e.EVENT_ADD_RANKING="addRanking",e.EVENT_REMOVE_RANKING="removeRanking",e.EVENT_DIRTY=g.b.EVENT_DIRTY,e.EVENT_DIRTY_HEADER=g.b.EVENT_DIRTY_HEADER,e.EVENT_DIRTY_VALUES=g.b.EVENT_DIRTY_VALUES,e.EVENT_ORDER_CHANGED=g.b.EVENT_ORDER_CHANGED,e.EVENT_ADD_DESC="addDesc",e.EVENT_CLEAR_DESC="clearDesc",e.EVENT_JUMP_TO_NEAREST="jumpToNearest",e.EVENT_GROUP_AGGREGATION_CHANGED=d.c.EVENT_AGGREGATE,e}(i.a);e.e=y},function(t,e,n){"use strict";var r=n(0),i=n(1),o=n(2),a=n(8),u=n(11),s=n(7),c=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.lookup=new Map,r.currentFilter=null,r.categories=Object(u.g)(n),r.missingCategory=n.missingCategory?Object(u.h)(n.missingCategory,NaN):null,r.categories.forEach(function(t){return r.lookup.set(t.name,t)}),r}var n;return r.c(e,t),n=e,e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_FILTER_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getValue=function(t){var e=this.getCategory(t);return e?e.name:null},e.prototype.getCategory=function(e){var n=t.prototype.getValue.call(this,e);if(!n)return this.missingCategory;var r=String(n);return this.lookup.has(r)?this.lookup.get(r):this.missingCategory},Object.defineProperty(e.prototype,"dataLength",{get:function(){return this.categories.length},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"labels",{get:function(){return this.categories.map(function(t){return t.label})},enumerable:!0,configurable:!0}),e.prototype.getLabel=function(t){var e=this.getCategory(t);return e?e.label:""},e.prototype.getColor=function(t){var e=this.getCategory(t);return e?e.color:this.color},e.prototype.getValues=function(t){var e=this.getCategory(t);return this.categories.map(function(t){return t===e})},e.prototype.getLabels=function(t){return this.getValues(t).map(String)},e.prototype.getMap=function(t){var e=this.categories;return this.getValues(t).map(function(t,n){return{key:e[n].label,value:t}})},e.prototype.getMapLabel=function(t){var e=this.categories;return this.getLabels(t).map(function(t,n){return{key:e[n].label,value:t}})},e.prototype.getSet=function(t){var e=this.getCategory(t),n=new Set;return e&&e!==this.missingCategory&&n.add(e),n},e.prototype.isMissing=function(t){return this.getCategory(t)===this.missingCategory},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return n.filter=this.currentFilter,n},e.prototype.restore=function(e,n){if(t.prototype.restore.call(this,e,n),"filter"in e){var r=e.filter;"string"==typeof r||Array.isArray(r)?this.currentFilter={filter:r,filterMissing:!1}:this.currentFilter=r}else this.currentFilter=null},e.prototype.isFiltered=function(){return null!=this.currentFilter},e.prototype.filter=function(t){return Object(u.d)(this.currentFilter,this.getCategory(t))},e.prototype.getFilter=function(){return null==this.currentFilter?null:Object.assign({},this.currentFilter)},e.prototype.setFilter=function(t){Object(u.e)(this.currentFilter,t)||this.fire([n.EVENT_FILTER_CHANGED,o.e.EVENT_DIRTY_VALUES,o.e.EVENT_DIRTY],this.currentFilter,this.currentFilter=t)},e.prototype.compare=function(t,e){return Object(u.a)(this.getCategory(t),this.getCategory(e))},e.prototype.group=function(t){var e=this.getCategory(t);return e?{name:e.label,color:e.color}:s.e},e.prototype.groupCompare=function(t,e){return Object(u.b)(t.rows,e.rows,this)},e.prototype.getGroupRenderer=function(){var e=t.prototype.getGroupRenderer.call(this);return e===this.desc.type&&this.isGroupedBy()>=0?"catdistributionbar":e},e.EVENT_FILTER_CHANGED="filterChanged",e=n=r.b([Object(i.l)("group","groupBy","sortGroupBy","filterCategorical"),Object(i.b)("categorical")],e)}(a.b);e.a=c},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r,i=n(0),o=n(1),a=n(2),u=n(7),s=n(8),c=n(74),l=n(50);!function(t){t.left="left",t.center="center",t.right="right"}(r||(r={}));var d=function(t){function e(e,n){var i=t.call(this,e,n)||this;return i.currentFilter=null,i.currentGroupCriteria=[],i.setDefaultWidth(200),i.alignment=n.alignment||r.left,i.escape=!1!==n.escape,i}var n;return i.c(e,t),n=e,e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_GROUPING_CHANGED,n.EVENT_FILTER_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getValue=function(e){var n=t.prototype.getValue.call(this,e);return null==n?"":String(n)},e.prototype.getLabel=function(t){return this.getValue(t)||""},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return this.currentFilter instanceof RegExp?n.filter="REGEX:"+this.currentFilter.source:n.filter=this.currentFilter,this.currentGroupCriteria&&(n.groupCriteria=this.currentGroupCriteria.map(function(t){return"string"==typeof t?t:"REGEX:"+t.source})),n},e.prototype.restore=function(e,n){t.prototype.restore.call(this,e,n),e.filter&&e.filter.startsWith("REGEX:")?this.currentFilter=new RegExp(e.filter.slice(6),"gm"):this.currentFilter=e.filter||null,e.groupCriteria&&(this.currentGroupCriteria=e.groupCriteria.map(function(t){return t.startsWith("REGEX:")?new RegExp(t.slice(6),"gm"):t}))},e.prototype.isFiltered=function(){return null!=this.currentFilter},e.prototype.filter=function(t){if(!this.isFiltered())return!0;var e=this.getLabel(t),r=this.currentFilter;return r===n.FILTER_MISSING?null!=e&&""!==e.trim():"string"==typeof r&&r.length>0?""!==e&&e.toLowerCase().indexOf(r.toLowerCase())>=0:!(r instanceof RegExp)||""!==e&&r.test(e)},e.prototype.getFilter=function(){return this.currentFilter},e.prototype.setFilter=function(t){""===t&&(t=null),this.currentFilter!==t&&this.fire([n.EVENT_FILTER_CHANGED,a.e.EVENT_DIRTY_VALUES,a.e.EVENT_DIRTY],this.currentFilter,this.currentFilter=t)},e.prototype.getGroupCriteria=function(){return this.currentGroupCriteria.slice()},e.prototype.setGroupCriteria=function(t){if(!Object(c.a)(this.currentGroupCriteria,t)){var e=this.getGroupCriteria();this.currentGroupCriteria=t.slice(),this.fire([n.EVENT_GROUPING_CHANGED,a.e.EVENT_DIRTY_VALUES,a.e.EVENT_DIRTY],e,t)}},e.prototype.compare=function(t,e){var n=this.getLabel(t),r=this.getLabel(e);return""===n?""===r?0:u.a:""===r?-u.a:n.toLowerCase().localeCompare(r.toLowerCase())},e.prototype.group=function(t){if(this.isMissing(t))return u.e;if(0===this.currentGroupCriteria.length)return l.a;var e=this.getLabel(t);if(!e)return l.a;for(var n=0,r=this.currentGroupCriteria;n<r.length;n++){var i=r[n];if(i instanceof RegExp&&i.test(e)||"string"==typeof i&&e.startsWith(i))return{name:"string"==typeof i?i:i.source,color:l.a.color}}return l.a},e.EVENT_FILTER_CHANGED="filterChanged",e.EVENT_GROUPING_CHANGED="groupingChanged",e.FILTER_MISSING="__FILTER_MISSING",e=n=i.b([Object(o.l)("search","groupBy","filterString"),Object(o.g)("group","groupString"),Object(o.b)("string")],e)}(s.b);e.d=d},function(t,e,n){"use strict";var r=n(0),i=n(56),o=n(26),a=n(79),u=n(60),s=n(31),c=n(11),l=n(7),d=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.setDefaultRenderer("upset"),r}return r.c(e,t),Object.defineProperty(e.prototype,"categories",{get:function(){return this.labels.map(function(t,e){return Object(c.h)(t,e)})},enumerable:!0,configurable:!0}),e.prototype.getSet=function(t){var e=this.getValues(t);return new Set(this.categories.filter(function(t,n){return e[n]}))},e.prototype.compare=function(t,e){var n=this.getValue(t),r=this.getValue(e);return null==n?null==r?0:l.a:null==r?-l.a:n.filter(function(t){return t}).length-r.filter(function(t){return t}).length},e}(s.d),p=n(33),f=n(20),h=n(35),g=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.lookup=new Map,r.categories=Object(c.g)(n),r.missingCategory=n.missingCategory?Object(c.h)(n.missingCategory,NaN):null,r.categories.forEach(function(t){return r.lookup.set(t.name,t)}),r}return r.c(e,t),e.prototype.parseValue=function(t){if(!t)return this.missingCategory;var e=String(t);return this.lookup.has(e)?this.lookup.get(e):this.missingCategory},e.prototype.getCategories=function(e){var n=this;return t.prototype.getValue.call(this,e).map(function(t){var e=t.key,r=t.value;return{key:e,value:n.parseValue(r)}})},e.prototype.getValue=function(t){return this.getCategories(t).map(function(t){var e=t.key,n=t.value;return{key:e,value:n?n.name:null}})},e.prototype.getLabels=function(t){return this.getCategories(t).map(function(t){var e=t.key,n=t.value;return{key:e,value:n?n.label:""}})},e}(h.a),m=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.lookup=new Map,r.categories=Object(c.g)(n),r.missingCategory=n.missingCategory?Object(c.h)(n.missingCategory,NaN):null,r.categories.forEach(function(t){return r.lookup.set(t.name,t)}),r}return r.c(e,t),e.prototype.getCategories=function(e){var n=this;return t.prototype.getValues.call(this,e).map(function(t){if(!t)return n.missingCategory;var e=String(t);return n.lookup.has(e)?n.lookup.get(e):n.missingCategory})},e.prototype.getSet=function(t){var e=new Set(this.getCategories(t));return e.delete(this.missingCategory),e},e.prototype.getValues=function(t){return this.getCategories(t).map(function(t){return t?t.name:null})},e.prototype.getLabels=function(t){return this.getCategories(t).map(function(t){return t?t.label:""})},e}(s.d),b=n(51),y=n(1),v=n(8),E=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.format=Object(b.a)(n.dateFormat||"%x"),r.parse=n.dateParse?Object(b.b)(n.dateParse):Object(b.b)(n.dateFormat||"%x"),r.setDefaultRenderer("default"),r}return r.c(e,t),e.prototype.getValue=function(e){var n=t.prototype.getValue.call(this,e);return Object(l.c)(n)?null:n instanceof Date?n:this.parse(String(n))},e.prototype.getLabel=function(t){var e=this.getValue(t);return e instanceof Date?this.format(e):""},e.prototype.compare=function(t,e){var n=this.getValue(t),r=this.getValue(e);return n===r?0:n instanceof Date?r instanceof Date?n.getTime()-r.getTime():-1*l.a:r instanceof Date?l.a:0},e=r.b([Object(y.b)("date")],e)}(v.b),C=n(89),w=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.format=Object(b.a)(n.dateFormat||"%x"),r.parse=n.dateParse?Object(b.b)(n.dateParse):Object(b.b)(n.dateFormat||"%x"),r.setDefaultRenderer("default"),r}return r.c(e,t),e.prototype.parseValue=function(t){return Object(l.c)(t)?null:t instanceof Date?t:this.parse(String(t))},e.prototype.getValue=function(e){var n=this;return t.prototype.getValue.call(this,e).map(function(t){var e=t.key,r=t.value;return{key:e,value:n.parseValue(r)}})},e.prototype.getLabels=function(t){var e=this;return this.getValue(t).map(function(t){var n=t.key,r=t.value;return{key:n,value:r instanceof Date?e.format(r):""}})},e}(h.a),A=n(61),x=n(34),N=n(63),T=n(64),O=n(72),R=n(10),M=n(9),D=n(2),_=n(23),L=n(4),S=n(12),I=n(27),j=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.currentFilter=Object(L.l)(),r.mapping=Object(S.e)(n),r.original=r.mapping.clone(),r.colorMapping=Object(I.h)(r.color,n),r.sort=n.sort||L.b.median,r.setDefaultRenderer("mapbars"),r}var n;return r.c(e,t),n=e,e.prototype.compare=function(t,e){return Object(L.d)(this,t,e)},e.prototype.getBoxPlotData=function(t){var e=this.getRawValue(t);return null==e?null:new M.g(e.map(function(t){return t.value}),this.mapping)},e.prototype.getRange=function(){return this.mapping.getRange(L.a)},e.prototype.getRawBoxPlotData=function(t){var e=this.getRawValue(t);return null==e?null:new M.g(e.map(function(t){return t.value}))},e.prototype.getNumber=function(t){return Object(L.f)(this,t,"normalized")},e.prototype.getRawNumber=function(t){return Object(L.f)(this,t,"raw")},e.prototype.getValue=function(t){var e=this;return this.getRawValue(t).map(function(t){var n=t.key,r=t.value;return{key:n,value:Object(l.c)(r)?NaN:e.mapping.apply(r)}})},e.prototype.getRawValue=function(e){var n=t.prototype.getValue.call(this,e);return null==n?[]:n},e.prototype.getExportValue=function(e,n){return"json"===n?this.getRawValue(e):t.prototype.getExportValue.call(this,e,n)},e.prototype.getLabels=function(t){return this.getValue(t).map(function(t){var e=t.key,n=t.value;return{key:e,value:Object(L.a)(n)}})},e.prototype.getSortMethod=function(){return this.sort},e.prototype.setSortMethod=function(t){this.sort!==t&&(this.fire([n.EVENT_SORTMETHOD_CHANGED],this.sort,this.sort=t),this.isSortedByMe().asc||this.sortByMe())},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return n.sortMethod=this.getSortMethod(),n.filter=Object(_.c)(this.currentFilter)?null:this.currentFilter,n.map=this.mapping.dump(),n.colorMapping=this.colorMapping.dump(),n},e.prototype.restore=function(e,n){t.prototype.restore.call(this,e,n),e.sortMethod&&(this.sort=e.sortMethod),e.filter&&(this.currentFilter=Object(_.g)(e.filter)),e.map?this.mapping=Object(S.c)(e.map):e.domain&&(this.mapping=new S.a(e.domain,"linear",e.range||[0,1])),e.colorMapping&&(this.colorMapping=Object(I.e)(this.color,e.colorMapping))},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_COLOR_MAPPING_CHANGED,n.EVENT_MAPPING_CHANGED,n.EVENT_SORTMETHOD_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getOriginalMapping=function(){return this.original.clone()},e.prototype.getMapping=function(){return this.mapping.clone()},e.prototype.setMapping=function(t){this.mapping.eq(t)||this.fire([n.EVENT_MAPPING_CHANGED,D.e.EVENT_DIRTY_VALUES,D.e.EVENT_DIRTY],this.mapping.clone(),this.mapping=t)},e.prototype.getColor=function(t){return R.c.prototype.getColor.call(this,t)},e.prototype.getColorMapping=function(){return this.colorMapping.clone()},e.prototype.setColorMapping=function(t){this.colorMapping.eq(t)||this.fire([n.EVENT_COLOR_MAPPING_CHANGED,D.e.EVENT_DIRTY_VALUES,D.e.EVENT_DIRTY],this.colorMapping.clone(),this.colorMapping=t)},e.prototype.isFiltered=function(){return R.c.prototype.isFiltered.call(this)},e.prototype.getFilter=function(){return R.c.prototype.getFilter.call(this)},e.prototype.setFilter=function(t){void 0===t&&(t={min:-1/0,max:1/0,filterMissing:!1}),R.c.prototype.setFilter.call(this,t)},e.prototype.filter=function(t){return R.c.prototype.filter.call(this,t)},e.EVENT_MAPPING_CHANGED=R.c.EVENT_MAPPING_CHANGED,e.EVENT_COLOR_MAPPING_CHANGED=R.c.EVENT_COLOR_MAPPING_CHANGED,e.EVENT_SORTMETHOD_CHANGED=R.c.EVENT_SORTMETHOD_CHANGED,e=n=r.b([Object(y.l)("filterNumber","colorMapped","editMapping"),Object(y.g)("sort","sortNumbers"),Object(y.c)("descending")],e)}(h.a),k=n(28),V=n(57),H=n(39),P=n(66),G=n(67),F=n(68),U=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.lookup=new Map,r.currentFilter=null,r.separator=new RegExp(n.separator||";"),r.categories=Object(c.g)(n),r.categories.forEach(function(t){return r.lookup.set(t.name,t)}),r.setDefaultRenderer("upset"),r.setDefaultGroupRenderer("upset"),r}return r.c(e,t),Object.defineProperty(e.prototype,"labels",{get:function(){return this.categories.map(function(t){return t.label})},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"dataLength",{get:function(){return this.categories.length},enumerable:!0,configurable:!0}),e.prototype.getValue=function(t){return this.getCategories(t).map(function(t){return t.name})},e.prototype.getLabel=function(t){return"("+this.getCategories(t).map(function(t){return t.label}).join(",")+")"},e.prototype.normalize=function(t){return"string"==typeof t?t.split(this.separator).map(function(t){return t.trim()}):Array.isArray(t)?t.map(function(t){return String(t).trim()}):t instanceof Set?Array.from(t).map(String):[]},e.prototype.getSet=function(e){var n=this,r=this.normalize(t.prototype.getValue.call(this,e)),i=new Set;return r.forEach(function(t){var e=n.lookup.get(t);e&&i.add(e)}),i},e.prototype.getCategories=function(t){return Array.from(this.getSet(t)).sort(function(t,e){return t.value===e.value?t.label.localeCompare(e.label):t.value-e.value})},e.prototype.isMissing=function(t){return 0===this.getSet(t).size},e.prototype.getValues=function(t){var e=new Set(this.getSet(t));return this.categories.map(function(t){return e.has(t)})},e.prototype.getLabels=function(t){return this.getValues(t).map(String)},e.prototype.getMap=function(t){return this.getCategories(t).map(function(t){return{key:t.label,value:!0}})},e.prototype.getMapLabel=function(t){return this.getCategories(t).map(function(t){return{key:t.label,value:"true"}})},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return n.filter=this.currentFilter,n},e.prototype.restore=function(e,n){if(t.prototype.restore.call(this,e,n),"filter"in e){var r=e.filter;"string"==typeof r||Array.isArray(r)?this.currentFilter={filter:r,filterMissing:!1}:this.currentFilter=r}else this.currentFilter=null},e.prototype.isFiltered=function(){return null!=this.currentFilter},e.prototype.filter=function(t){var e=this;return!this.currentFilter||Array.from(this.getSet(t)).some(function(t){return Object(c.d)(e.currentFilter,t)})},e.prototype.getFilter=function(){return f.a.prototype.getFilter.call(this)},e.prototype.setFilter=function(t){return f.a.prototype.setFilter.call(this,t)},e.prototype.compare=function(t,e){var n=this.getSet(t),r=this.getSet(e);if(0===n.size)return 0===r.size?0:l.a;if(0===r.size)return-l.a;if(n.size!==r.size)return n.size-r.size;for(var i=0,o=this.categories;i<o.length;i++){var a=o[i];if(n.has(a))return-1;if(r.has(a))return 1}return 0},e=r.b([Object(y.l)("filterCategorical"),Object(y.b)("categorical")],e)}(v.b),B=n(65),z=n(21),Y=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.setDefaultWidth(200),r.alignment=n.alignment||z.a.left,r.escape=!1!==n.escape,r.setDefaultRenderer("map"),r}return r.c(e,t),e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getValue=function(e){return t.prototype.getValue.call(this,e).map(function(t){var e=t.key,n=t.value;return{key:e,value:null==n?"":String(n)}})},e=r.b([Object(y.l)("search")],e)}(h.a),W=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.setDefaultWidth(200),r.alignment=n.alignment||z.a.left,r.escape=!1!==n.escape,r}return r.c(e,t),e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getValues=function(e){return t.prototype.getValues.call(this,e).map(function(t){return null==t?"":String(t)})},e=r.b([Object(y.l)("search")],e)}(s.d),q=n(62),K=n(32),J=n(69),Z=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.patternFunction=null,r.setDefaultWidth(200),r.alignment=n.alignment||z.a.left,r.escape=!1!==n.escape,r.pattern=n.pattern||"",r.patternTemplates=n.patternTemplates||[],r}var n;return r.c(e,t),n=e,e.prototype.setPattern=function(t){return K.d.prototype.setPattern.call(this,t)},e.prototype.getPattern=function(){return this.pattern},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_PATTERN_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getValues=function(t){return this.getLinks(t).map(function(t){return t?t.href:""})},e.prototype.getLabels=function(t){return this.getLinks(t).map(function(t){return t?t.alt:""})},e.prototype.transformValue=function(t,e,n){return null==t||""===t?null:"string"==typeof t?this.pattern?(this.patternFunction||(this.patternFunction=Object(_.f)(this.pattern,"item","index")),{alt:t,href:this.patternFunction.call(this,t,e.v,n)}):{alt:t,href:t}:t},e.prototype.getLinks=function(e){var n=this;return t.prototype.getValues.call(this,e).map(function(t,r){return n.transformValue(t,e,r)})},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return this.pattern!==this.desc.pattern&&(n.pattern=this.pattern),n},e.prototype.restore=function(e,n){e.pattern&&(this.pattern=e.pattern),t.prototype.restore.call(this,e,n)},e.EVENT_PATTERN_CHANGED=K.d.EVENT_PATTERN_CHANGED,e=n=r.b([Object(y.l)("search","editPattern")],e)}(s.d),Q=n(50),X=n(73),$=n(71),tt=n(13),et=n(36),nt=function(t){function e(e,n){return t.call(this,e,n)||this}return r.c(e,t),e.prototype.getLabel=function(){return""},e.prototype.getValue=function(){return""},e.prototype.compare=function(){return 0},e}(D.e),rt=n(85),it=n(14);function ot(t,e){void 0===e&&(e={});var n=function(t){function e(e,n){var r=t.call(this,e,n)||this;return"function"==typeof r.init&&r.init.apply(r,[].slice.apply(arguments)),r}return r.c(e,t),e.prototype.init=function(){},e}(v.b);return n.prototype.toString=function(){return t},n.prototype=Object.assign(n.prototype,e),n}function at(){return{actions:i.b,aggregate:o.c,annotate:a.a,boolean:u.c,booleans:d,boxplot:p.c,categorical:f.a,categoricalMap:g,categoricals:m,date:E,dateMap:w,dates:C.b,group:A.c,hierarchy:x.g,imposition:N.b,impositionBoxPlot:q.b,impositions:T.b,reduce:P.d,nested:O.b,number:R.c,numbers:k.c,numberMap:j,ordinal:V.b,rank:H.b,script:G.d,selection:F.c,set:U,stack:B.b,string:z.d,stringMap:Y,strings:W,link:K.d,linkMap:J.a,links:Z}}n.d(e,"fc",function(){return ot}),n.d(e,"Dc",function(){return at}),n.d(e,"Ac",function(){return y.k}),n.d(e,"l",function(){return y.b}),n.d(e,"Nb",function(){return y.d}),n.d(e,"Ib",function(){return y.c}),n.d(e,"k",function(){return y.a}),n.d(e,"Kc",function(){return y.l}),n.d(e,"hc",function(){return y.g}),n.d(e,"uc",function(){return l.c}),n.d(e,"Bc",function(){return l.d}),n.d(e,"y",function(){return l.b}),n.d(e,"x",function(){return l.a}),n.d(e,"Cc",function(){return l.e}),n.d(e,"ec",function(){return Q.a}),n.d(e,"qc",function(){return X.b}),n.d(e,"zc",function(){return c.f}),n.d(e,"Jc",function(){return c.h}),n.d(e,"Qb",function(){return c.a}),n.d(e,"jc",function(){return c.b}),n.d(e,"Ic",function(){return c.g}),n.d(e,"mc",function(){return c.c}),n.d(e,"oc",function(){return c.e}),n.d(e,"nc",function(){return c.d}),n.d(e,"p",function(){return L.a}),n.d(e,"Pb",function(){return L.d}),n.d(e,"ic",function(){return L.f}),n.d(e,"w",function(){return L.c}),n.d(e,"lc",function(){return L.g}),n.d(e,"t",function(){return L.b}),n.d(e,"yc",function(){return L.k}),n.d(e,"Fc",function(){return L.m}),n.d(e,"Ec",function(){return L.l}),n.d(e,"pc",function(){return L.h}),n.d(e,"xc",function(){return L.j}),n.d(e,"tc",function(){return $.b}),n.d(e,"kc",function(){return $.a}),n.d(e,"sc",function(){return S.d}),n.d(e,"Db",function(){return S.a}),n.d(e,"Fb",function(){return S.b}),n.d(e,"Xb",function(){return S.c}),n.d(e,"Hc",function(){return S.e}),n.d(e,"a",function(){return i.b}),n.d(e,"Rb",function(){return i.a}),n.d(e,"b",function(){return o.c}),n.d(e,"Sb",function(){return o.b}),n.d(e,"B",function(){return o.IAggregateGroupColumnDesc}),n.d(e,"c",function(){return a.a}),n.d(e,"d",function(){return s.d}),n.d(e,"C",function(){return s.IArrayColumnDesc}),n.d(e,"D",function(){return s.IArrayDesc}),n.d(e,"jb",function(){return s.ISplicer}),n.d(e,"e",function(){return u.c}),n.d(e,"E",function(){return u.IBooleanColumnDesc}),n.d(e,"F",function(){return u.IBooleanDesc}),n.d(e,"f",function(){return d}),n.d(e,"g",function(){return p.c}),n.d(e,"G",function(){return p.IBoxPlotColumnDesc}),n.d(e,"H",function(){return p.IBoxPlotDesc}),n.d(e,"h",function(){return f.a}),n.d(e,"i",function(){return g}),n.d(e,"j",function(){return m}),n.d(e,"m",function(){return D.e}),n.d(e,"P",function(){return D.IFlatColumn}),n.d(e,"N",function(){return D.IColumnParent}),n.d(e,"M",function(){return D.IColumnMetaData}),n.d(e,"L",function(){return D.a}),n.d(e,"n",function(){return tt.b}),n.d(e,"vc",function(){return tt.c}),n.d(e,"V",function(){return tt.IMultiLevelColumn}),n.d(e,"o",function(){return et.a}),n.d(e,"q",function(){return E}),n.d(e,"r",function(){return w}),n.d(e,"s",function(){return nt}),n.d(e,"z",function(){return A.c}),n.d(e,"Tb",function(){return A.b}),n.d(e,"v",function(){return A.a}),n.d(e,"A",function(){return x.g}),n.d(e,"K",function(){return x.ICategoryNode}),n.d(e,"cb",function(){return x.IPartialCategoryNode}),n.d(e,"Q",function(){return x.IHierarchyColumnDesc}),n.d(e,"R",function(){return x.IHierarchyDesc}),n.d(e,"J",function(){return x.ICategoryInternalNode}),n.d(e,"O",function(){return x.ICutOffNode}),n.d(e,"Gc",function(){return x.j}),n.d(e,"rc",function(){return x.i}),n.d(e,"gc",function(){return x.h}),n.d(e,"Ub",function(){return q.a}),n.d(e,"nb",function(){return q.b}),n.d(e,"Vb",function(){return N.a}),n.d(e,"ob",function(){return N.b}),n.d(e,"Wb",function(){return T.a}),n.d(e,"pb",function(){return T.b}),n.d(e,"tb",function(){return h.a}),n.d(e,"ub",function(){return rt.a}),n.d(e,"Yb",function(){return O.a}),n.d(e,"vb",function(){return O.b}),n.d(e,"wb",function(){return R.c}),n.d(e,"X",function(){return R.INumberColumnDesc}),n.d(e,"W",function(){return R.a}),n.d(e,"wc",function(){return R.d}),n.d(e,"xb",function(){return j}),n.d(e,"Z",function(){}),n.d(e,"Y",function(){}),n.d(e,"yb",function(){return k.c}),n.d(e,"bb",function(){return k.INumbersDesc}),n.d(e,"ab",function(){return k.INumbersColumnDesc}),n.d(e,"zb",function(){return V.b}),n.d(e,"I",function(){return V.ICategoricalNumberColumnDesc}),n.d(e,"Zb",function(){return H.a}),n.d(e,"Ab",function(){return H.b}),n.d(e,"Bb",function(){return it.b}),n.d(e,"ib",function(){return it.ISortCriteria}),n.d(e,"Cb",function(){return P.d}),n.d(e,"ac",function(){return P.c}),n.d(e,"eb",function(){return P.IReduceDesc}),n.d(e,"db",function(){return P.IReduceColumnDesc}),n.d(e,"Eb",function(){return G.d}),n.d(e,"bc",function(){return G.c}),n.d(e,"gb",function(){return G.IScriptDesc}),n.d(e,"fb",function(){return G.IScriptColumnDesc}),n.d(e,"Gb",function(){return F.c}),n.d(e,"cc",function(){return F.b}),n.d(e,"hb",function(){return F.ISelectionColumnDesc}),n.d(e,"Hb",function(){return U}),n.d(e,"Jb",function(){return B.b}),n.d(e,"dc",function(){return B.a}),n.d(e,"Kb",function(){return z.d}),n.d(e,"u",function(){return z.a}),n.d(e,"lb",function(){return z.IStringDesc}),n.d(e,"kb",function(){return z.IStringColumnDesc}),n.d(e,"Lb",function(){return Y}),n.d(e,"Mb",function(){return W}),n.d(e,"Ob",function(){return v.b}),n.d(e,"mb",function(){return v.IValueColumnDesc}),n.d(e,"qb",function(){return K.d}),n.d(e,"U",function(){return K.ILinkDesc}),n.d(e,"T",function(){return K.ILinkColumnDesc}),n.d(e,"S",function(){return K.ILink}),n.d(e,"rb",function(){return J.a}),n.d(e,"sb",function(){return Z})},function(t,e,n){"use strict";n.d(e,"f",function(){return o}),n.d(e,"c",function(){return a}),n.d(e,"g",function(){return u}),n.d(e,"d",function(){return s}),n.d(e,"h",function(){return c}),n.d(e,"i",function(){return l}),n.d(e,"a",function(){return p}),n.d(e,"e",function(){return f}),n.d(e,"b",function(){return h});var r=n(9),i=n(4);function o(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return new(Function.bind.apply(Function,[void 0,"value"].concat(e,["\n  const escapedValue = encodeURIComponent(String(value));\n  return `"+t+"`;\n "])))}function a(t){return!t.filterMissing&&!isFinite(t.min)&&!isFinite(t.max)}function u(t){return{min:null!=t.min&&isFinite(t.min)?t.min:-1/0,max:null!=t.max&&isFinite(t.max)?t.max:1/0,filterMissing:t.filterMissing}}function s(t){if(console.assert(t.length>0),1===t.length)return t[0];var e=t.map(function(t){return Object.assign({subGroups:[]},t)});e.slice(1).forEach(function(t,n){t.parent=e[n],e[n].subGroups.push(t)});var n={name:e.map(function(t){return t.name}).join(" ∩ "),color:e[0].color,parent:e[e.length-1]};return n.parent.subGroups.push(n),n}function c(t){return t.name}function l(t){if(!(t.length<=1)){var e=new Map,n=function(t){var r=t.name;if(t.parent){var i=n(t.parent);t.parent=i.g,r=i.id+".$[id}"}return e.has(r)?{g:e.get(r),id:r}:(t.parent&&t.parent.subGroups.push(t),t.subGroups=[],e.set(r,t),{g:t,id:r})};t.forEach(function(t){t.parent&&(t.parent=n(t.parent).g,t.parent.subGroups.push(t))})}}var d=["#1f77b4","#ff7f0e","#2ca02c","#d62728","#9467bd","#8c564b","#e377c2","#7f7f7f","#bcbd22","#17becf"].concat(["#8dd3c7","#ffffb3","#bebada","#fb8072","#80b1d3","#fdb462","#b3de69","#fccde5","#d9d9d9","#bc80bd","#ccebc5","#ffed6f"]);function p(){var t=0;return function(){return d[t++%d.length]}}function f(t,e){var n=t.map(function(t,n){return{i:n,v:e.getNumber(t),m:e.isMissing(t)}}).filter(function(t){return!t.m}).sort(function(t,e){return Object(i.m)(t.v,e.v)}),r=n[Math.floor(n.length/2)];return void 0===r?0:r.i}function h(t,e,n,o){var a=new r.g(t.map(function(t){return n.getNumber(t)})),u=new r.g(e.map(function(t){return n.getNumber(t)}));return Object(i.m)(a[o],u[o])}},function(t,e,n){"use strict";var r=function(t,e,n){t.prototype=e.prototype=n,n.constructor=t};function i(t,e){var n=Object.create(t.prototype);for(var r in e)n[r]=e[r];return n}function o(){}var a="\\s*([+-]?\\d+)\\s*",u="\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*",s="\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*",c=/^#([0-9a-f]{3})$/,l=/^#([0-9a-f]{6})$/,d=new RegExp("^rgb\\("+[a,a,a]+"\\)$"),p=new RegExp("^rgb\\("+[s,s,s]+"\\)$"),f=new RegExp("^rgba\\("+[a,a,a,u]+"\\)$"),h=new RegExp("^rgba\\("+[s,s,s,u]+"\\)$"),g=new RegExp("^hsl\\("+[u,s,s]+"\\)$"),m=new RegExp("^hsla\\("+[u,s,s,u]+"\\)$"),b={aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074};function y(t){var e;return t=(t+"").trim().toLowerCase(),(e=c.exec(t))?new A((e=parseInt(e[1],16))>>8&15|e>>4&240,e>>4&15|240&e,(15&e)<<4|15&e,1):(e=l.exec(t))?v(parseInt(e[1],16)):(e=d.exec(t))?new A(e[1],e[2],e[3],1):(e=p.exec(t))?new A(255*e[1]/100,255*e[2]/100,255*e[3]/100,1):(e=f.exec(t))?E(e[1],e[2],e[3],e[4]):(e=h.exec(t))?E(255*e[1]/100,255*e[2]/100,255*e[3]/100,e[4]):(e=g.exec(t))?N(e[1],e[2]/100,e[3]/100,1):(e=m.exec(t))?N(e[1],e[2]/100,e[3]/100,e[4]):b.hasOwnProperty(t)?v(b[t]):"transparent"===t?new A(NaN,NaN,NaN,0):null}function v(t){return new A(t>>16&255,t>>8&255,255&t,1)}function E(t,e,n,r){return r<=0&&(t=e=n=NaN),new A(t,e,n,r)}function C(t){return t instanceof o||(t=y(t)),t?new A((t=t.rgb()).r,t.g,t.b,t.opacity):new A}function w(t,e,n,r){return 1===arguments.length?C(t):new A(t,e,n,null==r?1:r)}function A(t,e,n,r){this.r=+t,this.g=+e,this.b=+n,this.opacity=+r}function x(t){return((t=Math.max(0,Math.min(255,Math.round(t)||0)))<16?"0":"")+t.toString(16)}function N(t,e,n,r){return r<=0?t=e=n=NaN:n<=0||n>=1?t=e=NaN:e<=0&&(t=NaN),new O(t,e,n,r)}function T(t,e,n,r){return 1===arguments.length?function(t){if(t instanceof O)return new O(t.h,t.s,t.l,t.opacity);if(t instanceof o||(t=y(t)),!t)return new O;if(t instanceof O)return t;var e=(t=t.rgb()).r/255,n=t.g/255,r=t.b/255,i=Math.min(e,n,r),a=Math.max(e,n,r),u=NaN,s=a-i,c=(a+i)/2;return s?(u=e===a?(n-r)/s+6*(n<r):n===a?(r-e)/s+2:(e-n)/s+4,s/=c<.5?a+i:2-a-i,u*=60):s=c>0&&c<1?0:u,new O(u,s,c,t.opacity)}(t):new O(t,e,n,null==r?1:r)}function O(t,e,n,r){this.h=+t,this.s=+e,this.l=+n,this.opacity=+r}function R(t,e,n){return 255*(t<60?e+(n-e)*t/60:t<180?n:t<240?e+(n-e)*(240-t)/60:e)}r(o,y,{displayable:function(){return this.rgb().displayable()},hex:function(){return this.rgb().hex()},toString:function(){return this.rgb()+""}}),r(A,w,i(o,{brighter:function(t){return t=null==t?1/.7:Math.pow(1/.7,t),new A(this.r*t,this.g*t,this.b*t,this.opacity)},darker:function(t){return t=null==t?.7:Math.pow(.7,t),new A(this.r*t,this.g*t,this.b*t,this.opacity)},rgb:function(){return this},displayable:function(){return 0<=this.r&&this.r<=255&&0<=this.g&&this.g<=255&&0<=this.b&&this.b<=255&&0<=this.opacity&&this.opacity<=1},hex:function(){return"#"+x(this.r)+x(this.g)+x(this.b)},toString:function(){var t=this.opacity;return(1===(t=isNaN(t)?1:Math.max(0,Math.min(1,t)))?"rgb(":"rgba(")+Math.max(0,Math.min(255,Math.round(this.r)||0))+", "+Math.max(0,Math.min(255,Math.round(this.g)||0))+", "+Math.max(0,Math.min(255,Math.round(this.b)||0))+(1===t?")":", "+t+")")}})),r(O,T,i(o,{brighter:function(t){return t=null==t?1/.7:Math.pow(1/.7,t),new O(this.h,this.s,this.l*t,this.opacity)},darker:function(t){return t=null==t?.7:Math.pow(.7,t),new O(this.h,this.s,this.l*t,this.opacity)},rgb:function(){var t=this.h%360+360*(this.h<0),e=isNaN(t)||isNaN(this.s)?0:this.s,n=this.l,r=n+(n<.5?n:1-n)*e,i=2*n-r;return new A(R(t>=240?t-240:t+120,i,r),R(t,i,r),R(t<120?t+240:t-120,i,r),this.opacity)},displayable:function(){return(0<=this.s&&this.s<=1||isNaN(this.s))&&0<=this.l&&this.l<=1&&0<=this.opacity&&this.opacity<=1}}));var M=Math.PI/180,D=180/Math.PI,_=.96422,L=1,S=.82521,I=4/29,j=6/29,k=3*j*j,V=j*j*j;function H(t){if(t instanceof G)return new G(t.l,t.a,t.b,t.opacity);if(t instanceof W){if(isNaN(t.h))return new G(t.l,0,0,t.opacity);var e=t.h*M;return new G(t.l,Math.cos(e)*t.c,Math.sin(e)*t.c,t.opacity)}t instanceof A||(t=C(t));var n,r,i=z(t.r),o=z(t.g),a=z(t.b),u=F((.2225045*i+.7168786*o+.0606169*a)/L);return i===o&&o===a?n=r=u:(n=F((.4360747*i+.3850649*o+.1430804*a)/_),r=F((.0139322*i+.0971045*o+.7141733*a)/S)),new G(116*u-16,500*(n-u),200*(u-r),t.opacity)}function P(t,e,n,r){return 1===arguments.length?H(t):new G(t,e,n,null==r?1:r)}function G(t,e,n,r){this.l=+t,this.a=+e,this.b=+n,this.opacity=+r}function F(t){return t>V?Math.pow(t,1/3):t/k+I}function U(t){return t>j?t*t*t:k*(t-I)}function B(t){return 255*(t<=.0031308?12.92*t:1.055*Math.pow(t,1/2.4)-.055)}function z(t){return(t/=255)<=.04045?t/12.92:Math.pow((t+.055)/1.055,2.4)}function Y(t,e,n,r){return 1===arguments.length?function(t){if(t instanceof W)return new W(t.h,t.c,t.l,t.opacity);if(t instanceof G||(t=H(t)),0===t.a&&0===t.b)return new W(NaN,0,t.l,t.opacity);var e=Math.atan2(t.b,t.a)*D;return new W(e<0?e+360:e,Math.sqrt(t.a*t.a+t.b*t.b),t.l,t.opacity)}(t):new W(t,e,n,null==r?1:r)}function W(t,e,n,r){this.h=+t,this.c=+e,this.l=+n,this.opacity=+r}r(G,P,i(o,{brighter:function(t){return new G(this.l+18*(null==t?1:t),this.a,this.b,this.opacity)},darker:function(t){return new G(this.l-18*(null==t?1:t),this.a,this.b,this.opacity)},rgb:function(){var t=(this.l+16)/116,e=isNaN(this.a)?t:t+this.a/500,n=isNaN(this.b)?t:t-this.b/200;return new A(B(3.1338561*(e=_*U(e))-1.6168667*(t=L*U(t))-.4906146*(n=S*U(n))),B(-.9787684*e+1.9161415*t+.033454*n),B(.0719453*e-.2289914*t+1.4052427*n),this.opacity)}})),r(W,Y,i(o,{brighter:function(t){return new W(this.h,this.c,this.l+18*(null==t?1:t),this.opacity)},darker:function(t){return new W(this.h,this.c,this.l-18*(null==t?1:t),this.opacity)},rgb:function(){return H(this).rgb()}}));var q=-.14861,K=1.78277,J=-.29227,Z=-.90649,Q=1.97294,X=Q*Z,$=Q*K,tt=K*J-Z*q;function et(t,e,n,r){return 1===arguments.length?function(t){if(t instanceof nt)return new nt(t.h,t.s,t.l,t.opacity);t instanceof A||(t=C(t));var e=t.r/255,n=t.g/255,r=t.b/255,i=(tt*r+X*e-$*n)/(tt+X-$),o=r-i,a=(Q*(n-i)-J*o)/Z,u=Math.sqrt(a*a+o*o)/(Q*i*(1-i)),s=u?Math.atan2(a,o)*D-120:NaN;return new nt(s<0?s+360:s,u,i,t.opacity)}(t):new nt(t,e,n,null==r?1:r)}function nt(t,e,n,r){this.h=+t,this.s=+e,this.l=+n,this.opacity=+r}r(nt,et,i(o,{brighter:function(t){return t=null==t?1/.7:Math.pow(1/.7,t),new nt(this.h,this.s,this.l*t,this.opacity)},darker:function(t){return t=null==t?.7:Math.pow(.7,t),new nt(this.h,this.s,this.l*t,this.opacity)},rgb:function(){var t=isNaN(this.h)?0:(this.h+120)*M,e=+this.l,n=isNaN(this.s)?0:this.s*e*(1-e),r=Math.cos(t),i=Math.sin(t);return new A(255*(e+n*(q*r+K*i)),255*(e+n*(J*r+Z*i)),255*(e+n*(Q*r)),this.opacity)}})),n.d(e,"a",function(){return y}),n.d(e,"f",function(){return w}),n.d(e,"d",function(){return T}),n.d(e,"e",function(){return P}),n.d(e,"c",function(){return Y}),n.d(e,"b",function(){return et})},function(t,e,n){"use strict";n.d(e,"a",function(){return r}),n.d(e,"e",function(){return i}),n.d(e,"b",function(){return o});var r=19,i=30,o=500},function(t,e,n){"use strict";n.d(e,"b",function(){return o});var r=n(0),i=n(1);function o(t){return void 0===t&&(t="Aggregate Groups"),{type:"aggregate",label:t,fixed:!0}}var a=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.setDefaultWidth(20),r}var n;return r.c(e,t),n=e,Object.defineProperty(e.prototype,"frozen",{get:function(){return!1!==this.desc.frozen},enumerable:!0,configurable:!0}),e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_AGGREGATE])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.isAggregated=function(t){var e=this.findMyRanker();return!!this.desc.isAggregated&&this.desc.isAggregated(e,t)},e.prototype.setAggregated=function(t,e){var r=this.findMyRanker();return(this.desc.isAggregated&&this.desc.isAggregated(r,t))===e||(this.desc.setAggregated&&this.desc.setAggregated(r,t,e),this.fire(n.EVENT_AGGREGATE,r,t,e),!1)},e.EVENT_AGGREGATE="aggregate",e=n=r.b([Object(i.d)(),Object(i.b)("support")],e)}(n(2).e);e.c=a},function(t,e,n){"use strict";n.d(e,"c",function(){return c}),n.d(e,"a",function(){return l}),n.d(e,"d",function(){return p}),n.d(e,"b",function(){return f}),n.d(e,"i",function(){return h}),n.d(e,"f",function(){return g}),n.d(e,"g",function(){return m}),n.d(e,"e",function(){return M}),n.d(e,"h",function(){return D});var r=n(45),i=n(2),o=n(74),a=n(59),u=function(){function t(t,e,n){this.name=t,this.type=e,this.apply=n}return t.prototype.dump=function(){return this.name},t.prototype.clone=function(){return this},t.prototype.eq=function(t){return t===this},t}(),s=function(){function t(t){this.color=t}return Object.defineProperty(t.prototype,"type",{get:function(){return"solid"},enumerable:!0,configurable:!0}),t.prototype.apply=function(){return this.color},t.prototype.dump=function(){return this.color},t.prototype.clone=function(){return this},t.prototype.eq=function(e){return e instanceof t&&e.color===this.color},t}(),c=function(){function t(t,e){this.base=t,this.steps=e}return Object.defineProperty(t.prototype,"type",{get:function(){return"quantized"},enumerable:!0,configurable:!0}),t.prototype.apply=function(t){return this.base.apply(function(t,e){var n=1/e;if(t<=n)return 0;if(t>=1-n)return 1;for(var r=0;r<1;r+=n)if(t<r)return r-n/2;return t}(t,this.steps))},t.prototype.dump=function(){return{base:this.base.dump(),steps:this.steps}},t.prototype.clone=function(){return this},t.prototype.eq=function(e){return e instanceof t&&e.base.eq(this.base)&&e.steps===this.steps},t}(),l=function(){function t(t){this.entries=t,this.scale=Object(a.a)(),this.scale.domain(t.map(function(t){return t.value})).range(t.map(function(t){return t.color})).clamp(!0)}return Object.defineProperty(t.prototype,"type",{get:function(){return"custom"},enumerable:!0,configurable:!0}),t.prototype.apply=function(t){return this.scale(t)},t.prototype.dump=function(){return this.entries},t.prototype.clone=function(){return new t(this.entries)},t.prototype.eq=function(e){return e instanceof t&&Object(o.a)(this.entries,e.entries)},t}(),d=new Map;function p(t){if(d.has(t))return d.get(t);var e=new s(t);return d.set(t,e),e}for(var f=p(i.e.DEFAULT_COLOR),h=[],g=[],m=new Map,b={interpolateBlues:r.a,interpolateGreens:r.h,interpolateGreys:r.i,interpolateOranges:r.m,interpolatePurples:r.u,interpolateReds:r.B,interpolateCool:r.e,interpolateCubehelixDefault:r.f,interpolateWarm:r.E,interpolatePlasma:r.p,interpolateMagma:r.k,interpolateViridis:r.D,interpolateInferno:r.j,interpolateYlOrRd:r.I,interpolateYlOrBr:r.H,interpolateBuGn:r.c,interpolateBuPu:r.d,interpolateGnBu:r.g,interpolateOrRd:r.l,interpolatePuBuGn:r.r,interpolatePuBu:r.q,interpolatePuRd:r.t,interpolateRdPu:r.y,interpolateYlGnBu:r.G,interpolateYlGn:r.F,interpolateRainbow:r.v},y={interpolateBrBG:r.b,interpolatePRGn:r.n,interpolatePiYG:r.o,interpolatePuOr:r.s,interpolateRdBu:r.w,interpolateRdGy:r.x,interpolateRdYlBu:r.z,interpolateRdYlGn:r.A,interpolateSpectral:r.C},v=0,E=Object.keys(b);v<E.length;v++){var C=E[v];h.push(new u(C,"sequential",b[C]))}for(var w=0,A=Object.keys(y);w<A.length;w++)C=A[w],g.push(new u(C,"divergent",y[C]));for(var x=0,N=h;x<N.length;x++){var T=N[x];m.set(T.name,T)}for(var O=0,R=g;O<R.length;O++)T=R[O],m.set(T.name,T);function M(t,e){if(!e)return t?p(t):f;if("string"==typeof e){var n=m.get(e);return n||p(e)}return"function"==typeof e?new u("custom","sequential",e):e.base&&e.steps?new c(M(t,e.base),e.steps):Array.isArray(e)?new l(e):f}function D(t,e){return e.colorMapping?M(t,e.colorMapping):t?p(t):f}},function(t,e,n){"use strict";var r=n(0),i=n(9),o=n(1),a=n(31),u=n(2),s=n(23),c=n(4),l=n(12),d=n(7),p=n(10),f=n(27),h=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.currentFilter=Object(c.l)(),r.mapping=Object(l.e)(n),r.original=r.mapping.clone(),r.colorMapping=Object(f.h)(r.color,n),r.sort=n.sort||c.b.median,r.dataLength&&r.setDefaultWidth(Math.min(Math.max(100,10*r.dataLength),500)),r.setDefaultRenderer("heatmap"),r.setDefaultGroupRenderer("heatmap"),r.setDefaultSummaryRenderer("histogram"),r}var n;return r.c(e,t),n=e,e.prototype.compare=function(t,e){return Object(c.d)(this,t,e)},e.prototype.getRawNumbers=function(t){return this.getRawValue(t)},e.prototype.getBoxPlotData=function(t){var e=this.getRawValue(t);return null==e?null:new i.g(e,this.mapping)},e.prototype.getRange=function(){return this.mapping.getRange(c.a)},e.prototype.getRawBoxPlotData=function(t){var e=this.getRawValue(t);return null==e?null:new i.g(e)},e.prototype.getNumbers=function(t){return this.getValue(t)},e.prototype.getNumber=function(t){return Object(c.f)(this,t,"normalized")},e.prototype.getRawNumber=function(t){return Object(c.f)(this,t,"raw")},e.prototype.getValue=function(t){var e=this;return this.getRawValue(t).map(function(t){return Object(d.c)(t)?NaN:e.mapping.apply(t)})},e.prototype.getRawValue=function(e){var n=t.prototype.getValue.call(this,e);return null==n?[]:n},e.prototype.getExportValue=function(e,n){return"json"===n?this.getRawValue(e):t.prototype.getExportValue.call(this,e,n)},e.prototype.getLabels=function(t){return this.getValue(t).map(c.a)},e.prototype.getSortMethod=function(){return this.sort},e.prototype.setSortMethod=function(t){this.sort!==t&&(this.fire([n.EVENT_SORTMETHOD_CHANGED],this.sort,this.sort=t),this.isSortedByMe().asc||this.sortByMe())},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return n.sortMethod=this.getSortMethod(),n.filter=Object(s.c)(this.currentFilter)?null:this.currentFilter,n.map=this.mapping.dump(),n.colorMapping=this.colorMapping.dump(),n},e.prototype.restore=function(e,n){t.prototype.restore.call(this,e,n),e.sortMethod&&(this.sort=e.sortMethod),e.filter&&(this.currentFilter=Object(s.g)(e.filter)),e.map?this.mapping=Object(l.c)(e.map):e.domain&&(this.mapping=new l.a(e.domain,"linear",e.range||[0,1])),e.colorMapping&&(this.colorMapping=Object(f.e)(this.color,e.colorMapping))},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_COLOR_MAPPING_CHANGED,n.EVENT_MAPPING_CHANGED,n.EVENT_SORTMETHOD_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getOriginalMapping=function(){return this.original.clone()},e.prototype.getMapping=function(){return this.mapping.clone()},e.prototype.setMapping=function(t){this.mapping.eq(t)||this.fire([n.EVENT_MAPPING_CHANGED,u.e.EVENT_DIRTY_HEADER,u.e.EVENT_DIRTY_VALUES,u.e.EVENT_DIRTY],this.mapping.clone(),this.mapping=t)},e.prototype.getColor=function(t){return p.c.prototype.getColor.call(this,t)},e.prototype.getColorMapping=function(){return this.colorMapping.clone()},e.prototype.setColorMapping=function(t){this.colorMapping.eq(t)||this.fire([n.EVENT_COLOR_MAPPING_CHANGED,u.e.EVENT_DIRTY_HEADER,u.e.EVENT_DIRTY_VALUES,u.e.EVENT_DIRTY],this.colorMapping.clone(),this.colorMapping=t)},e.prototype.isFiltered=function(){return p.c.prototype.isFiltered.call(this)},e.prototype.getFilter=function(){return p.c.prototype.getFilter.call(this)},e.prototype.setFilter=function(t){void 0===t&&(t={min:-1/0,max:1/0,filterMissing:!1}),p.c.prototype.setFilter.call(this,t)},e.prototype.filter=function(t){return p.c.prototype.filter.call(this,t)},e.EVENT_MAPPING_CHANGED=p.c.EVENT_MAPPING_CHANGED,e.EVENT_COLOR_MAPPING_CHANGED=p.c.EVENT_COLOR_MAPPING_CHANGED,e.EVENT_SORTMETHOD_CHANGED=p.c.EVENT_SORTMETHOD_CHANGED,e.CENTER=0,e=n=r.b([Object(o.l)("filterNumber","colorMapped","editMapping"),Object(o.g)("sort","sortNumbers"),Object(o.c)("descending")],e)}(a.d);e.c=h},,function(t,e,n){"use strict";n.d(e,"b",function(){return i}),n.d(e,"a",function(){return o});var r=n(3);function i(t,e,n){var r=function(t){var e=t;n?e.dataset.active="":delete e.dataset.active};r(t);var i=t.closest(".lu");i&&Array.from(i.querySelectorAll('[data-col-id="'+e.id+'"] i[title^=Filter]')).forEach(r)}function o(t,e,n,i){var o=Object(r.j)(i),a=e.getSortMethod();n.forEach(function(e){return t.insertAdjacentHTML("beforeend",'<div class="checkbox"><input id="'+o+e+'" type="radio" name="multivaluesort" value="'+e+'"  '+(a===e?"checked":"")+' ><label for="'+o+e+'">'+(e.slice(0,1).toUpperCase()+e.slice(1))+"</label></div>")}),Object(r.d)(t,"input[name=multivaluesort]",function(t){t.addEventListener("change",function(){return e.setSortMethod(t.value)},{passive:!0})})}},function(t,e,n){"use strict";var r=n(0),i=n(1),o=n(2),a=n(8),u=n(18),s=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r._dataLength=null==n.dataLength||isNaN(n.dataLength)?null:n.dataLength,r.originalLabels=n.labels||Object(u.k)(null==r._dataLength?0:r._dataLength).map(function(t,e){return"Column "+e}),r.splicer={length:r._dataLength,splice:function(t){return t}},r}var n;return r.c(e,t),n=e,e.prototype.setSplicer=function(t){this.fire([n.EVENT_SPLICE_CHANGED,o.e.EVENT_DIRTY_VALUES,o.e.EVENT_DIRTY_HEADER,o.e.EVENT_DIRTY],this.splicer,this.splicer=t)},Object.defineProperty(e.prototype,"labels",{get:function(){return this.splicer?this.splicer.splice(this.originalLabels):this.originalLabels},enumerable:!0,configurable:!0}),e.prototype.getSplicer=function(){return this.splicer},Object.defineProperty(e.prototype,"dataLength",{get:function(){return this.splicer?this.splicer.length:this._dataLength},enumerable:!0,configurable:!0}),e.prototype.getValue=function(t){return this.getValues(t)},e.prototype.getValues=function(e){var n=t.prototype.getValue.call(this,e);return this.splicer&&null!=n&&(n=this.splicer.splice(n)),null==n?[]:n},e.prototype.getLabels=function(t){return this.getValues(t).map(String)},e.prototype.getLabel=function(t){var e=this.getLabels(t);return e?e.toString():""},e.prototype.getMap=function(t){var e=this.labels;return this.getValues(t).map(function(t,n){return{key:e[n],value:t}})},e.prototype.getMapLabel=function(t){var e=this.labels;return this.getLabels(t).map(function(t,n){return{key:e[n],value:t}})},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_SPLICE_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.EVENT_SPLICE_CHANGED="spliceChanged",e=n=r.b([Object(i.b)("array")],e)}(a.b);e.d=s},function(t,e,n){"use strict";var r=n(0),i=n(1),o=n(2),a=n(23),u=n(8),s=n(21),c=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.patternFunction=null,r.currentFilter=null,r.currentGroupCriteria=[],r.setDefaultWidth(200),r.alignment=n.alignment||s.a.left,r.escape=!1!==n.escape,r.pattern=n.pattern||"",r.patternTemplates=n.patternTemplates||[],r.pattern&&(r.setDefaultRenderer("link"),r.setDefaultGroupRenderer("link")),r}var n;return r.c(e,t),n=e,e.prototype.setPattern=function(t){t!==this.pattern&&(this.patternFunction=null,this.fire([n.EVENT_PATTERN_CHANGED,o.e.EVENT_DIRTY_HEADER,o.e.EVENT_DIRTY_VALUES,o.e.EVENT_DIRTY],this.pattern,this.pattern=t))},e.prototype.getPattern=function(){return this.pattern},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_PATTERN_CHANGED,n.EVENT_GROUPING_CHANGED,n.EVENT_FILTER_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getValue=function(t){var e=this.getLink(t);return null==e?"":e.href},e.prototype.getLink=function(e){var n=t.prototype.getValue.call(this,e);return this.transformValue(n,e)},e.prototype.transformValue=function(t,e){return null==t||""===t?null:"string"==typeof t?this.pattern?(this.patternFunction||(this.patternFunction=Object(a.f)(this.pattern,"item")),{alt:t,href:this.patternFunction.call(this,t,e.v)}):{alt:t,href:t}:t},e.prototype.getLabel=function(t){var e=this.getLink(t);return null==e?"":e.alt},e.prototype.dump=function(t){var e=s.d.prototype.dump.call(this,t);return this.pattern!==this.desc.pattern&&(e.pattern=this.pattern),e},e.prototype.restore=function(t,e){s.d.prototype.restore.call(this,t,e),t.pattern&&(this.pattern=t.pattern)},e.prototype.isFiltered=function(){return null!=this.currentFilter},e.prototype.filter=function(t){return s.d.prototype.filter.call(this,t)},e.prototype.getFilter=function(){return this.currentFilter},e.prototype.setFilter=function(t){return s.d.prototype.setFilter.call(this,t)},e.prototype.getGroupCriteria=function(){return this.currentGroupCriteria.slice()},e.prototype.setGroupCriteria=function(t){return s.d.prototype.setGroupCriteria.call(this,t)},e.prototype.compare=function(t,e){return s.d.prototype.compare.call(this,t,e)},e.prototype.group=function(t){return s.d.prototype.group.call(this,t)},e.EVENT_FILTER_CHANGED=s.d.EVENT_FILTER_CHANGED,e.EVENT_GROUPING_CHANGED=s.d.EVENT_GROUPING_CHANGED,e.EVENT_PATTERN_CHANGED="patternChanged",e=n=r.b([Object(i.l)("search","groupBy","filterString","editPattern"),Object(i.g)("group","groupString"),Object(i.b)("string")],e)}(u.b);e.d=c},function(t,e,n){"use strict";var r=n(0),i=n(48),o=n(1),a=n(2),u=n(8),s=n(23),c=n(4),l=n(12),d=n(10),p=n(27),f=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.currentFilter=Object(c.l)(),r.mapping=Object(l.e)(n),r.original=r.mapping.clone(),r.colorMapping=Object(p.h)(r.color,n),r.sort=n.sort||c.c.min,r}var n;return r.c(e,t),n=e,e.prototype.compare=function(t,e){return Object(c.d)(this,t,e)},e.prototype.getBoxPlotData=function(t){return this.getValue(t)},e.prototype.getRange=function(){return this.mapping.getRange(n.DEFAULT_FORMATTER)},e.prototype.getRawBoxPlotData=function(t){return this.getRawValue(t)},e.prototype.getRawValue=function(e){return t.prototype.getValue.call(this,e)},e.prototype.getExportValue=function(e,n){return"json"===n?this.getRawValue(e):t.prototype.getExportValue.call(this,e,n)},e.prototype.getValue=function(t){var e=this,n=this.getRawValue(t);if(null==n)return n;var r={min:this.mapping.apply(n.min),max:this.mapping.apply(n.max),median:this.mapping.apply(n.median),q1:this.mapping.apply(n.q1),q3:this.mapping.apply(n.q3)};return n.outlier&&Object.assign(r,{outlier:n.outlier.map(function(t){return e.mapping.apply(t)})}),null!=n.whiskerLow&&Object.assign(r,{whiskerLow:this.mapping.apply(n.whiskerLow)}),null!=n.whiskerHigh&&Object.assign(r,{whiskerHigh:this.mapping.apply(n.whiskerHigh)}),r},e.prototype.getNumber=function(t){return Object(c.f)(this,t,"normalized")},e.prototype.getRawNumber=function(t){return Object(c.f)(this,t,"raw")},e.prototype.getLabel=function(t){var e=this.getRawValue(t);if(null==e)return"";var r=n.DEFAULT_FORMATTER;return"BoxPlot(min = "+r(e.min)+", q1 = "+r(e.q1)+", median = "+r(e.median)+", q3 = "+r(e.q3)+", max = "+r(e.max)+")"},e.prototype.getSortMethod=function(){return this.sort},e.prototype.setSortMethod=function(t){this.sort!==t&&(this.fire(n.EVENT_SORTMETHOD_CHANGED,this.sort,this.sort=t),this.isSortedByMe().asc||this.sortByMe())},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return n.sortMethod=this.getSortMethod(),n.filter=Object(s.c)(this.currentFilter)?null:this.currentFilter,n.map=this.mapping.dump(),n.colorMapping=this.colorMapping.dump(),n},e.prototype.restore=function(e,n){t.prototype.restore.call(this,e,n),e.sortMethod&&(this.sort=e.sortMethod),e.filter&&(this.currentFilter=Object(s.g)(e.filter)),e.map?this.mapping=Object(l.c)(e.map):e.domain&&(this.mapping=new l.a(e.domain,"linear",e.range||[0,1])),e.colorMapping&&(this.colorMapping=Object(p.e)(this.color,e.colorMapping))},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_SORTMETHOD_CHANGED,n.EVENT_COLOR_MAPPING_CHANGED,n.EVENT_MAPPING_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getOriginalMapping=function(){return this.original.clone()},e.prototype.getMapping=function(){return this.mapping.clone()},e.prototype.setMapping=function(t){this.mapping.eq(t)||this.fire([n.EVENT_MAPPING_CHANGED,a.e.EVENT_DIRTY_HEADER,a.e.EVENT_DIRTY_VALUES,a.e.EVENT_DIRTY],this.mapping.clone(),this.mapping=t)},e.prototype.getColor=function(t){return d.c.prototype.getColor.call(this,t)},e.prototype.getColorMapping=function(){return this.colorMapping.clone()},e.prototype.setColorMapping=function(t){this.colorMapping.eq(t)||this.fire([n.EVENT_COLOR_MAPPING_CHANGED,a.e.EVENT_DIRTY_HEADER,a.e.EVENT_DIRTY_VALUES,a.e.EVENT_DIRTY],this.colorMapping.clone(),this.colorMapping=t)},e.prototype.isFiltered=function(){return d.c.prototype.isFiltered.call(this)},e.prototype.getFilter=function(){return d.c.prototype.getFilter.call(this)},e.prototype.setFilter=function(t){void 0===t&&(t={min:-1/0,max:1/0,filterMissing:!1}),d.c.prototype.setFilter.call(this,t)},e.prototype.filter=function(t){return d.c.prototype.filter.call(this,t)},e.EVENT_MAPPING_CHANGED=d.c.EVENT_MAPPING_CHANGED,e.EVENT_COLOR_MAPPING_CHANGED=d.c.EVENT_COLOR_MAPPING_CHANGED,e.EVENT_SORTMETHOD_CHANGED="sortMethodChanged",e.DEFAULT_FORMATTER=Object(i.a)(".3n"),e=n=r.b([Object(o.l)("filterNumber","colorMapped","editMapping"),Object(o.g)("sort","sortBoxPlot"),Object(o.b)("array"),Object(o.c)("descending")],e)}(u.b);e.c=f},function(t,e,n){"use strict";n.d(e,"j",function(){return d}),n.d(e,"i",function(){return p}),n.d(e,"h",function(){return f});var r=n(0),i=n(1),o=n(20),a=n(2),u=n(23),s=n(7),c=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.currentMaxDepth=1/0,r.currentLeaves=[],r.currentLeavesNameCache=new Map,r.currentLeavesPathCache=new Map,r.hierarchySeparator=n.hierarchySeparator||".",r.hierarchy=r.initHierarchy(n.hierarchy),r.currentNode=r.hierarchy,r.currentLeaves=l(r.currentNode,r.currentMaxDepth),r.updateCaches(),r.setDefaultRenderer("categorical"),r}var n;return r.c(e,t),n=e,e.prototype.initHierarchy=function(t){var e=Object(u.a)(),n=this.hierarchySeparator,r=function(t,i){var o=null==i.name?String(i.value):i.name,a=(i.children||[]).map(function(i){if("string"==typeof i){var a=t+i;return{path:a,name:i,label:a,color:e(),value:0,children:[]}}var u=r(""+t+o+n,i);return u.color||(u.color=e()),u}),u=t+o,s=i.label?""+i.label:u;return{path:u,name:o,children:a,label:s,color:i.color,value:0}};return r("",t)},Object.defineProperty(e.prototype,"categories",{get:function(){return this.currentLeaves},enumerable:!0,configurable:!0}),e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_CUTOFF_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getCutOff=function(){return{node:this.currentNode,maxDepth:this.currentMaxDepth}},e.prototype.setCutOff=function(t){var e=null==t.maxDepth?1/0:t.maxDepth;if(this.currentNode!==t.node||this.currentMaxDepth!==e){var r=this.getCutOff();this.currentNode=t.node,this.currentMaxDepth=e,this.currentLeaves=l(t.node,e),this.updateCaches(),this.fire([n.EVENT_CUTOFF_CHANGED,a.e.EVENT_DIRTY_HEADER,a.e.EVENT_DIRTY_VALUES,a.e.EVENT_DIRTY],r,this.getCutOff())}},e.prototype.getCategory=function(e){var n=this,r=t.prototype.getValue.call(this,e);return null==r||""===r?null:(r=r.trim(),this.currentLeavesNameCache.has(r)?this.currentLeavesNameCache.get(r):this.currentLeavesPathCache.has(r)?this.currentLeavesPathCache.get(r):this.currentLeaves.find(function(t){return t.path===r||t.name===r||r.startsWith(t.path+n.hierarchySeparator)})||null)},Object.defineProperty(e.prototype,"dataLength",{get:function(){return this.currentLeaves.length},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"labels",{get:function(){return this.currentLeaves.map(function(t){return t.label})},enumerable:!0,configurable:!0}),e.prototype.getValue=function(t){var e=this.getCategory(t);return e?e.name:null},e.prototype.getLabel=function(t){return o.a.prototype.getLabel.call(this,t)},e.prototype.getColor=function(t){return o.a.prototype.getColor.call(this,t)},e.prototype.getLabels=function(t){return o.a.prototype.getLabels.call(this,t)},e.prototype.getValues=function(t){return o.a.prototype.getValues.call(this,t)},e.prototype.getMap=function(t){return o.a.prototype.getMap.call(this,t)},e.prototype.getMapLabel=function(t){return o.a.prototype.getMapLabel.call(this,t)},e.prototype.getSet=function(t){return o.a.prototype.getSet.call(this,t)},e.prototype.compare=function(t,e){return o.a.prototype.compare.call(this,t,e)},e.prototype.group=function(e){if(this.isMissing(e))return s.e;var n=this.getCategory(e);return n?{name:n.label,color:n.color}:t.prototype.group.call(this,e)},e.prototype.updateCaches=function(){var t=this;this.currentLeavesPathCache.clear(),this.currentLeavesNameCache.clear(),this.currentLeaves.forEach(function(e){t.currentLeavesPathCache.set(e.path,e),t.currentLeavesNameCache.set(e.name,e)})},e.EVENT_CUTOFF_CHANGED="cutOffChanged",e=n=r.b([Object(i.l)("cutoff","group","groupBy"),Object(i.b)("categorical")],e)}(n(8).b);function l(t,e){void 0===e&&(e=1/0);var n=[],r=function(t,i){i>=e||0===t.children.length?n.push(t):t.children.forEach(function(t){return r(t,i+1)})};return r(t,0),n}function d(t){for(var e=[t],n=0;n<e.length;){var r=e[n++];e.push.apply(e,r.children)}return e}function p(t){return 0!==t.length&&"string"!=typeof t[0]&&t.some(function(t){return null!=t.parent})}function f(t){var e=new Map;t.forEach(function(t){var n=t.parent||"",r=Object.assign({children:[],label:t.name,name:t.name,color:a.e.DEFAULT_COLOR,value:0},e.get(t.name)||{},t);e.set(t.name,r),e.has(n)||e.set(n,{name:n,children:[],label:n,value:0,color:a.e.DEFAULT_COLOR}),e.get(n).children.push(r)});var n=e.get("");return console.assert(void 0!==n,"hierarchy with no root"),1===n.children.length?n.children[0]:n}e.g=c},function(t,e,n){"use strict";var r=n(0),i=n(1),o=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.setDefaultWidth(200),r}return r.c(e,t),e.prototype.getValue=function(e){return function(t){return t?t instanceof Map?Array.from(t.entries()).map(function(t){var e=t[0],n=t[1];return{key:e,value:n}}).sort(a):Array.isArray(t)?t:Object.keys(t).map(function(e){return{key:e,value:t[e]}}).sort(a):[]}(t.prototype.getValue.call(this,e))},e.prototype.getLabels=function(t){return this.getValue(t).map(function(t){var e=t.key,n=t.value;return{key:e,value:String(n)}})},e.prototype.getMap=function(t){return this.getValue(t)},e.prototype.getMapLabel=function(t){return this.getLabels(t)},e.prototype.getLabel=function(t){return"{"+this.getLabels(t).map(function(t){return t.key+": "+t.value}).join(", ")+"}"},e=r.b([Object(i.b)("map")],e)}(n(8).b);function a(t,e){return t===e?0:t.key.localeCompare(e.key)}e.a=o},function(t,e,n){"use strict";var r=n(0),i=n(48),o=n(13),a=n(7),u=n(10),s=n(1),c=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.missingValue=NaN,r.numberFormat=Object(i.a)(".3n"),n.numberFormat&&(r.numberFormat=Object(i.a)(n.numberFormat)),void 0!==n.missingValue&&(r.missingValue=n.missingValue),r}return r.c(e,t),e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return n.missingValue=this.missingValue,n},e.prototype.restore=function(e,n){void 0!==e.missingValue&&(this.missingValue=e.missingValue),e.numberFormat&&(this.numberFormat=Object(i.a)(e.numberFormat)),t.prototype.restore.call(this,e,n)},e.prototype.getLabel=function(t){if(!this.isLoaded())return"";var e=this.getValue(t);return String("number"==typeof e&&!isNaN(e)&&isFinite(e)?this.numberFormat(e):e)},e.prototype.getValue=function(t){if(!this.isLoaded())return null;var e=this.compute(t);return Object(a.c)(e)?this.missingValue:e},e.prototype.compute=function(t){return NaN},e.prototype.getNumber=function(t){var e=this.getValue(t);return null==e?NaN:e},e.prototype.getRawNumber=function(t){return this.getNumber(t)},e.prototype.getExportValue=function(e,n){return"json"===n?{value:this.getRawNumber(e),children:this.children.map(function(t){return t.getExportValue(e,n)})}:t.prototype.getExportValue.call(this,e,n)},e.prototype.isMissing=function(t){return Object(a.c)(this.compute(t))},e.prototype.compare=function(t,e){return u.c.prototype.compare.call(this,t,e)},e.prototype.groupCompare=function(t,e){return u.c.prototype.groupCompare.call(this,t,e)},e.prototype.getRenderer=function(){return u.c.prototype.getRenderer.call(this)},e=r.b([Object(s.c)("descending")],e)}(o.b);e.a=c},function(t,e,n){"use strict";n.d(e,"a",function(){return o});var r=n(55),i=n(42);function o(){return{toolbar:Object.assign({},i.g),renderers:Object.assign({},r.q),labelRotation:0,summaryHeader:!0,animated:!0,expandLineOnHover:!1,sidePanel:!0,sidePanelCollapsed:!1,hierarchyIndicator:!0,defaultSlopeGraphMode:"item",overviewMode:!1,rowHeight:18,groupHeight:40,groupPadding:5,rowPadding:2,levelOfDetail:function(){return"high"},customRowUpdate:function(){},dynamicHeight:function(){return null},flags:{disableFrozenColumns:!0},ignoreUnsupportedBrowser:!1}}},function(t,e,n){"use strict";var r=n(80);n.d(e,"e",function(){return r.b}),n.d(e,"c",function(){return r.a});var i=n(86);n.d(e,"d",function(){return i.a});var o=n(93);n.d(e,"a",function(){return o.IRule}),n.d(e,"b",function(){return o.IRuleInstance});var a=n(87);n.d(e,"f",function(){return a.a})},function(t,e,n){"use strict";n.d(e,"a",function(){return o});var r=n(0),i=n(1);function o(t){return void 0===t&&(t="Rank"),{type:"rank",label:t}}var a=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.setDefaultWidth(50),r}return r.c(e,t),Object.defineProperty(e.prototype,"frozen",{get:function(){return!1!==this.desc.frozen},enumerable:!0,configurable:!0}),e=r.b([Object(i.d)(),Object(i.b)("support")],e)}(n(8).b);e.b=a},function(t,e,n){"use strict";n.d(e,"a",function(){return o}),n.d(e,"b",function(){return a}),n.d(e,"c",function(){return u}),n.d(e,"d",function(){return s});var r=n(3),i="Filter rows containing missing values";function o(t,e){var n=Object(r.j)(e);return'<div class="lu-checkbox"><input type="checkbox" '+(t?'checked="checked"':"")+' id="'+n+'"><label for="'+n+'" class="lu-filter-missing">'+i+"</label></div>"}function a(t,e,n){var i=Object(r.j)(n);return'<div class="lu-checkbox"><input type="checkbox" '+(t?'checked="checked"':"")+" "+(0===e?"disabled":"")+' id="'+i+'"><label for="'+i+'" class="lu-filter-missing" '+(0===e?'class="lu-disabled"':"")+">Filter "+e+" missing value rows</label></div>"}function u(t){return t.querySelector(".lu-filter-missing").previousElementSibling}function s(t,e){var n=t.querySelector("input");e>0&&(t.classList.remove("lu-disabled"),n.disabled=!1),n.checked||(t.lastElementChild.textContent="Filter "+e+" remaining missing value rows")}},function(t,e,n){"use strict";var r=n(0),i=n(37),o=n(81),a=n(82),u=n(75),s=n(76),c=n(25);n.d(e,"a",function(){return c.ILineUpOptions});var l=function(t){function e(e,n,r){void 0===r&&(r={});var a=t.call(this,e,n,r&&!0===r.ignoreUnsupportedBrowser)||this;return a.options=Object(i.a)(),Object(o.a)(a.options,r),a.isBrowserSupported?(a.node.classList.add("lu"),a.renderer=new u.a(n,a.node,a.options),a.options.sidePanel?(a.panel=new s.b(a.renderer.ctx,a.node.ownerDocument,{collapseable:!a.options.sidePanelCollapsed||"collapsed",hierarchy:a.options.hierarchyIndicator}),a.renderer.pushUpdateAble(function(t){return a.panel.update(t)}),a.node.insertBefore(a.panel.node,a.node.firstChild)):a.panel=null,a.forward(a.renderer,u.a.EVENT_HIGHLIGHT_CHANGED+".main"),a):(a.renderer=null,a.panel=null,a)}return r.c(e,t),e.prototype.destroy=function(){this.node.classList.remove("lu"),this.renderer&&this.renderer.destroy(),this.panel&&this.panel.destroy(),t.prototype.destroy.call(this)},e.prototype.update=function(){this.renderer&&this.renderer.update()},e.prototype.setDataProvider=function(e,n){t.prototype.setDataProvider.call(this,e,n),this.renderer&&(this.renderer.setDataProvider(e),this.update(),this.panel&&this.panel.update(this.renderer.ctx))},e.prototype.setHighlight=function(t,e){return void 0===e&&(e=!0),null!=this.renderer&&this.renderer.setHighlight(t,e)},e.prototype.getHighlight=function(){return this.renderer?this.renderer.getHighlight():-1},e.prototype.enableHighlightListening=function(t){this.renderer&&this.renderer.enableHighlightListening(t)},e}(a.a);e.b=l},function(t,e,n){"use strict";var r=n(1),i=n(0),o=n(17),a=n(3),u=function(t){function e(e,n,r){var i=t.call(this,n)||this;return i.column=e,i.ctx=r,i}return i.c(e,t),e.prototype.build=function(t){var e=this,n=this.column.getRenderer(),r=this.column.getGroupRenderer(),i=this.column.getSummaryRenderer(),o=this.ctx.getPossibleRenderer(this.column),u=o.item,s=o.group,c=o.summary;console.assert(u.length>1||s.length>1||c.length>1);var l=Object(a.j)(this.dialog.idPrefix);t.insertAdjacentHTML("beforeend","\n      <strong>Item Visualization</strong>\n      "+u.sort(function(t,e){return t.label.localeCompare(e.label)}).map(function(t){return' <div class="lu-checkbox"><input id="'+l+"0"+t.type+'" type="radio" name="renderer" value="'+t.type+'" '+(n===t.type?"checked":"")+'><label for="'+l+"0"+t.type+'">'+t.label+"</label></div>"}).join("")+"\n      <strong>Group Visualization</strong>\n      "+s.sort(function(t,e){return t.label.localeCompare(e.label)}).map(function(t){return' <div class="lu-checkbox"><input id="'+l+"1"+t.type+'" type="radio" name="group" value="'+t.type+'" '+(r===t.type?"checked":"")+'><label for="'+l+"1"+t.type+'">'+t.label+"</label></div>"}).join("")+"\n      <strong>Summary Visualization</strong>\n      "+c.sort(function(t,e){return t.label.localeCompare(e.label)}).map(function(t){return' <div class="lu-checkbox"><input id="'+l+"2"+t.type+'" type="radio" name="summary" value="'+t.type+'" '+(i===t.type?"checked":"")+'><label for="'+l+"2"+t.type+'">'+t.label+"</label></div>"}).join("")+"\n    "),this.forEach('input[name="renderer"]',function(t){t.addEventListener("change",function(){return e.column.setRenderer(t.value)},{passive:!0})}),this.forEach('input[name="group"]',function(t){t.addEventListener("change",function(){return e.column.setGroupRenderer(t.value)},{passive:!0})}),this.forEach('input[name="summary"]',function(t){t.addEventListener("change",function(){return e.column.setSummaryRenderer(t.value)},{passive:!0})})},e}(o.c),s=n(49),c=function(t){function e(e,n,r){var i=t.call(this,n)||this;return i.column=e,i.ctx=r,i}return i.c(e,t),e.prototype.build=function(t){t.classList.add("lu-more-options"),Object(s.c)(t,this.dialog.level+1,this.column,this.ctx),Object(s.f)(t,this.column)},e}(o.c),l=n(2),d=n(45),p=function(t){function e(e,n){var r=t.call(this,n,{fullDialog:!0})||this;return r.column=e,r.before=e.getMetaData(),r}return i.c(e,t),e.prototype.build=function(t){t.classList.add("lu-rename-dialog");var e=Object(a.j)("col");t.insertAdjacentHTML("beforeend",'\n      <datalist id="'+e+'L"><option value="'+l.e.DEFAULT_COLOR+'"></option>'+d.K.slice(1).map(function(t){return"<option>"+t+"</option>"}).join("")+'</datalist>\n      <input type="text" value="'+this.column.label+'" required autofocus placeholder="name">\n      <input type="color" value="'+this.column.color+'" required placeholder="color" list="'+e+'L">\n      <textarea rows="5" placeholder="Description">'+this.column.description+"</textarea>")},e.prototype.reset=function(){this.findInput('input[type="text"]').value=this.before.label,this.findInput('input[type="color"]').value=this.before.color||l.e.DEFAULT_COLOR,this.node.querySelector("textarea").value=this.before.description,this.column.setMetaData(this.before)},e.prototype.submit=function(){var t=this.findInput('input[type="text"]').value,e=this.findInput('input[type="color"]').value,n=this.node.querySelector("textarea").value;return this.column.setMetaData({label:t,color:e,description:n}),!0},e}(o.c),f=n(30),h=function(t){function e(e,n){var r=t.call(this,n,{fullDialog:!0})||this;return r.column=e,r.before=r.column.getFilter(),r}return i.c(e,t),e.prototype.build=function(t){var e=Object(a.j)(this.dialog.idPrefix);t.insertAdjacentHTML("beforeend",'\n     <div class="lu-checkbox"><input id="'+e+'0" type="radio" name="boolean_check" value="null" '+(null==this.before?'checked="checked"':"")+'><label id="'+e+'0">No Filter</label></div>\n     <div class="lu-checkbox"><input id="'+e+'1" type="radio" name="boolean_check" value="true" '+(!0===this.before?'checked="checked"':"")+'><label id="'+e+'1">True</label></div>\n     <div class="lu-checkbox"><input id="'+e+'2" type="radio" name="boolean_check" value="false" '+(!1===this.before?'checked="checked"':"")+'><label id="'+e+'2">False</label></div>\n    ')},e.prototype.updateFilter=function(t){Object(f.b)(this.attachment,this.column,null!=t),this.column.setFilter(t)},e.prototype.reset=function(){this.forEach('input[type="radio"]',function(t){return t.checked="null"===t.value}),this.updateFilter(null)},e.prototype.submit=function(){var t=this.findInput('input[type="radio"][value="true"]').checked,e=this.findInput('input[type="radio"][value="false"]').checked;return this.updateFilter(!!t||!e&&null),!0},e}(o.c),g=n(11),m=n(40),b=function(t){function e(e,n){var r=t.call(this,n,{fullDialog:!0})||this;return r.column=e,r.before=r.column.getFilter()||{filter:r.column.categories.map(function(t){return t.name}),filterMissing:!1},r}return i.c(e,t),e.prototype.build=function(t){var e=this;t.classList.add("lu-filter-table");var n=Object(a.j)(this.dialog.idPrefix);t.insertAdjacentHTML("beforeend",'<div>\n        <div class="lu-checkbox"><input id="'+n+'" type="checkbox" checked><label for="'+n+'"><span></span><div>Un/Select All</div></label></div>\n        '+this.column.categories.map(function(t){return'<div class="lu-checkbox"><input id="'+n+t.name+'" data-cat="'+t.name+'" type="checkbox"'+(Object(g.d)(e.before,t)?"checked":"")+'><label for="'+n+t.name+'"><span style="background-color: '+t.color+'"></span><div>'+t.label+"</div></label></div>"}).join("")+"\n    </div>"),this.findInput("input:not([data-cat])").onchange=function(){var e=this;Object(a.d)(t,"input[data-cat]",function(t){return t.checked=e.checked})},t.insertAdjacentHTML("beforeend",Object(m.a)(this.before.filterMissing,this.dialog.idPrefix))},e.prototype.updateFilter=function(t,e){var n=null==t&&!1===e;Object(f.b)(this.attachment,this.column,!n),this.column.setFilter(n?null:{filter:t,filterMissing:e})},e.prototype.reset=function(){this.forEach("input[data-cat]",function(t){return t.checked=!0}),this.updateFilter(null,!1)},e.prototype.submit=function(){var t=this.forEach("input[data-cat]",function(t){return t.checked?t.dataset.cat:""}).filter(Boolean);t.length===this.column.categories.length&&(t=null);var e=Object(m.c)(this.node).checked;return this.updateFilter(t,e),!0},e}(o.c),y=n(9),v=function(t){function e(e,n){var r=t.call(this,n,{fullDialog:!0})||this;return r.column=e,r.before=r.column.getFilter()||{filter:r.column.categories.map(function(t){return t.name}),filterMissing:!1},r}return i.c(e,t),e.prototype.build=function(t){var e=this;t.classList.add("lu-filter-table");var n=this.column.categories.map(function(t){return Object.assign({range:Object(y.k)(100*t.value,2)})});n.sort(function(t,e){return t.label.localeCompare(e.label)});var r=Object(a.j)(this.dialog.idPrefix);t.insertAdjacentHTML("beforeend",'<div>\n        <div class="lu-checkbox"><input id="'+r+'" type="checkbox" checked><label for="'+r+'"><div>Un/Select All</div></label></div>\n        '+n.map(function(t){var n=t.name,i=t.color,o=t.label,a=t.range;return'<input id="'+r+n+'" data-cat="'+n+'" type="checkbox"'+(Object(g.d)(e.before,n)?"checked":"")+'>\n        <div class="lu-checkbox"><input type="number" value="'+a+'" min="0" max="100" size="5"><label for="'+r+n+'"><div><div style="background-color: '+i+"; width: "+a+'%"></div></div><div>'+o+"</div></label></div>"}).join("")+"\n    </div>"),this.findInput("input[type=checkbox]:not([data-cat])").onchange=function(){var e=this;Object(a.d)(t,"[data-cat]",function(t){return t.checked=e.checked})},this.forEach("input[type=number]",function(t){t.oninput=function(){t.nextElementSibling.firstElementChild.style.width=t.value+"%"}}),t.insertAdjacentHTML("beforeend",Object(m.a)(this.before.filterMissing,this.dialog.idPrefix))},e.prototype.updateFilter=function(t,e){var n=null==t&&!1===e;Object(f.b)(this.attachment,this.column,!n),this.column.setFilter(n?null:{filter:t,filterMissing:e})},e.prototype.reset=function(){this.forEach("[data-cat]",function(t){t.checked=!1,t.nextElementSibling.value="50"}),this.updateFilter(null,!1),this.column.setMapping(this.column.categories.map(function(){return 1}))},e.prototype.submit=function(){var t=this.forEach("input[data-cat]",function(t){return{checked:t.checked,cat:t.dataset.cat,range:parseFloat(t.nextElementSibling.value)}}),e=t.filter(function(t){return t.checked}).map(function(t){return t.cat});e.length===this.column.categories.length&&(e=null);var n=Object(m.c)(this.node).checked;return this.updateFilter(e,n),this.column.setMapping(t.map(function(t){return t.range/100})),!0},e}(o.c),E=n(16),C=n(54),w=n(13),A=function(t){function e(e,n,r){var i=t.call(this,n)||this;return i.column=e,i.ctx=r,i.id=".dialog"+Math.random().toString(36).slice(-8).substr(0,3),i}return i.c(e,t),e.prototype.destroy=function(){this.column.on(Object(E.b)(this.id,w.b.EVENT_ADD_COLUMN,w.b.EVENT_REMOVE_COLUMN),null),t.prototype.destroy.call(this)},e.prototype.build=function(t){var e=this;t.classList.add("lu-sub-nested");var n=function(){e.column.children.forEach(function(n){var r=Object(s.a)(n,e.ctx,{mergeDropAble:!1,resizeable:!1,level:e.dialog.level+1});r.className="lu-header",Object(s.e)(r,n);var i=e.ctx.summaryRenderer(n,!1);r.insertAdjacentHTML("beforeend",i.template);var o=r.lastElementChild;o.dataset.renderer=n.getSummaryRenderer(),o.classList.add("lu-summary"),i.update(o,e.ctx.statsOf(n)),t.appendChild(r)})};n(),this.column.on(Object(E.b)(this.id,w.b.EVENT_ADD_COLUMN,w.b.EVENT_REMOVE_COLUMN),Object(C.a)(function(){t.parentElement?(t.innerHTML="",n()):e.destroy()}))},e}(o.c),x=n(34),N=function(t){function e(e,n,r){var i=t.call(this,n,{fullDialog:!0})||this;return i.column=e,i.idPrefix=r,i.innerNodes=Object(x.j)(i.column.hierarchy),i.innerNodePaths=i.innerNodes.map(function(t){return t.path}),i.before=e.getCutOff(),i}return i.c(e,t),e.prototype.build=function(t){t.insertAdjacentHTML("beforeend",'\n        <input type="text" value="'+this.before.node.label+'" required="required" autofocus="autofocus" list="ui'+this.idPrefix+'lineupHierarchyList" placeholder="cut off node">\n        <input type="number" value="'+(isFinite(this.before.maxDepth)?this.before.maxDepth:"")+'" placeholder="max depth (&infin;)">\n        <datalist id="ui'+this.idPrefix+'lineupHierarchyList">'+this.innerNodes.map(function(t){return'<option value="'+t.path+'">'+t.label+"</option>"})+"</datalist>");var e=this.innerNodePaths;this.findInput('input[type="text"]').addEventListener("change",function(){var t=this.value;e.indexOf(t)<0?this.setCustomValidity("invalid node"):this.setCustomValidity("")},{passive:!0})},e.prototype.reset=function(){this.findInput('input[type="text"]').value=this.before.node.label,this.findInput('input[type="number"]').value=isFinite(this.before.maxDepth)?String(this.before.maxDepth):"",this.column.setCutOff(this.before)},e.prototype.submit=function(){var t=this.findInput('input[type="text"]').value,e=this.innerNodePaths.indexOf(t),n=this.innerNodes[e],r=this.findInput('input[type="number"]').value,i=""===r?1/0:parseInt(r,10);return this.column.setCutOff({node:n,maxDepth:i}),!0},e}(o.c),T=function(t){function e(e,n,r){var i=t.call(this,n,{fullDialog:!0})||this;return i.column=e,i.idPrefix=r,i}return i.c(e,t),e.prototype.build=function(t){var e=this.column.patternTemplates;t.insertAdjacentHTML("beforeend",'<strong>Edit Pattern (access via ${value}, ${item})</strong><input\n        type="text"\n        size="30"\n        value="'+this.column.getPattern()+'"\n        required\n        autofocus\n        placeholder="pattern (access via ${value}, ${item})"\n        '+(e.length>0?'list="ui'+this.idPrefix+'lineupPatternList"':"")+"\n      >"),e.length>0&&t.insertAdjacentHTML("beforeend",'<datalist id="ui'+this.idPrefix+'lineupPatternList">'+e.map(function(t){return'<option value="'+t+'">'})+"</datalist>")},e.prototype.reset=function(){this.node.querySelector("input").value="",this.column.setPattern("")},e.prototype.submit=function(){var t=this.node.querySelector("input").value;return this.column.setPattern(t),!0},e}(o.c),O=function(t){function e(e,n,r){var i=t.call(this,n)||this;return i.column=e,i.ctx=r,i.summary=r.summaryRenderer(i.column,!0),i}return i.c(e,t),e.prototype.build=function(t){t.classList.add("lu-dialog-mapper"),t.insertAdjacentHTML("beforeend",this.summary.template);var e=t.lastElementChild;e.classList.add("lu-summary","lu-renderer"),e.dataset.renderer=this.column.getSummaryRenderer(),e.dataset.interactive="",t.appendChild(e),this.summary.update(this.node.querySelector(".lu-summary"),this.ctx.statsOf(this.column))},e}(o.c),R=n(4),M=function(t){function e(e,n){var r=t.call(this,n)||this;return r.column=e,r}return i.c(e,t),e.prototype.build=function(t){var e=this,n={getSortMethod:function(){return e.column.getReduce()},setSortMethod:function(t){return e.column.setReduce(t)}};Object(f.a)(t,n,Object.keys(R.b),this.dialog.idPrefix)},e}(o.c),D=function(t){function e(e,n){var r=t.call(this,n,{fullDialog:!0})||this;return r.column=e,r.before=e.getScript(),r}return i.c(e,t),e.prototype.build=function(t){t.insertAdjacentHTML("beforeend",'<textarea autofocus="true" rows="5" autofocus="autofocus" style="width: 95%;">'+this.column.getScript()+"</textarea>")},e.prototype.reset=function(){this.node.querySelector("textarea").value=this.before,this.column.setScript(this.before)},e.prototype.submit=function(){return this.column.setScript(this.node.querySelector("textarea").value),!0},e}(o.c),_=function(t){function e(e,n,r){var i=t.call(this,n)||this;return i.column=e,i.provider=r,i}return i.c(e,t),e.prototype.build=function(t){var e=this,n=Object(a.j)(this.dialog.idPrefix);t.insertAdjacentHTML("beforeend",'<input type="text" size="20" value="" required autofocus placeholder="search... (>= 3 chars)"><div class="lu-checkbox"><input id="'+n+'" type="checkbox"><label for="'+n+'">RegExp</label></div>');var r=t.querySelector('input[type="text"]'),i=t.querySelector('input[type="checkbox"]'),o=function(){var t=r.value;t.length<3?r.setCustomValidity("at least 3 characters"):(r.setCustomValidity(""),i.checked&&(t=new RegExp(t)),e.provider.searchAndJump(t,e.column))};r.addEventListener("input",o,{passive:!0}),i.addEventListener("change",o,{passive:!0})},e}(o.c),L=n(21),S=function(t){function e(e,n){var r=t.call(this,n,{fullDialog:!0})||this;return r.column=e,r}return i.c(e,t),e.prototype.updateFilter=function(t){Object(f.b)(this.attachment,this.column,null!=t&&""!==t),this.column.setFilter(t)},e.prototype.reset=function(){this.findInput('input[type="text"]').value="",this.forEach("input[type=checkbox",function(t){return t.checked=!1}),this.updateFilter(null)},e.prototype.submit=function(){if(Object(m.c)(this.node).checked)return this.updateFilter(L.d.FILTER_MISSING),!0;var t=this.findInput('input[type="text"]').value,e=this.findInput('input[type="checkbox"]').checked;return this.updateFilter(e?new RegExp(t):t),!0},e.prototype.build=function(t){var e=this,n=this.column.getFilter()||"",r=n===L.d.FILTER_MISSING;r&&(n="");var i=Object(a.j)(this.dialog.idPrefix);t.insertAdjacentHTML("beforeend",'<input type="text" placeholder="Filter '+this.column.desc.label+'..." autofocus value="'+(n instanceof RegExp?n.source:n)+'" style="width: 100%">\n    <span class="lu-checkbox"><input id="'+i+'" type="checkbox" '+(n instanceof RegExp?'checked="checked"':"")+'><label for="'+i+'">RegExp</label></span>\n    '+Object(m.a)(r,this.dialog.idPrefix));var o=Object(m.c)(t),u=t.querySelector('input[type="text"]'),s=t.querySelector('input[type="checkbox"]'),c=function(){if(u.disabled=o.checked,s.disabled=o.checked,o.checked)e.updateFilter(L.d.FILTER_MISSING);else{var t=u.value.trim();o.disabled=t.length>0,t.length<=0?e.updateFilter(null):e.updateFilter(s.checked?new RegExp(u.value,"gm"):u.value)}};o.onchange=c,u.onchange=c,s.onchange=c},e}(o.c),I=function(t){function e(e,n){var r=t.call(this,n,{fullDialog:!0})||this;return r.column=e,r.weights=r.column.getWeights(),r}return i.c(e,t),e.prototype.reset=function(){var t=this;Object(a.d)(this.node,"input[type=number]",function(e){var n=Object(y.k)(100/t.weights.length,2);e.value=String(n),e.nextElementSibling.firstElementChild.style.width=n+"%"}),this.column.setWeights(this.weights.slice().fill(100/this.weights.length))},e.prototype.build=function(t){t.classList.add("lu-filter-table","lu-weights-table");var e=this.column.children;t.insertAdjacentHTML("beforeend","<div>\n        "+this.weights.map(function(t,n){return'<div><input type="number" value="'+Object(y.k)(100*t,2)+'" min="0" max="100" step="any"><span><span style="background-color: '+e[n].color+"; width: "+Object(y.k)(100*t,2)+'%"></span></span>'+e[n].label+"</div>"}).join("")+"\n    </div>"),this.forEach("input[type=number]",function(t){t.oninput=function(){t.nextElementSibling.firstElementChild.style.width=t.value+"%"}})},e.prototype.submit=function(){var t=this.forEach("input[type=number]",function(t){return parseFloat(t.value)/100});return this.column.setWeights(t),!0},e}(o.c),j=function(t){function e(e,n,r){var i=t.call(this,n)||this;return i.column=e,i.ctx=r,i.addons=gt(i.column,"group",r),i}return i.c(e,t),e.prototype.build=function(t){for(var e=0,n=this.addons;e<n.length;e++){var r=n[e];this.node.insertAdjacentHTML("beforeend","<strong>"+r.title+"</strong>"),r.append(this.column,this.node,this.dialog,this.ctx)}!function(t,e,n){var r=e.findMyRanker(),i=r.getGroupCriteria(),o=i.indexOf(e),u=o>=0;o<0&&(o=i.length);var s=Object(a.j)(n);t.insertAdjacentHTML("afterbegin",'\n        <strong>Group By</strong>\n        <div class="lu-checkbox"><input id="'+s+'B" type="radio" name="grouped" value="true" '+(u?"checked":"")+' ><label for="'+s+'B">Enabled</label></div>\n        <div class="lu-checkbox"><input id="'+s+'N" type="radio" name="grouped" value="false" '+(u?"":"checked")+' ><label for="'+s+'N">Disabled</label></div>\n        <strong>Group Priority</strong>\n        <input type="number" id="'+s+'P" step="1" min="1" max="'+(i.length+1)+'" value="'+(o+1)+'">\n    ');var c=function(e){Object(a.d)(t,"input:not([name=grouped]), select, textarea",function(t){t.disabled=e})};c(!u);var l=function(){r.groupBy(e,u?o:-1),c(!u)};Object(a.d)(t,"input[name=grouped]",function(t){t.addEventListener("change",function(){u="true"===t.value,l()},{passive:!0})});var d=t.querySelector("#"+s+"P");d.addEventListener("change",function(){o=parseInt(d.value,10)-1,l()},{passive:!0})}(t,this.column,this.dialog.idPrefix)},e}(o.c),k=function(t){function e(e,n,r,i){var o=t.call(this,r)||this;return o.column=e,o.group=n,o.ctx=i,o.addons=gt(o.column,n?"sortGroup":"sort",i),o}return i.c(e,t),e.prototype.build=function(t){for(var e=0,n=this.addons;e<n.length;e++){var r=n[e];this.node.insertAdjacentHTML("beforeend","<strong>"+r.title+"</strong>"),r.append(this.column,this.node,this.dialog,this.ctx)}!function(t,e,n,r){void 0===r&&(r=!1);var i=e.findMyRanker(),o=r?i.getGroupSortCriteria():i.getSortCriteria(),u=Object.assign({},r?e.isGroupSortedByMe():e.isSortedByMe());void 0===u.priority&&(u.priority=o.length);var s=Object(a.j)(n);t.insertAdjacentHTML("afterbegin",'\n        <strong>Sort Order</strong>\n        <div class="lu-checkbox"><input id="'+s+'B" type="radio" name="sortorder" value="asc"  '+("asc"===u.asc?"checked":"")+' ><label for="'+s+'B">Ascending</label></div>\n        <div class="lu-checkbox"><input id="'+s+'D" type="radio" name="sortorder" value="desc"  '+("desc"===u.asc?"checked":"")+' ><label for="'+s+'D">Decending</label></div>\n        <div class="lu-checkbox"><input id="'+s+'N" type="radio" name="sortorder" value="none"  '+(void 0===u.asc?"checked":"")+' ><label for="'+s+'N">Unsorted</label></div>\n        <strong>Sort Priority</strong>\n        <input type="number" id="'+s+'P" step="1" min="1" max="'+(o.length+1)+'" value="'+(u.priority+1)+'">\n    ');var c=function(e){Object(a.d)(t,"input:not([name=sortorder]), select, textarea",function(t){t.disabled=e})};c(void 0===u.asc);var l=function(){r?i.groupSortBy(e,"asc"===u.asc,void 0===u.asc?-1:u.priority):i.sortBy(e,"asc"===u.asc,void 0===u.asc?-1:u.priority),c(void 0===u.asc)};Object(a.d)(t,"input[name=sortorder]",function(t){t.addEventListener("change",function(){u.asc="none"===t.value?void 0:t.value,l()},{passive:!0})});var d=t.querySelector("#"+s+"P");d.addEventListener("change",function(){u.priority=parseInt(d.value,10)-1,l()},{passive:!0})}(t,this.column,this.dialog.idPrefix,this.group)},e}(o.c),V=n(89),H=n(74),P=n(27),G=function(t){function e(e,n){var r=t.call(this,n)||this;return r.column=e,r}return i.c(e,t),e.prototype.build=function(t){t.classList.add("lu-dialog-color");var e=Object(a.j)("col"),n=this.column.getColorMapping(),r="custom"===n.type?n.entries:[],i="";i+='<datalist id="'+e+'L">'+d.K.map(function(t){return"<option>"+t+'"</option>'}).join("")+"</datalist>",i+='<datalist id="'+e+'LW"><option>#FFFFFF"</option>'+d.K.slice(0,-1).map(function(t){return"<option>"+t+"</option>"}).join("")+"</datalist>",i+='<strong>Quantization</strong>\n    <div class="lu-checkbox">\n      <input id="'+e+'KC" name="kind" type="radio" value="continuous" '+("quantized"!==n.type?"checked":"")+'>\n      <label for="'+e+'CK">Continuous</label>\n    </div>\n    <div class="lu-checkbox">\n      <input id="'+e+'KQ" name="kind" type="radio" value="quantized" '+("quantized"===n.type?"checked":"")+'>\n      <label for="'+e+'KQ"><input type="number" id="'+e+'KQS" min="2" step="1" value="'+("quantized"===n.type?n.steps:5)+'">&nbsp; steps</label>\n    </div>',i+='<strong data-toggle="'+("solid"===n.type?"open":"")+'">Solid Color</strong>',i+="<div>";for(var o="solid"===n.type?n.color:"",u=!1,s=0,c=[d.K,d.J,d.L,d.M,d.N,d.O,d.P,d.Q];s<c.length;s++){var p=c[s];u=u||p.includes(o),i+='<div class="lu-color-line">\n          '+p.map(function(t){return'<div class="lu-checkbox-color">\n              <input id="'+e+Object(H.d)(t)+'" name="color" type="radio" value="'+t+'" '+(t===o?'checked="checked"':"")+'>\n              <label for="'+e+Object(H.d)(t)+'" style="background: '+t+'"></label>\n            </div>'}).join("")+"\n        </div>"}i+='<div class="lu-checkbox"><input id="'+e+'O" name="color" type="radio" value="custom:solid" '+(o&&!u?'checked="checked"':"")+'>\n        <label for="'+e+'O"><input type="color" name="solid" list="'+e+'L" value="'+("solid"===n.type?n.color:l.e.DEFAULT_COLOR)+'" '+(o&&!u?"":"disabled")+"></label>\n      </div>",i+="</div>",i+='<strong data-toggle="'+("sequential"===n.type||"custom"===n.type&&2===r.length?"open":"")+'">Sequential Color</strong>',i+="<div>";for(var f="sequential"===n.type?n.name:"",h=0,g=P.i;h<g.length;h++)p=g[h],i+='<div class="lu-checkbox lu-color-gradient"><input id="'+e+p.name+'" name="color" type="radio" value="'+p.name+'" '+(p.name===f?'checked="checked"':"")+'>\n        <label for="'+e+p.name+'" data-c="'+p.name+'" style="background: '+F(p.apply,9)+'"></label>\n      </div>';var m=2===r.length;i+='<div class="lu-checkbox lu-color-gradient">\n        <input id="'+e+'S" name="color" type="radio" value="custom:sequential" '+(m?"checked":"")+'>\n        <label for="'+e+'S" class="lu-color-custom">\n          <input type="color" name="interpolate0" list="'+e+'LW" '+(m?'value="'+r[0].color+'"':"disabled")+'>\n          <input type="color" name="interpolate1" list="'+e+'LW" '+(m?'value="'+r[r.length-1].color+'"':"disabled")+">\n        </label>\n      </div>",i+="</div>",i+='<strong data-toggle="'+("divergent"===n.type||"custom"===n.type&&3===r.length?"open":"")+'">Diverging Color</strong>',i+="<div>",f="sequential"===n.type?n.name:"";for(var b=0,y=P.f;b<y.length;b++)p=y[b],i+='<div class="lu-checkbox lu-color-gradient"><input id="'+e+p.name+'" name="color" type="radio" value="'+p.name+'" '+(p.name===f?'checked="checked"':"")+'>\n        <label for="'+e+p.name+'" data-c="'+p.name+'" style="background: '+F(p.apply,11)+'"></label>\n      </div>';m=3===r.length,i+='<div class="lu-checkbox lu-color-gradient"><input id="'+e+'D" name="color" type="radio" value="custom:divergent" '+(m?"checked":"")+'>\n        <label for="'+e+'D" class="lu-color-custom">\n          <input type="color" name="divergingm1" list="'+e+'L" '+(m?'value="'+r[0].color+'"':"disabled")+'>\n          <input type="color" name="diverging0" list="'+e+'LW" '+(m?'value="'+r[1].color+'"':"disabled")+'>\n          <input type="color" name="diverging1" list="'+e+'L" '+(m?'value="'+r[2].color+'"':"disabled")+">\n          </label>\n      </div>",i+="</div>",t.insertAdjacentHTML("beforeend",i),this.interactive(t,e)},e.prototype.interactive=function(t,e){var n=this,r=this.findInput("#"+e+"KC"),i=this.findInput("#"+e+"KQ"),o=this.findInput("#"+e+"KQS"),a=Array.from(t.querySelectorAll("strong[data-toggle]"));r.onchange=function(){r.checked&&n.updateGradients(-1)},i.onchange=o.onchange=function(){i.checked?("open"===a[0].dataset.toggle&&(a[0].dataset.toggle="",a[1].dataset.toggle="open"),n.updateGradients(parseInt(o.value,10))):n.updateGradients(-1)};for(var u=function(t){t.onclick=function(e){e.preventDefault(),e.stopPropagation();for(var n=0,r=a;n<r.length;n++){var i=r[n];i.dataset.toggle="open"===i.dataset.toggle||t!==i?"":"open"}}},s=0,c=a;s<c.length;s++)u(c[s]);var l=[];this.forEach("input[name=color]",function(e){e.value.startsWith("custom:")&&l.push(e),e.onchange=function(){if(e.checked){for(var r=function(t){Array.from(t.nextElementSibling.querySelectorAll("input")).forEach(function(n){return n.disabled=t!==e})},a=0,u=l;a<u.length;a++)r(u[a]);var s=function(e){switch(e.value){case"custom:solid":return Object(P.d)(t.querySelector("input[name=solid]").value);case"custom:sequential":var r=t.querySelector("input[name=interpolate0]").value,i=t.querySelector("input[name=interpolate1]").value;return new P.a([{color:r,value:0},{color:i,value:1}]);case"custom:diverging":var o=t.querySelector("input[name=divergentm1]").value,a=t.querySelector("input[name=divergent0]").value,u=t.querySelector("input[name=divergent1]").value;return new P.a([{color:o,value:0},{color:a,value:.5},{color:u,value:1}])}return Object(P.e)(n.column.color,e.value)}(e);i.checked&&"solid"!==s.type?n.column.setColorMapping(new P.c(s,parseInt(o.value,10))):n.column.setColorMapping(s)}}}),this.forEach("label > input[type=color]",function(t){t.onchange=function(){var e=t.parentElement.previousElementSibling;e.onchange.call(e,null)}})},e.prototype.updateGradients=function(t){this.forEach("label[data-c]",function(e){var n=P.g.get(e.dataset.c);e.style.background=t<0?F(n.apply,"sequential"===n.type?9:11):function(t,e){if(void 0===e&&(e=2),1===e)return""+t(0);for(var n="linear-gradient(to right",r=1/e,i=r/2,o=0;o<e;++o){var a=0===o?0:o===e-1?r:i,u=t(o*r+a);n+=", "+u+" "+Object(y.k)(o*r*100,2)+"%, "+u+" "+Object(y.k)((o+1)*r*100,2)+"%"}return n+=")"}(n.apply,t)})},e}(o.c);function F(t,e){if(void 0===e&&(e=2),e<=1)return""+t(0);for(var n=1/(e-1),r="linear-gradient(to right",i=0;i<e;++i)r+=", "+t(i*n)+" "+Object(y.k)(i*n*100,2)+"%";return r+=")"}var U=n(12),B=n(7),z=n(78);function Y(t){return Math.max(Math.min(t,100),0)}var W=function(t){function e(e,n,r){var i=t.call(this,n)||this;return i.line=e,i.adapter=r,i}return i.c(e,t),e.prototype.build=function(t){var e=this,n=this.adapter.domain();t.insertAdjacentHTML("beforeend",'\n        <strong>Input Domain Value (min ... max)</strong>\n        <input type="number" value="'+Object(y.k)(this.adapter.unnormalizeRaw(this.line.domain),3)+'" '+(this.line.frozen?"readonly":"")+' autofocus required min="'+n[0]+'" max="'+n[1]+'" step="any">\n        <strong>Output Normalized Value (0 ... 1)</strong>\n        <input type="number" value="'+Object(y.k)(this.line.range/100,3)+'" required min="0" max="1" step="any">\n        <button type="button" '+(this.line.frozen?"disabled":"")+" >Remove Mapping Line</button>\n      "),this.forEach("input",function(t){return t.onchange=function(){return e.submit()}}),this.find("button").addEventListener("click",function(){e.destroy(),e.line.destroy()},{passive:!0})},e.prototype.submit=function(){if(!this.node.checkValidity())return!1;var t=this.adapter.normalizeRaw(parseFloat(this.findInput("input[type=number]").value)),e=100*parseFloat(this.findInput("input[type=number]:last-of-type").value);return this.line.update(t,e),!0},e}(o.c),q=function(){function t(t,e,n,r){var o=this;this.domain=e,this.range=n,this.adapter=r,t.insertAdjacentHTML("beforeend",'<g class="lu-mapping" transform="translate('+e+',0)">\n      <line x1="0" x2="'+(n-e)+'" y2="60"></line>\n      <line x1="0" x2="'+(n-e)+'" y2="60"></line>\n      <circle r="3"></circle>\n      <circle cx="'+(n-e)+'" cy="60" r="3"></circle>\n      <title>Drag the anchor circle to change the mapping, shift click to edit</title>\n    </g>'),this.node=t.lastElementChild,this.node.classList.toggle("lu-frozen",Object(y.l)(0,e)||Object(y.l)(e,100));var a,u,s,c,l=function(e){return 100*e/t.getBoundingClientRect().width},d={container:t.parentElement,filter:function(t){return 0===t.button&&!t.shiftKey},onStart:function(t,e){a=o.domain,u=o.range;var n=l(e);s=o.domain-n,c=o.range-n},onEnd:function(){Object(y.l)(a,o.domain)&&Object(y.l)(u,o.range)||o.adapter.updated(o)}},p=this.node.querySelector("line:first-of-type");Object(z.a)(p,i.a({},d,{onDrag:function(t,e){var n=l(e);o.update(Y(n+s),Y(n+c))}}));var f=this.node.querySelector("circle:first-of-type");Object(z.a)(f,i.a({},d,{onDrag:function(t,e){var n=l(e);o.update(Y(n),o.range)}}));var h=this.node.querySelector("circle:last-of-type");Object(z.a)(h,i.a({},d,{onDrag:function(t,e){var n=l(e);o.update(o.domain,Y(n))}})),this.node.onclick=function(t){if(t.shiftKey){var e={manager:o.adapter.dialog.manager,level:o.adapter.dialog.level+1,attachment:o.node,idPrefix:o.adapter.dialog.idPrefix};new W(o,e,o.adapter).open()}}}return Object.defineProperty(t.prototype,"frozen",{get:function(){return this.node.classList.contains("lu-frozen")},enumerable:!0,configurable:!0}),t.prototype.destroy=function(){this.node.remove(),this.adapter.destroyed(this)},t.prototype.update=function(t,e){if(!Object(y.l)(t,this.domain)||!Object(y.l)(e,this.range)){this.frozen&&(t=this.domain),this.domain=t,this.range=e,this.node.setAttribute("transform","translate("+t+",0)");var n=e-t;Array.from(this.node.querySelectorAll("line")).forEach(function(t){return t.setAttribute("x2",String(n))}),this.node.querySelector("circle[cx]").setAttribute("cx",String(n))}},t}(),K=function(t){function e(e,n,r){var i=t.call(this,n,{fullDialog:!0})||this;i.column=e,i.mappingLines=[],i.loadedData=null,i.mappingAdapter={destroyed:function(t){i.mappingLines.splice(i.mappingLines.indexOf(t),1)},updated:function(){return i.updateLines(i.computeScale())},domain:function(){return i.rawDomain},normalizeRaw:i.normalizeRaw.bind(i),unnormalizeRaw:i.unnormalizeRaw.bind(i),dialog:i.dialog},i.idPrefix="me"+r.idPrefix,i.scale=i.column.getMapping().clone();var o=i.scale.domain;return i.rawDomain=[o[0],o[o.length-1]],i.data=Promise.resolve(r.provider.mappingSample(e)),i}return i.c(e,t),Object.defineProperty(e.prototype,"scaleType",{get:function(){if(!(this.scale instanceof U.a))return"script";var t=this.scale.scaleType;if("linear"!==t)return t;var e=this.scale.range;return 2===e.length&&1===e[0]&&0===e[1]?"linear_invert":3===e.length&&1===e[0]&&0===e[1]&&1===e[2]?"linear_abs":"linear"},enumerable:!0,configurable:!0}),e.prototype.build=function(t){var e=this;t.classList.add("lu-dialog-mapper");var n=this.column.findMyRanker(),r=n?n.flatColumns.filter(function(t){return Object(U.d)(t)&&t!==e.column}):[];t.insertAdjacentHTML("beforeend",'\n        <div><label for="'+this.idPrefix+'mapping_type"><strong>Normalization Scaling:</strong></label><select id="'+this.idPrefix+'mapping_type" class="browser-default">\n        <option value="linear">Linear</option>\n        <option value="linear_invert">Invert</option>\n        <option value="linear_abs">Absolute</option>\n        <option value="log">Log</option>\n        <option value="pow1.1">Pow 1.1</option>\n        <option value="pow2">Pow 2</option>\n        <option value="pow3">Pow 3</option>\n        <option value="sqrt">Sqrt</option>\n        <option value="script">Custom Script</option>\n        '+(r.length>0?'<optgroup label="Copy From">'+r.map(function(t){return'<option value="copy_'+t.id+'">'+t.label+"</option>"}).join("")+"</optgroup>":"")+'\n      </select>\n      </div>\n        <div class="lu-details"><strong>Domain (min - max): </strong><input id="'+this.idPrefix+'min" required type="number" value="'+Object(y.k)(this.rawDomain[0],3)+'" step="any"> - <input id="'+this.idPrefix+'max" required type="number" value="'+Object(y.k)(this.rawDomain[1],3)+'" step="any"></div>\n        <strong class="lu-details" style="text-align: center">Input Domain (min - max)</strong>\n        <svg class="lu-details" viewBox="0 0 106 66">\n           <g transform="translate(3,3)">\n              <line x2="100"></line>\n              <rect y="-3" width="100" height="10"></rect>\n              <line y1="60" x2="100" y2="60"></line>\n              <rect y="36" width="100" height="10"></rect>\n           </g>\n        </svg>\n        <strong class="lu-details" style="text-align: center; margin-top: 0">Output Normalized Domain (0 - 1)</strong>\n        <div class="lu-script">\n          <strong>Custom Normalization Script</strong>\n          <textarea></textarea>\n        </div>');var i=t.querySelector(".lu-details > g");this.forEach(".lu-details rect",function(t){return t.onclick=function(n){n.preventDefault(),n.stopPropagation();var r=t.getBoundingClientRect(),o=Object(y.k)(100*(n.x-r.left)/r.width,2),a=new q(i,o,o,e.mappingAdapter);e.mappingLines.push(a)}});var o=this.find("select"),a=this.find("textarea");o.onchange=function(n){var r=n.currentTarget;switch(r.value){case"linear_invert":e.scale=new U.a(e.rawDomain.slice(),"linear",[1,0]);break;case"linear_abs":e.scale=new U.a([e.rawDomain[0],(e.rawDomain[1]-e.rawDomain[0])/2,e.rawDomain[1]],"linear",[1,0,1]);break;case"script":var i=new U.b(e.rawDomain.slice());e.scale=i,a.value=i.code;break;default:if(r.value.startsWith("copy_"))return void e.copyMapping(r.value.slice("copy_".length));e.scale=new U.a(e.rawDomain.slice(),r.value)}e.createMappings(),t.dataset.scale=r.value,e.updateLines()};var u=t.dataset.scale=this.scaleType;o.selectedIndex=Array.from(o.options).findIndex(function(t){return t.value===u}),"script"===u&&(a.value=this.scale.code),this.createMappings(),this.forEach(".lu-details input[type=number]",function(t,n){return t.onchange=function(){var r=parseFloat(t.value);if(r!==e.rawDomain[n]){var i=e.rawDomain[1-n];isNaN(r)||0===n&&r>=i||1===n&&r<=i?t.setCustomValidity("value has to be "+(0===n?"<= max":">= min")):(t.setCustomValidity(""),e.rawDomain[n]=r,e.scale.domain=e.rawDomain.slice(),e.loadedData&&(e.applyMapping(e.scale),e.updateLines()))}else t.setCustomValidity("")}}),this.data.then(function(t){e.loadedData=t,Array.from(t).forEach(function(t){Object(B.c)(t)||i.insertAdjacentHTML("afterbegin",'<line data-v="'+t+'" x1="'+Object(y.k)(e.normalizeRaw(t),2)+'" x2="'+Object(y.k)(100*e.scale.apply(t),2)+'" y2="60"></line>')})})},e.prototype.createMappings=function(){var t,e=this;if(this.mappingLines.splice(0,this.mappingLines.length).forEach(function(t){return t.destroy()}),this.scale instanceof U.a){var n=this.node.querySelector(".lu-details > g"),r=this.scale.domain,i=this.scale.range;(t=this.mappingLines).push.apply(t,r.map(function(t,r){return new q(n,e.normalizeRaw(t),100*i[r],e.mappingAdapter)}))}},e.prototype.update=function(){var t=this.node.dataset.scale=this.scaleType,e=this.find("select");e.selectedIndex=Array.from(e.options).findIndex(function(e){return e.value===t}),"script"===t&&(this.find("textarea").value=this.scale.code);var n=this.scale.domain;this.forEach(".lu-details input[type=number]",function(t,e){t.value=String(n[e])})},e.prototype.updateLines=function(t){var e=this;void 0===t&&(t=this.scale),this.forEach(".lu-details > g > line[x1]",function(n){var r=parseFloat(n.getAttribute("data-v"));n.setAttribute("x1",Object(y.k)(e.normalizeRaw(r),2).toString()),n.setAttribute("x2",Object(y.k)(100*t.apply(r),2).toString())})},e.prototype.applyMapping=function(t){this.column.setMapping(t)},e.prototype.reset=function(){this.scale=this.column.getOriginalMapping(),this.rawDomain=this.scale.domain.slice(),this.applyMapping(this.scale),this.update(),this.updateLines(),this.createMappings()},e.prototype.copyMapping=function(t){var e=this.column.findMyRanker();if(e){var n=e.find(t);this.scale=n.getMapping().clone(),this.rawDomain=this.scale.domain.slice(),this.applyMapping(this.scale),this.update(),this.updateLines()}},e.prototype.normalizeRaw=function(t){var e=100*(t-this.rawDomain[0])/(this.rawDomain[1]-this.rawDomain[0]);return Math.max(Math.min(e,100),0)},e.prototype.unnormalizeRaw=function(t){return t*(this.rawDomain[1]-this.rawDomain[0])/100+this.rawDomain[0]},e.prototype.computeScale=function(){var t=this,e=this.scale.clone();return"script"===this.scaleType&&(e.code=this.node.querySelector("textarea").value,e.domain=this.rawDomain.slice()),e instanceof U.a&&(this.mappingLines.sort(function(t,e){return t.domain-e.domain}),e.domain=this.mappingLines.map(function(e){return t.unnormalizeRaw(e.domain)}),e.range=this.mappingLines.map(function(t){return t.range/100})),e},e.prototype.submit=function(){var t=this;return!!this.node.checkValidity()&&("script"===this.scaleType&&(this.scale.code=this.node.querySelector("textarea").value,this.scale.domain=this.rawDomain.slice()),this.scale instanceof U.a&&(this.mappingLines.sort(function(t,e){return t.domain-e.domain}),this.scale.domain=this.mappingLines.map(function(e){return t.unnormalizeRaw(e.domain)}),this.scale.range=this.mappingLines.map(function(t){return t.range/100})),this.applyMapping(this.scale),!0)},e}(o.c);function J(t,e,n){return void 0===n&&(n={}),{title:t,onClick:e,options:n}}function Z(t,e,n){return{attachment:n.currentTarget,level:e,manager:t.dialogManager,idPrefix:t.idPrefix}}function Q(t,e,n,r){return void 0===n&&(n=function(){return[]}),void 0===r&&(r={}),{title:t,onClick:function(t,r,i,o){(new(e.bind.apply(e,[void 0,t,Z(i,o,r)].concat(n(i))))).open()},options:r}}function X(t){return t=t.sort(function(t,e){return t.toLowerCase().localeCompare(e.toLowerCase())}),{title:"Sort By",order:2,append:function(e,n,r){Object(f.a)(n,e,t,r.idPrefix)}}}n.d(e,"b",function(){return Z}),n.d(e,"g",function(){return dt}),n.d(e,"a",function(){return ht}),n.d(e,"c",function(){return gt}),n.d(e,"f",function(){return mt}),n.d(e,"d",function(){return bt}),n.d(e,"e",function(){return yt});var $={title:"Sort",onClick:function(t,e,n,i){if(n.dialogManager.removeAboveLevel(i),e.ctrlKey){var o=t.findMyRanker(),a=o.getSortCriteria(),u=t.isSortedByMe(),s=Object(r.j)(t);if(void 0!==u.priority){var c=void 0;c=s?u.asc?"desc":void 0:u.asc?void 0:"asc",o.sortBy(t,"asc"===c,c?u.priority:-1)}else o.sortBy(t,s,a.length)}else t.toggleMySorting()},options:{shortcut:"only",order:1}},tt={title:"Sort By &hellip;",onClick:function(t,e,n,r){new k(t,!1,Z(n,r,e),n).open()},options:{shortcut:!1,order:1}},et={title:"Sort Groups By &hellip;",onClick:function(t,e,n,r){new k(t,!0,Z(n,r,e),n).open()},options:{shortcut:!1,order:3}},nt={title:"Rename + Color &hellip;",onClick:function(t,e,n,r){new p(t,Z(n,r,e)).open()},options:{order:4}},rt={title:"Visualization &hellip;",onClick:function(t,e,n,r){new u(t,Z(n,r,e),n).open()},options:{}},it={title:"Clone",onClick:function(t,e,n){n.dialogManager.removeAll(),n.provider.takeSnapshot(t)},options:{order:80}},ot={title:"More &hellip;",onClick:function(t,e,n,r){new c(t,Z(n,r,e),n).open()},options:{shortcut:!0,order:100}},at={title:"Remove",onClick:function(t,e,n){n.dialogManager.removeAll();var i=t.findMyRanker();i.children.every(function(e){return Object(r.k)(e)||e.fixed||e===t})?(n.provider.removeRanking(i),n.provider.ensureOneRanking()):t.removeMe()},options:{order:90}},ut=J("Group",function(t,e,n,r){if(n.dialogManager.removeAboveLevel(r),e.ctrlKey){var i=t.findMyRanker(),o=i.getGroupCriteria(),a=o.indexOf(t);i.groupBy(t,a>=0?-1:o.length)}else t.groupByMe()},{shortcut:"only",order:2}),st=J("Group By &hellip;",function(t,e,n,r){new j(t,Z(n,r,e),n).open()},{shortcut:!1,order:2}),ct=J("Compress",function(t,e,n,r){n.dialogManager.removeAboveLevel(r);var i=t;i.setCollapsed(!i.getCollapsed()),e.currentTarget.title=i.getCollapsed()?"Expand":"Compress"}),lt={sortNumber:X(Object.keys(R.b)),sortNumbers:X(Object.keys(R.b)),sortBoxPlot:X(Object.keys(R.c)),sortDates:X(Object.keys(V.a)),sortGroup:X(["count","name"]),groupNumber:{title:"Split",order:2,append:function(t,e,n){var r=t.getMapping().domain,i=t.getGroupThresholds(),o=i.length<=1,u=t.findMyRanker();e.insertAdjacentHTML("beforeend",'\n    <div class="lu-checkbox">\n      <input type="radio" name="threshold" value="threshold" id="'+n.idPrefix+'T1" '+(o?"checked":"")+'>\n      <label for="'+n.idPrefix+'T1">at&nbsp;<input type="number" size="10" id="'+n.idPrefix+'N1" value="'+(i.length>0?i[0]:Object(y.k)((r[1]-r[0])/2,2))+'"\n          required min="'+r[0]+'" max="'+r[1]+'" step="any" '+(o?"":"disabled")+'>\n      </label>\n    </div>\n    <div class="lu-checkbox">\n      <input type="radio" name="threshold" value="bins" id="'+n.idPrefix+'T2" '+(o?"":"checked")+'>\n      <label for="'+n.idPrefix+'T2"> in&nbsp;<input type="number" size="5" id="'+n.idPrefix+'N2" value="'+(i.length>1?i.length:Object(y.j)(u.getOrder().length))+'"\n          required min="2" step="1" '+(o?"disabled":"")+">&nbsp;bins\n      </label>\n    </div>\n  ");var s=e.querySelector("#"+n.idPrefix+"N1"),c=e.querySelector("#"+n.idPrefix+"N2"),l=function(){if(s.disabled=!o,c.disabled=o,o)t.setGroupThresholds([parseFloat(s.value)]);else{for(var e=parseInt(c.value,10),n=(r[1]-r[0])/e,i=r[0]+n,a=[i],u=1;u<e-1;++u)i+=n,a.push(i);t.setGroupThresholds(a)}};s.addEventListener("change",l,{passive:!0}),c.addEventListener("change",l,{passive:!0}),Object(a.d)(e,"input[name=threshold]",function(t){t.addEventListener("change",function(){o="threshold"===t.value,l()},{passive:!0})})}},groupString:{title:"Groups",order:2,append:function(t,e,n){var r=t.getGroupCriteria(),i=r.length>0&&r[0]instanceof RegExp;e.insertAdjacentHTML("beforeend",'\n    <div class="lu-checkbox">\n      <input type="radio" name="regex" value="startsWith" id="'+n.idPrefix+'RW" '+(i?"":"checked")+'>\n      <label for="'+n.idPrefix+'RW">Text starts with &hellip;</label>\n    </div>\n    <div class="lu-checkbox">\n      <input type="radio" name="regex" value="regex" id="'+n.idPrefix+'RE" '+(i?"checked":"")+'>\n      <label for="'+n.idPrefix+'RE">RegExp</label>\n    </div>\n    <textarea required rows="5" placeholder="e.g. Test,a.*" id="'+n.idPrefix+'T">'+r.map(function(t){return"string"==typeof t?t:t.source}).join("\n")+'</textarea>\n    <button id="'+n.idPrefix+'A">Apply</button>\n  ');var o=e.querySelector("#"+n.idPrefix+"A"),a=e.querySelector("#"+n.idPrefix+"RE"),u=e.querySelector("#"+n.idPrefix+"T");o.onclick=function(e){e.preventDefault(),e.stopPropagation();var n=u.value.trim().split("\n").map(function(t){return t.trim()}).filter(function(t){return t.length>0}),r=0===n.length;u.setCustomValidity(r?"At least one group is required":""),r?u.reportValidity():(a.checked&&(n=n.map(function(t){return new RegExp(t.toString(),"gm")})),t.setGroupCriteria(n))}}}},dt=Object.assign({group:ut,groupBy:st,collapse:ct,sort:$,sortBy:tt,sortGroupBy:et,more:ot,clone:it,remove:at,rename:nt,search:Q("Search &hellip;",_,function(t){return[t.provider]},{shortcut:!0,order:3}),filterNumber:Q("Filter &hellip;",O,function(t){return[t]},{shortcut:!0}),filterString:Q("Filter &hellip;",S,function(){return[]},{shortcut:!0}),filterCategorical:Q("Filter &hellip;",b,function(){return[]},{shortcut:!0}),filterOrdinal:Q("Filter &hellip;",v,function(){return[]},{shortcut:!0}),filterBoolean:Q("Filter &hellip;",h,function(){return[]},{shortcut:!0}),colorMapped:Q("Color Mapping &hellip;",G,function(){return[]},{shortcut:!1}),script:Q("Edit Combine Script &hellip;",D,function(){return[]},{shortcut:!0}),reduce:Q("Reduce by &hellip;",M),cutoff:Q("Set Cut Off &hellip;",N,function(t){return[t.idPrefix]}),editMapping:Q("Data Mapping &hellip;",K,function(t){return[t]}),editPattern:Q("Edit Pattern &hellip;",T,function(t){return[t.idPrefix]}),editWeights:Q("Edit Weights &hellip;",I,function(){return[]},{shortcut:!0}),compositeContained:Q("Contained Columns &hellip;",A,function(t){return[t]}),splitCombined:J("Split Combined Column",function(t,e,n,r){n.dialogManager.removeAboveLevel(r),t.children.reverse().forEach(function(e){return t.insertAfterMe(e)}),t.removeMe()}),invertSelection:J("Invert Selection",function(t,e,n,r){n.dialogManager.removeAboveLevel(r-1);var i=n.provider.getSelection(),o=t.findMyRanker().getOrder();if(0!==i.length){var a=new Set(i),u=o.filter(function(t){return!a.has(t)});n.provider.setSelection(u)}else n.provider.setSelection(o)})},lt),pt=new Map,ft=new Map;function ht(t,e){if(pt.has(t.desc.type))return pt.get(t.desc.type);var n=e.toolbar,i=new Set;t.fixed||i.add(at);var o=e.getPossibleRenderer(t);(o.item.length>2||o.group.length>2||o.summary.length>2)&&i.add(rt),Object(r.k)(t)||(i.add($),i.add(tt),i.add(nt),i.add(it)),Object(r.h)(t).forEach(function(e){n.hasOwnProperty(e)?i.add(n[e]):console.warn("cannot find: ",t.desc.type,e)}),i.size>0&&i.add(ot);var a=Array.from(i).sort(function(t,e){return t.options.order===e.options.order?t.title.localeCompare(e.title):(t.options.order||50)-(e.options.order||50)});return pt.set(t.desc.type,a),a}function gt(t,e,n){var i=t.desc.type+"@"+e;if(ft.has(i))return ft.get(i);var o=n.toolbar,a=new Set;Object(r.i)(t,e).forEach(function(e){o.hasOwnProperty(e)?a.add(o[e]):console.warn("cannot find: ",t.desc.type,e)});var u=Array.from(a).sort(function(t,e){return t.order===e.order?t.title.localeCompare(e.title):(t.order||50)-(e.order||50)});return ft.set(i,u),u}function mt(t,e){return ht(t,e).includes(tt)}function bt(t,e){return ht(t,e).includes(st)}function yt(t,e){return ht(t,e).includes(et)}},function(t,e,n){"use strict";var r=n(0),i=function(t){function e(e){void 0===e&&(e={});var n=t.call(this)||this;return n.options={formatItem:function(t){return t.text},doc:document,placeholder:"Select..."},n.values=[],Object.assign(n.options,e),n.node=n.options.doc.createElement("div"),n.node.classList.add("lu-search"),n.node.innerHTML='<input type="search" placeholder="'+n.options.placeholder+'"><ul></ul>',n.search=n.node.querySelector("input"),n.body=n.node.querySelector("ul"),n.search.onfocus=function(){return n.focus()},n.search.onblur=function(){return n.blur()},n.search.oninput=function(){return n.filter()},n.search.onkeydown=function(t){return n.handleKey(t)},n}return r.c(e,t),Object.defineProperty(e.prototype,"data",{get:function(){return this.values},set:function(t){this.values=t,this.body.innerHTML="",this.buildDialog(this.body,this.values)},enumerable:!0,configurable:!0}),e.prototype.buildDialog=function(t,e){var n=this;e.forEach(function(e){if(function(t){return void 0!==t.id}(e)){t.insertAdjacentHTML("beforeend",'<li class="lu-search-item"><span></span></li>');var r=t.lastElementChild;r.onmousedown=function(t){t.preventDefault()},r.onclick=function(){return n.select(e)},r.onmouseenter=function(){return n.highlighted=r},r.onmouseleave=function(){return n.highlighted=null}}else{t.insertAdjacentHTML("beforeend",'<li class="lu-search-group"><span></span><ul></ul></li>');var i=t.lastElementChild.lastElementChild;n.buildDialog(i,e.children)}var o=t.lastElementChild.firstElementChild;o.innerHTML=n.options.formatItem(e,o)})},e.prototype.handleKey=function(t){switch(t.which){case 27:this.search.blur();break;case 13:var e=this.highlighted;e&&e.click();break;case 38:this.highlightPrevious();break;case 40:this.highlightNext()}},e.prototype.select=function(t){this.search.value="",this.search.blur(),this.fire(e.EVENT_SELECT,t)},e.prototype.focus=function(){this.body.style.width=this.search.offsetWidth+"px",this.highlighted=this.body.firstElementChild||null,this.node.classList.add("lu-search-open")},Object.defineProperty(e.prototype,"highlighted",{get:function(){return this.body.querySelector(".lu-search-highlighted")||null},set:function(t){var e=this.highlighted;e!==t&&(e&&e.classList.remove("lu-search-highlighted"),t&&t.classList.add("lu-search-highlighted"))},enumerable:!0,configurable:!0}),e.prototype.highlightNext=function(){var t=this.highlighted;if(t&&!t.classList.contains("hidden")){var e=Array.from(this.body.querySelectorAll(".lu-search-item:not(.hidden)")),n=e.indexOf(t);this.highlighted=e[n+1]||null}else this.highlighted=this.body.querySelector(".lu-search-item:not(.hidden)")||null},e.prototype.highlightPrevious=function(){var t=this.highlighted,e=Array.from(this.body.querySelectorAll(".lu-search-item:not(.hidden)"));if(t&&!t.classList.contains("hidden")){var n=e.indexOf(t);this.highlighted=e[n-1]||null}else this.highlighted=e[e.length-1]||null},e.prototype.blur=function(){this.search.value="",this.filterResults(this.body,""),this.node.classList.remove("lu-search-open")},e.prototype.filter=function(){var t=this.filterResults(this.body,this.search.value.toLowerCase());this.body.classList.toggle("lu-search-empty",t)},e.prototype.filterResults=function(t,e){var n=this;if(""===e)return Array.from(t.querySelectorAll(".hidden")).forEach(function(t){return t.classList.remove("hidden")}),!1;var r=Array.from(t.children);return r.forEach(function(t){var r=!t.firstElementChild.innerHTML.toLowerCase().includes(e);if(t.classList.contains("lu-search-group")){var i=t.lastElementChild,o=n.filterResults(i,e);r=r&&o}t.classList.toggle("hidden",r)}),r.every(function(t){return t.classList.contains("hidden")})},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([e.EVENT_SELECT])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.EVENT_SELECT="select",e}(n(16).a);e.d=i},function(t,e,n){"use strict";n.d(e,"b",function(){return s}),n.d(e,"d",function(){return l}),n.d(e,"c",function(){return d}),n.d(e,"e",function(){return p});var r=n(18),i=n(1),o=n(4),a=n(2),u=n(23);function s(t){return t.delete(null),t.delete(void 0),t.delete(""),t.delete("NA"),t.delete("NaN"),t.delete("na"),Array.from(t).map(String).sort()}function c(t,e,n,i,o){var a,u,c={type:"string",label:t,column:n};if("number"==typeof e)return c.type="number",c.domain=Object(r.d)(i,function(t){return t[n]}),c;if(e&&e instanceof Date)return c.type="date",c;if("boolean"==typeof e)return c.type="boolean",c;if("string"==typeof e)return(l=new Set(i.map(function(t){return t[n]}))).size<i.length*o.categoricalThreshold&&(c.type="categorical",c.categories=s(l)),c;if(Array.isArray(e)){c.type="strings",c.dataLength=e.length;var l,d=e[0];if("number"==typeof d)return c.type="numbers",c.domain=Object(r.d)((a=[]).concat.apply(a,i.map(function(t){return t[n]}))),c;if(d&&e instanceof Date)return c.type="dates",c;if("boolean"==typeof e)return c.type="booleans",c;if("string"==typeof e)return(l=new Set((u=[]).concat.apply(u,i.map(function(t){return t[n]})))).size<i.length*o.categoricalThreshold&&(c.type="categoricals",c.categories=s(l)),c}return console.log("cannot infer type of column:",n),c}function l(t,e){void 0===e&&(e={});var n=Object.assign({categoricalThreshold:.7,columns:[]},e);if(0===t.length)return[];var r=t[0];return Array.isArray(r)?r.map(function(e,r){return c("Col"+r,e,r,t,n)}):(n.columns.length>0?n.columns:Object.keys(r)).map(function(e){return c(e,r[e],e,t,n)})}function d(t){var e=Object(u.a)();return t.forEach(function(t){switch(t.type){case"number":t.color=t.color||e()||a.e.DEFAULT_COLOR}}),t}function p(t,e,n){void 0===n&&(n={});var r=Object.assign({separator:"\t",newline:"\n",header:!0,quote:!1,quoteChar:'"',filter:function(t){return!Object(i.k)(t)},verboseColumnHeaders:!1},n),a=new RegExp("["+r.quoteChar+"]","g");function u(t,e){var n=String(t);return!(r.quote||n.indexOf("\n")>=0)||e&&Object(o.i)(e)?n:""+r.quoteChar+n.replace(a,r.quoteChar+r.quoteChar)+r.quoteChar}var s=t.flatColumns.filter(function(t){return r.filter(t)}),c=t.getOrder(),l=[];return r.header&&l.push(s.map(function(t){return u(t.label+(r.verboseColumnHeaders&&t.description?"\n"+t.description:""))}).join(r.separator)),e.forEach(function(t,e){l.push(s.map(function(n){return u(n.getExportValue({v:t,i:c[e]},"text"),n)}).join(r.separator))}),l.join(r.newline)}},function(t,e,n){"use strict";var r=function(t){for(var e=t.length/6|0,n=new Array(e),r=0;r<e;)n[r]="#"+t.slice(6*r,6*++r);return n},i=r("1f77b4ff7f0e2ca02cd627289467bd8c564be377c27f7f7fbcbd2217becf"),o=r("7fc97fbeaed4fdc086ffff99386cb0f0027fbf5b17666666"),a=r("1b9e77d95f027570b3e7298a66a61ee6ab02a6761d666666"),u=(r("a6cee31f78b4b2df8a33a02cfb9a99e31a1cfdbf6fff7f00cab2d66a3d9affff99b15928"),r("fbb4aeb3cde3ccebc5decbe4fed9a6ffffcce5d8bdfddaecf2f2f2")),s=r("b3e2cdfdcdaccbd5e8f4cae4e6f5c9fff2aef1e2cccccccc"),c=r("e41a1c377eb84daf4a984ea3ff7f00ffff33a65628f781bf999999"),l=r("66c2a5fc8d628da0cbe78ac3a6d854ffd92fe5c494b3b3b3"),d=r("8dd3c7ffffb3bebadafb807280b1d3fdb462b3de69fccde5d9d9d9bc80bdccebc5ffed6f"),p=n(52),f=function(t){return Object(p.d)(t[t.length-1])},h=f(new Array(3).concat("d8b365f5f5f55ab4ac","a6611adfc27d80cdc1018571","a6611adfc27df5f5f580cdc1018571","8c510ad8b365f6e8c3c7eae55ab4ac01665e","8c510ad8b365f6e8c3f5f5f5c7eae55ab4ac01665e","8c510abf812ddfc27df6e8c3c7eae580cdc135978f01665e","8c510abf812ddfc27df6e8c3f5f5f5c7eae580cdc135978f01665e","5430058c510abf812ddfc27df6e8c3c7eae580cdc135978f01665e003c30","5430058c510abf812ddfc27df6e8c3f5f5f5c7eae580cdc135978f01665e003c30").map(r)),g=f(new Array(3).concat("af8dc3f7f7f77fbf7b","7b3294c2a5cfa6dba0008837","7b3294c2a5cff7f7f7a6dba0008837","762a83af8dc3e7d4e8d9f0d37fbf7b1b7837","762a83af8dc3e7d4e8f7f7f7d9f0d37fbf7b1b7837","762a839970abc2a5cfe7d4e8d9f0d3a6dba05aae611b7837","762a839970abc2a5cfe7d4e8f7f7f7d9f0d3a6dba05aae611b7837","40004b762a839970abc2a5cfe7d4e8d9f0d3a6dba05aae611b783700441b","40004b762a839970abc2a5cfe7d4e8f7f7f7d9f0d3a6dba05aae611b783700441b").map(r)),m=f(new Array(3).concat("e9a3c9f7f7f7a1d76a","d01c8bf1b6dab8e1864dac26","d01c8bf1b6daf7f7f7b8e1864dac26","c51b7de9a3c9fde0efe6f5d0a1d76a4d9221","c51b7de9a3c9fde0eff7f7f7e6f5d0a1d76a4d9221","c51b7dde77aef1b6dafde0efe6f5d0b8e1867fbc414d9221","c51b7dde77aef1b6dafde0eff7f7f7e6f5d0b8e1867fbc414d9221","8e0152c51b7dde77aef1b6dafde0efe6f5d0b8e1867fbc414d9221276419","8e0152c51b7dde77aef1b6dafde0eff7f7f7e6f5d0b8e1867fbc414d9221276419").map(r)),b=f(new Array(3).concat("998ec3f7f7f7f1a340","5e3c99b2abd2fdb863e66101","5e3c99b2abd2f7f7f7fdb863e66101","542788998ec3d8daebfee0b6f1a340b35806","542788998ec3d8daebf7f7f7fee0b6f1a340b35806","5427888073acb2abd2d8daebfee0b6fdb863e08214b35806","5427888073acb2abd2d8daebf7f7f7fee0b6fdb863e08214b35806","2d004b5427888073acb2abd2d8daebfee0b6fdb863e08214b358067f3b08","2d004b5427888073acb2abd2d8daebf7f7f7fee0b6fdb863e08214b358067f3b08").map(r)),y=f(new Array(3).concat("ef8a62f7f7f767a9cf","ca0020f4a58292c5de0571b0","ca0020f4a582f7f7f792c5de0571b0","b2182bef8a62fddbc7d1e5f067a9cf2166ac","b2182bef8a62fddbc7f7f7f7d1e5f067a9cf2166ac","b2182bd6604df4a582fddbc7d1e5f092c5de4393c32166ac","b2182bd6604df4a582fddbc7f7f7f7d1e5f092c5de4393c32166ac","67001fb2182bd6604df4a582fddbc7d1e5f092c5de4393c32166ac053061","67001fb2182bd6604df4a582fddbc7f7f7f7d1e5f092c5de4393c32166ac053061").map(r)),v=f(new Array(3).concat("ef8a62ffffff999999","ca0020f4a582bababa404040","ca0020f4a582ffffffbababa404040","b2182bef8a62fddbc7e0e0e09999994d4d4d","b2182bef8a62fddbc7ffffffe0e0e09999994d4d4d","b2182bd6604df4a582fddbc7e0e0e0bababa8787874d4d4d","b2182bd6604df4a582fddbc7ffffffe0e0e0bababa8787874d4d4d","67001fb2182bd6604df4a582fddbc7e0e0e0bababa8787874d4d4d1a1a1a","67001fb2182bd6604df4a582fddbc7ffffffe0e0e0bababa8787874d4d4d1a1a1a").map(r)),E=f(new Array(3).concat("fc8d59ffffbf91bfdb","d7191cfdae61abd9e92c7bb6","d7191cfdae61ffffbfabd9e92c7bb6","d73027fc8d59fee090e0f3f891bfdb4575b4","d73027fc8d59fee090ffffbfe0f3f891bfdb4575b4","d73027f46d43fdae61fee090e0f3f8abd9e974add14575b4","d73027f46d43fdae61fee090ffffbfe0f3f8abd9e974add14575b4","a50026d73027f46d43fdae61fee090e0f3f8abd9e974add14575b4313695","a50026d73027f46d43fdae61fee090ffffbfe0f3f8abd9e974add14575b4313695").map(r)),C=f(new Array(3).concat("fc8d59ffffbf91cf60","d7191cfdae61a6d96a1a9641","d7191cfdae61ffffbfa6d96a1a9641","d73027fc8d59fee08bd9ef8b91cf601a9850","d73027fc8d59fee08bffffbfd9ef8b91cf601a9850","d73027f46d43fdae61fee08bd9ef8ba6d96a66bd631a9850","d73027f46d43fdae61fee08bffffbfd9ef8ba6d96a66bd631a9850","a50026d73027f46d43fdae61fee08bd9ef8ba6d96a66bd631a9850006837","a50026d73027f46d43fdae61fee08bffffbfd9ef8ba6d96a66bd631a9850006837").map(r)),w=f(new Array(3).concat("fc8d59ffffbf99d594","d7191cfdae61abdda42b83ba","d7191cfdae61ffffbfabdda42b83ba","d53e4ffc8d59fee08be6f59899d5943288bd","d53e4ffc8d59fee08bffffbfe6f59899d5943288bd","d53e4ff46d43fdae61fee08be6f598abdda466c2a53288bd","d53e4ff46d43fdae61fee08bffffbfe6f598abdda466c2a53288bd","9e0142d53e4ff46d43fdae61fee08be6f598abdda466c2a53288bd5e4fa2","9e0142d53e4ff46d43fdae61fee08bffffbfe6f598abdda466c2a53288bd5e4fa2").map(r)),A=f(new Array(3).concat("e5f5f999d8c92ca25f","edf8fbb2e2e266c2a4238b45","edf8fbb2e2e266c2a42ca25f006d2c","edf8fbccece699d8c966c2a42ca25f006d2c","edf8fbccece699d8c966c2a441ae76238b45005824","f7fcfde5f5f9ccece699d8c966c2a441ae76238b45005824","f7fcfde5f5f9ccece699d8c966c2a441ae76238b45006d2c00441b").map(r)),x=f(new Array(3).concat("e0ecf49ebcda8856a7","edf8fbb3cde38c96c688419d","edf8fbb3cde38c96c68856a7810f7c","edf8fbbfd3e69ebcda8c96c68856a7810f7c","edf8fbbfd3e69ebcda8c96c68c6bb188419d6e016b","f7fcfde0ecf4bfd3e69ebcda8c96c68c6bb188419d6e016b","f7fcfde0ecf4bfd3e69ebcda8c96c68c6bb188419d810f7c4d004b").map(r)),N=f(new Array(3).concat("e0f3dba8ddb543a2ca","f0f9e8bae4bc7bccc42b8cbe","f0f9e8bae4bc7bccc443a2ca0868ac","f0f9e8ccebc5a8ddb57bccc443a2ca0868ac","f0f9e8ccebc5a8ddb57bccc44eb3d32b8cbe08589e","f7fcf0e0f3dbccebc5a8ddb57bccc44eb3d32b8cbe08589e","f7fcf0e0f3dbccebc5a8ddb57bccc44eb3d32b8cbe0868ac084081").map(r)),T=f(new Array(3).concat("fee8c8fdbb84e34a33","fef0d9fdcc8afc8d59d7301f","fef0d9fdcc8afc8d59e34a33b30000","fef0d9fdd49efdbb84fc8d59e34a33b30000","fef0d9fdd49efdbb84fc8d59ef6548d7301f990000","fff7ecfee8c8fdd49efdbb84fc8d59ef6548d7301f990000","fff7ecfee8c8fdd49efdbb84fc8d59ef6548d7301fb300007f0000").map(r)),O=f(new Array(3).concat("ece2f0a6bddb1c9099","f6eff7bdc9e167a9cf02818a","f6eff7bdc9e167a9cf1c9099016c59","f6eff7d0d1e6a6bddb67a9cf1c9099016c59","f6eff7d0d1e6a6bddb67a9cf3690c002818a016450","fff7fbece2f0d0d1e6a6bddb67a9cf3690c002818a016450","fff7fbece2f0d0d1e6a6bddb67a9cf3690c002818a016c59014636").map(r)),R=f(new Array(3).concat("ece7f2a6bddb2b8cbe","f1eef6bdc9e174a9cf0570b0","f1eef6bdc9e174a9cf2b8cbe045a8d","f1eef6d0d1e6a6bddb74a9cf2b8cbe045a8d","f1eef6d0d1e6a6bddb74a9cf3690c00570b0034e7b","fff7fbece7f2d0d1e6a6bddb74a9cf3690c00570b0034e7b","fff7fbece7f2d0d1e6a6bddb74a9cf3690c00570b0045a8d023858").map(r)),M=f(new Array(3).concat("e7e1efc994c7dd1c77","f1eef6d7b5d8df65b0ce1256","f1eef6d7b5d8df65b0dd1c77980043","f1eef6d4b9dac994c7df65b0dd1c77980043","f1eef6d4b9dac994c7df65b0e7298ace125691003f","f7f4f9e7e1efd4b9dac994c7df65b0e7298ace125691003f","f7f4f9e7e1efd4b9dac994c7df65b0e7298ace125698004367001f").map(r)),D=f(new Array(3).concat("fde0ddfa9fb5c51b8a","feebe2fbb4b9f768a1ae017e","feebe2fbb4b9f768a1c51b8a7a0177","feebe2fcc5c0fa9fb5f768a1c51b8a7a0177","feebe2fcc5c0fa9fb5f768a1dd3497ae017e7a0177","fff7f3fde0ddfcc5c0fa9fb5f768a1dd3497ae017e7a0177","fff7f3fde0ddfcc5c0fa9fb5f768a1dd3497ae017e7a017749006a").map(r)),_=f(new Array(3).concat("edf8b17fcdbb2c7fb8","ffffcca1dab441b6c4225ea8","ffffcca1dab441b6c42c7fb8253494","ffffccc7e9b47fcdbb41b6c42c7fb8253494","ffffccc7e9b47fcdbb41b6c41d91c0225ea80c2c84","ffffd9edf8b1c7e9b47fcdbb41b6c41d91c0225ea80c2c84","ffffd9edf8b1c7e9b47fcdbb41b6c41d91c0225ea8253494081d58").map(r)),L=f(new Array(3).concat("f7fcb9addd8e31a354","ffffccc2e69978c679238443","ffffccc2e69978c67931a354006837","ffffccd9f0a3addd8e78c67931a354006837","ffffccd9f0a3addd8e78c67941ab5d238443005a32","ffffe5f7fcb9d9f0a3addd8e78c67941ab5d238443005a32","ffffe5f7fcb9d9f0a3addd8e78c67941ab5d238443006837004529").map(r)),S=f(new Array(3).concat("fff7bcfec44fd95f0e","ffffd4fed98efe9929cc4c02","ffffd4fed98efe9929d95f0e993404","ffffd4fee391fec44ffe9929d95f0e993404","ffffd4fee391fec44ffe9929ec7014cc4c028c2d04","ffffe5fff7bcfee391fec44ffe9929ec7014cc4c028c2d04","ffffe5fff7bcfee391fec44ffe9929ec7014cc4c02993404662506").map(r)),I=f(new Array(3).concat("ffeda0feb24cf03b20","ffffb2fecc5cfd8d3ce31a1c","ffffb2fecc5cfd8d3cf03b20bd0026","ffffb2fed976feb24cfd8d3cf03b20bd0026","ffffb2fed976feb24cfd8d3cfc4e2ae31a1cb10026","ffffccffeda0fed976feb24cfd8d3cfc4e2ae31a1cb10026","ffffccffeda0fed976feb24cfd8d3cfc4e2ae31a1cbd0026800026").map(r)),j=f(new Array(3).concat("deebf79ecae13182bd","eff3ffbdd7e76baed62171b5","eff3ffbdd7e76baed63182bd08519c","eff3ffc6dbef9ecae16baed63182bd08519c","eff3ffc6dbef9ecae16baed64292c62171b5084594","f7fbffdeebf7c6dbef9ecae16baed64292c62171b5084594","f7fbffdeebf7c6dbef9ecae16baed64292c62171b508519c08306b").map(r)),k=f(new Array(3).concat("e5f5e0a1d99b31a354","edf8e9bae4b374c476238b45","edf8e9bae4b374c47631a354006d2c","edf8e9c7e9c0a1d99b74c47631a354006d2c","edf8e9c7e9c0a1d99b74c47641ab5d238b45005a32","f7fcf5e5f5e0c7e9c0a1d99b74c47641ab5d238b45005a32","f7fcf5e5f5e0c7e9c0a1d99b74c47641ab5d238b45006d2c00441b").map(r)),V=f(new Array(3).concat("f0f0f0bdbdbd636363","f7f7f7cccccc969696525252","f7f7f7cccccc969696636363252525","f7f7f7d9d9d9bdbdbd969696636363252525","f7f7f7d9d9d9bdbdbd969696737373525252252525","fffffff0f0f0d9d9d9bdbdbd969696737373525252252525","fffffff0f0f0d9d9d9bdbdbd969696737373525252252525000000").map(r)),H=f(new Array(3).concat("efedf5bcbddc756bb1","f2f0f7cbc9e29e9ac86a51a3","f2f0f7cbc9e29e9ac8756bb154278f","f2f0f7dadaebbcbddc9e9ac8756bb154278f","f2f0f7dadaebbcbddc9e9ac8807dba6a51a34a1486","fcfbfdefedf5dadaebbcbddc9e9ac8807dba6a51a34a1486","fcfbfdefedf5dadaebbcbddc9e9ac8807dba6a51a354278f3f007d").map(r)),P=f(new Array(3).concat("fee0d2fc9272de2d26","fee5d9fcae91fb6a4acb181d","fee5d9fcae91fb6a4ade2d26a50f15","fee5d9fcbba1fc9272fb6a4ade2d26a50f15","fee5d9fcbba1fc9272fb6a4aef3b2ccb181d99000d","fff5f0fee0d2fcbba1fc9272fb6a4aef3b2ccb181d99000d","fff5f0fee0d2fcbba1fc9272fb6a4aef3b2ccb181da50f1567000d").map(r)),G=f(new Array(3).concat("fee6cefdae6be6550d","feeddefdbe85fd8d3cd94701","feeddefdbe85fd8d3ce6550da63603","feeddefdd0a2fdae6bfd8d3ce6550da63603","feeddefdd0a2fdae6bfd8d3cf16913d948018c2d04","fff5ebfee6cefdd0a2fdae6bfd8d3cf16913d948018c2d04","fff5ebfee6cefdd0a2fdae6bfd8d3cf16913d94801a636037f2704").map(r)),F=n(24),U=Object(p.b)(Object(F.b)(300,.5,0),Object(F.b)(-240,.5,1)),B=Object(p.b)(Object(F.b)(-100,.75,.35),Object(F.b)(80,1.5,.8)),z=Object(p.b)(Object(F.b)(260,.75,.35),Object(F.b)(80,1.5,.8)),Y=Object(F.b)(),W=function(t){(t<0||t>1)&&(t-=Math.floor(t));var e=Math.abs(t-.5);return Y.h=360*t-100,Y.s=1.5-1.5*e,Y.l=.8-.9*e,Y+""};function q(t){var e=t.length;return function(n){return t[Math.max(0,Math.min(e-1,Math.floor(n*e)))]}}Object(F.f)(),Math.PI,Math.PI;var K=q(r("44015444025645045745055946075a46085c460a5d460b5e470d60470e6147106347116447136548146748166848176948186a481a6c481b6d481c6e481d6f481f70482071482173482374482475482576482677482878482979472a7a472c7a472d7b472e7c472f7d46307e46327e46337f463480453581453781453882443983443a83443b84433d84433e85423f854240864241864142874144874045884046883f47883f48893e49893e4a893e4c8a3d4d8a3d4e8a3c4f8a3c508b3b518b3b528b3a538b3a548c39558c39568c38588c38598c375a8c375b8d365c8d365d8d355e8d355f8d34608d34618d33628d33638d32648e32658e31668e31678e31688e30698e306a8e2f6b8e2f6c8e2e6d8e2e6e8e2e6f8e2d708e2d718e2c718e2c728e2c738e2b748e2b758e2a768e2a778e2a788e29798e297a8e297b8e287c8e287d8e277e8e277f8e27808e26818e26828e26828e25838e25848e25858e24868e24878e23888e23898e238a8d228b8d228c8d228d8d218e8d218f8d21908d21918c20928c20928c20938c1f948c1f958b1f968b1f978b1f988b1f998a1f9a8a1e9b8a1e9c891e9d891f9e891f9f881fa0881fa1881fa1871fa28720a38620a48621a58521a68522a78522a88423a98324aa8325ab8225ac8226ad8127ad8128ae8029af7f2ab07f2cb17e2db27d2eb37c2fb47c31b57b32b67a34b67935b77937b87838b9773aba763bbb753dbc743fbc7340bd7242be7144bf7046c06f48c16e4ac16d4cc26c4ec36b50c46a52c56954c56856c66758c7655ac8645cc8635ec96260ca6063cb5f65cb5e67cc5c69cd5b6ccd5a6ece5870cf5773d05675d05477d1537ad1517cd2507fd34e81d34d84d44b86d54989d5488bd6468ed64590d74393d74195d84098d83e9bd93c9dd93ba0da39a2da37a5db36a8db34aadc32addc30b0dd2fb2dd2db5de2bb8de29bade28bddf26c0df25c2df23c5e021c8e020cae11fcde11dd0e11cd2e21bd5e21ad8e219dae319dde318dfe318e2e418e5e419e7e419eae51aece51befe51cf1e51df4e61ef6e620f8e621fbe723fde725")),J=q(r("00000401000501010601010802010902020b02020d03030f03031204041405041606051806051a07061c08071e0907200a08220b09240c09260d0a290e0b2b100b2d110c2f120d31130d34140e36150e38160f3b180f3d19103f1a10421c10441d11471e114920114b21114e22115024125325125527125829115a2a115c2c115f2d11612f116331116533106734106936106b38106c390f6e3b0f703d0f713f0f72400f74420f75440f764510774710784910784a10794c117a4e117b4f127b51127c52137c54137d56147d57157e59157e5a167e5c167f5d177f5f187f601880621980641a80651a80671b80681c816a1c816b1d816d1d816e1e81701f81721f817320817521817621817822817922827b23827c23827e24828025828125818326818426818627818827818928818b29818c29818e2a81902a81912b81932b80942c80962c80982d80992d809b2e7f9c2e7f9e2f7fa02f7fa1307ea3307ea5317ea6317da8327daa337dab337cad347cae347bb0357bb2357bb3367ab5367ab73779b83779ba3878bc3978bd3977bf3a77c03a76c23b75c43c75c53c74c73d73c83e73ca3e72cc3f71cd4071cf4070d0416fd2426fd3436ed5446dd6456cd8456cd9466bdb476adc4869de4968df4a68e04c67e24d66e34e65e44f64e55064e75263e85362e95462ea5661eb5760ec5860ed5a5fee5b5eef5d5ef05f5ef1605df2625df2645cf3655cf4675cf4695cf56b5cf66c5cf66e5cf7705cf7725cf8745cf8765cf9785df9795df97b5dfa7d5efa7f5efa815ffb835ffb8560fb8761fc8961fc8a62fc8c63fc8e64fc9065fd9266fd9467fd9668fd9869fd9a6afd9b6bfe9d6cfe9f6dfea16efea36ffea571fea772fea973feaa74feac76feae77feb078feb27afeb47bfeb67cfeb77efeb97ffebb81febd82febf84fec185fec287fec488fec68afec88cfeca8dfecc8ffecd90fecf92fed194fed395fed597fed799fed89afdda9cfddc9efddea0fde0a1fde2a3fde3a5fde5a7fde7a9fde9aafdebacfcecaefceeb0fcf0b2fcf2b4fcf4b6fcf6b8fcf7b9fcf9bbfcfbbdfcfdbf")),Z=q(r("00000401000501010601010802010a02020c02020e03021004031204031405041706041907051b08051d09061f0a07220b07240c08260d08290e092b10092d110a30120a32140b34150b37160b39180c3c190c3e1b0c411c0c431e0c451f0c48210c4a230c4c240c4f260c51280b53290b552b0b572d0b592f0a5b310a5c320a5e340a5f3609613809623909633b09643d09653e0966400a67420a68440a68450a69470b6a490b6a4a0c6b4c0c6b4d0d6c4f0d6c510e6c520e6d540f6d550f6d57106e59106e5a116e5c126e5d126e5f136e61136e62146e64156e65156e67166e69166e6a176e6c186e6d186e6f196e71196e721a6e741a6e751b6e771c6d781c6d7a1d6d7c1d6d7d1e6d7f1e6c801f6c82206c84206b85216b87216b88226a8a226a8c23698d23698f24699025689225689326679526679727669827669a28659b29649d29649f2a63a02a63a22b62a32c61a52c60a62d60a82e5fa92e5eab2f5ead305dae305cb0315bb1325ab3325ab43359b63458b73557b93556ba3655bc3754bd3853bf3952c03a51c13a50c33b4fc43c4ec63d4dc73e4cc83f4bca404acb4149cc4248ce4347cf4446d04545d24644d34743d44842d54a41d74b3fd84c3ed94d3dda4e3cdb503bdd513ade5238df5337e05536e15635e25734e35933e45a31e55c30e65d2fe75e2ee8602de9612bea632aeb6429eb6628ec6726ed6925ee6a24ef6c23ef6e21f06f20f1711ff1731df2741cf3761bf37819f47918f57b17f57d15f67e14f68013f78212f78410f8850ff8870ef8890cf98b0bf98c0af98e09fa9008fa9207fa9407fb9606fb9706fb9906fb9b06fb9d07fc9f07fca108fca309fca50afca60cfca80dfcaa0ffcac11fcae12fcb014fcb216fcb418fbb61afbb81dfbba1ffbbc21fbbe23fac026fac228fac42afac62df9c72ff9c932f9cb35f8cd37f8cf3af7d13df7d340f6d543f6d746f5d949f5db4cf4dd4ff4df53f4e156f3e35af3e55df2e661f2e865f2ea69f1ec6df1ed71f1ef75f1f179f2f27df2f482f3f586f3f68af4f88ef5f992f6fa96f8fb9af9fc9dfafda1fcffa4")),Q=q(r("0d088710078813078916078a19068c1b068d1d068e20068f2206902406912605912805922a05932c05942e05952f059631059733059735049837049938049a3a049a3c049b3e049c3f049c41049d43039e44039e46039f48039f4903a04b03a14c02a14e02a25002a25102a35302a35502a45601a45801a45901a55b01a55c01a65e01a66001a66100a76300a76400a76600a76700a86900a86a00a86c00a86e00a86f00a87100a87201a87401a87501a87701a87801a87a02a87b02a87d03a87e03a88004a88104a78305a78405a78606a68707a68808a68a09a58b0aa58d0ba58e0ca48f0da4910ea3920fa39410a29511a19613a19814a099159f9a169f9c179e9d189d9e199da01a9ca11b9ba21d9aa31e9aa51f99a62098a72197a82296aa2395ab2494ac2694ad2793ae2892b02991b12a90b22b8fb32c8eb42e8db52f8cb6308bb7318ab83289ba3388bb3488bc3587bd3786be3885bf3984c03a83c13b82c23c81c33d80c43e7fc5407ec6417dc7427cc8437bc9447aca457acb4679cc4778cc4977cd4a76ce4b75cf4c74d04d73d14e72d24f71d35171d45270d5536fd5546ed6556dd7566cd8576bd9586ada5a6ada5b69db5c68dc5d67dd5e66de5f65de6164df6263e06363e16462e26561e26660e3685fe4695ee56a5de56b5de66c5ce76e5be76f5ae87059e97158e97257ea7457eb7556eb7655ec7754ed7953ed7a52ee7b51ef7c51ef7e50f07f4ff0804ef1814df1834cf2844bf3854bf3874af48849f48948f58b47f58c46f68d45f68f44f79044f79143f79342f89441f89540f9973ff9983ef99a3efa9b3dfa9c3cfa9e3bfb9f3afba139fba238fca338fca537fca636fca835fca934fdab33fdac33fdae32fdaf31fdb130fdb22ffdb42ffdb52efeb72dfeb82cfeba2cfebb2bfebd2afebe2afec029fdc229fdc328fdc527fdc627fdc827fdca26fdcb26fccd25fcce25fcd025fcd225fbd324fbd524fbd724fad824fada24f9dc24f9dd25f8df25f8e125f7e225f7e425f6e626f6e826f5e926f5eb27f4ed27f3ee27f3f027f2f227f1f426f1f525f0f724f0f921"));n.d(e,"K",function(){return i}),n.d(e,"J",function(){return o}),n.d(e,"L",function(){return a}),n.d(e,"M",function(){return u}),n.d(e,"N",function(){return s}),n.d(e,"O",function(){return c}),n.d(e,"P",function(){return l}),n.d(e,"Q",function(){return d}),n.d(e,"b",function(){return h}),n.d(e,"n",function(){return g}),n.d(e,"o",function(){return m}),n.d(e,"s",function(){return b}),n.d(e,"w",function(){return y}),n.d(e,"x",function(){return v}),n.d(e,"z",function(){return E}),n.d(e,"A",function(){return C}),n.d(e,"C",function(){return w}),n.d(e,"c",function(){return A}),n.d(e,"d",function(){return x}),n.d(e,"g",function(){return N}),n.d(e,"l",function(){return T}),n.d(e,"r",function(){return O}),n.d(e,"q",function(){return R}),n.d(e,"t",function(){return M}),n.d(e,"y",function(){return D}),n.d(e,"G",function(){return _}),n.d(e,"F",function(){return L}),n.d(e,"H",function(){return S}),n.d(e,"I",function(){return I}),n.d(e,"a",function(){return j}),n.d(e,"h",function(){return k}),n.d(e,"i",function(){return V}),n.d(e,"u",function(){return H}),n.d(e,"B",function(){return P}),n.d(e,"m",function(){return G}),n.d(e,"f",function(){return U}),n.d(e,"v",function(){return W}),n.d(e,"E",function(){return B}),n.d(e,"e",function(){return z}),n.d(e,"D",function(){return K}),n.d(e,"k",function(){return J}),n.d(e,"j",function(){return Z}),n.d(e,"p",function(){return Q})},function(t,e,n){"use strict";var r=new Date,i=new Date;function o(t,e,n,a){function u(e){return t(e=new Date(+e)),e}return u.floor=u,u.ceil=function(n){return t(n=new Date(n-1)),e(n,1),t(n),n},u.round=function(t){var e=u(t),n=u.ceil(t);return t-e<n-t?e:n},u.offset=function(t,n){return e(t=new Date(+t),null==n?1:Math.floor(n)),t},u.range=function(n,r,i){var o,a=[];if(n=u.ceil(n),i=null==i?1:Math.floor(i),!(n<r&&i>0))return a;do{a.push(o=new Date(+n)),e(n,i),t(n)}while(o<n&&n<r);return a},u.filter=function(n){return o(function(e){if(e>=e)for(;t(e),!n(e);)e.setTime(e-1)},function(t,r){if(t>=t)if(r<0)for(;++r<=0;)for(;e(t,-1),!n(t););else for(;--r>=0;)for(;e(t,1),!n(t););})},n&&(u.count=function(e,o){return r.setTime(+e),i.setTime(+o),t(r),t(i),Math.floor(n(r,i))},u.every=function(t){return t=Math.floor(t),isFinite(t)&&t>0?t>1?u.filter(a?function(e){return a(e)%t==0}:function(e){return u.count(0,e)%t==0}):u:null}),u}var a=o(function(){},function(t,e){t.setTime(+t+e)},function(t,e){return e-t});a.every=function(t){return t=Math.floor(t),isFinite(t)&&t>0?t>1?o(function(e){e.setTime(Math.floor(e/t)*t)},function(e,n){e.setTime(+e+n*t)},function(e,n){return(n-e)/t}):a:null};var u=a,s=(a.range,6e4),c=6048e5,l=o(function(t){t.setTime(1e3*Math.floor(t/1e3))},function(t,e){t.setTime(+t+1e3*e)},function(t,e){return(e-t)/1e3},function(t){return t.getUTCSeconds()}),d=l,p=(l.range,o(function(t){t.setTime(Math.floor(t/s)*s)},function(t,e){t.setTime(+t+e*s)},function(t,e){return(e-t)/s},function(t){return t.getMinutes()})),f=p,h=(p.range,o(function(t){var e=t.getTimezoneOffset()*s%36e5;e<0&&(e+=36e5),t.setTime(36e5*Math.floor((+t-e)/36e5)+e)},function(t,e){t.setTime(+t+36e5*e)},function(t,e){return(e-t)/36e5},function(t){return t.getHours()})),g=h,m=(h.range,o(function(t){t.setHours(0,0,0,0)},function(t,e){t.setDate(t.getDate()+e)},function(t,e){return(e-t-(e.getTimezoneOffset()-t.getTimezoneOffset())*s)/864e5},function(t){return t.getDate()-1})),b=m;function y(t){return o(function(e){e.setDate(e.getDate()-(e.getDay()+7-t)%7),e.setHours(0,0,0,0)},function(t,e){t.setDate(t.getDate()+7*e)},function(t,e){return(e-t-(e.getTimezoneOffset()-t.getTimezoneOffset())*s)/c})}m.range;var v=y(0),E=y(1),C=y(2),w=y(3),A=y(4),x=y(5),N=y(6),T=(v.range,E.range,C.range,w.range,A.range,x.range,N.range,o(function(t){t.setDate(1),t.setHours(0,0,0,0)},function(t,e){t.setMonth(t.getMonth()+e)},function(t,e){return e.getMonth()-t.getMonth()+12*(e.getFullYear()-t.getFullYear())},function(t){return t.getMonth()})),O=T,R=(T.range,o(function(t){t.setMonth(0,1),t.setHours(0,0,0,0)},function(t,e){t.setFullYear(t.getFullYear()+e)},function(t,e){return e.getFullYear()-t.getFullYear()},function(t){return t.getFullYear()}));R.every=function(t){return isFinite(t=Math.floor(t))&&t>0?o(function(e){e.setFullYear(Math.floor(e.getFullYear()/t)*t),e.setMonth(0,1),e.setHours(0,0,0,0)},function(e,n){e.setFullYear(e.getFullYear()+n*t)}):null};var M=R,D=(R.range,o(function(t){t.setUTCSeconds(0,0)},function(t,e){t.setTime(+t+e*s)},function(t,e){return(e-t)/s},function(t){return t.getUTCMinutes()})),_=D,L=(D.range,o(function(t){t.setUTCMinutes(0,0,0)},function(t,e){t.setTime(+t+36e5*e)},function(t,e){return(e-t)/36e5},function(t){return t.getUTCHours()})),S=L,I=(L.range,o(function(t){t.setUTCHours(0,0,0,0)},function(t,e){t.setUTCDate(t.getUTCDate()+e)},function(t,e){return(e-t)/864e5},function(t){return t.getUTCDate()-1})),j=I;function k(t){return o(function(e){e.setUTCDate(e.getUTCDate()-(e.getUTCDay()+7-t)%7),e.setUTCHours(0,0,0,0)},function(t,e){t.setUTCDate(t.getUTCDate()+7*e)},function(t,e){return(e-t)/c})}I.range;var V=k(0),H=k(1),P=k(2),G=k(3),F=k(4),U=k(5),B=k(6),z=(V.range,H.range,P.range,G.range,F.range,U.range,B.range,o(function(t){t.setUTCDate(1),t.setUTCHours(0,0,0,0)},function(t,e){t.setUTCMonth(t.getUTCMonth()+e)},function(t,e){return e.getUTCMonth()-t.getUTCMonth()+12*(e.getUTCFullYear()-t.getUTCFullYear())},function(t){return t.getUTCMonth()})),Y=z,W=(z.range,o(function(t){t.setUTCMonth(0,1),t.setUTCHours(0,0,0,0)},function(t,e){t.setUTCFullYear(t.getUTCFullYear()+e)},function(t,e){return e.getUTCFullYear()-t.getUTCFullYear()},function(t){return t.getUTCFullYear()}));W.every=function(t){return isFinite(t=Math.floor(t))&&t>0?o(function(e){e.setUTCFullYear(Math.floor(e.getUTCFullYear()/t)*t),e.setUTCMonth(0,1),e.setUTCHours(0,0,0,0)},function(e,n){e.setUTCFullYear(e.getUTCFullYear()+n*t)}):null};var q=W;W.range,n.d(e,"c",function(){return u}),n.d(e,"n",function(){return u}),n.d(e,"g",function(){return d}),n.d(e,"r",function(){return d}),n.d(e,"d",function(){return f}),n.d(e,"b",function(){return g}),n.d(e,"a",function(){return b}),n.d(e,"j",function(){return v}),n.d(e,"h",function(){return v}),n.d(e,"e",function(){return E}),n.d(e,"i",function(){return A}),n.d(e,"f",function(){return O}),n.d(e,"k",function(){return M}),n.d(e,"o",function(){return _}),n.d(e,"m",function(){return S}),n.d(e,"l",function(){return j}),n.d(e,"u",function(){return V}),n.d(e,"s",function(){return V}),n.d(e,"p",function(){return H}),n.d(e,"t",function(){return F}),n.d(e,"q",function(){return Y}),n.d(e,"v",function(){return q})},function(t,e,n){
/*! lineupengine - v1.1.2 - 2018
* https://sgratzl.github.io/lineupengine/
* Copyright (c) 2018 Samuel Gratzl; Licensed MIT*/
window,t.exports=function(t){var e={};function n(r){if(e[r])return e[r].exports;var i=e[r]={i:r,l:!1,exports:{}};return t[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=t,n.c=e,n.d=function(t,e,r){n.o(t,e)||Object.defineProperty(t,e,{enumerable:!0,get:r})},n.r=function(t){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0})},n.t=function(t,e){if(1&e&&(t=n(t)),8&e)return t;if(4&e&&"object"==typeof t&&t&&t.__esModule)return t;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:t}),2&e&&"string"!=typeof t)for(var i in t)n.d(r,i,function(e){return t[e]}.bind(null,i));return r},n.n=function(t){var e=t&&t.__esModule?function(){return t.default}:function(){return t};return n.d(e,"a",e),e},n.o=function(t,e){return Object.prototype.hasOwnProperty.call(t,e)},n.p="",n(n.s=2)}([,function(t,e){},function(t,e,n){"use strict";n.r(e);
/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
var r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)};function i(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)}var o=Symbol("aborted");function a(t){return{then:function(e){var n,r=null,i=function(){return null===r},a=new Promise(function(t){return r=t}),u=t.then(function(t){return i()?o:Promise.resolve(e(t)).then(function(t){return i()?o:t})}),s=Promise.race([a,u]);return(n={abort:function(){null!==r&&(r(o),r=null)},then:s.then.bind(s),catch:s.catch.bind(s)})[Symbol.toStringTag]=s[Symbol.toStringTag],n}}}function u(t){return void 0!==t&&null!==t&&t&&"function"==typeof t.then&&"function"==typeof t.abort}var s,c=function(){function t(t,e){var n=this;this.context=t,this.key=e,this.cache=[],this.lastFilled=0,this.key2index=new Map,this.context.exceptions.forEach(function(t){n.cache[t.index]=t.y,n.key2index.set(e(t.index),t.index)})}return t.prototype.findValidStart=function(t){for(var e=t-1;e>=0;--e)if(void 0!==this.cache[e])return e;return-1},t.prototype.posByKey=function(t){if(this.key2index.has(t)){var e=this.key2index.get(t);return{index:e,pos:this.pos(e)}}return this.fillCacheTillKey(t)},t.prototype.pos=function(t){if(0===this.context.exceptions.length)return t*this.context.defaultRowHeight;var e=this.cache[t];if(void 0!==e)return e;var n=this.findValidStart(t);return n<0?this.fillCache(0,t,0):this.fillCache(n+1,t,this.cache[n]+this.heightOf(n)),this.cache[t]},t.prototype.fillCache=function(t,e,n,r){if(e<=this.lastFilled){if(!r)return;for(var i=t;i<=e;++i)r(i,this.key(i),this.cache[i])}else{var o=n;for(i=t;i<=e;++i){this.cache[i]=o;var a=this.key(i);this.key2index.set(a,i),r&&r(i,a,o),o+=this.heightOf(i)}}},t.prototype.heightOf=function(t){var e=this.context.exceptionsLookup;return e.has(t)?e.get(t):this.context.defaultRowHeight},t.prototype.exceptionHeightOf=function(t,e){void 0===e&&(e=!1);var n=this.context.padding(t),r=this.context.exceptionsLookup;return r.has(t)?r.get(t)-n:e?this.context.defaultRowHeight-n:null},t.prototype.padding=function(t){return this.context.padding(t)},t.prototype.fillCacheTillKey=function(t){for(var e=0,n=this.lastFilled;n<this.context.numberOfRows;++n,++this.lastFilled){var r=this.cache[n];if(void 0===r){var i=this.key(n);if(this.cache[n]=e,this.key2index.set(i,n),i===t)return{index:n,pos:e};e+=this.heightOf(n)}else e=r+this.heightOf(n)}return{index:-1,pos:-1}},t.prototype.positions=function(t,e,n,r){this.fillCache(t,e,n,r)},t}();!function(t){t[t.UPDATE=0]="UPDATE",t[t.UPDATE_CREATE=1]="UPDATE_CREATE",t[t.UPDATE_REMOVE=2]="UPDATE_REMOVE",t[t.SHOW=3]="SHOW",t[t.HIDE=4]="HIDE"}(s||(s={}));var l=1;function d(t,e,n){var r=t.previous,i=t.mode,o=t.nodeY,a=t.current,u=null==r.height?e:r.height,c=null==a.height?n:a.height;return i===s.UPDATE&&Math.abs(r.y-o)<=l&&Math.abs(u-c)<=l}var p,f=[{delay:0,apply:function(t){var e=t.mode,n=t.previous,r=t.nodeY,i=t.current,o=t.node;o.dataset.animation=s[e].toLowerCase(),o.style.transform="translate(0, "+(n.y-r)+"px)",e===s.SHOW?o.style.height=null!==i.height?i.height+"px":null:o.style.height=n.height+"px",o.style.opacity=e===s.SHOW?"0":e===s.HIDE?"1":null}},{delay:10,apply:function(t){var e=t.mode,n=t.current,r=t.nodeY,i=t.node;i.style.transform=e===s.HIDE||e===s.UPDATE_REMOVE?"translate(0, "+(n.y-r)+"px)":null,e!==s.HIDE&&(i.style.height=null!==n.height?n.height+"px":null),i.style.opacity=e===s.SHOW?"1":e===s.HIDE?"0":null}},{delay:1100,apply:function(t){var e=t.node;delete e.dataset.animation,e.style.opacity=null,e.style.transform=null}}],h=function(){function t(t,e,n){this.index=t,this.y=e,this.height=n}return Object.defineProperty(t.prototype,"y2",{get:function(){return this.y+this.height},enumerable:!0,configurable:!0}),t}();function g(t,e,n){return void 0===n&&(n=0),{exceptions:[],exceptionsLookup:{keys:function(){return[].values()},get:function(){return e},has:function(){return!1},size:0},totalHeight:t*(e+=n),numberOfRows:t,defaultRowHeight:e,padding:function(){return n}}}function m(t,e,n){void 0===e&&(e=NaN),void 0===n&&(n=0);var r=new Map,i=[],o="function"==typeof n?n:function(){return n};isNaN(e)&&(e=function(t){var e=new Map;if(t.forEach(function(t){e.set(t,(e.get(t)||0)+1)}),0===e.size)return 20;var n=Array.from(e).sort(function(t,e){return t[1]!==e[1]?e[1]-t[1]:t[0]-e[0]}),r=n[0][0];return 0===r?n.length>1?n[1][0]:20:r}(t)),e+=o(-1);var a=-1,u=0,s=0,c=0;return t.forEach(function(t,n){if(t+=o(n),s+=t,c++,t!==e){r.set(n,t);var l=(n-a-1)*e;a=n;var d=u+l;u=d+t,i.push(new h(n,d,t))}}),{exceptionsLookup:r,exceptions:i,totalHeight:s,defaultRowHeight:e,numberOfRows:c,padding:o}}function b(t,e,n,r,i,o){void 0===n&&(n=2),void 0===r&&(r=10*e),void 0===i&&(i=.2),void 0===o&&(o=Date.now());var a=o,u=function(){var t=1e4*Math.sin(a++);return t-Math.floor(t)};return m({forEach:function(o){for(var a=0;a<t;++a)o(u()<i?n+Math.round(u()*(r-n)):e,a)}},e)}function y(t,e,n,r,i){if(0===i)return{first:0,last:-1,firstRowPos:0,endPos:0};if(1===i)return{first:0,last:0,firstRowPos:0,endPos:0===r.length?n:r[0].y2};var o=t,a=o+e;function u(t,e){return Math.min(i-1,e+Math.max(0,Math.floor(t/n)))}function s(t,e,r){void 0===r&&(r=!1);var s=a-t,c=u(o-t,e),l=u(s,e),d=t+(c-e)*n,p=t+(l+1-e)*n;return console.assert(!r||!(d>o||p<a&&l<i-1),"error",r,d,p,o,a,e,t),{first:c,last:l,firstRowPos:d,endPos:p}}var c=s(0,0,!0);if(0===r.length)return c;if(c.last<r[0].index)return c;if(c.last===r[0].index&&r[0].height>n)return Object.assign(c,{endPos:r[0].y2});var l=r[r.length-1];if(o>=l.y){var d=s(l.y2,l.index+1);return o<l.y2?Object.assign(d,{first:l.index,firstRowPos:l.y}):d}for(var p=[],f=r[0],h=0,g=r;h<g.length;h++){var m=g[h],b=m.y,y=m.y2;if(b>=a)break;y<=o?f=m:p.push(m)}if(0===p.length)return s(f.y2,f.index+1);var v=p[0],E=p[p.length-1],C=Math.max(0,v.index-Math.max(0,Math.ceil((v.y-o)/n))),w=E.index;a>=E.y2&&(w=u(a-E.y2,E.index+1));var A=v.y-(v.index-C)*n,x=E.y2+(w-E.index)*n;return console.assert(A<=o&&(x>=a||w===i-1),"error",A,x,o,a,v,E),{first:C,last:w,firstRowPos:A,endPos:x}}function v(t,e){var n=t.length,r=e.length;if(0===n)return{added:e,removed:[],common:0};if(0===r)return{added:[],removed:t,common:0};if(n===r)return{added:[],removed:[],common:n};var i=t.slice(Math.min(r,n));return{added:e.slice(Math.min(r,n)),removed:i,common:n-i.length}}function E(t,e,n){for(var r=0===t.length?0:t[t.length-1]+1,i=[],o=[],a=t.length-1;a>=0&&t[a]>=n;--a)o.push(t.pop());for(a=r;a<n;++a)e[a].frozen&&(i.push(a),t.push(a));return{target:t,added:i,removed:o}}!function(t){t[t.NONE=0]="NONE",t[t.ALL=1]="ALL",t[t.SOME=2]="SOME",t[t.SOME_BOTTOM=3]="SOME_BOTTOM",t[t.SOME_TOP=4]="SOME_TOP"}(p||(p={}));var C="animation",w=function(){function t(t){var e=this;this.node=t,this.handlers=new Map,this.animationWaiting=!1,this.immediateTimeout=-1,this.timersWaiting=new Set,this.prevs=new Map,t.addEventListener("scroll",function(){e.handle("sync"),e.handleAnimation(),e.handleImmediate(),e.handleTimeouts()},{passive:!0})}return t.prototype.has=function(t){return this.handlers.has(t)&&this.handlers.get(t).length>0},t.prototype.handle=function(t){var e=this.handlers.get(t)||[];if(e&&!(e.length<=0)){var n=this.asInfo();if(this.prevs.has(t)){var r=this.prevs.get(t);if(Math.abs(n.left-r.left)+Math.abs(n.top-r.top)<4)return}this.prevs.set(t,n);for(var i=0,o=e;i<o.length;i++)(0,o[i])(n)}},t.prototype.handleAnimation=function(){var t=this;!this.animationWaiting&&this.has("animation")&&(this.animationWaiting=!0,requestAnimationFrame(function(){t.animationWaiting&&(t.animationWaiting=!1,t.handle("animation"))}))},t.prototype.handleImmediate=function(){var t=this;this.immediateTimeout>=0||!this.has("immediate")||(this.immediateTimeout=self.setImmediate(function(){t.immediateTimeout<0||(t.immediateTimeout=-1,t.handle("immediate"))}))},t.prototype.handleTimeouts=function(){var t=this,e=Array.from(this.handlers.keys()).filter(function(e){return"number"==typeof e&&!t.timersWaiting.has(e)});if(0!==e.length)for(var n=function(e){r.timersWaiting.add(e),self.setTimeout(function(){t.timersWaiting.delete(e),t.handle(e)},e)},r=this,i=0,o=e;i<o.length;i++)n(o[i])},t.prototype.asInfo=function(){return{left:this.node.scrollLeft,top:this.node.scrollTop,width:this.node.clientWidth,height:this.node.clientHeight}},t.prototype.push=function(t,e){"immediate"===t&&"function"!=typeof self.setImmediate&&(t=0),this.handlers.has(t)?this.handlers.get(t).push(e):this.handlers.set(t,[e])},t.prototype.remove=function(t){return Array.from(this.handlers.values()).some(function(e){var n=e.indexOf(t);return n>=0&&e.splice(n,1),n>=0})},t}();function A(t,e,n){t.__le_scroller__||(t.__le_scroller__=new w(t));var r=t.__le_scroller__;return r.push(e,n),r.asInfo()}var x=function(){function t(t,e){void 0===e&&(e={});var n=this;this.body=t,this.pool=[],this.loadingPool=[],this.loading=new Map,this.visible={first:0,forcedFirst:0,last:-1,forcedLast:-1},this.visibleFirstRowPos=0,this.scrollListener=null,this.abortAnimation=function(){},this.options={async:C,minScrollDelta:10,mixins:[],scrollingHint:!1,batchSize:5},this.adapter=this.createAdapter(),Object.assign(this.options,e),this.mixins=this.options.mixins.map(function(t){return new t(n.adapter)}),this.fragment=t.ownerDocument.createDocumentFragment()}return t.prototype.addMixin=function(t,e){this.mixins.push(new t(this.adapter,e))},t.prototype.createAdapter=function(){var t=this,e={visible:this.visible,addAtBeginning:this.addAtBeginning.bind(this),addAtBottom:this.addAtBottom.bind(this),removeFromBeginning:this.removeFromBeginning.bind(this),removeFromBottom:this.removeFromBottom.bind(this),updateOffset:this.updateOffset.bind(this),scroller:this.bodyScroller};return Object.defineProperties(e,{visibleFirstRowPos:{get:function(){return t.visibleFirstRowPos},enumerable:!0},context:{get:function(){return t.context},enumerable:!0}}),e},Object.defineProperty(t.prototype,"bodyScroller",{get:function(){return this.body.parentElement},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"bodySizer",{get:function(){var t=this.bodyScroller,e=Array.from(t.children).find(function(t){return"footer"===t.tagName.toLowerCase()});if(e)return e;var n=t.ownerDocument.createElement("footer");return t.insertBefore(n,t.firstChild),n},enumerable:!0,configurable:!0}),t.prototype.init=function(){var t=this,e=this.bodyScroller,n=A(e,this.options.async,this.scrollListener=function(r){if(!(Math.abs(n.top-r.top)<t.options.minScrollDelta&&Math.abs(n.height-r.height)<t.options.minScrollDelta)){var i=r.top>n.top;n=r,t.onScrolledVertically(r.top,r.height,i),t.options.scrollingHint&&e.classList.remove("le-scrolling")}});this.options.scrollingHint&&A(e,"animation",function(){return e.classList.add("le-scrolling")}),this.recreate()},t.prototype.destroy=function(){!function(t,e){t.__le_scroller__&&t.__le_scroller__.remove(e)}(this.bodyScroller,this.scrollListener),this.body.remove()},t.cleanUp=function(t){t.style.height&&(t.style.height=null)},t.prototype.select=function(t){var e,n;return this.pool.length>0?(e=this.pool.pop(),n=this.updateRow(e,t)):this.loadingPool.length>0?((e=this.loadingPool.pop()).classList.remove("loading"),n=this.createRow(e,t)):(e=this.body.ownerDocument.createElement("div"),n=this.createRow(e,t)),e.dataset.index=String(t),{item:e,result:n}},t.prototype.selectProxy=function(){var t;return this.loadingPool.length>0?t=this.loadingPool.pop():(t=this.body.ownerDocument.createElement("div")).classList.add("loading"),t},t.prototype.recycle=function(e){t.cleanUp(e),this.loading.has(e)?this.loading.get(e).abort():this.pool.push(e)},t.prototype.proxy=function(e,n){var r=this;if(!u(n))return e;var i=n,a=e,s=this.selectProxy();return s.dataset.index=a.dataset.index,s.style.height=a.style.height,this.loading.set(s,i),i.then(function(e){e===o?(t.cleanUp(a),r.pool.push(a)):r.body.replaceChild(a,s),r.loading.delete(s),t.cleanUp(s),r.loadingPool.push(s)}),s},t.prototype.create=function(t){var e=this.select(t),n=e.item,r=e.result,i=this.context,o=i.exceptionsLookup,a=i.padding;return o.has(t)&&(n.style.height=o.get(t)-a(t)+"px"),this.proxy(n,r)},t.prototype.removeAll=function(){var t=this,e=Array.from(this.body.children);this.body.innerHTML="",e.forEach(function(e){t.recycle(e)})},t.prototype.update=function(){var t=this,e=this.visible.first,n=this.fragment,r=Array.from(this.body.children);this.body.innerHTML="",r.forEach(function(r,i){if(!t.loading.has(r)){var o=t.updateRow(r,i+e);n.appendChild(t.proxy(r,o))}}),this.body.appendChild(n)},t.prototype.forEachRow=function(t,e){var n=this;void 0===e&&(e=!1);var r=Array.from(this.body.children),i=this.fragment;e||(this.body.innerHTML=""),r.forEach(function(r,o){r.classList.contains("loading")||"update_remove"===r.dataset.animation||"hide"===r.dataset.animation||t(r,o+n.visible.first),e||i.appendChild(r)}),e||this.body.appendChild(i)},t.prototype.removeFromBeginning=function(t,e){return this.remove(t,e,!0)},t.prototype.removeFromBottom=function(t,e){return this.remove(t,e,!1)},t.prototype.remove=function(t,e,n){if(!(e<t))for(var r=t;r<=e;++r){var i=n?this.body.firstChild:this.body.lastChild;i.remove(),this.recycle(i)}},t.prototype.addAtBeginning=function(t,e){if(!(e<t))if(t!==e){for(var n=this.fragment,r=t;r<=e;++r)n.appendChild(this.create(r));this.body.insertBefore(n,this.body.firstChild)}else this.body.insertBefore(this.create(t),this.body.firstChild)},t.prototype.addAtBottom=function(t,e){if(!(e<t))if(t!==e){for(var n=this.fragment,r=t;r<=e;++r)n.appendChild(this.create(r));this.body.appendChild(n)}else this.body.appendChild(this.create(t))},t.prototype.updateOffset=function(t){this.visibleFirstRowPos=t,this.body.classList.toggle("odd",this.visible.first%2==1),this.updateSizer(t)},t.prototype.updateSizer=function(t){var e=this.context.totalHeight;this.body.style.transform="translate(0, "+t.toFixed(0)+"px)",this.bodySizer.style.transform="translate(0, "+Math.max(0,e-1).toFixed(0)+"px)"},t.prototype.recreate=function(t){return this.abortAnimation(),t?this.recreateAnimated(t):this.recreatePure()},t.prototype.recreatePure=function(){var t=this.context,e=this.bodyScroller;this.updateOffset(0),this.removeAll(),this.clearPool();var n=y(e.scrollTop,e.clientHeight,t.defaultRowHeight,t.exceptions,t.numberOfRows),r=n.first,i=n.last,o=n.firstRowPos;this.visible.first=this.visible.forcedFirst=r,this.visible.last=this.visible.forcedLast=i,r<0?this.updateOffset(0):(this.addAtBottom(r,i),this.updateOffset(o))},t.prototype.recreateAnimated=function(t){var e=this,n=new Map,r=new c(t.previous,t.previousKey),i=new c(this.context,t.currentKey),o=y(this.bodyScroller.scrollTop,this.bodyScroller.clientHeight,i.context.defaultRowHeight,i.context.exceptions,i.context.numberOfRows),a=Array.from(this.body.children),u=Object.assign({},this.visible);this.body.innerHTML="",r.positions(u.first,Math.min(u.last,u.first+a.length),this.visibleFirstRowPos,function(t,e,r){var i=a[t];i&&n.set(e,{n:i,pos:r,i:t})}),this.visible.first=this.visible.forcedFirst=o.first,this.visible.last=this.visible.forcedLast=o.last;var l=this.fragment,d=[],p=o.firstRowPos;i.positions(o.first,o.last,o.firstRowPos,function(t,o,a){var u,c,f=s.UPDATE;if(n.has(o)){var h=n.get(o);n.delete(o),h.n.dataset.index=String(t),u=e.proxy(h.n,e.updateRow(h.n,t)),c={index:h.i,y:h.pos,height:r.exceptionHeightOf(h.i,!0)}}else{var g=r.posByKey(o);u=e.create(t),f=g.index<0?s.SHOW:s.UPDATE_CREATE,c={index:g.index,y:g.pos>=0?g.pos:a,height:g.index<0?i.exceptionHeightOf(t,!0):r.exceptionHeightOf(g.index,!0)}}d.push({node:u,key:o,mode:f,previous:c,nodeY:p,nodeYCurrentHeight:a,current:{index:t,y:a,height:i.exceptionHeightOf(t)}}),u.style.transform="translate(0, "+(p-a)+"px)",p+=c.height+(c.index<0?i.padding(t):r.padding(c.index)),l.appendChild(u)});var h=o.endPos;n.forEach(function(t,e){var n=i.posByKey(e),o=n.pos>=0?n.pos:t.pos,a=t.n;a.style.transform="translate(0, "+(t.pos-p)+"px)",l.appendChild(a);var u=r.exceptionHeightOf(t.i,!0);d.push({node:t.n,key:e,mode:n.index<0?s.HIDE:s.UPDATE_REMOVE,previous:{index:t.i,y:t.pos,height:u},nodeY:p,nodeYCurrentHeight:h,current:{index:n.index,y:o,height:n.index<0?null:i.exceptionHeightOf(n.index)}}),h+=n.index<0?i.context.defaultRowHeight:i.exceptionHeightOf(n.index,!0)+i.padding(n.index),p+=u+r.padding(t.i)}),this.updateOffset(o.firstRowPos),this.animate(d,t.phases||f,r,i,l)},t.prototype.animate=function(t,e,n,r,i){var o=this;if(t.length<=0)this.body.appendChild(i);else{for(var a=-1,u=0,c=function(e,i){void 0===i&&(i=t),i.forEach(function(t){return e.apply(t,n,r)})},l=function(){if(console.assert(t[0].node.offsetTop>=0,"dummy log for forcing dom update"),c(e[u++]),u<e.length){var n=e[u];a=self.setTimeout(l,n.delay)}else{var r=o.body.classList;Array.from(r).forEach(function(t){t.startsWith("le-")&&t.endsWith("-animation")&&r.remove(t)}),t.forEach(function(t){var e=t.node,n=t.mode;n!==s.UPDATE_REMOVE&&n!==s.HIDE||(e.remove(),e.style.transform=null,o.recycle(e))}),o.abortAnimation=function(){},a=-1}};0===e[u].delay;)c(e[u++]);var p=this.body;this.body.appendChild(i);var f=[];t=t.filter(function(t){return!d(t,n.context.defaultRowHeight,r.context.defaultRowHeight)||(f.push(t),!1)}),f.length>0&&e.slice(u).forEach(function(t){return c(t,f)}),0!==t.length&&(p.classList.add("le-row-animation"),new Set(t.map(function(t){return t.mode})).forEach(function(t){p.classList.add("le-"+s[t].toLowerCase().split("_")[0]+"-animation")}),this.abortAnimation=function(){a<=0||(clearTimeout(a),a=-1,u=e.length-1,l())},a=self.setTimeout(l,e[u].delay))}},t.prototype.clearPool=function(){this.pool.splice(0,this.pool.length)},t.prototype.revalidate=function(){var t=this.bodyScroller;this.onScrolledVertically(t.scrollTop,t.clientHeight,!0),this.updateOffset(this.visibleFirstRowPos)},t.prototype.onScrolledVertically=function(t,e,n){var r=this.onScrolledImpl(t,e);return this.mixins.forEach(function(t){return t.onScrolled(n,r)}),r},t.prototype.shiftLast=function(t,e){if(e>=this.options.batchSize)return t;var n=this.context.numberOfRows;return Math.min(n-1,t+(this.options.batchSize-e))},t.prototype.shiftFirst=function(t,e,n){if(n>=this.options.batchSize||t<=0)return{first:t,firstRowPos:e};for(var r=Math.max(0,t-(this.options.batchSize-n)),i=this.context,o=i.exceptionsLookup,a=i.defaultRowHeight,u=e,s=r;s<t;++s)o.has(s)?u-=o.get(s):u-=a;return{first:r,firstRowPos:u}},t.prototype.onScrolledImpl=function(t,e){var n=this.context,r=y(t,e,n.defaultRowHeight,n.exceptions,n.numberOfRows),i=r.first,o=r.last,a=r.firstRowPos,u=this.visible;if(u.forcedFirst=i,u.forcedLast=o,i-u.first>=0&&o-u.last<=0)return p.NONE;var s=p.SOME;if(i>u.last||o<u.first)this.removeAll(),this.addAtBottom(i,o),s=p.ALL;else if(i<u.first){u.last-(o+1)>=this.options.batchSize?this.removeFromBottom(o+1,u.last):o=u.last;var c=this.shiftFirst(i,a,u.first-1-i);i=c.first,a=c.firstRowPos,this.addAtBeginning(i,u.first-1),s=p.SOME_TOP}else i-1-u.first>=this.options.batchSize?this.removeFromBeginning(u.first,i-1):(i=u.first,a=this.visibleFirstRowPos),o=this.shiftLast(o,o-u.last+1),this.addAtBottom(u.last+1,o),s=p.SOME_BOTTOM;return u.first=i,u.last=o,this.updateOffset(a),s},t}(),N=x,T=function(){function t(t){this.rules=new Map,this.node=t.ownerDocument.createElement("style"),t.appendChild(this.node)}return t.prototype.destroy=function(){this.node.remove()},t.prototype.updateRules=function(){this.node.innerHTML=Array.from(this.rules.values()).join("\n")},t.prototype.addRule=function(t,e,n){return void 0===n&&(n=!0),this.rules.set(t,e),n&&this.updateRules(),t},t.prototype.updateRule=function(t,e,n){return void 0===n&&(n=!0),this.rules.set(t,e),n&&this.updateRules(),t},t.prototype.deleteRule=function(t,e){void 0===e&&(e=!0),this.rules.get(t)&&e&&this.updateRules()},Object.defineProperty(t.prototype,"ruleNames",{get:function(){return Array.from(this.rules.keys())},enumerable:!0,configurable:!0}),t}(),O="\n  <header>\n    <article></article>\n  </header>\n  <main>\n    <footer>&nbsp;</footer>\n    <article></article>\n  </main>";function R(t){return t.innerHTML=O,t}function M(t,e){t.dataset.id=e.id}var D=function(t){function e(e,n){var r=t.call(this,e)||this;r.id=n;var i=e.querySelector("header"),o=e.querySelector("main");return self.setTimeout(function(){var t=function(t){var e=t.ownerDocument.body;e.insertAdjacentHTML("beforeend",'\n    <div class="lineup-engine-scrollbar-tester"><div></div></div>\n  ');var n=e.lastElementChild,r=n.offsetWidth-n.clientWidth,i=n.offsetHeight-n.clientHeight;return n.remove(),{width:r,height:i}}(e).width;r.updateRule("__scollBarFix2",r.hashedId+" > header > article:last-of-type {\n        border-right: "+t+"px solid transparent;\n      }")},20),A(o,"animation",function(t){var n=i.scrollLeft,r=t.left;n!==r&&(i.scrollLeft=r),e.classList.toggle("le-shifted",t.left>0)}),r}return i(e,t),Object.defineProperty(e.prototype,"hashedId",{get:function(){return this.id.startsWith("#")?this.id:"#"+this.id},enumerable:!0,configurable:!0}),e.prototype.update=function(t,e,n,r,i){void 0===i&&(i="px");var o=void 0!==r?this.tableIds(r,!0):{header:this.id+" > header > article",body:this.id+" > main > article"};this.updateRule("__heightsRule"+o.body,o.body+" > div {\n      height: "+t+"px;\n    }",!1),this.updateColumns(e,o,n,i),this.updateRules()},e.prototype.remove=function(t){var e=this.tableIds(t,!0);this.deleteRule("__heightsRule"+e.body,!1),this.deleteRule("__widthRule"+e.body,!1);for(var n="__col"+e.body+"_",r=this.ruleNames.reduce(function(t,e){return t+(e.startsWith(n)?1:0)},0),i=0;i<r;++i)this.deleteRule(""+n+i,!1);this.updateRules()},e.prototype.tableIds=function(t,e){void 0===e&&(e=!1);var n=this.id.startsWith("#")?this.id.slice(1):this.id;return{header:(e?"#":"")+n+"_H"+t,body:(e?"#":"")+n+"_B"+t}},e.prototype.updateColumns=function(t,e,n,r){var i=this;void 0===r&&(r="px");var o="__col"+e.body+"_",a=this.ruleNames.reduce(function(t,e){return t+(e.startsWith(o)?1:0)},0),u=0,s=0;t.forEach(function(t){var a=e.body+' > div > [data-id="'+t.id+'"], '+e.header+' [data-id="'+t.id+'"] {\n        width: '+t.width+r+";\n        "+(t.frozen?"left: "+u+"px;":"")+"\n      }";if(0!==n&&t.frozen){var c=e.body+' > div > [data-id="'+t.id+'"] {\n          width: '+t.width+r+";\n          left: "+(u+n)+"px;\n        }";a=e.header+' [data-id="'+t.id+'"] {\n          width: '+t.width+r+";\n          left: "+u+"px;\n        }",i.updateRule(""+o+s++,c,!1)}t.frozen&&(u+=t.width),i.updateRule(""+o+s++,a,!1)});for(var c=s-1;c<a;++c)this.deleteRule(""+o+c,!1)},e}(T),_=function(){function t(t,e,n){for(var r=[],i=3;i<arguments.length;i++)r[i-3]=arguments[i];var o=this;this.header=t,this.style=e,this.tableId=n,this.cellPool=[],this.visibleColumns={frozen:[],first:0,forcedFirst:0,last:-1,forcedLast:-1},this.visibleFirstColumnPos=0,this.columnAdapter=this.createColumnAdapter(),this.columnMixins=r.map(function(t){return new t(o.columnAdapter)}),this.columnFragment=t.ownerDocument.createDocumentFragment()}return t.prototype.leftShift=function(){var t=this.context,e=this.visibleColumns.frozen.reduce(function(e,n){return e+t.columns[n].width+t.column.padding(n)},0);return this.visibleFirstColumnPos-e},Object.defineProperty(t.prototype,"headerScroller",{get:function(){return this.header.parentElement},enumerable:!0,configurable:!0}),t.prototype.addColumnMixin=function(t,e){this.columnMixins.push(new t(this.columnAdapter,e))},t.prototype.createColumnAdapter=function(){var t=this,e={visible:this.visibleColumns,addAtBeginning:this.addColumnAtStart.bind(this),addAtBottom:this.addColumnAtEnd.bind(this),removeFromBeginning:this.removeColumnFromStart.bind(this),removeFromBottom:this.removeColumnFromEnd.bind(this),updateOffset:this.updateColumnOffset.bind(this),scroller:this.headerScroller,syncFrozen:this.syncFrozen.bind(this)};return Object.defineProperties(e,{visibleFirstRowPos:{get:function(){return t.visibleFirstColumnPos},enumerable:!0},context:{get:function(){return t.context.column},enumerable:!0}}),e},t.prototype.init=function(){var t=this,e=this.context;this.style.update(e.defaultRowHeight-e.padding(-1),e.columns,0,this.tableId),e.columns.forEach(function(){t.cellPool.push([])})},t.prototype.onScrolledHorizontally=function(t,e,n){var r=this.onScrolledHorizontallyImpl(t,e);return this.columnMixins.forEach(function(t){return t.onScrolled(n,r)}),r},t.prototype.removeColumnFromStart=function(t,e,n){var r=this;void 0===n&&(n=this.visibleColumns.frozen.length),this.forEachRow(function(i){r.removeCellFromStart(i,t,e,n)})},t.prototype.removeCellFromStart=function(t,e,n,r){for(var i=e;i<=n;++i){var o=0===r?t.firstElementChild:t.children[r];o.remove(),this.recycleCell(o,i)}},t.prototype.removeColumnFromEnd=function(t,e){var n=this;this.forEachRow(function(r){n.removeCellFromEnd(r,t,e)})},t.prototype.removeCellFromEnd=function(t,e,n){for(var r=n;r>=e;--r){var i=t.lastElementChild;i.remove(),this.recycleCell(i,r)}},t.prototype.removeFrozenCells=function(t,e,n){for(var r=0,i=e;r<i.length;r++){var o=i[r],a=t.children[n];a.remove(),this.recycleCell(a,o)}},t.prototype.removeFrozenColumns=function(t,e){var n=this;this.forEachRow(function(r){n.removeFrozenCells(r,t,e)})},t.prototype.removeAllColumns=function(t){var e=this;this.forEachRow(function(n){e.removeAllCells(n,t)})},t.prototype.removeAllCells=function(t,e,n){var r=this;void 0===n&&(n=this.visibleColumns.first);var i=Array.from(t.children),o=this.visibleColumns.frozen;if(t.innerHTML="",e||0===o.length)for(var a=0,u=o;a<u.length;a++){var s=u[a];this.recycleCell(i.shift(),s)}else for(var c=0,l=o;c<l.length;c++)l[c],t.appendChild(i.shift());i.forEach(function(t,e){r.recycleCell(t,e+n)})},t.prototype.selectCell=function(t,e,n){var r=this.cellPool[e],i=n[e];if(r.length>0){var o=r.pop(),a=this.updateCell(o,t,i);return a&&a!==o&&M(a,i),a||o}var u=this.createCell(this.header.ownerDocument,t,i);return M(u,i),u},t.prototype.recycleCell=function(t,e){this.cellPool[e].push(t)},t.prototype.addColumnAtStart=function(t,e,n){var r=this;void 0===n&&(n=this.visibleColumns.frozen.length);var i=this.context.columns;this.forEachRow(function(o,a){r.addCellAtStart(o,a,t,e,n,i)})},t.prototype.addCellAtStart=function(t,e,n,r,i,o){for(var a=r;a>=n;--a){var u=this.selectCell(e,a,o);t.insertBefore(u,i>0?t.children[i]:t.firstChild)}},t.prototype.insertFrozenCells=function(t,e,n,r,i){for(var o=t.children[r],a=0,u=n;a<u.length;a++){var s=u[a],c=this.selectCell(e,s,i);o?t.insertBefore(c,o):t.appendChild(c)}},t.prototype.insertFrozenColumns=function(t,e){var n=this,r=this.context.columns;this.forEachRow(function(i,o){n.insertFrozenCells(i,o,t,e,r)})},t.prototype.addColumnAtEnd=function(t,e){var n=this,r=this.context.columns;this.forEachRow(function(i,o){n.addCellAtEnd(i,o,t,e,r)})},t.prototype.verifyRows=function(){var t=this.context.columns;this.forEachRow(function(e,n){return function(t,e,n){var r=Array.from(t.children);if(!(r.length<=1)){var i=r.map(function(t){return n.find(function(e){return e.id===t.dataset.id})});console.assert(i.every(function(t){return Boolean(t)}),"all columns must exist",e),console.assert(i.every(function(t,e){return 0===e||t.index>=i[e-1].index}),"all columns in ascending order",e),console.assert(new Set(i).size===i.length,"unique columns",i)}}(e,n,t)})},t.prototype.addCellAtEnd=function(t,e,n,r,i){for(var o=n;o<=r;++o){var a=this.selectCell(e,o,i);t.appendChild(a)}},t.prototype.updateHeaders=function(){var t=this,e=this.context.columns;Array.from(this.header.children).forEach(function(n,r){t.updateHeader(n,e[r])})},t.prototype.recreate=function(t,e){var n=this,r=this.context;this.style.update(r.defaultRowHeight-r.padding(-1),r.columns,-this.leftShift(),this.tableId),this.clearPool();for(var i=this.cellPool.length;i<r.columns.length;++i)this.cellPool.push([]);var o=this.columnFragment,a=o.ownerDocument;this.header.innerHTML="",r.columns.forEach(function(t){var e=n.createHeader(a,t);M(e,t),o.appendChild(e)}),this.header.appendChild(o);var u=y(t,e,r.column.defaultRowHeight,r.column.exceptions,r.column.numberOfRows),s=u.first,c=u.last,l=u.firstRowPos;if(this.visibleColumns.first=this.visibleColumns.forcedFirst=s,this.visibleColumns.last=this.visibleColumns.forcedLast=c,r.columns.some(function(t){return t.frozen})){var d=E([],r.columns,s).target;this.visibleColumns.frozen=d}else this.visibleColumns.frozen=[];this.updateColumnOffset(l)},t.prototype.clearPool=function(){this.cellPool.forEach(function(t){return t.splice(0,t.length)})},t.prototype.updateColumnOffset=function(t){var e=t!==this.visibleFirstColumnPos;if(this.visibleFirstColumnPos=t,e){var n=this.context;this.style.update(n.defaultRowHeight-n.padding(-1),n.columns,-this.leftShift(),this.tableId)}},t.prototype.createRow=function(t,e){var n=this.context.columns,r=this.visibleColumns;if(r.frozen.length>0)for(var i=0,o=r.frozen;i<o.length;i++){var a=o[i],u=this.selectCell(e,a,n);t.appendChild(u)}for(a=r.first;a<=r.last;++a)u=this.selectCell(e,a,n),t.appendChild(u)},t.prototype.updateRow=function(t,e){var n=this.context.columns,r=this.visibleColumns,i=Array.from(t.children);switch(i.length){case 0:r.frozen.length>0&&this.insertFrozenCells(t,e,r.frozen,0,n),this.addCellAtEnd(t,e,r.first,r.last,n);break;case 1:var o=i[0],a=o.dataset.id,u=n.findIndex(function(t){return t.id===a});t.removeChild(o),u>=0&&this.recycleCell(o,u),r.frozen.length>0&&this.insertFrozenCells(t,e,r.frozen,0,n),this.addCellAtEnd(t,e,r.first,r.last,n);break;default:this.mergeColumns(t,e,i)}},t.prototype.mergeColumns=function(t,e,n){var r=this,i=this.context.columns,o=this.visibleColumns;t.innerHTML="";var a=new Map(n.map(function(t){return[t.dataset.id,t]})),u=function(n){var o=i[n],u=a.get(o.id);if(u){var s=r.updateCell(u,e,o);s&&s!==u&&M(s,o),t.appendChild(s||u)}else{var c=r.selectCell(e,n,i);t.appendChild(c)}};o.frozen.forEach(u);for(var s=o.first;s<=o.last;++s)u(s)},t.prototype.syncFrozen=function(t){var e=this.context.columns,n=this.visibleColumns;if(!e.some(function(t){return t.frozen}))return 0;if(0===t)return n.frozen.length>0&&(this.removeFrozenColumns(n.frozen,0),n.frozen=[]),0;var r=n.frozen.length,i=E(n.frozen,e,t),o=i.target,a=i.added,u=i.removed;return u.length>0&&this.removeFrozenColumns(u,r-u.length),a.length>0&&this.insertFrozenColumns(a,r-u.length),n.frozen=o,o.length},t.prototype.onScrolledHorizontallyImpl=function(t,e){var n=this.context.column,r=y(t,e,n.defaultRowHeight,n.exceptions,n.numberOfRows),i=r.first,o=r.last,a=r.firstRowPos,u=this.visibleColumns;if(u.forcedFirst=i,u.forcedLast=o,i-u.first>=0&&o-u.last<=0)return p.NONE;var s=p.SOME,c=this.syncFrozen(i);return i>u.last||o<u.first?(this.removeAllColumns(!1),this.addColumnAtEnd(i,o),s=p.ALL):i<u.first?(this.removeColumnFromEnd(o+1,u.last),this.addColumnAtStart(i,u.first-1,c),s=p.SOME_TOP):(this.removeColumnFromStart(u.first,i-1,c),this.addColumnAtEnd(u.last+1,o),s=p.SOME_BOTTOM),u.first=i,u.last=o,this.updateColumnOffset(a),s},t}(),L=function(t){function e(e,n,r){void 0===r&&(r={});var o=t.call(this,R(e).querySelector("main > article"),r)||this;o.root=e,e.classList.add("lineup-engine"),o.style=new D(o.root,n);var a=o,u=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return i(e,t),Object.defineProperty(e.prototype,"context",{get:function(){return a.context},enumerable:!0,configurable:!0}),e.prototype.createHeader=function(t,e){return a.createHeader(t,e)},e.prototype.updateHeader=function(t,e){return a.updateHeader(t,e)},e.prototype.createCell=function(t,e,n){return a.createCell(t,e,n)},e.prototype.updateCell=function(t,e,n){return a.updateCell(t,e,n)},e.prototype.updateColumnOffset=function(e){t.prototype.updateColumnOffset.call(this,e),a.updateOffset(a.visibleFirstRowPos)},e.prototype.forEachRow=function(t){return a.forEachRow(t)},e}(_);return o.cell=new(u.bind.apply(u,[void 0,o.header,o.style,void 0].concat(r.mixins||[]))),o}return i(e,t),Object.defineProperty(e.prototype,"header",{get:function(){return this.root.querySelector("header > article")},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"headerScroller",{get:function(){return this.root.querySelector("header")},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"visibleColumns",{get:function(){return this.cell.visibleColumns},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"visibleFirstColumnPos",{get:function(){return this.cell.visibleFirstColumnPos},enumerable:!0,configurable:!0}),e.prototype.addColumnMixin=function(t,e){this.cell.addColumnMixin(t,e)},e.prototype.init=function(){var e=this;this.cell.init();var n=A(this.body.parentElement,this.options.async,function(t){if(!(Math.abs(n.left-t.left)<e.options.minScrollDelta&&Math.abs(n.width-t.width)<e.options.minScrollDelta)){var r=t.left>n.left;n=t,e.onScrolledHorizontally(t.left,t.width,r)}});t.prototype.init.call(this)},e.prototype.destroy=function(){t.prototype.destroy.call(this),this.root.remove()},e.prototype.onScrolledHorizontally=function(t,e,n){return this.cell.onScrolledHorizontally(t,e,n)},e.prototype.updateHeaders=function(){this.cell.updateHeaders()},e.prototype.updateColumnWidths=function(){var t=this.context;this.style.update(t.defaultRowHeight-t.padding(-1),t.columns,-this.cell.leftShift())},e.prototype.updateSizer=function(t){var e=this.context,n=e.totalHeight,r=e.column.totalHeight;this.updateShifts(t,this.cell.leftShift()),this.bodySizer.style.transform="translate("+Math.max(0,r-1).toFixed(0)+"px, "+Math.max(0,n-1).toFixed(0)+"px)"},e.prototype.updateShifts=function(t,e){this.body.style.transform="translate("+e.toFixed(0)+"px, "+t.toFixed(0)+"px)"},e.prototype.recreate=function(e){var n=this.bodyScroller,r=n.scrollLeft;this.cell.recreate(r,n.clientWidth),t.prototype.recreate.call(this,e),n.scrollLeft=r},e.prototype.clearPool=function(){t.prototype.clearPool.call(this),this.cell.clearPool()},e.prototype.createRow=function(t,e){this.cell.createRow(t,e)},e.prototype.updateRow=function(t,e){this.cell.updateRow(t,e)},e}(x),S=n(1),I=function(){function t(t,e){return this.adapter=t,this.prefetchTimeout=-1,this.cleanupTimeout=-1,this.options={prefetchRows:20,cleanUpRows:3,delay:50},Object.assign(this.options,e),this}return t.prototype.prefetchDown=function(){this.prefetchTimeout=-1;var t=this.adapter.context,e=Math.min(this.adapter.visible.forcedLast+this.options.prefetchRows,t.numberOfRows-1);this.adapter.visible.last===e&&this.adapter.visible.last>=this.adapter.visible.forcedLast+this.options.prefetchRows||(this.adapter.addAtBottom(this.adapter.visible.last+1,e),this.adapter.visible.last=e)},t.prototype.prefetchUp=function(){if(this.prefetchTimeout=-1,!(this.adapter.visible.first<=this.adapter.visible.forcedFirst-this.options.prefetchRows)){var t=this.adapter.context,e=this.adapter.scroller,n=y(Math.max(e.scrollTop-this.options.prefetchRows*t.defaultRowHeight,0),e.clientHeight,t.defaultRowHeight,t.exceptions,t.numberOfRows),r=n.first,i=n.firstRowPos;if(r!==this.adapter.visible.first){var o=this.adapter.syncFrozen?this.adapter.syncFrozen(r):0;this.adapter.addAtBeginning(r,this.adapter.visible.first-1,o),this.adapter.visible.first=r,this.adapter.updateOffset(i)}}},t.prototype.triggerPrefetch=function(t){this.prefetchTimeout>=0&&clearTimeout(this.prefetchTimeout);var e=this.adapter.visible.last<this.adapter.visible.forcedLast+this.options.prefetchRows,n=this.adapter.visible.first>this.adapter.visible.forcedFirst-this.options.prefetchRows,r=this.adapter.visible.last===this.adapter.context.numberOfRows,i=0===this.adapter.visible.first;if((!t||e||r)&&(t||n||i)){var o=t||i?this.prefetchDown.bind(this):this.prefetchUp.bind(this);this.prefetchTimeout=self.setTimeout(o,this.options.delay)}},t.prototype.cleanUpTop=function(t){this.cleanupTimeout=-1;var e=Math.max(0,t-this.options.cleanUpRows);if(!(e<=this.adapter.visible.first)){var n=this.adapter.syncFrozen?this.adapter.syncFrozen(e):0;this.adapter.removeFromBeginning(this.adapter.visible.first,e-1,n);var r=this.adapter.context,i=(e-this.adapter.visible.first)*r.defaultRowHeight;if(r.exceptions.length>0)for(var o=this.adapter.visible.first;o<e;++o)r.exceptionsLookup.has(o)&&(i+=r.exceptionsLookup.get(o)-r.defaultRowHeight);this.adapter.visible.first=e,this.adapter.updateOffset(this.adapter.visibleFirstRowPos+i),this.prefetchDown()}},t.prototype.cleanUpBottom=function(t){this.cleanupTimeout=-1;var e=t+this.options.cleanUpRows;this.adapter.visible.last<=e||(this.adapter.removeFromBottom(e+1,this.adapter.visible.last),this.adapter.visible.last=e,this.prefetchUp())},t.prototype.triggerCleanUp=function(t,e,n){this.cleanupTimeout>=0&&clearTimeout(this.cleanupTimeout),n&&t-this.options.cleanUpRows<=this.adapter.visible.first||!n&&this.adapter.visible.last<=e+this.options.cleanUpRows||(this.cleanupTimeout=self.setTimeout(n?this.cleanUpTop.bind(this):this.cleanUpBottom.bind(this),this.options.delay,n?t:e))},t.prototype.onScrolled=function(t,e){e!==p.ALL&&this.options.cleanUpRows>0&&this.triggerCleanUp(this.adapter.visible.forcedFirst,this.adapter.visible.forcedLast,t),e!==p.NONE&&this.options.prefetchRows>0&&this.triggerPrefetch(t)},t}(),j=function(t){function e(e,n,r,o,a){void 0===a&&(a={});var u=t.call(this,n,a)||this;u.header=e,u.body=n,u.tableId=r,u.style=o;var s=u,c=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return i(e,t),Object.defineProperty(e.prototype,"context",{get:function(){return s.context},enumerable:!0,configurable:!0}),e.prototype.createHeader=function(t,e){return s.createHeader(t,e)},e.prototype.updateHeader=function(t,e){return s.updateHeader(t,e)},e.prototype.createCell=function(t,e,n){return s.createCell(t,e,n)},e.prototype.updateColumnOffset=function(e){t.prototype.updateColumnOffset.call(this,e),s.updateOffset(s.visibleFirstRowPos)},e.prototype.updateCell=function(t,e,n){return s.updateCell(t,e,n)},e.prototype.forEachRow=function(t){return s.forEachRow(t)},e}(_);return u.cell=new(c.bind.apply(c,[void 0,u.header,u.style,r].concat(a.mixins||[]))),u}return i(e,t),e.prototype.addColumnMixin=function(t,e){this.cell.addColumnMixin(t,e)},Object.defineProperty(e.prototype,"width",{get:function(){return this.context.column.totalHeight},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"visibleColumns",{get:function(){return this.cell.visibleColumns},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"visibleFirstColumnPos",{get:function(){return this.cell.visibleFirstColumnPos},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"height",{get:function(){return this.context.totalHeight},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"hidden",{get:function(){return this.header.classList.contains("loading")},set:function(t){this.hidden!==t&&(this.header.classList.toggle("loading",t),this.body.classList.toggle("loading",t),this.onVisibilityChanged(!t))},enumerable:!0,configurable:!0}),e.prototype.updateSizer=function(t){this.updateShifts(t,this.cell.leftShift())},e.prototype.updateShifts=function(t,e){this.body.style.transform="translate("+e.toFixed(0)+"px, "+t.toFixed(0)+"px)"},e.prototype.onVisibilityChanged=function(t){},e.prototype.hide=function(){this.hidden=!0},e.prototype.show=function(t,e,n){var r=this.hidden;this.hidden=!1,r?this.revalidate():this.onScrolledHorizontally(t,e,n)},e.prototype.init=function(){this.hide(),this.cell.init(),t.prototype.init.call(this)},e.prototype.destroy=function(){t.prototype.destroy.call(this),this.header.remove(),this.style.remove(this.tableId)},e.prototype.onScrolledVertically=function(e,n,r){return this.hidden?p.NONE:t.prototype.onScrolledVertically.call(this,e,n,r)},e.prototype.onScrolledHorizontally=function(t,e,n){var r=this.cell.onScrolledHorizontally(t,e,n);return this.updateSizer(this.visibleFirstRowPos),r},e.prototype.updateHeaders=function(){this.cell.updateHeaders()},e.prototype.updateColumnWidths=function(){var t=this.context;this.style.update(t.defaultRowHeight-t.padding(-1),t.columns,-this.cell.leftShift(),this.tableId)},e.prototype.recreate=function(e){var n=this.bodyScroller,r=n.scrollLeft;this.cell.recreate(r,n.clientWidth),t.prototype.recreate.call(this,e),n.scrollLeft=r},e.prototype.clearPool=function(){t.prototype.clearPool.call(this),this.cell.clearPool()},e.prototype.createRow=function(t,e){this.cell.createRow(t,e)},e.prototype.updateRow=function(t,e){this.cell.updateRow(t,e)},e}(N),k=function(){function t(t,e,n){void 0===n&&(n={});var r=this;this.node=t,this.tableId=0,this.sections=[],this.options={columnPadding:0,async:C,minScrollDelta:30},Object.assign(this.options,n),t.id=e.startsWith("#")?e.slice(1):e,t.innerHTML="<header><footer>&nbsp;</footer></header><main><footer>&nbsp;</footer></main>",t.classList.add("lineup-engine","lineup-multi-engine"),this.style=new D(this.node,e);var i=A(this.main,this.options.async,function(t){if(!(Math.abs(i.left-t.left)<r.options.minScrollDelta&&Math.abs(i.width-t.width)<r.options.minScrollDelta)){var e=t.left>i.left;i=t,r.onScrolledHorizontally(t.left,t.width,e)}})}return t.prototype.update=function(){var t=this;this.onScrolledHorizontally(this.main.scrollLeft,this.main.clientWidth,!1);var e=0;this.sections.forEach(function(n){n.body.style.left=n.header.style.left=e+"px",e+=n.width+t.options.columnPadding})},t.prototype.onScrolledHorizontally=function(t,e,n){var r=this,i=0,o=t+e;this.sections.forEach(function(e){var a=i+e.width;a<t||i>o?e.hide():e.show(Math.max(0,t-i),Math.min(o-i,e.width),n),i=a+r.options.columnPadding}),this.updateOffset()},t.prototype.updateOffset=function(){var t=this,e=this.header.querySelector("footer"),n=this.main.querySelector("footer"),r=Math.max.apply(Math,[0].concat(this.sections.map(function(t){return t.height}))),i=this.sections.reduce(function(e,n){return e+n.width+t.options.columnPadding},0);e.style.transform="translate("+i+"px,0)",n.style.transform="translate("+i+"px, "+r+"px)"},t.prototype.destroy=function(){this.sections.forEach(function(t){return t.destroy()}),this.node.remove()},Object.defineProperty(t.prototype,"doc",{get:function(){return this.node.ownerDocument},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"header",{get:function(){return this.node.querySelector("header")},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"main",{get:function(){return this.node.querySelector("main")},enumerable:!0,configurable:!0}),t.prototype.pushTable=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];var r=this.doc.createElement("article"),i=this.doc.createElement("article"),o="T"+this.tableId++,a=this.style.tableIds(o);r.id=a.header,i.id=a.body,this.header.insertBefore(r,this.header.lastElementChild),this.main.appendChild(i);var u=t.apply(void 0,[r,i,o,this.style].concat(e));return u.init(),this.sections.push(u),this.update(),u},t.prototype.pushSeparator=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];var r=this.doc.createElement("section"),i=this.doc.createElement("section");this.header.insertBefore(r,this.header.lastElementChild),this.main.appendChild(i);var o=t.apply(void 0,[r,i,this.style].concat(e));return o.init(),this.sections.push(o),this.update(),o},t.prototype.remove=function(t){var e=this.sections.indexOf(t);return!(e<0||(this.sections.splice(e,1),t.destroy(),this.update(),0))},t.prototype.clear=function(){this.sections.splice(0,this.sections.length).forEach(function(t){return t.destroy()}),this.update()},t.prototype.widthChanged=function(){this.update()},t}();n.d(e,"version",function(){return V}),n.d(e,"ACellRenderer",function(){return L}),n.d(e,"ARowRenderer",function(){return x}),n.d(e,"uniformContext",function(){return g}),n.d(e,"nonUniformContext",function(){return m}),n.d(e,"randomContext",function(){return b}),n.d(e,"range",function(){return y}),n.d(e,"frozenDelta",function(){return v}),n.d(e,"updateFrozen",function(){return E}),n.d(e,"ABORTED",function(){return o}),n.d(e,"isAbortAble",function(){return u}),n.d(e,"abortAble",function(){return a}),n.d(e,"IColumn",function(){return S.IColumn}),n.d(e,"GridStyleManager",function(){return D}),n.d(e,"setColumn",function(){return M}),n.d(e,"TEMPLATE",function(){return O}),n.d(e,"setTemplate",function(){return R}),n.d(e,"StyleManager",function(){return T}),n.d(e,"IMixin",function(){}),n.d(e,"IMixinAdapter",function(){}),n.d(e,"IMixinClass",function(){}),n.d(e,"EScrollResult",function(){return p}),n.d(e,"PrefetchMixin",function(){return I}),n.d(e,"ITableCellRenderContext",function(){}),n.d(e,"ACellTableSection",function(){return j}),n.d(e,"IMultiTableRowRendererOptions",function(){}),n.d(e,"ISeparatorFactory",function(){}),n.d(e,"ITableFactory",function(){}),n.d(e,"ITableSection",function(){}),n.d(e,"MultiTableRowRenderer",function(){return k}),n.d(e,"KeyFinder",function(){return c}),n.d(e,"EAnimationMode",function(){return s}),n.d(e,"noAnimationChange",function(){return d}),n.d(e,"defaultPhases",function(){return f});var V="1.1.2"}])},function(t,e,n){"use strict";var r=function(t,e){if((n=(t=e?t.toExponential(e-1):t.toExponential()).indexOf("e"))<0)return null;var n,r=t.slice(0,n);return[r.length>1?r[0]+r.slice(2):r,+t.slice(n+1)]},i=function(t){return(t=r(Math.abs(t)))?t[1]:NaN},o=/^(?:(.)?([<>=^]))?([+\-( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?(~)?([a-z%])?$/i;function a(t){return new u(t)}function u(t){if(!(e=o.exec(t)))throw new Error("invalid format: "+t);var e;this.fill=e[1]||" ",this.align=e[2]||">",this.sign=e[3]||"-",this.symbol=e[4]||"",this.zero=!!e[5],this.width=e[6]&&+e[6],this.comma=!!e[7],this.precision=e[8]&&+e[8].slice(1),this.trim=!!e[9],this.type=e[10]||""}a.prototype=u.prototype,u.prototype.toString=function(){return this.fill+this.align+this.sign+this.symbol+(this.zero?"0":"")+(null==this.width?"":Math.max(1,0|this.width))+(this.comma?",":"")+(null==this.precision?"":"."+Math.max(0,0|this.precision))+(this.trim?"~":"")+this.type};var s,c,l,d,p=function(t){t:for(var e,n=t.length,r=1,i=-1;r<n;++r)switch(t[r]){case".":i=e=r;break;case"0":0===i&&(i=r),e=r;break;default:if(i>0){if(!+t[r])break t;i=0}}return i>0?t.slice(0,i)+t.slice(e+1):t},f=function(t,e){var n=r(t,e);if(!n)return t+"";var i=n[0],o=n[1];return o<0?"0."+new Array(-o).join("0")+i:i.length>o+1?i.slice(0,o+1)+"."+i.slice(o+1):i+new Array(o-i.length+2).join("0")},h={"%":function(t,e){return(100*t).toFixed(e)},b:function(t){return Math.round(t).toString(2)},c:function(t){return t+""},d:function(t){return Math.round(t).toString(10)},e:function(t,e){return t.toExponential(e)},f:function(t,e){return t.toFixed(e)},g:function(t,e){return t.toPrecision(e)},o:function(t){return Math.round(t).toString(8)},p:function(t,e){return f(100*t,e)},r:f,s:function(t,e){var n=r(t,e);if(!n)return t+"";var i=n[0],o=n[1],a=o-(s=3*Math.max(-8,Math.min(8,Math.floor(o/3))))+1,u=i.length;return a===u?i:a>u?i+new Array(a-u+1).join("0"):a>0?i.slice(0,a)+"."+i.slice(a):"0."+new Array(1-a).join("0")+r(t,Math.max(0,e+a-1))[0]},X:function(t){return Math.round(t).toString(16).toUpperCase()},x:function(t){return Math.round(t).toString(16)}},g=function(t){return t},m=["y","z","a","f","p","n","µ","m","","k","M","G","T","P","E","Z","Y"];c=function(t){var e=t.grouping&&t.thousands?function(t,e){return function(n,r){for(var i=n.length,o=[],a=0,u=t[0],s=0;i>0&&u>0&&(s+u+1>r&&(u=Math.max(1,r-s)),o.push(n.substring(i-=u,i+u)),!((s+=u+1)>r));)u=t[a=(a+1)%t.length];return o.reverse().join(e)}}(t.grouping,t.thousands):g,n=t.currency,r=t.decimal,o=t.numerals?function(t){return function(e){return e.replace(/[0-9]/g,function(e){return t[+e]})}}(t.numerals):g,u=t.percent||"%";function c(t){var i=(t=a(t)).fill,c=t.align,l=t.sign,d=t.symbol,f=t.zero,g=t.width,b=t.comma,y=t.precision,v=t.trim,E=t.type;"n"===E?(b=!0,E="g"):h[E]||(null==y&&(y=12),v=!0,E="g"),(f||"0"===i&&"="===c)&&(f=!0,i="0",c="=");var C="$"===d?n[0]:"#"===d&&/[boxX]/.test(E)?"0"+E.toLowerCase():"",w="$"===d?n[1]:/[%p]/.test(E)?u:"",A=h[E],x=/[defgprs%]/.test(E);function N(t){var n,a,u,d=C,h=w;if("c"===E)h=A(t)+h,t="";else{var N=(t=+t)<0;if(t=A(Math.abs(t),y),v&&(t=p(t)),N&&0==+t&&(N=!1),d=(N?"("===l?l:"-":"-"===l||"("===l?"":l)+d,h=("s"===E?m[8+s/3]:"")+h+(N&&"("===l?")":""),x)for(n=-1,a=t.length;++n<a;)if(48>(u=t.charCodeAt(n))||u>57){h=(46===u?r+t.slice(n+1):t.slice(n))+h,t=t.slice(0,n);break}}b&&!f&&(t=e(t,1/0));var T=d.length+t.length+h.length,O=T<g?new Array(g-T+1).join(i):"";switch(b&&f&&(t=e(O+t,O.length?g-h.length:1/0),O=""),c){case"<":t=d+t+h+O;break;case"=":t=d+O+t+h;break;case"^":t=O.slice(0,T=O.length>>1)+d+t+h+O.slice(T);break;default:t=O+d+t+h}return o(t)}return y=null==y?6:/[gprs]/.test(E)?Math.max(1,Math.min(21,y)):Math.max(0,Math.min(20,y)),N.toString=function(){return t+""},N}return{format:c,formatPrefix:function(t,e){var n=c(((t=a(t)).type="f",t)),r=3*Math.max(-8,Math.min(8,Math.floor(i(e)/3))),o=Math.pow(10,-r),u=m[8+r/3];return function(t){return n(o*t)+u}}}}({decimal:".",thousands:",",grouping:[3],currency:["$",""]}),l=c.format,d=c.formatPrefix;var b=function(t){return Math.max(0,-i(Math.abs(t)))},y=function(t,e){return Math.max(0,3*Math.max(-8,Math.min(8,Math.floor(i(e)/3)))-i(Math.abs(t)))},v=function(t,e){return t=Math.abs(t),e=Math.abs(e)-t,Math.max(0,i(e)-i(t))+1};n.d(e,"a",function(){return l}),n.d(e,"b",function(){return d}),n.d(e,"c",function(){return a}),n.d(e,"d",function(){return b}),n.d(e,"e",function(){return y}),n.d(e,"f",function(){return v})},function(t,e,n){"use strict";var r=n(25);function i(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];var r=t.dataTransfer.types;return"function"==typeof r.indexOf?e.some(function(t){return r.indexOf(t)>=0}):"function"==typeof r.includes?e.some(function(t){return r.includes(t)}):"function"==typeof r.contains&&e.some(function(t){return r.contains(t)})}n(74);var o=new Map;function a(t){return o.size>0&&i(t,"text/plain")}function u(t){var e=t.dataTransfer;!function(t){var e=t.dataTransfer;return Boolean(t.ctrlKey&&e.effectAllowed.match(/copy/gi)||!e.effectAllowed.match(/move/gi))}(t)?e.dropEffect="move":e.dropEffect="copy"}var s=0;function c(t,e,n,r,s){void 0===r&&(r=null),void 0===s&&(s=!1),t.addEventListener("dragenter",function(n){if(!t.classList.contains("lu-dragging")&&(i.apply(void 0,[n].concat(e))||a(n)))return t.classList.add("lu-dragover"),s&&n.stopPropagation(),!1;t.classList.remove("lu-dragover")}),t.addEventListener("dragover",function(n){if(!t.classList.contains("lu-dragging")&&(i.apply(void 0,[n].concat(e))||a(n)))return n.preventDefault(),u(n),t.classList.add("lu-dragover"),s&&n.stopPropagation(),r&&r(n),!1}),t.addEventListener("dragleave",function(t){t.target.classList.remove("lu-dragover")}),t.addEventListener("drop",function(r){r.preventDefault(),s&&r.stopPropagation(),u(r);var c=r.dataTransfer.dropEffect;if(t.classList.remove("lu-dragover"),a(r)){var l=r.dataTransfer.getData("text/plain"),d=parseInt(l.indexOf(":")>=0?l.substring(0,l.indexOf(":")):l,10);if(o.has(d)){var p=o.get(d);return o.delete(d),!n({effect:c,data:p},r)}}else if(i.apply(void 0,[r].concat(e))){var f={};return e.forEach(function(t){var e=r.dataTransfer.getData(t);""!==e&&(f[t]=e)}),!n({effect:c,data:f},r)}})}var l=n(4),d=n(11),p=n(71),f=n(64),h=n(62),g=n(63),m=n(66),b=n(65),y=n(72),v=n(1),E=n(13),C=n(42);function w(t,e,n){void 0===n&&(n={}),n=Object.assign({dragAble:!0,mergeDropAble:!0,rearrangeAble:!0,resizeable:!0,level:0},n);var o=e.document.createElement("section");return o.innerHTML='\n    <div class="lu-label">'+(t.getWidth()<r.e?"&nbsp;":t.label)+'</div>\n    <div class="lu-toolbar"></div>\n    <div class="lu-spacing"></div>\n    <div class="lu-handle"></div>\n  ',T(o.querySelector("div.lu-toolbar"),n.level,t,e),R(o,t),n.dragAble&&D(o,t,e),n.mergeDropAble&&function(t,e,n){var r=function(t){var r=t.data,i="copy"===t.effect,o=M,a=Object.keys(r).find(function(t){return t.startsWith(o)&&t.endsWith("-ref")});if(a){var u=r[a],s=n.provider.find(u);if(i)s=n.provider.clone(s);else{if(s===e)return null;s.removeMe()}return s}var c=Object.keys(r).find(function(t){return t.startsWith(o)});if(!c)return null;var l=JSON.parse(c);return n.provider.create(n.provider.fromDescRef(l))},o=function(t){var n=r(t);return null!=n&&null!=e.push(n)},a=function(t,r){if(null==t)return!1;var i=e.findMyRanker(),o=i.indexOf(e),a=n.provider.create(r);return e.removeMe(),a.push(e),a.push(t),null!=i.insert(a,o)},u=function(t){return function(e){var n=r(e);return a(n,t)}},s=[M+"-ref",M],p=[M+"-number-ref",M+"-number"],v=[M+"-categorical-ref",M+"-categorical"],C=[M+"-boxplot-ref",M+"-boxplot"],w=[M+"-numbers-ref",M+"-numbers"];t.dataset.draginfo="+",e instanceof g.b?c(t,v.concat(p),o,function(e){i.apply(void 0,[e].concat(v))?t.dataset.draginfo="Color by":i.apply(void 0,[e].concat(p))&&(t.dataset.draginfo="Wrap")}):e instanceof h.b?c(t,v.concat(C),o,function(e){i.apply(void 0,[e].concat(v))?t.dataset.draginfo="Color by":i.apply(void 0,[e].concat(C))&&(t.dataset.draginfo="Wrap")}):e instanceof f.b?c(t,v.concat(w),o,function(e){i.apply(void 0,[e].concat(v))?t.dataset.draginfo="Color by":i.apply(void 0,[e].concat(w))&&(t.dataset.draginfo="Wrap")}):Object(E.c)(e)?c(t,e.canJustAddNumbers?p:s,o):e instanceof E.b?c(t,e.canJustAddNumbers?p:s,o):Object(l.k)(e)?(t.dataset.draginfo="Color by",c(t,v,u(Object(f.a)()))):Object(l.g)(e)?(t.dataset.draginfo="Color by",c(t,v,u(Object(h.a)()))):Object(l.i)(e)?(t.dataset.draginfo="Merge",c(t,v.concat(p),function(t,e){var n=r(t);return null!=n&&(Object(d.c)(n)?a(n,Object(g.a)()):!!Object(l.i)(n)&&a(n,e.shiftKey?Object(m.c)():Object(b.a)()))},function(e){i.apply(void 0,[e].concat(v))?t.dataset.draginfo="Color by":i.apply(void 0,[e].concat(p))&&(t.dataset.draginfo=e.shiftKey?"Min/Max":"Sum")})):(t.dataset.draginfo="Group",c(t,s,u(Object(y.a)())))}(o,t,e),n.rearrangeAble&&function(t,e,n){c(t,[M+"-ref",M],function(t){var r=null,i=t.data;if(!(M+"-ref"in i)){var o=JSON.parse(i[M]);return null!=(r=n.provider.create(n.provider.fromDescRef(o)))&&null!=e.insertAfterMe(r)}var a=i[M+"-ref"];if(!(r=n.provider.find(a))||r===e&&!t.effect.startsWith("copy"))return!1;if(t.effect.startsWith("copy"))return null!=(r=n.provider.clone(r))&&null!=e.insertAfterMe(r);var u=r.parent;return u?u===e.parent?null!=u.moveAfter(r,e):(r.removeMe(),null!=e.insertAfterMe(r)):null!=e.insertAfterMe(r)},null,!0)}(o.querySelector(".lu-handle"),t,e),n.resizeable&&function(t,e){var n,r=e.querySelector(".lu-handle"),i=0,o=function(n){n.stopPropagation(),n.preventDefault();var r=n.clientX,o=r-i,a=Math.max(0,t.getWidth()+o);Math.abs(i-r)<2||(i=r,e.style.width=a+"px",t.setWidth(a),R(e,t))},a=function(r){r.stopPropagation(),r.preventDefault();var u=r.clientX;if(e.classList.remove("lu-change-width"),n.removeEventListener("mousemove",o),n.removeEventListener("mouseup",a),n.removeEventListener("mouseleave",a),e.style.width=null,!(Math.abs(i-u)<2)){var s=u-i,c=Math.max(0,t.getWidth()+s);t.setWidth(c),R(e,t)}};r.onmousedown=function(t){t.stopPropagation(),t.preventDefault(),e.classList.add("lu-change-width"),i=t.clientX,(n=e.closest("header")).addEventListener("mousemove",o),n.addEventListener("mouseup",a),n.addEventListener("mouseleave",a)},r.onclick=function(t){t.stopPropagation(),t.preventDefault()}}(t,o),o}function A(t,e){t.querySelector(".lu-label").innerHTML=e.getWidth()<r.e?"&nbsp;":e.label,t.title=e.description?e.label+"\n"+e.description:e.label,t.dataset.colId=e.id,t.dataset.type=e.desc.type,t.dataset.typeCat=Object(v.e)(e).name,x(t,e)}function x(t,e){var n=t.querySelector("i[title='Sort']");if(n){var r=e.isSortedByMe(),i=r.asc,o=r.priority;n.dataset.sort=void 0!==i?i:"",void 0!==o?n.dataset.priority=(o+1).toString():delete n.dataset.priority}var a=t.querySelector("i[title='Sort Group']");if(a){var u=e.isGroupSortedByMe();i=u.asc,o=u.priority,a.dataset.sort=void 0!==i?i:"",void 0!==o?a.dataset.priority=(o+1).toString():delete a.dataset.priority}var s=t.querySelector("i[title^='Group']");if(s){var c=e.isGroupedBy();s.dataset.group=c>=0?"true":"false",c>=0?s.dataset.priority=(c+1).toString():delete s.dataset.priority}var l=t.querySelector("i[title^='Filter']");l&&(e.isFiltered()?l.dataset.active="":delete l.dataset.active)}function N(t,e,n,r,i){return function(o,a){t.insertAdjacentHTML("beforeend",'<i title="'+o+'" class="lu-action"><span'+(i?"":' aria-hidden="true"')+">"+o+"</span> </i>");var u=t.lastElementChild;return u.onclick=function(t){t.stopPropagation(),n.dialogManager.setHighlightColumn(e),a(e,t,n,r,!i)},u}}function T(t,e,n,r){return function(t,e,n,r){var i=N(t,n,r,e,!1);Object(C.a)(n,r).filter(function(t){return t.options.shortcut}).forEach(function(t){return i(t.title,t.onClick)})}(t,e,n,r)}function O(t,e,n,r){var i=N(t,n,r,e,!0);Object(C.a)(n,r).filter(function(t){return!t.title.startsWith("More")&&"only"!==t.options.shortcut}).forEach(function(t){return i(t.title,t.onClick)})}function R(t,e,n){void 0===n&&(n=22.5),function(t,e,n){void 0===n&&(n=22.5);var i=t.querySelector(".lu-label");if(e.getWidth()<r.e)i.classList.remove("lu-rotated");else{var o=i.clientWidth<=0?e.label.length*n/3*.6>e.getWidth():.6*i.scrollWidth>i.clientWidth;i.classList.toggle("lu-rotated",o)}}(t,e,n);var i=t.querySelector(".lu-toolbar");if(0!==i.childElementCount){var o=i.querySelector("[title^=More]"),a=e.getWidth()-(o&&o.clientWidth?o.clientWidth:n),u=Array.from(i.children).filter(function(t){return t!==o}).reverse();u.forEach(function(t){t.classList.remove("hidden")}),u.forEach(function(t){var e=u.reduce(function(t,e){var r=e.clientWidth;return r>0?t+r:e.classList.contains("hidden")?t:t+n},0);t.classList.toggle("hidden",e>a)})}}n.d(e,"a",function(){return w}),n.d(e,"e",function(){return A}),n.d(e,"f",function(){return x}),n.d(e,"b",function(){return T}),n.d(e,"c",function(){return O}),n.d(e,"d",function(){return D});var M="text/x-caleydo-lineup-column";function D(t,e,n){!function(t,e,n){void 0===n&&(n=!1);var r=++s;t.draggable=!0,t.addEventListener("dragstart",function(i){t.classList.add("lu-dragging");var a=e();if(i.dataTransfer.effectAllowed=a.effectAllowed,n&&i.stopPropagation(),!Object.keys(a.data).every(function(t){try{return i.dataTransfer.setData(t,a.data[t]),!0}catch(t){return!1}})){var u=a.data["text/plain"]||"";i.dataTransfer.setData("text/plain",r+(u?": "+u:"")),o.set(r,a.data)}}),t.addEventListener("dragend",function(e){t.classList.remove("lu-dragging"),n&&e.stopPropagation(),o.size>0&&o.delete(r);var i=t.ownerDocument.querySelector(".lu-dragover");i&&i.classList.remove("lu-dragover")})}(t,function(){var t,r=JSON.stringify(n.provider.toDescRef(e.desc)),i=((t={"text/plain":e.label})[M+"-ref"]=e.id,t[M]=r,t);return Object(l.i)(e)&&(i[M+"-number"]=r,i[M+"-number-ref"]=e.id),Object(d.c)(e)&&(i[M+"-categorical"]=r,i[M+"-categorical-ref"]=e.id),Object(l.g)(e)&&(i[M+"-boxplot"]=r,i[M+"-boxplot-ref"]=e.id),Object(p.b)(e)&&(i[M+"-map"]=r,i[M+"-map-ref"]=e.id),Object(p.a)(e)&&(i[M+"-array"]=r,i[M+"-array-ref"]=e.id),Object(l.k)(e)&&(i[M+"-numbers"]=r,i[M+"-numbers-ref"]=e.id),{effectAllowed:"copyMove",data:i}},!0)}},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r={name:"Default",color:"gray"}},function(t,e,n){"use strict";var r=n(46);function i(t){if(0<=t.y&&t.y<100){var e=new Date(-1,t.m,t.d,t.H,t.M,t.S,t.L);return e.setFullYear(t.y),e}return new Date(t.y,t.m,t.d,t.H,t.M,t.S,t.L)}function o(t){if(0<=t.y&&t.y<100){var e=new Date(Date.UTC(-1,t.m,t.d,t.H,t.M,t.S,t.L));return e.setUTCFullYear(t.y),e}return new Date(Date.UTC(t.y,t.m,t.d,t.H,t.M,t.S,t.L))}function a(t){return{y:t,m:0,d:1,H:0,M:0,S:0,L:0}}var u,s,c,l,d,p={"-":"",_:" ",0:"0"},f=/^\s*\d+/,h=/^%/,g=/[\\^$*+?|[\]().{}]/g;function m(t,e,n){var r=t<0?"-":"",i=(r?-t:t)+"",o=i.length;return r+(o<n?new Array(n-o+1).join(e)+i:i)}function b(t){return t.replace(g,"\\$&")}function y(t){return new RegExp("^(?:"+t.map(b).join("|")+")","i")}function v(t){for(var e={},n=-1,r=t.length;++n<r;)e[t[n].toLowerCase()]=n;return e}function E(t,e,n){var r=f.exec(e.slice(n,n+1));return r?(t.w=+r[0],n+r[0].length):-1}function C(t,e,n){var r=f.exec(e.slice(n,n+1));return r?(t.u=+r[0],n+r[0].length):-1}function w(t,e,n){var r=f.exec(e.slice(n,n+2));return r?(t.U=+r[0],n+r[0].length):-1}function A(t,e,n){var r=f.exec(e.slice(n,n+2));return r?(t.V=+r[0],n+r[0].length):-1}function x(t,e,n){var r=f.exec(e.slice(n,n+2));return r?(t.W=+r[0],n+r[0].length):-1}function N(t,e,n){var r=f.exec(e.slice(n,n+4));return r?(t.y=+r[0],n+r[0].length):-1}function T(t,e,n){var r=f.exec(e.slice(n,n+2));return r?(t.y=+r[0]+(+r[0]>68?1900:2e3),n+r[0].length):-1}function O(t,e,n){var r=/^(Z)|([+-]\d\d)(?::?(\d\d))?/.exec(e.slice(n,n+6));return r?(t.Z=r[1]?0:-(r[2]+(r[3]||"00")),n+r[0].length):-1}function R(t,e,n){var r=f.exec(e.slice(n,n+2));return r?(t.m=r[0]-1,n+r[0].length):-1}function M(t,e,n){var r=f.exec(e.slice(n,n+2));return r?(t.d=+r[0],n+r[0].length):-1}function D(t,e,n){var r=f.exec(e.slice(n,n+3));return r?(t.m=0,t.d=+r[0],n+r[0].length):-1}function _(t,e,n){var r=f.exec(e.slice(n,n+2));return r?(t.H=+r[0],n+r[0].length):-1}function L(t,e,n){var r=f.exec(e.slice(n,n+2));return r?(t.M=+r[0],n+r[0].length):-1}function S(t,e,n){var r=f.exec(e.slice(n,n+2));return r?(t.S=+r[0],n+r[0].length):-1}function I(t,e,n){var r=f.exec(e.slice(n,n+3));return r?(t.L=+r[0],n+r[0].length):-1}function j(t,e,n){var r=f.exec(e.slice(n,n+6));return r?(t.L=Math.floor(r[0]/1e3),n+r[0].length):-1}function k(t,e,n){var r=h.exec(e.slice(n,n+1));return r?n+r[0].length:-1}function V(t,e,n){var r=f.exec(e.slice(n));return r?(t.Q=+r[0],n+r[0].length):-1}function H(t,e,n){var r=f.exec(e.slice(n));return r?(t.Q=1e3*+r[0],n+r[0].length):-1}function P(t,e){return m(t.getDate(),e,2)}function G(t,e){return m(t.getHours(),e,2)}function F(t,e){return m(t.getHours()%12||12,e,2)}function U(t,e){return m(1+r.a.count(Object(r.k)(t),t),e,3)}function B(t,e){return m(t.getMilliseconds(),e,3)}function z(t,e){return B(t,e)+"000"}function Y(t,e){return m(t.getMonth()+1,e,2)}function W(t,e){return m(t.getMinutes(),e,2)}function q(t,e){return m(t.getSeconds(),e,2)}function K(t){var e=t.getDay();return 0===e?7:e}function J(t,e){return m(r.h.count(Object(r.k)(t),t),e,2)}function Z(t,e){var n=t.getDay();return t=n>=4||0===n?Object(r.i)(t):r.i.ceil(t),m(r.i.count(Object(r.k)(t),t)+(4===Object(r.k)(t).getDay()),e,2)}function Q(t){return t.getDay()}function X(t,e){return m(r.e.count(Object(r.k)(t),t),e,2)}function $(t,e){return m(t.getFullYear()%100,e,2)}function tt(t,e){return m(t.getFullYear()%1e4,e,4)}function et(t){var e=t.getTimezoneOffset();return(e>0?"-":(e*=-1,"+"))+m(e/60|0,"0",2)+m(e%60,"0",2)}function nt(t,e){return m(t.getUTCDate(),e,2)}function rt(t,e){return m(t.getUTCHours(),e,2)}function it(t,e){return m(t.getUTCHours()%12||12,e,2)}function ot(t,e){return m(1+r.l.count(Object(r.v)(t),t),e,3)}function at(t,e){return m(t.getUTCMilliseconds(),e,3)}function ut(t,e){return at(t,e)+"000"}function st(t,e){return m(t.getUTCMonth()+1,e,2)}function ct(t,e){return m(t.getUTCMinutes(),e,2)}function lt(t,e){return m(t.getUTCSeconds(),e,2)}function dt(t){var e=t.getUTCDay();return 0===e?7:e}function pt(t,e){return m(r.s.count(Object(r.v)(t),t),e,2)}function ft(t,e){var n=t.getUTCDay();return t=n>=4||0===n?Object(r.t)(t):r.t.ceil(t),m(r.t.count(Object(r.v)(t),t)+(4===Object(r.v)(t).getUTCDay()),e,2)}function ht(t){return t.getUTCDay()}function gt(t,e){return m(r.p.count(Object(r.v)(t),t),e,2)}function mt(t,e){return m(t.getUTCFullYear()%100,e,2)}function bt(t,e){return m(t.getUTCFullYear()%1e4,e,4)}function yt(){return"+0000"}function vt(){return"%"}function Et(t){return+t}function Ct(t){return Math.floor(+t/1e3)}u=function(t){var e=t.dateTime,n=t.date,u=t.time,s=t.periods,c=t.days,l=t.shortDays,d=t.months,f=t.shortMonths,h=y(s),g=v(s),m=y(c),b=v(c),wt=y(l),At=v(l),xt=y(d),Nt=v(d),Tt=y(f),Ot=v(f),Rt={a:function(t){return l[t.getDay()]},A:function(t){return c[t.getDay()]},b:function(t){return f[t.getMonth()]},B:function(t){return d[t.getMonth()]},c:null,d:P,e:P,f:z,H:G,I:F,j:U,L:B,m:Y,M:W,p:function(t){return s[+(t.getHours()>=12)]},Q:Et,s:Ct,S:q,u:K,U:J,V:Z,w:Q,W:X,x:null,X:null,y:$,Y:tt,Z:et,"%":vt},Mt={a:function(t){return l[t.getUTCDay()]},A:function(t){return c[t.getUTCDay()]},b:function(t){return f[t.getUTCMonth()]},B:function(t){return d[t.getUTCMonth()]},c:null,d:nt,e:nt,f:ut,H:rt,I:it,j:ot,L:at,m:st,M:ct,p:function(t){return s[+(t.getUTCHours()>=12)]},Q:Et,s:Ct,S:lt,u:dt,U:pt,V:ft,w:ht,W:gt,x:null,X:null,y:mt,Y:bt,Z:yt,"%":vt},Dt={a:function(t,e,n){var r=wt.exec(e.slice(n));return r?(t.w=At[r[0].toLowerCase()],n+r[0].length):-1},A:function(t,e,n){var r=m.exec(e.slice(n));return r?(t.w=b[r[0].toLowerCase()],n+r[0].length):-1},b:function(t,e,n){var r=Tt.exec(e.slice(n));return r?(t.m=Ot[r[0].toLowerCase()],n+r[0].length):-1},B:function(t,e,n){var r=xt.exec(e.slice(n));return r?(t.m=Nt[r[0].toLowerCase()],n+r[0].length):-1},c:function(t,n,r){return St(t,e,n,r)},d:M,e:M,f:j,H:_,I:_,j:D,L:I,m:R,M:L,p:function(t,e,n){var r=h.exec(e.slice(n));return r?(t.p=g[r[0].toLowerCase()],n+r[0].length):-1},Q:V,s:H,S:S,u:C,U:w,V:A,w:E,W:x,x:function(t,e,r){return St(t,n,e,r)},X:function(t,e,n){return St(t,u,e,n)},y:T,Y:N,Z:O,"%":k};function _t(t,e){return function(n){var r,i,o,a=[],u=-1,s=0,c=t.length;for(n instanceof Date||(n=new Date(+n));++u<c;)37===t.charCodeAt(u)&&(a.push(t.slice(s,u)),null!=(i=p[r=t.charAt(++u)])?r=t.charAt(++u):i="e"===r?" ":"0",(o=e[r])&&(r=o(n,i)),a.push(r),s=u+1);return a.push(t.slice(s,u)),a.join("")}}function Lt(t,e){return function(n){var i,u,s=a(1900);if(St(s,t,n+="",0)!=n.length)return null;if("Q"in s)return new Date(s.Q);if("p"in s&&(s.H=s.H%12+12*s.p),"V"in s){if(s.V<1||s.V>53)return null;"w"in s||(s.w=1),"Z"in s?(i=(u=(i=o(a(s.y))).getUTCDay())>4||0===u?r.p.ceil(i):Object(r.p)(i),i=r.l.offset(i,7*(s.V-1)),s.y=i.getUTCFullYear(),s.m=i.getUTCMonth(),s.d=i.getUTCDate()+(s.w+6)%7):(i=(u=(i=e(a(s.y))).getDay())>4||0===u?r.e.ceil(i):Object(r.e)(i),i=r.a.offset(i,7*(s.V-1)),s.y=i.getFullYear(),s.m=i.getMonth(),s.d=i.getDate()+(s.w+6)%7)}else("W"in s||"U"in s)&&("w"in s||(s.w="u"in s?s.u%7:"W"in s?1:0),u="Z"in s?o(a(s.y)).getUTCDay():e(a(s.y)).getDay(),s.m=0,s.d="W"in s?(s.w+6)%7+7*s.W-(u+5)%7:s.w+7*s.U-(u+6)%7);return"Z"in s?(s.H+=s.Z/100|0,s.M+=s.Z%100,o(s)):e(s)}}function St(t,e,n,r){for(var i,o,a=0,u=e.length,s=n.length;a<u;){if(r>=s)return-1;if(37===(i=e.charCodeAt(a++))){if(i=e.charAt(a++),!(o=Dt[i in p?e.charAt(a++):i])||(r=o(t,n,r))<0)return-1}else if(i!=n.charCodeAt(r++))return-1}return r}return Rt.x=_t(n,Rt),Rt.X=_t(u,Rt),Rt.c=_t(e,Rt),Mt.x=_t(n,Mt),Mt.X=_t(u,Mt),Mt.c=_t(e,Mt),{format:function(t){var e=_t(t+="",Rt);return e.toString=function(){return t},e},parse:function(t){var e=Lt(t+="",i);return e.toString=function(){return t},e},utcFormat:function(t){var e=_t(t+="",Mt);return e.toString=function(){return t},e},utcParse:function(t){var e=Lt(t,o);return e.toString=function(){return t},e}}}({dateTime:"%x, %X",date:"%-m/%-d/%Y",time:"%-I:%M:%S %p",periods:["AM","PM"],days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]}),s=u.format,c=u.parse,l=u.utcFormat,d=u.utcParse,Date.prototype.toISOString||l("%Y-%m-%dT%H:%M:%S.%LZ"),+new Date("2000-01-01T00:00:00.000Z")||d("%Y-%m-%dT%H:%M:%S.%LZ"),n.d(e,"a",function(){return s}),n.d(e,"b",function(){return c}),n.d(e,"c",function(){return l})},function(t,e,n){"use strict";var r=n(24);function i(t,e,n,r,i){var o=t*t,a=o*t;return((1-3*t+3*o-a)*e+(4-6*o+3*a)*n+(1+3*t+3*o-3*a)*r+a*i)/6}var o=function(t){return function(){return t}};function a(t,e){return function(n){return t+n*e}}function u(t,e){var n=e-t;return n?a(t,n>180||n<-180?n-360*Math.round(n/360):n):o(isNaN(t)?e:t)}function s(t,e){var n=e-t;return n?a(t,n):o(isNaN(t)?e:t)}var c=function t(e){var n=function(t){return 1==(t=+t)?s:function(e,n){return n-e?function(t,e,n){return t=Math.pow(t,n),e=Math.pow(e,n)-t,n=1/n,function(r){return Math.pow(t+r*e,n)}}(e,n,t):o(isNaN(e)?n:e)}}(e);function i(t,e){var i=n((t=Object(r.f)(t)).r,(e=Object(r.f)(e)).r),o=n(t.g,e.g),a=n(t.b,e.b),u=s(t.opacity,e.opacity);return function(e){return t.r=i(e),t.g=o(e),t.b=a(e),t.opacity=u(e),t+""}}return i.gamma=t,i}(1);function l(t){return function(e){var n,i,o=e.length,a=new Array(o),u=new Array(o),s=new Array(o);for(n=0;n<o;++n)i=Object(r.f)(e[n]),a[n]=i.r||0,u[n]=i.g||0,s[n]=i.b||0;return a=t(a),u=t(u),s=t(s),i.opacity=1,function(t){return i.r=a(t),i.g=u(t),i.b=s(t),i+""}}}var d,p,f,h,g=l(function(t){var e=t.length-1;return function(n){var r=n<=0?n=0:n>=1?(n=1,e-1):Math.floor(n*e),o=t[r],a=t[r+1],u=r>0?t[r-1]:2*o-a,s=r<e-1?t[r+2]:2*a-o;return i((n-r/e)*e,u,o,a,s)}}),m=(l(function(t){var e=t.length;return function(n){var r=Math.floor(((n%=1)<0?++n:n)*e),o=t[(r+e-1)%e],a=t[r%e],u=t[(r+1)%e],s=t[(r+2)%e];return i((n-r/e)*e,o,a,u,s)}}),function(t,e){return e-=t=+t,function(n){return t+e*n}}),b=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,y=new RegExp(b.source,"g"),v=function(t,e){var n,i=typeof e;return null==e||"boolean"===i?o(e):("number"===i?m:"string"===i?(n=Object(r.a)(e))?(e=n,c):function(t,e){var n,r,i,o=b.lastIndex=y.lastIndex=0,a=-1,u=[],s=[];for(t+="",e+="";(n=b.exec(t))&&(r=y.exec(e));)(i=r.index)>o&&(i=e.slice(o,i),u[a]?u[a]+=i:u[++a]=i),(n=n[0])===(r=r[0])?u[a]?u[a]+=r:u[++a]=r:(u[++a]=null,s.push({i:a,x:m(n,r)})),o=y.lastIndex;return o<e.length&&(i=e.slice(o),u[a]?u[a]+=i:u[++a]=i),u.length<2?s[0]?function(t){return function(e){return t(e)+""}}(s[0].x):function(t){return function(){return t}}(e):(e=s.length,function(t){for(var n,r=0;r<e;++r)u[(n=s[r]).i]=n.x(t);return u.join("")})}:e instanceof r.a?c:e instanceof Date?function(t,e){var n=new Date;return e-=t=+t,function(r){return n.setTime(t+e*r),n}}:Array.isArray(e)?function(t,e){var n,r=e?e.length:0,i=t?Math.min(r,t.length):0,o=new Array(i),a=new Array(r);for(n=0;n<i;++n)o[n]=v(t[n],e[n]);for(;n<r;++n)a[n]=e[n];return function(t){for(n=0;n<i;++n)a[n]=o[n](t);return a}}:"function"!=typeof e.valueOf&&"function"!=typeof e.toString||isNaN(e)?function(t,e){var n,r={},i={};for(n in null!==t&&"object"==typeof t||(t={}),null!==e&&"object"==typeof e||(e={}),e)n in t?r[n]=v(t[n],e[n]):i[n]=e[n];return function(t){for(n in r)i[n]=r[n](t);return i}}:m)(t,e)},E=function(t,e){return e-=t=+t,function(n){return Math.round(t+e*n)}},C=180/Math.PI,w={translateX:0,translateY:0,rotate:0,skewX:0,scaleX:1,scaleY:1},A=function(t,e,n,r,i,o){var a,u,s;return(a=Math.sqrt(t*t+e*e))&&(t/=a,e/=a),(s=t*n+e*r)&&(n-=t*s,r-=e*s),(u=Math.sqrt(n*n+r*r))&&(n/=u,r/=u,s/=u),t*r<e*n&&(t=-t,e=-e,s=-s,a=-a),{translateX:i,translateY:o,rotate:Math.atan2(e,t)*C,skewX:Math.atan(s)*C,scaleX:a,scaleY:u}};function x(t,e,n,r){function i(t){return t.length?t.pop()+" ":""}return function(o,a){var u=[],s=[];return o=t(o),a=t(a),function(t,r,i,o,a,u){if(t!==i||r!==o){var s=a.push("translate(",null,e,null,n);u.push({i:s-4,x:m(t,i)},{i:s-2,x:m(r,o)})}else(i||o)&&a.push("translate("+i+e+o+n)}(o.translateX,o.translateY,a.translateX,a.translateY,u,s),function(t,e,n,o){t!==e?(t-e>180?e+=360:e-t>180&&(t+=360),o.push({i:n.push(i(n)+"rotate(",null,r)-2,x:m(t,e)})):e&&n.push(i(n)+"rotate("+e+r)}(o.rotate,a.rotate,u,s),function(t,e,n,o){t!==e?o.push({i:n.push(i(n)+"skewX(",null,r)-2,x:m(t,e)}):e&&n.push(i(n)+"skewX("+e+r)}(o.skewX,a.skewX,u,s),function(t,e,n,r,o,a){if(t!==n||e!==r){var u=o.push(i(o)+"scale(",null,",",null,")");a.push({i:u-4,x:m(t,n)},{i:u-2,x:m(e,r)})}else 1===n&&1===r||o.push(i(o)+"scale("+n+","+r+")")}(o.scaleX,o.scaleY,a.scaleX,a.scaleY,u,s),o=a=null,function(t){for(var e,n=-1,r=s.length;++n<r;)u[(e=s[n]).i]=e.x(t);return u.join("")}}}function N(t){return function(e,n){var i=t((e=Object(r.d)(e)).h,(n=Object(r.d)(n)).h),o=s(e.s,n.s),a=s(e.l,n.l),u=s(e.opacity,n.opacity);return function(t){return e.h=i(t),e.s=o(t),e.l=a(t),e.opacity=u(t),e+""}}}function T(t){return function(e,n){var i=t((e=Object(r.c)(e)).h,(n=Object(r.c)(n)).h),o=s(e.c,n.c),a=s(e.l,n.l),u=s(e.opacity,n.opacity);return function(t){return e.h=i(t),e.c=o(t),e.l=a(t),e.opacity=u(t),e+""}}}function O(t){return function e(n){function i(e,i){var o=t((e=Object(r.b)(e)).h,(i=Object(r.b)(i)).h),a=s(e.s,i.s),u=s(e.l,i.l),c=s(e.opacity,i.opacity);return function(t){return e.h=o(t),e.s=a(t),e.l=u(Math.pow(t,n)),e.opacity=c(t),e+""}}return n=+n,i.gamma=e,i}(1)}x(function(t){return"none"===t?w:(d||(d=document.createElement("DIV"),p=document.documentElement,f=document.defaultView),d.style.transform=t,t=f.getComputedStyle(p.appendChild(d),null).getPropertyValue("transform"),p.removeChild(d),t=t.slice(7,-1).split(","),A(+t[0],+t[1],+t[2],+t[3],+t[4],+t[5]))},"px, ","px)","deg)"),x(function(t){return null==t?w:(h||(h=document.createElementNS("http://www.w3.org/2000/svg","g")),h.setAttribute("transform",t),(t=h.transform.baseVal.consolidate())?(t=t.matrix,A(t.a,t.b,t.c,t.d,t.e,t.f)):w)},", ",")",")"),Math.SQRT2,N(u),N(s),T(u),T(s),O(u);var R=O(s);n.d(e,"a",function(){return v}),n.d(e,"c",function(){return m}),n.d(e,"e",function(){return E}),n.d(e,"d",function(){return g}),n.d(e,"b",function(){return R})},function(t,e,n){"use strict";var r=n(0),i=n(47),o=n(25),a=n(16),u=n(54),s=n(73),c=n(13),l=n(8),d=n(2),p=n(14),f=n(65),h=n(5);function g(t){var e=new Map,n=new Map;return t.forEach(function(t,r){Object(s.b)(t)?t.rows.forEach(function(t){return e.set(t.i,r)}):t.group&&0===t.relativeIndex&&n.set(t.group.name,r)}),{item2groupIndex:e,group2firstItemIndex:n}}function m(t){return Object(s.b)(t)?t.name:t.i.toString()}var b=n(9),y=n(58),v=n(49),E=function(){function t(t,e,n,r){this.c=t,this.index=e,this.ctx=n,this.flags=r,this.renderers=null}return Object.defineProperty(t.prototype,"width",{get:function(){return this.c.getWidth()},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"id",{get:function(){return this.c.id},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"frozen",{get:function(){return!this.flags.disableFrozenColumns&&this.c.frozen},enumerable:!0,configurable:!0}),t.prototype.singleRenderer=function(){if(!this.renderers||!this.renderers.single)return null;if(this.renderers.singleTemplate)return this.renderers.singleTemplate.cloneNode(!0);var t=C(this.ctx.document,this.renderers.single.template);return t.dataset.renderer=this.renderers.singleId,t.dataset.group="d",this.renderers.singleTemplate=t.cloneNode(!0),t},t.prototype.groupRenderer=function(){if(!this.renderers||!this.renderers.group)return null;if(this.renderers.groupTemplate)return this.renderers.groupTemplate.cloneNode(!0);var t=C(this.ctx.document,this.renderers.group.template);return t.dataset.renderer=this.renderers.groupId,t.dataset.group="g",this.renderers.groupTemplate=t.cloneNode(!0),t},t.prototype.summaryRenderer=function(){if(!this.renderers||!this.renderers.summary)return null;if(this.renderers.summaryTemplate)return this.renderers.summaryTemplate.cloneNode(!0);var t=C(this.ctx.document,this.renderers.summary.template);return t.dataset.renderer=this.renderers.summaryId,t.classList.add("lu-summary"),this.renderers.summaryTemplate=t.cloneNode(!0),t},t.prototype.createHeader=function(){var t=Object(v.a)(this.c,this.ctx);if(t.className="lu-header",t.classList.toggle("frozen",this.frozen),this.renderers&&this.renderers.summary){var e=this.summaryRenderer();t.appendChild(e)}return this.updateHeader(t),t},t.prototype.updateHeader=function(t){if(Object(v.e)(t,this.c),this.renderers&&this.renderers.summary){var e=t.querySelector(".lu-summary");e.dataset.renderer!==this.renderers.summaryId&&(e.remove(),e=this.summaryRenderer(),t.appendChild(e)),this.renderers.summary.update(e,this.ctx.statsOf(this.c))}},t.prototype.createCell=function(t){var e=this.ctx.isGroup(t)?this.groupRenderer():this.singleRenderer();return this.updateCell(e,t),e},t.prototype.updateCell=function(t,e){t.classList.toggle("frozen",this.frozen);var n=this.ctx.isGroup(e),r=t.dataset.renderer,i=n?this.renderers.groupId:this.renderers.singleId,o=t.dataset.group;if(r===i&&o===(n?"g":"d")||(t=n?this.groupRenderer():this.singleRenderer()),n){var a=this.ctx.getGroup(e);this.renderers.group.update(t,a,a.rows)}else{var u=this.ctx.getRow(e);this.renderers.single.update(t,u,u.relativeIndex,u.group)}return t},t.prototype.renderCell=function(t,e){var n=this.ctx.getRow(e);this.renderers.single.render(t,n,n.relativeIndex,n.group)},t}();function C(t,e){var n=t.createElement("div");n.innerHTML=e;var r=n.firstElementChild;return n.innerHTML="",r}var w=function(t){function e(e,n,r,i){var o=t.call(this,e,n,r,i)||this;return o.summaries=[],o}return r.c(e,t),Object.defineProperty(e.prototype,"mc",{get:function(){return this.c},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"width",{get:function(){return this.c.getWidth()+h.d*this.mc.length},enumerable:!0,configurable:!0}),e.prototype.createHeader=function(){var e=this,n=t.prototype.createHeader.call(this),r=this.ctx.document.createElement("div");return r.classList.add("lu-nested"),r.classList.add(Object(y.c)(this.c)),n.appendChild(r),this.summaries.splice(0,this.summaries.length),this.mc.children.forEach(function(t,n){var i=Object(v.a)(t,e.ctx,{mergeDropAble:!1});if(i.classList.add("lu-header"),i.style.gridColumnStart=(n+1).toString(),r.appendChild(i),e.renderers&&e.renderers.summary){var o=e.ctx.summaryRenderer(t,!1);i.insertAdjacentHTML("beforeend",o.template);var a=i.lastElementChild;a.classList.add("lu-summary"),a.dataset.renderer=t.getSummaryRenderer(),e.summaries.push(o),o.update(a,e.ctx.statsOf(t))}}),this.updateNested(r),n},e.prototype.updateHeader=function(e){t.prototype.updateHeader.call(this,e);var n=e.querySelector(".lu-nested");return n?(e.appendChild(n),this.updateNested(n),e):e},e.prototype.updateWidthRule=function(t){var e=this.mc.children.map(function(t){return"minmax(0, "+Object(b.k)(t.getWidth())+"fr)"}),n=Object(y.c)(this.c);return t.updateRule("stacked-"+this.c.id,".lineup-engine ."+n+" {\n      display: grid;\n      grid-template-columns: "+e.join(" ")+";\n    }"),n},e.prototype.updateNested=function(t){var e=this,n=this.mc.children,r=Array.from(t.children);n.forEach(function(t,n){var i=r[n];if(Object(v.e)(i,t),e.renderers&&e.renderers.summary){var o=i.querySelector(".lu-summary"),a=o.dataset.renderer,u=t.getSummaryRenderer();if(a!==u){var s=e.ctx.summaryRenderer(t,!1);o.remove(),o.innerHTML=s.template,(o=o.firstElementChild).classList.add("lu-summary"),o.dataset.renderer=u,e.summaries[n]=s,i.appendChild(o)}e.summaries[n].update(o,e.ctx.statsOf(t))}})},e}(E),A=n(88),x=n(91),N=function(t){function e(e,n){var r=t.call(this)||this;r.ctx=e,r.body=n,r.start=null;var i=n.parentElement.parentElement,o=i.querySelector("hr");o||(o=i.ownerDocument.createElement("hr"),i.appendChild(o)),r.hr=o;var a=function(t){r.showHint(t)},u=function(t){if(r.body.removeEventListener("mousemove",a),r.body.removeEventListener("mouseup",u),r.body.removeEventListener("mouseleave",u),r.start){var e=r.start.node.classList.contains("lu-row")?r.start.node:r.start.node.closest(".lu-row"),n=r.body.ownerDocument.elementFromPoint(t.clientX,t.clientY),i=n.classList.contains("lu-row")?n:n.closest(".lu-row");r.start=null,r.body.classList.remove("lu-selection-active"),r.hr.classList.remove("lu-selection-active"),r.select(t.ctrlKey,e,i)}};return n.addEventListener("mousedown",function(t){var e=i.getBoundingClientRect();r.start={x:t.clientX,y:t.clientY,xShift:e.left,yShift:e.top,node:t.target},r.body.classList.add("lu-selection-active"),n.addEventListener("mousemove",a,{passive:!0}),n.addEventListener("mouseup",u,{passive:!0}),n.addEventListener("mouseleave",u,{passive:!0})},{passive:!0}),r}return r.c(e,t),e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([e.EVENT_SELECT_RANGE])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.select=function(t,n,r){var i=this;if(n&&r&&n!==r){var o=parseInt(n.dataset.index,10),a=parseInt(r.dataset.index,10),u=Math.min(o,a),s=Math.max(o,a);u!==s&&requestAnimationFrame(function(){return i.fire(e.EVENT_SELECT_RANGE,u,s,t)})}},e.prototype.showHint=function(t){var n=this.start,r=n.y,i=t.clientY,o=Math.abs(r-i)>e.MIN_DISTANCE;this.hr.classList.toggle("lu-selection-active",o),this.hr.style.transform="translate("+(n.x-n.xShift)+"px,"+(r-n.yShift)+"px)scale(1,"+Math.abs(i-r)+")rotate("+(i>r?90:-90)+"deg)"},e.prototype.remove=function(t){t.onclick=void 0},e.prototype.add=function(t){var e=this;t.onclick=function(n){var r=parseInt(t.dataset.i,10);if(n.shiftKey){var i=parseInt(t.dataset.index,10),o=t.parentElement.dataset.ranking;if(Object(x.b)(e.ctx.provider,o,r,i,n.ctrlKey))return}e.ctx.provider.toggleSelection(r,n.ctrlKey)}},e.prototype.selectRange=function(t,e){void 0===e&&(e=!1);var n=new A.a(e?this.ctx.provider.getSelection():[]),r=function(t){n.has(t)?n.delete(t):n.add(t)};t.forEach(function(t){Object(s.b)(t)?t.rows.forEach(function(t){return r(t.i)}):r(t.i)}),this.ctx.provider.setSelection(Array.from(n))},e.prototype.updateState=function(t,e){this.ctx.provider.isSelected(e)?t.classList.add("lu-selected"):t.classList.remove("lu-selected")},e.prototype.update=function(t,e){var n=parseInt(t.dataset.i,10);e.has(n)?t.classList.add("lu-selected"):t.classList.remove("lu-selected")},e.EVENT_SELECT_RANGE="selectRange",e.MIN_DISTANCE=10,e}(a.a),T=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.c(e,t),e.prototype.fire=function(e){for(var n=[],r=1;r<arguments.length;r++)n[r-1]=arguments[r];t.prototype.fire.apply(this,[e].concat(n))},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([e.EVENT_WIDTH_CHANGED,e.EVENT_UPDATE_DATA,e.EVENT_UPDATE_HIST,e.EVENT_HIGHLIGHT_CHANGED])},e.EVENT_WIDTH_CHANGED="widthChanged",e.EVENT_UPDATE_DATA="updateData",e.EVENT_UPDATE_HIST="updateHist",e.EVENT_HIGHLIGHT_CHANGED="highlightChanged",e}(a.a),O=function(t){function e(n,r,a,c,l,f,g){void 0===g&&(g={});var m=t.call(this,r,a,c,l,{mixins:[i.PrefetchMixin],batchSize:10})||this;m.ranking=n,m.ctx=f,m.data=[],m.highlight=-1,m.canvasPool=[],m.oldLeft=0,m.events=new T,m.roptions={animation:!0,levelOfDetail:function(){return"high"},customRowUpdate:function(){},flags:{disableFrozenColumns:!1}},m.canvasMouseHandler={timer:-1,enter:function(t){var e=m.canvasMouseHandler;e.timer>0&&self.clearTimeout(e.timer);var n=t.currentTarget;n.addEventListener("mouseleave",e.leave,{passive:!0}),e.timer=self.setTimeout(function(){return m.updateHoveredRow(n,!0)},o.b)},leave:function(t){var n=m.canvasMouseHandler;n.timer>0&&(self.clearTimeout(n.timer),n.timer=-1);var r=t.currentTarget;e.isCanvasRenderedRow(r)||self.setTimeout(function(){return m.updateHoveredRow(r,!1)}),r.removeEventListener("mouseleave",n.leave)}},m.highlightHandler={enabled:!1,enter:function(t){if(m.highlight>=0){var n=m.body.querySelector(".le-highlighted");n&&n.classList.remove("le-highlighted"),m.highlight=-1}var r=t.currentTarget,i=parseInt(r.dataset.i||"-1",10);m.events.fire(e.EVENT_HIGHLIGHT_CHANGED,i)},leave:function(){if(m.highlight>=0){var t=m.body.querySelector(".le-highlighted");t&&t.classList.remove("le-highlighted"),m.highlight=-1}m.events.fire(e.EVENT_HIGHLIGHT_CHANGED,-1)}},Object.assign(m.roptions,g),a.classList.add("lu-row-body"),a.dataset.ranking=n.id;var b=m;return m.delayedUpdate=Object(u.a)(function(){this.type===p.b.EVENT_DIRTY_VALUES?this.primaryType!==d.e.EVENT_RENDERER_TYPE_CHANGED&&this.primaryType!==d.e.EVENT_GROUP_RENDERER_TYPE_CHANGED&&this.primaryType!==d.e.EVENT_LABEL_CHANGED&&b.updateBody():b.events.fire(e.EVENT_UPDATE_DATA)},50,function(t,e){return t.self.type===p.b.EVENT_ORDER_CHANGED?t:e}),m.delayedUpdateAll=Object(u.a)(function(){return m.updateAll()},50),m.delayedUpdateColumnWidths=Object(u.a)(function(){return m.updateColumnWidths()},50),n.on(p.b.EVENT_ADD_COLUMN+".hist",function(t,e){m.columns.splice(e,0,m.createCol(t,e)),m.reindex(),m.updateHist(t),m.delayedUpdateAll()}),n.on(p.b.EVENT_REMOVE_COLUMN+".body",function(t,n){e.disableListener(t),m.columns.splice(n,1),m.reindex(),m.delayedUpdateAll()}),n.on(p.b.EVENT_MOVE_COLUMN+".body",function(t,e,n){var r=m.columns.splice(n,1)[0];console.assert(r.c===t),m.columns.splice(n<e?e-1:e,0,r),m.reindex(),m.delayedUpdateAll()}),n.on(p.b.EVENT_COLUMN_VISIBILITY_CHANGED+".body",function(t,r,i){if(i){var o=n.children.indexOf(t);m.columns.splice(o,0,m.createCol(t,o)),m.updateHist(t)}else o=m.columns.findIndex(function(e){return e.c===t}),e.disableListener(t),m.columns.splice(o,1);m.reindex(),m.delayedUpdateAll()}),n.on(p.b.EVENT_ORDER_CHANGED+".body",m.delayedUpdate),m.selection=new N(m.ctx,a),m.selection.on(N.EVENT_SELECT_RANGE,function(t,e,n){m.selection.selectRange(m.data.slice(t,e+1),n)}),m.renderCtx=Object.assign({isGroup:function(t){return Object(s.b)(m.data[t])},getRow:function(t){return m.data[t]},getGroup:function(t){return m.data[t]}},f),m.columns=n.children.filter(function(t){return t.isVisible()}).map(function(t,e){return m.createCol(t,e)}),m._context=Object.assign({columns:m.columns,column:Object(i.nonUniformContext)(m.columns.map(function(t){return t.width}),100,h.d)},Object(i.uniformContext)(0,20)),m.columns.forEach(function(t){t instanceof w&&t.updateWidthRule(m.style),t.renderers=m.ctx.createRenderer(t.c)}),m}return r.c(e,t),e.prototype.on=function(t,e){return this.events.on(t,e),this},Object.defineProperty(e.prototype,"id",{get:function(){return this.ranking.id},enumerable:!0,configurable:!0}),e.prototype.onVisibilityChanged=function(e){t.prototype.onVisibilityChanged.call(this,e),e&&this.delayedUpdate.call({type:p.b.EVENT_ORDER_CHANGED})},e.prototype.updateHeaders=function(){return t.prototype.updateHeaders.call(this)},Object.defineProperty(e.prototype,"currentData",{get:function(){return this.data},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"context",{get:function(){return this._context},enumerable:!0,configurable:!0}),e.prototype.createHeader=function(t,e){return e.createHeader()},e.prototype.updateHeader=function(t,e){return e instanceof w&&e.updateWidthRule(this.style),e.updateHeader(t)},e.prototype.createCell=function(t,e,n){return n.createCell(e)},e.prototype.updateCell=function(t,e,n){return n.updateCell(t,e)},e.prototype.selectCanvas=function(){if(this.canvasPool.length>0)return this.canvasPool.pop();var t=this.body.ownerDocument.createElement("canvas");return t.classList.add("lu-detail-canvas"),t},e.prototype.rowFlags=function(t){var e=t,n=e.__lu__;return null==n?e.__lu__={}:n},e.prototype.visibleRenderedWidth=function(){for(var t=0,e=0,n=this.visibleColumns.frozen;e<n.length;e++){var r=n[e];t+=this.columns[r].width+h.d}for(r=this.visibleColumns.first;r<=this.visibleColumns.last;++r)t+=this.columns[r].width+h.d;return t>0&&(t-=h.d),t},e.prototype.renderRow=function(t,e,n){void 0===n&&(n=this.visibleRenderedWidth()),t.width=n,t.style.width=n+"px",t.height=h.c;var r=t.getContext("2d");r.imageSmoothingEnabled=!1,r.clearRect(0,0,t.width,t.height),r.save();for(var i=0,o=this.visibleColumns.frozen;i<o.length;i++){var a=o[i];(s=this.columns[a]).renderCell(r,e);var u=s.width+h.d;r.translate(u,0)}for(a=this.visibleColumns.first;a<=this.visibleColumns.last;++a){var s;(s=this.columns[a]).renderCell(r,e),u=s.width+h.d,r.translate(u,0)}r.restore()},e.prototype.updateCanvasCell=function(t,e,n,r){var i=t.getContext("2d");i.clearRect(r-1,0,n.width+2,t.height),i.save(),i.translate(r,0),n.renderCell(i,e),i.restore()},e.prototype.reindex=function(){this.columns.forEach(function(t,e){t.index=e})},e.prototype.updateAll=function(){var n=this;this.columns.forEach(function(t,e){t.index=e,t.renderers=n.ctx.createRenderer(t.c)}),this._context=Object.assign({},this._context,{column:Object(i.nonUniformContext)(this.columns.map(function(t){return t.width}),100,h.d)}),t.prototype.recreate.call(this),this.events.fire(e.EVENT_WIDTH_CHANGED)},e.prototype.updateBody=function(){var n=this;this.hidden||(this.events.fire(e.EVENT_WIDTH_CHANGED),t.prototype.forEachRow.call(this,function(t,e){return n.updateRow(t,e)}))},e.prototype.updateHeaderOf=function(t){var e=this._context.columns.findIndex(function(e){return e.c===t});if(e<0)return!1;var n=this.header.children[e],r=this._context.columns[e];return n&&r&&this.updateHeader(n,r),n&&r},e.prototype.createRow=function(e,n){if(e.classList.add("lu-row"),this.roptions.customRowUpdate(e,n),this.highlightHandler.enabled&&(e.addEventListener("mouseenter",this.highlightHandler.enter,{passive:!0}),this.rowFlags(e).highlight=!0),this.renderCtx.isGroup(n))return e.dataset.agg="group",void t.prototype.createRow.call(this,e,n);var r=this.renderCtx.getRow(n),i=r.i,o=r.meta;e.classList.toggle("le-highlighted",this.highlight===i),e.dataset.i=i.toString(),e.dataset.agg="detail",o?e.dataset.meta=o:delete e.dataset.meta,this.selection.updateState(e,i),this.selection.add(e);var a=this.roptions.levelOfDetail(n);if("high"===a?delete e.dataset.lod:e.dataset.lod=a,"high"===a||o||this.ctx.provider.isSelected(i))t.prototype.createRow.call(this,e,n);else{var u=this.selectCanvas();e.appendChild(u),this.renderRow(u,n),e.addEventListener("mouseenter",this.canvasMouseHandler.enter,{passive:!0})}},e.prototype.updateRow=function(e,n,r){this.roptions.customRowUpdate(e,n);var i=this.roptions.levelOfDetail(n),o=r||i,a=e.dataset.lod||"high",u=this.renderCtx.isGroup(n),s="group"===e.dataset.agg;if("high"===i?delete e.dataset.lod:e.dataset.lod=i,this.highlightHandler.enabled&&!this.rowFlags(e).highlight&&(e.addEventListener("mouseenter",this.highlightHandler.enter,{passive:!0}),this.rowFlags(e).highlight=!0),e.removeEventListener("mouseenter",this.canvasMouseHandler.enter),u!==s&&(e.innerHTML="",e.dataset.agg=u?"group":"detail",u?(e.dataset.i="",this.selection.remove(e)):this.selection.add(e)),u)return e.classList.remove("le-highlighted"),void t.prototype.updateRow.call(this,e,n);var c=this.renderCtx.getRow(n),l=c.i,d=c.meta;e.classList.toggle("le-highlighted",this.highlight===l),e.dataset.i=l.toString(),d?e.dataset.meta=d:delete e.dataset.meta,this.selection.updateState(e,l);var p=Array.from(e.children).find(function(t){return t.classList.contains("lu-detail-canvas")});if("high"===o||d||this.ctx.provider.isSelected(l))return p&&(this.canvasPool.push(p),p.remove()),void t.prototype.updateRow.call(this,e,n);if(e.addEventListener("mouseenter",this.canvasMouseHandler.enter,{passive:!0}),"high"!==a&&p)this.renderRow(p,n);else{e.innerHTML="",e.dataset.agg="detail";var f=this.selectCanvas();e.appendChild(f),this.renderRow(f,n)}},e.prototype.updateCanvasBody=function(){var n=this,r=this.visibleRenderedWidth();t.prototype.forEachRow.call(this,function(t,i){e.isCanvasRenderedRow(t)&&n.renderRow(t.querySelector("canvas"),i,r)})},e.prototype.updateShifts=function(e,n){t.prototype.updateShifts.call(this,e,n),n!==this.oldLeft&&(this.oldLeft=n,this.updateCanvasBody())},e.prototype.enableHighlightListening=function(e){var n=this;if(this.highlightHandler.enabled!==e){if(this.highlightHandler.enabled=e,e)return this.body.addEventListener("mouseleave",this.highlightHandler.leave,{passive:!0}),void t.prototype.forEachRow.call(this,function(t){t.addEventListener("mouseenter",n.highlightHandler.enter,{passive:!0}),n.rowFlags(t).highlight=!0});this.body.removeEventListener("mouseleave",this.highlightHandler.leave),t.prototype.forEachRow.call(this,function(t){t.removeEventListener("mouseenter",n.highlightHandler.enter),n.rowFlags(t).highlight=!1})}},e.prototype.updateHoveredRow=function(t,n){if(e.isCanvasRenderedRow(t)===n){var r=parseInt(t.dataset.index,10);this.updateRow(t,r,n?"high":"low")}},e.prototype.forEachRow=function(n,r){return void 0===r&&(r=!1),t.prototype.forEachRow.call(this,function(t,r){e.isCanvasRenderedRow(t)||n(t,r)},r)},e.prototype.updateSelection=function(e){var n=this;t.prototype.forEachRow.call(this,function(t,r){n.renderCtx.isGroup(r)?n.updateRow(t,r):n.selection.update(t,e)},!0)},e.prototype.updateColumnWidths=function(){var n=this;this._context.column=Object(i.nonUniformContext)(this._context.columns.map(function(t){return t.width}),100,h.d),t.prototype.updateColumnWidths.call(this),this.context.columns.forEach(function(t){t instanceof w&&t.updateWidthRule(n.style),t.renderers=n.ctx.createRenderer(t.c)}),this.events.fire(e.EVENT_WIDTH_CHANGED)},e.prototype.updateHist=function(t){this.events.fire(e.EVENT_UPDATE_HIST,t)},e.prototype.updateColumn=function(n){var r=this,o=this.context.columns,a=o[n];if(!a)return!1;for(var u=0,s=this.visibleColumns.first;s<n;++s)u+=o[s].width+h.d;return t.prototype.forEachRow.call(this,function(t,o){if(e.isCanvasRenderedRow(t))r.updateCanvasCell(t.querySelector("canvas"),o,a,u);else{var s=t.children[n];if(s){var c=r.updateCell(s,o,a);s!==c&&c&&(Object(i.setColumn)(c,a),t.replaceChild(c,s))}}}),!0},e.prototype.destroy=function(){t.prototype.destroy.call(this),this.ranking.flatColumns.forEach(function(t){return e.disableListener(t)})},e.prototype.groupData=function(t){var e=this,n=this.ranking.getGroups(),r=this.ctx.provider,i=function(t,e){return 1===e?"first last":0===t?"first":t===e-1?"last":void 0};if(1===n.length)return r.isAggregated(this.ranking,n[0])?[Object.assign({rows:t},n[0])]:t.map(function(e,r){return Object.assign({group:n[0],relativeIndex:r,meta:i(r,t.length)},e)});var o=0,a=[];return n.forEach(function(n){var u=n.order.length,s=t.slice(o,o+u);o+=u,r.isAggregated(e.ranking,n)?a.push(Object.assign({rows:s},n)):a.push.apply(a,s.map(function(t,e){return Object.assign({group:n,relativeIndex:e,meta:i(e,s.length)},t)}))}),a},e.prototype.render=function(e,n){var r=this,o=this._context,a=this.data;if(this.data=e,this.renderCtx.totalNumberOfRows=e.length,this.columns.forEach(function(t,e){t.index=e,t.renderers=r.ctx.createRenderer(t.c)}),this._context=Object.assign({columns:this.columns,column:Object(i.nonUniformContext)(this.columns.map(function(t){return t.width}),100,h.d)},n),this.bodyScroller)return t.prototype.recreate.call(this,this.roptions.animation?function(t,e,n){var r,o=function(t){return m(e[t])},a=function(t){return m(n[t])};if(e.reduce(function(t,e){return t+(Object(s.b)(e)?1:0)},0)===n.reduce(function(t,e){return t+(Object(s.b)(e)?1:0)},0))return{currentKey:a,previous:t,previousKey:o};var u,c=[Object.assign({},i.defaultPhases[0],{apply:function(t,a){i.defaultPhases[0].apply(t),t.mode===i.EAnimationMode.SHOW&&(t.node.style.transform="translate(0, "+(function(t,i,a){r||(r=g(e));var u=n[t],c=Object(s.b)(u)?r.group2firstItemIndex.get(u.name):r.item2groupIndex.get(u.i);if(void 0===c)return a;var l=i.posByKey(o(c));return l.pos>=0?l.pos:a}(t.current.index,a,t.previous.y)-t.nodeY)+"px)")}}),Object.assign({},i.defaultPhases[1],{apply:function(t,r,o){i.defaultPhases[1].apply(t),t.mode===i.EAnimationMode.HIDE&&(t.node.style.transform="translate(0, "+(function(t,r,i){u||(u=g(n));var o=e[t],c=Object(s.b)(o)?u.group2firstItemIndex.get(o.name):u.item2groupIndex.get(o.i);if(void 0===c)return i;var l=r.posByKey(a(c));return l.pos>=0?l.pos:i}(t.previous.index,o,t.current.y)-t.nodeY)+"px)")}}),i.defaultPhases[i.defaultPhases.length-1]];return{previous:t,previousKey:o,currentKey:a,phases:c}}(o,a,this.data):void 0)},e.prototype.setHighlight=function(t){this.highlight=t;var e=this.body.querySelector("[data-i].le-highlighted");if(e&&e.classList.remove("le-highlighted"),!(t<0)){var n=this.body.querySelector('[data-i="'+t+'"]');return n&&n.classList.add("le-highlighted"),null!=n}},e.prototype.findNearest=function(t){var e=this,n=t.find(function(t){return Boolean(e.body.querySelectorAll('[data-i="'+t+'"]'))});if(null!=n)return n;for(var r=this.visible,i=new Set(t),o=-1,a=-1,u=r.first;u>=0;--u){var c=this.data[u];if(!Object(s.b)(c)&&i.has(c.i)){o=u;break}}for(u=r.last;u<this.data.length;++u)if(c=this.data[u],!Object(s.b)(c)&&i.has(c.i)){a=u;break}if(o<0&&o<0)return-1;var l=o>=0&&r.first-o<a-r.last?o:a;return this.data[l].i},e.prototype.scrollIntoView=function(t){var e=this,n=this.body.querySelector('[data-i="'+t+'"]');if(n)return n.scrollIntoView(!0),!0;var r=this.data.findIndex(function(e){return!Object(s.b)(e)&&e.i===t});if(r<0)return!1;var i=function(){var t=e._context;if(0===t.exceptions.length||r<t.exceptions[0].index)return r*t.defaultRowHeight;var n=t.exceptions.reverse().find(function(t){return t.index<=r});if(!n)return-1;if(n.index===r)return n.y;var i=r-n.index-1;return n.y2+i*t.defaultRowHeight}();if(i<0)return!1;var o=this.bodyScroller;if(!o)return!1;var a=o.scrollTop;o.scrollTop=Math.min(i,o.scrollHeight-o.clientHeight),this.onScrolledVertically(o.scrollTop,o.clientHeight,a<o.scrollTop);var u=this.body.querySelector('[data-i="'+t+'"]');return!!u&&(u.scrollIntoView(!0),!0)},e.prototype.getHighlight=function(){var t=this.body.querySelector("[data-i]:hover, [data-i].le-highlighted");return t?parseInt(t.dataset.i,10):this.highlight},e.prototype.createCol=function(t,n){var r=this,i=Object(c.c)(t)&&!t.getCollapsed()?new w(t,n,this.renderCtx,this.roptions.flags):new E(t,n,this.renderCtx,this.roptions.flags);t.on(d.e.EVENT_WIDTH_CHANGED+".body",function(){r.delayedUpdateColumnWidths()}),t.on(l.b.EVENT_DATA_LOADED+".hist",function(){return r.updateHist(t)});var o=Object(u.a)(function(){r.updateColumn(i.index)||e.disableListener(t)},25);t.on([d.e.EVENT_RENDERER_TYPE_CHANGED+".body",d.e.EVENT_GROUP_RENDERER_TYPE_CHANGED+".body"],function(){i.renderers=r.ctx.createRenderer(t),o()});var a=this;return t.on(d.e.EVENT_DIRTY_HEADER+".body",function(){a.updateHeaderOf(i.c)||e.disableListener(t),this.primaryType!==d.e.EVENT_SUMMARY_RENDERER_TYPE_CHANGED&&this.primaryType!==d.e.EVENT_LABEL_CHANGED&&this.primaryType!==d.e.EVENT_METADATA_CHANGED&&self.setTimeout(function(){return a.updateHist(i.c)},50)}),t.on(d.e.EVENT_SUMMARY_RENDERER_TYPE_CHANGED+".body",function(){i.renderers=r.ctx.createRenderer(t),r.updateHeaderOf(i.c)||e.disableListener(t)}),t.on(d.e.EVENT_DIRTY_VALUES+".body",o),Object(c.c)(t)&&!t.getCollapsed()&&(i.updateWidthRule(this.style),t.on(f.b.EVENT_MULTI_LEVEL_CHANGED+".body",function(){i.updateWidthRule(r.style)}),t.on(f.b.EVENT_MULTI_LEVEL_CHANGED+".bodyUpdate",o)),i},e.isCanvasRenderedRow=function(t){return"low"===t.dataset.lod&&1===t.childElementCount&&t.firstElementChild.classList.contains("lu-detail-canvas")},e.disableListener=function(t){t.on(d.e.EVENT_WIDTH_CHANGED+".body",null),t.on(l.b.EVENT_DATA_LOADED+".hist",null),t.on([d.e.EVENT_RENDERER_TYPE_CHANGED+".body",d.e.EVENT_GROUP_RENDERER_TYPE_CHANGED+".body",d.e.EVENT_SUMMARY_RENDERER_TYPE_CHANGED+".body",d.e.EVENT_LABEL_CHANGED+".body"],null),t.on(p.b.EVENT_DIRTY_HEADER+".body",null),t.on(p.b.EVENT_DIRTY_VALUES+".body",null),Object(c.c)(t)&&(t.on(f.b.EVENT_MULTI_LEVEL_CHANGED+".body",null),t.on(f.b.EVENT_MULTI_LEVEL_CHANGED+".bodyUpdate",null))},e.EVENT_WIDTH_CHANGED=T.EVENT_WIDTH_CHANGED,e.EVENT_UPDATE_DATA=T.EVENT_UPDATE_DATA,e.EVENT_UPDATE_HIST=T.EVENT_UPDATE_HIST,e.EVENT_HIGHLIGHT_CHANGED=T.EVENT_HIGHLIGHT_CHANGED,e}(i.ACellTableSection);e.c=O},function(t,e,n){"use strict";function r(t,e,n){void 0===e&&(e=100);var r=-1,i=null;return function(){for(var o=[],a=0;a<arguments.length;a++)o[a]=arguments[a];r>=0&&(clearTimeout(r),r=-1);var u={self:this,args:o};i=i&&n?n(i,u):u,r=self.setTimeout(function(){console.assert(null!=i),t.call.apply(t,[i.self].concat(i.args)),i=null},e)}}n.d(e,"a",function(){return r})},function(t,e,n){"use strict";var r=n(56),i=n(6),o=n(3),a=function(){function t(){this.title="Default"}return t.prototype.canRender=function(t,e){return t instanceof r.b&&e!==i.a.SUMMARY},t.prototype.create=function(t){var e=t.actions;return{template:"<div class='actions lu-hover-only'>"+e.map(function(t){return"<span title='"+t.name+"' class='"+(t.className||"")+"'>"+(t.icon||"")+"</span>"}).join("")+"</div>",update:function(t,n){Object(o.e)(t,function(t,r){t.onclick=function(t){t.preventDefault(),t.stopPropagation(),e[r].action(n)}})},render:o.h}},t.prototype.createGroup=function(t){var e=t.groupActions;return{template:"<div class='actions lu-hover-only'>"+e.map(function(t){return"<span title='"+t.name+"' class='"+(t.className||"")+"'>"+(t.icon||"")+"</span>"}).join("")+"</div>",update:function(t,n,r){Object(o.e)(t,function(t,i){t.onclick=function(t){t.preventDefault(),t.stopPropagation(),e[i].action(n,r)}})}}},t.prototype.createSummary=function(){return o.g},t}(),u=n(26),s=n(5),c=function(){function t(){this.title="Default"}return t.prototype.canRender=function(t){return t instanceof u.c},t.prototype.create=function(t,e){var n=e.colWidth(t);return{template:'<div title="Collapse Group"></div>',update:function(e,n,r,i){e.onclick=function(e){e.preventDefault(),e.stopPropagation(),t.setAggregated(i,!0)}},render:function(t){t.fillStyle=s.a.color,t.fillRect(n-s.a.width-1,0,s.a.strokeWidth,s.c)}}},t.prototype.createGroup=function(t){return{template:'<div title="Expand Group"></div>',update:function(e,n){e.onclick=function(e){e.preventDefault(),e.stopPropagation(),t.setAggregated(n,!1)}}}},t.prototype.createSummary=function(t,e){return{template:'<div title="(Un)Aggregate All" data-icon="caret-down"></div>',update:function(n){var r=t.findMyRanker(),i=Boolean(r&&r.getGroups().every(function(e){return t.isAggregated(e)}));n.dataset.icon=i?"caret-right":"caret-down",n.onclick=function(r){r.stopPropagation();var i=t.findMyRanker();if(i&&e){var o="caret-down"===n.dataset.icon;n.dataset.icon=o?"caret-right":"caret-down",e.provider.aggregateAllOf(i,o)}}}}},t}(),l=n(0),d=n(79),p=n(21),f=n(40),h=n(15),g=function(){function t(){this.title="Default"}return t.prototype.canRender=function(t){return t instanceof p.d},t.prototype.create=function(t){var e=t.alignment||"left";return{template:"<div"+("left"!==e?' class="lu-'+e+'"':"")+"> </div>",update:function(e,n){Object(h.b)(e,t,n),t.escape?Object(o.i)(e,t.getLabel(n)):e.innerHTML=t.getLabel(n)},render:o.h}},t.exampleText=function(t,e){for(var n=[],r=0,i=e;r<i.length;r++){var o=i[r];if(!t.isMissing(o)){var a=t.getLabel(o);if(n.push(a),n.length>=5)break}}return n.join(", ")+(n.length<e.length?", &hellip;":"")},t.prototype.createGroup=function(e){return{template:"<div> </div>",update:function(n,r,i){n.innerHTML=""+t.exampleText(e,i)}}},t.interactiveSummary=function(t,e){var n=e,r=Object(f.c)(e),i=e.querySelector('input[type="text"]'),o=e.querySelector('input[type="checkbox"]'),a=function(){if(i.disabled=r.checked,o.disabled=r.checked,r.checked)t.setFilter(p.d.FILTER_MISSING);else{var e=i.value.trim();r.disabled=e.length>0,e.length<=0?t.setFilter(null):t.setFilter(o.checked?new RegExp(i.value):i.value)}};return r.onchange=a,i.onchange=a,o.onchange=a,n.onsubmit=function(t){return t.preventDefault(),t.stopPropagation(),a(),!1},function(e){var n=(t=e).getFilter()||"",a=n===p.d.FILTER_MISSING;a&&(n=""),r.checked=a,i.value=n instanceof RegExp?n.source:n,o.checked=n instanceof RegExp,r.disabled=i.value.trim().length>0,i.disabled=r.checked,o.disabled=r.checked}},t.prototype.createSummary=function(e,n,r){if(!r)return{template:"<div></div>",update:function(t){var n=e.getFilter()||"";t.textContent=function(t){return null==t||""===t||t===p.d.FILTER_MISSING?"":t instanceof RegExp?t.source:String(t)}(n)}};var i,a=e.getFilter()||"",u=a===p.d.FILTER_MISSING;u&&(a="");var s=Object(o.j)(n.idPrefix);return{template:'<form><input type="text" placeholder="Filter '+e.desc.label+'..." autofocus value="'+(a instanceof RegExp?a.source:a)+'">\n          <div class="lu-checkbox"><input id="'+s+'" type="checkbox" '+(a instanceof RegExp?'checked="checked"':"")+'><label for="'+s+'">RegExp</label></div>\n          '+Object(f.a)(u,n.idPrefix)+"</form>",update:function(n){i||(i=t.interactiveSummary(e,n)),i(e)}}},t}(),m=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.title="Default",e}return l.c(e,t),e.prototype.canRender=function(e){return t.prototype.canRender.call(this,e)&&e instanceof d.a},e.prototype.create=function(t){return{template:"<div class='annotations text'>\n        <input class='lu-hover-only'>\n        <span class='text lu-not-hover'></span>\n       </div>",update:function(e,n){var r=e.firstElementChild;r.onchange=function(){t.setValue(n,r.value)},r.onclick=function(t){t.stopPropagation()},e.lastElementChild.textContent=r.value=t.getLabel(n)},render:o.h}},e}(g),b=n(4),y=n(2),v=n(12);function E(t,e,n,r){return n&&n.color?n.color(e,r):e?t.getColor(e):Object(v.d)(t)&&null!=r?t.getColorMapping().apply(r):t.color}var C=function(){function t(t){void 0===t&&(t=!1),this.renderValue=t,this.title="Bar"}return t.prototype.canRender=function(t,e){return e===i.a.CELL&&Object(b.i)(t)&&!Object(b.k)(t)},t.prototype.create=function(t,e,n,r){var i=e.colWidth(t);return{template:'<div title="">\n          <div style=\'background-color: '+t.color+"'>\n            <span "+(this.renderValue?"":'class="lu-hover-only"')+"></span>\n          </div>\n        </div>",update:function(e,n){var i=t.getNumber(n),a=Object(h.b)(e,t,n),u=isNaN(i)?0:Math.round(100*i*100)/100,s=t.getLabel(n);e.title=s;var c=e.firstElementChild;c.style.width=a?"100%":u+"%";var l=E(t,n,r,i);c.style.backgroundColor=a?null:l,Object(o.i)(c.firstElementChild,s);var d=c.firstElementChild;Object(o.i)(d,s),Object(o.a)(d,l||y.e.DEFAULT_COLOR,s,u/100)},render:function(e,n){if(!Object(h.a)(e,t,n,i)){var o=t.getNumber(n);e.fillStyle=E(t,n,r,o)||y.e.DEFAULT_COLOR;var a=i*o;e.fillRect(0,0,isNaN(a)?0:a,s.c)}}}},t.prototype.createGroup=function(){return o.g},t.prototype.createSummary=function(){return o.g},t}(),w=n(60),A=function(){function t(){this.title="String",this.groupTitle="None",this.summaryTitle="None"}return t.prototype.canRender=function(t,e){return!0},t.prototype.create=function(t){return{template:"<div> </div>",update:function(e,n){Object(h.b)(e,t,n),Object(o.i)(e,t.getLabel(n))},render:o.h}},t.prototype.createGroup=function(t){return o.g},t.prototype.createSummary=function(){return o.g},t}(),x=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.title="Default",e}return l.c(e,t),e.prototype.canRender=function(t,e){return t instanceof w.c&&e===i.a.CELL},e.prototype.create=function(e){var n=t.prototype.create.call(this,e);return n.template='<div class="lu-center"> </div>',n},e}(A),N=n(9),T=n(33),O=n(10),R=function(){function t(){this.title="Box Plot"}return t.prototype.canRender=function(t,e){return Object(b.g)(t)&&e===i.a.CELL||Object(b.i)(t)&&e!==i.a.CELL},t.prototype.create=function(t,e,n,r){var i=t.getSortMethod(),o=void 0!==t.isSortedByMe().asc,a=e.colWidth(t);return{template:'<div title="">\n                    <div><div></div><div></div></div>\n                 </div>',update:function(e,n){var a=t.getBoxPlotData(n);a&&!t.isMissing(n)?(e.classList.remove("lu-missing"),M(e,a,t.getRawBoxPlotData(n),o?i:"",E(t,n,r))):e.classList.add("lu-missing")},render:function(e,n){if(!Object(h.a)(e,t,n,a)){var u=t.getBoxPlotData(n);u&&function(t,e,n,r,i,o){var a=null!=e.whiskerLow?e.whiskerLow:Math.max(e.q1-1.5*(e.q3-e.q1),e.min),u=null!=e.whiskerHigh?e.whiskerHigh:Math.min(e.q3+1.5*(e.q3-e.q1),e.max);t.fillStyle=r||s.b.box,t.strokeStyle=s.b.stroke,t.beginPath(),t.rect(e.q1,0,e.q3-e.q1,i),t.fill(),t.stroke();var c=i-0,l=i/2;t.beginPath(),t.moveTo(a,l),t.lineTo(e.q1,l),t.moveTo(a,0),t.lineTo(a,c),t.moveTo(e.median,0),t.lineTo(e.median,i),t.moveTo(e.q3,l),t.lineTo(u,l),t.moveTo(u,0),t.lineTo(u,c),t.stroke(),t.fill(),""!==n&&(t.strokeStyle=s.b.sort,t.beginPath(),t.moveTo(e[n],0),t.lineTo(e[n],i-0),t.stroke(),t.fill()),e.outlier&&(t.fillStyle=s.b.outlier,e.outlier.forEach(function(e){t.fillRect(Math.max(e-s.f.size/2,0),l-s.f.size/2,s.f.size,s.f.size)}))}(e,{min:u.min*a,median:u.median*a,q1:u.q1*a,q3:u.q3*a,max:u.max*a,outlier:u.outlier?u.outlier.map(function(t){return t*a}):void 0,whiskerLow:null!=u.whiskerLow?u.whiskerLow*a:void 0,whiskerHigh:null!=u.whiskerHigh?u.whiskerHigh*a:void 0},o?i:"",E(t,n,r),s.c)}}}},t.createAggregatedBoxPlot=function(t,e,n){var r;void 0===n&&(n=!1);var i=(r=[]).concat.apply(r,e.map(function(e){return n?t.getRawNumbers(e):t.getNumber(e)}));return new N.g(i)},t.prototype.createGroup=function(e,n,r,i){var o=e instanceof O.c&&void 0!==e.isGroupSortedByMe().asc?e.getSortMethod():"";return{template:'<div title="">\n                    <div><div></div><div></div></div>\n                 </div>',update:function(n,r,a){var u,s;a.every(function(t){return e.isMissing(t)})?Object(h.b)(n,e,a[0]):(n.classList.remove("lu-missing"),Object(b.k)(e)?(u=t.createAggregatedBoxPlot(e,a),s=t.createAggregatedBoxPlot(e,a,!0)):(u=new N.g(a.map(function(t){return e.getNumber(t)})),s=new N.g(a.map(function(t){return e.getRawNumber(t)}))),M(n,u,s,o,E(e,null,i)))}}},t.prototype.createSummary=function(t,e,n,r){var i='<div title="">\n    <div><div></div><div></div></div>';if(Object(v.d)(t)){var o=t.getRange();i+="<span>"+o[0]+"</span><span>"+o[1]+"</span>"}return{template:i+="</div>",update:function(e,n){if(null!=n&&0!==n.count){e.classList.remove("lu-missing");var i=t instanceof O.c&&void 0!==t.isGroupSortedByMe().asc?t.getSortMethod():"";if(Object(v.d)(t)){var o=t.getRange();Array.from(e.querySelectorAll("span")).forEach(function(t,e){return t.textContent=o[e]})}M(e,n,n,i,E(t,null,r),Object(v.d)(t))}else e.classList.add("lu-missing")}}},t}();function M(t,e,n,r,i,o){void 0===o&&(o=!1),t.title=function(t){if(null==t)return"";var e=T.c.DEFAULT_FORMATTER,n=null!=t.mean?"mean = "+e(t.mean)+"\n":"";return"min = "+e(t.min)+"\nq1 = "+e(t.q1)+"\nmedian = "+e(t.median)+"\n"+n+"q3 = "+e(t.q3)+"\nmax = "+e(t.max)}(n);var a=t.firstElementChild,u=a.firstElementChild,s=a.lastElementChild,c=null!=e.whiskerLow?e.whiskerLow:Math.max(e.q1-1.5*(e.q3-e.q1),e.min),l=null!=e.whiskerHigh?e.whiskerHigh:Math.min(e.q3+1.5*(e.q3-e.q1),e.max);a.style.left=Object(N.k)(100*c,2)+"%";var d=l-c;a.style.width=Object(N.k)(100*d,2)+"%",u.style.left=Object(N.k)((e.q1-c)/d*100,2)+"%",u.style.width=Object(N.k)((e.q3-e.q1)/d*100,2)+"%",u.style.backgroundColor=i,s.style.left=Object(N.k)((e.median-c)/d*100,2)+"%";var p=Array.from(t.children).slice(1,o?-2:void 0),f=e.outlier?e.outlier.length:0;if(p.splice(f,p.length-f).forEach(function(t){return t.remove()}),a.dataset.sort=r,e.outlier&&0!==f){for(var h=p.length;h<f;++h){var g=t.ownerDocument.createElement("div");p.unshift(g),a.insertAdjacentElement("afterend",g)}e.outlier.forEach(function(t,e){delete p[e].dataset.sort,p[e].style.left=Object(N.k)(100*t,2)+"%"}),"min"===r&&e.outlier[0]<=c?(a.dataset.sort="",p[0].dataset.sort="min",p.length>1&&p[p.length-1].insertAdjacentElement("afterend",p[0])):"max"===r&&e.outlier[p.length-1]>=l&&(a.dataset.sort="",p[p.length-1].dataset.sort="max")}}var D=n(24);function _(t,e,n,r){if((null==t||isNaN(t))&&(t=1),r||!Object(v.d)(n))return(i=Object(D.d)(E(n,e,r,t)||y.e.DEFAULT_COLOR)).l=1-t,i.toString();var i,o=n.getColorMapping(),a=o.apply(t);return"solid"===o.type?((i=Object(D.d)(a)).l=1-t,i.toString()):a}var L=function(){function t(){this.title="Brightness"}return t.prototype.canRender=function(t,e){return Object(b.i)(t)&&e===i.a.CELL&&!Object(b.k)(t)},t.prototype.create=function(t,e,n,r){var i=e.colWidth(t);return{template:'<div title="">\n        <div style="background-color: '+t.color+'"></div><div> </div>\n      </div>',update:function(e,n){var i=Object(h.b)(e,t,n);e.title=t.getLabel(n),e.firstElementChild.style.backgroundColor=i?null:_(t.getNumber(n),n,t,r),Object(o.i)(e.lastElementChild,e.title)},render:function(e,n){Object(h.a)(e,t,n,i)||(e.fillStyle=_(t.getNumber(n),n,t,r),e.fillRect(0,0,i,s.c))}}},t.prototype.createGroup=function(){return o.g},t.prototype.createSummary=function(){return o.g},t}(),S=n(25),I=n(11),j=n(20),k=n(57),V=function(){function t(){this.title="Color",this.groupTitle="Histogram"}return t.prototype.canRender=function(t){return Object(I.c)(t)},t.prototype.create=function(t,e){var n=e.colWidth(t);return{template:"<div>\n        <div></div><div></div>\n      </div>",update:function(e,n){Object(h.b)(e,t,n);var r=t.getCategory(n);e.firstElementChild.style.backgroundColor=r?r.color:null,Object(o.i)(e.lastElementChild,t.getLabel(n))},render:function(e,r){if(!Object(h.a)(e,t,r,n)){var i=t.getCategory(r);e.fillStyle=i?i.color:"",e.fillRect(0,0,n,s.c)}}}},t.prototype.createGroup=function(t,e,n){var r=H(t,!1),i=r.template,o=r.update;return{template:i+"</div>",update:function(e,r,i){var a=Object(N.h)(i,function(e){return t.getCategory(e)},t.categories),u=a.maxBin,s=a.hist,c=Math.max(u,n?n.maxBin:0);o(e,c,s)}}},t.prototype.createSummary=function(t,e,n){return t instanceof j.a||t instanceof k.b?function(t,e,n){var r,i=H(t,e||Object(o.k)(t)),a=i.template,u=i.update;return{template:""+a+(e?Object(f.b)(!1,0,n):"")+"</div>",update:function(e,n){r||(r=P(t,e)),r(n?n.missing:0,t),e.classList.toggle("lu-missing",!n),n&&u(e,n.maxBin,n.hist)}}}(t,n,e.idPrefix):function(t,e){var n=H(t,e),r=n.template,i=n.update;return{template:r+"</div>",update:function(t,e){t.classList.toggle("lu-missing",!e),e&&i(t,e.maxBin,e.hist)}}}(t,n)},t}();function H(t,e){var n=t.categories.map(function(t){return'<div title="'+t.label+': 0" data-cat="'+t.name+'" '+(e?'data-title="'+t.label+'"':"")+'><div style="height: 0; background-color: '+t.color+'"></div></div>'}).join("");return{template:"<div"+(t.dataLength>S.a?'class="lu-dense"':"")+">"+n,update:function(e,n,r){Object(o.d)(e,"[data-cat]",function(e,i){var o=r[i].y;e.title=t.categories[i].label+": "+o,e.firstElementChild.style.height=Math.round(100*o/n)+"%"})}}}function P(t,e){var n=Array.from(e.querySelectorAll("[data-cat]"));n.forEach(function(e,n){var r=t.categories[n];e.onclick=function(i){i.preventDefault(),i.stopPropagation();var o=t.getFilter();if(null==o||!Array.isArray(o.filter)){var a=t.categories.slice();return e.dataset.filtered="filtered",a.splice(n,1),void t.setFilter({filterMissing:!!o&&o.filterMissing,filter:a.map(function(t){return t.name})})}var u=o.filter.slice(),s=u.indexOf(r.name);s>=0?(e.dataset.filtered="filtered",u.splice(s,1)):(delete e.dataset.filtered,u.push(r.name)),t.setFilter({filterMissing:o.filterMissing,filter:u})}});var r=e.querySelector("input");return r&&(r.onchange=function(){var e=r.checked,n=t.getFilter();null==n?t.setFilter(e?{filterMissing:e,filter:t.categories.map(function(t){return t.name})}:null):t.setFilter({filterMissing:e,filter:n.filter})}),function(e,i){var o=(t=i).categories,a=t.getFilter();n.forEach(function(t,e){Object(I.d)(a,o[e])?delete t.dataset.filtered:t.dataset.filtered="filtered"}),r&&(r.checked=null!=a&&a.filterMissing,Object(f.d)(r.parentElement,e))}}var G=function(){function t(){this.title="Matrix"}return t.prototype.canRender=function(t,e){return Object(I.f)(t)&&e!==i.a.SUMMARY},t.calculateSetPath=function(t,e){var n=[];return t.forEach(function(t,e){return t?n.push(e):-1}),{left:n[0]*e+e/2,right:n[n.length-1]*e+e/2}},t.createDOMContext=function(t){for(var e="",n=0,r=t.categories;n<r.length;n++)e+='<div title="'+r[n].label+'"></div>';return{templateRow:e,render:function(t,e){Array.from(t.children).slice(1).forEach(function(t,n){var r=e[n];t.classList.toggle("enabled",r)});var n=t.firstElementChild,r=e.findIndex(function(t){return t}),i=e.length-1-e.reverse().findIndex(function(t){return t});r<0||r===i?n.style.display="none":(n.style.display=null,n.style.left=Math.round(100*(r+.5)/e.length)+"%",n.style.width=Math.round(100*(i-r)/e.length)+"%")}}},t.prototype.create=function(e,n){var r=t.createDOMContext(e),i=r.templateRow,o=r.render,a=n.colWidth(e),u=a/e.dataLength;return{template:"<div><div></div>"+i+"</div>",update:function(t,n){Object(h.b)(t,e,n)||o(t,e.getValues(n))},render:function(n,r){if(!Object(h.a)(n,e,r,a)){var i=e.getValues(r),o=i.some(function(t){return t});if(n.save(),n.fillStyle=s.h.circle,n.strokeStyle=s.h.stroke,o){var c=t.calculateSetPath(i,u),l=c.left,d=c.right;n.beginPath(),n.moveTo(l,s.c/2),n.lineTo(d,s.c/2),n.stroke()}i.forEach(function(t,e){var r=e*u;n.beginPath(),n.globalAlpha=t?1:s.h.inactive,n.fillRect(r,0,u,s.c),n.fill()}),n.restore()}}}},t.prototype.createGroup=function(e){var n=t.createDOMContext(e),r=n.templateRow,i=n.render;return{template:"<div><div></div>"+r+"</div>",update:function(t,n,r){var o=F(e,r);i(t,o)}}},t.prototype.createSummary=function(){return o.g},t}();function F(t,e){var n=new Set;return e.forEach(function(e){t.getSet(e).forEach(function(t){return n.add(t)})}),t.categories.map(function(t){return n.has(t)})}var U=function(){function t(){this.title="Heatmap"}return t.prototype.canRender=function(t){return Object(I.f)(t)},t.createDOMContext=function(t){for(var e="",n=0,r=t.categories;n<r.length;n++){var i=r[n];e+='<div title="'+i.label+'" style="background-color: '+i.color+'"></div>'}return{templateRow:e,render:function(t,e){Object(o.e)(t,function(t,n){var r=e[n];t.style.visibility=r?null:"hidden"})}}},t.prototype.create=function(e,n){var r=t.createDOMContext(e),i=r.templateRow,o=r.render,a=n.colWidth(e),u=a/e.dataLength,c=e.categories;return{template:"<div>"+i+"</div>",update:function(t,n){Object(h.b)(t,e,n)||o(t,e.getValues(n))},render:function(t,n){if(!Object(h.a)(t,e,n,a)){var r=e.getValues(n);t.save(),c.forEach(function(e,n){if(r[n]){var i=n*u;t.fillStyle=e.color,t.fillRect(i,0,u,s.c)}}),t.restore()}}}},t.prototype.createGroup=function(e){var n=t.createDOMContext(e),r=n.templateRow,i=n.render;return{template:"<div>"+r+"</div>",update:function(t,n,r){var o=F(e,r);i(t,o)}}},t.prototype.createSummary=function(t){for(var e=t.categories,n="<div>",r=Object(o.k)(t),i=0,a=e;i<a.length;i++){var u=a[i];n+='<div title="'+u.label+'"'+(r?' data-title="'+u.label+'"':"")+' style="background-color: '+u.color+'"></div>'}return{template:n+="</div>",update:o.h}},t}(),B=function(){function t(){this.title="Distribution Bar"}return t.prototype.canRender=function(t,e){return Object(I.c)(t)&&e!==i.a.CELL},t.prototype.create=function(){return o.g},t.prototype.createGroup=function(t){var e=z(t),n=e.template,r=e.update;return{template:n+"</div>",update:function(e,n,i){var o=Object(N.h)(i,function(e){return t.getCategory(e)},t.categories),a=o.hist,u=o.missing;r(e,a,u)}}},t.prototype.createSummary=function(t,e,n){return t instanceof j.a||t instanceof k.b?function(t,e,n){var r,i=z(t),o=i.template,a=i.update;return{template:""+o+(e?Object(f.b)(!1,0,n):"")+"</div>",update:function(e,n){r||(r=P(t,e)),r(n?n.missing:0,t),e.classList.toggle("lu-missing",!n),n&&a(e,n.hist,n.missing)}}}(t,n,e.idPrefix):function(t){var e=z(t),n=e.template,r=e.update;return{template:n+"</div>",update:function(t,e){t.classList.toggle("lu-missing",!e),e&&r(t,e.hist,e.missing)}}}(t)},t}();function z(t){var e=t.categories;return{template:"<div>"+e.map(function(t){return'<div style="background-color: '+t.color+"; color: "+Object(o.b)(t.color)+'" title="'+t.label+': 0" data-cat="'+t.name+'"><span>'+t.label+"</span></div>"}).join("")+'<div title="Missing Values"></div>',update:function(t,n,r){var i=n.reduce(function(t,e){return t+e.y},r);Object(o.e)(t,function(t,o){var a,u;o>=n.length?(a=r,u="Missing Values"):(a=n[o].y,u=e[o].label),t.style.flexGrow=""+Math.round(0===i?0:a),t.title=u+": "+a})}}}var Y=function(){function t(){this.title="Proportional Symbol"}return t.prototype.canRender=function(t,e){return Object(b.i)(t)&&e===i.a.CELL&&!Object(b.k)(t)},t.prototype.create=function(t,e,n,r){return{template:'<div style="background: radial-gradient(circle closest-side, red 100%, transparent 100%)" title="">\n              <div class="lu-hover-only"></div>\n          </div>',update:function(e,n){var i=t.getNumber(n),a=Math.round(100*i),u=Object(h.b)(e,t,n);Object(o.c)(e,{},{background:u?null:"radial-gradient(circle closest-side, "+E(t,n,r)+" "+a+"%, transparent "+a+"%)"}),Object(o.i)(e.firstElementChild,t.getLabel(n))},render:o.h}},t.prototype.createGroup=function(){return o.g},t.prototype.createSummary=function(){return o.g},t}(),W=n(7),q=function(){function t(){this.title="Dot",this.groupTitle="Dots"}return t.prototype.canRender=function(t,e){return Object(b.i)(t)&&e!==i.a.SUMMARY},t.getDOMRenderer=function(t){for(var e=Object(b.k)(t)?t.dataLength:1,n="",r=0;r<e;++r)n+="<div style='background-color: "+t.color+"' title=''></div>";return{template:"<div>"+n+"</div>",update:function(t,e,n,r){if(t.children.length!==e.length){for(var i="",a=0;a<e.length;++a)i+="<div style='background-color: "+r[a]+"' title='"+n[a]+"'></div>";t.innerHTML=i}Object(o.e)(t,function(t,i){var a=e[i];Object(o.c)(t,{title:n[i]},{display:Object(W.c)(a)?"none":null,left:Math.round(100*a)+"%",top:e.length>1?Math.round(80*Math.random()+10)+"%":null,"background-color":r[i]})})},render:function(t,e,n,r){t.save(),t.globalAlpha=s.f.opacity,e.forEach(function(e,i){t.fillStyle=n[i]||s.f.color,t.fillRect(Math.max(0,e*r-s.f.size/2),0,s.f.size,s.c)}),t.restore()}}},t.prototype.create=function(e,n,r,i){var o=t.getDOMRenderer(e),a=o.template,u=o.render,s=o.update,c=n.colWidth(e);return{template:a,update:function(t,n){if(!Object(h.b)(t,e,n)){var r=E(e,n,i),o=e.getValue(n);if(!Object(b.k)(e))return s(t,[o],[e.getLabel(n)],[r]);var a=o.filter(function(t){return!Object(W.c)(t)});return s(t,a,a.map(b.a),a.map(function(t){return r}))}},render:function(t,n){if(!Object(h.a)(t,e,n,c)){var r=E(e,n,i),o=e.getValue(n);if(!Object(b.k)(e))return u(t,[o],[r],c);var a=o.filter(function(t){return!Object(W.c)(t)});return u(t,a,a.map(function(t){return r}),c)}}}},t.prototype.createGroup=function(e,n,r,i){var o=t.getDOMRenderer(e),a=o.template,u=o.update;return{template:a,update:function(t,n,r){var o,a=r.map(function(t){return e.getValue(t)}),s=r.map(function(t){return E(e,t,i)});if(!Object(b.k)(e))return u(t,a,r.map(function(t){return e.getLabel(t)}),s);var c=(o=[]).concat.apply(o,a.filter(function(t){return!Object(W.c)(t)}));return u(t,c,c.map(b.a),a.map(function(t,e){return s[e]}))}}},t.prototype.createSummary=function(){return o.g},t}(),K=n(61),J=function(){function t(){this.title="Default"}return t.prototype.canRender=function(t){return t instanceof K.c},t.prototype.create=function(){return{template:"<div><div></div></div>",update:function(t,e,n,r){var i=t.firstElementChild;0!==n?i.innerText="":Array.isArray(r.order)?i.innerText=r.name+" ("+r.order.length+")":i.innerText=r.name},render:o.h}},t.prototype.createGroup=function(){return{template:"<div><div></div></div>",update:function(t,e,n){t.firstElementChild.innerText=e.name+" ("+n.length+")"}}},t.prototype.createSummary=function(){return o.g},t}(),Z=function(){function t(){}return t.choose=function(t,e){for(var n=e.map(function(e){return{n:t.getNumbers(e),raw:t.getRawNumbers(e)}}),r=t.dataLength,i=[],o=[],a=function(e){var r=n.map(function(t){return{n:t.n[e],raw:t.raw[e]}}).filter(function(t){return!Object(W.c)(t.n)});if(0===r.length)i.push(NaN),o.push(NaN);else{var a=new N.g(r.map(function(t){return t.n})),u=new N.g(r.map(function(t){return t.raw}));i.push(a[t.getSortMethod()]),o.push(u[t.getSortMethod()])}},u=0;u<r;++u)a(u);return{normalized:i,raw:o}},t.prototype.create=function(t,e,n,r){var i=e.colWidth(t),o=this.createContext(t,e,r),a=o.templateRow,u=o.render,s=o.update;return{template:"<div>"+a+"</div>",update:function(e,n){Object(h.b)(e,t,n)||s(e,t.getNumbers(n),t.getRawNumbers(n),n)},render:function(e,n){Object(h.a)(e,t,n,i)||u(e,t.getNumbers(n),n)}}},t.prototype.createGroup=function(e,n,r,i){var o=this.createContext(e,n,i),a=o.templateRow,u=o.update;return{template:"<div>"+a+"</div>",update:function(n,r,i){var o=t.choose(e,i),a=o.normalized,s=o.raw;u(n,a,s,i[0])}}},t}(),Q=function(){function t(){this.title="Heatmap"}return t.prototype.canRender=function(t){return Object(b.k)(t)&&Boolean(t.dataLength)},t.prototype.createContext=function(t,e,n,r){var i=e.colWidth(t),o=i/t.dataLength,a=t.labels;return{template:'<canvas height="20" title=""></canvas>',render:function(e,n,i,a){n.forEach(function(n,u){var s=u*o;Object(W.c)(n)?Object(h.c)(e,o,a,s,0):(e.fillStyle=_(n,i,t,r),e.fillRect(s,0,o,a))})},width:i,mover:function(e,n){return function(r){var o=r.offsetX/i,u=Math.max(0,Math.min(t.dataLength-1,Math.floor(o*(t.dataLength-1)+.5)));e.title=a[u]+": "+n[u]}}}},t.prototype.create=function(t,e,n,r){var i=this.createContext(t,e,n,r),o=i.template,a=i.render,u=i.mover,c=i.width;return{template:o,update:function(e,n){var r=e.getContext("2d");r.canvas.width=c,r.clearRect(0,0,r.canvas.width,r.canvas.height),Object(h.b)(e,t,n)||(e.onmousemove=u(e,t.getRawNumbers(n).map(b.a)),e.onmouseleave=function(){return e.title=""},a(r,t.getNumbers(n),n,20))},render:function(e,n){a(e,t.getNumbers(n),n,s.c)}}},t.prototype.createGroup=function(t,e,n,r){var i=this.createContext(t,e,n,r),o=i.template,a=i.render,u=i.mover,s=i.width;return{template:o,update:function(e,n,r){var i=Z.choose(t,r),o=i.normalized,c=i.raw,l=e.getContext("2d");l.canvas.width=s,l.clearRect(0,0,l.canvas.width,l.canvas.height),e.onmousemove=u(e,c.map(b.a)),e.onmouseleave=function(){return e.title=""},a(l,o,r[0],20)}}},t.prototype.createSummary=function(t){for(var e=t.labels.slice();e.length>0&&!Object(o.k)(t,e.length);)e=e.filter(function(t,e){return e%2==0});for(var n="<div>",r=0,i=e;r<i.length;r++){var a=i[r];n+='<div title="'+a+'" data-title="'+a+'"></div>'}return{template:n+="</div>",update:o.h}},t}(),X=function(t){function e(e,n,r){void 0===r&&(r={});var i=t.call(this,e)||this;return i.callback=n,i.ioptions={min:NaN,max:NaN,step:"any",value:NaN,label:null},Object.assign(i.ioptions,r),i}return l.c(e,t),e.prototype.build=function(t){var e=this.ioptions;t.insertAdjacentHTML("beforeend",'\n     <input type="number" value="'+(isNaN(e.value)?"":String(e.value))+'" required autofocus placeholder="'+(e.label?e.label:"enter number")+'" '+(isNaN(e.min)?"":' min="'+e.min+'"')+" "+(isNaN(e.max)?"":' max="'+e.max+'"')+' step="'+e.step+'">\n    ')},e.prototype.submit=function(){return this.callback(parseFloat(this.findInput("input[type=number]").value)),!0},e}(n(17).c),$=n(78),tt=function(){function t(){this.title="Histogram"}return t.prototype.canRender=function(t,e){return Object(b.i)(t)&&e!==i.a.CELL||Object(b.k)(t)&&e===i.a.CELL},t.prototype.create=function(t,e,n,r){var i=nt(e.totalNumberOfRows,t,r),a=i.template,u=i.render,s=i.guessedBins;return{template:a+"</div>",update:function(e,r){Object(h.b)(e,t,r)||u(e,et(n,s,[r],t))},render:o.h}},t.prototype.createGroup=function(t,e,n,r){var i=nt(e.totalNumberOfRows,t,r),o=i.template,a=i.render,u=i.guessedBins;return{template:o+"</div>",update:function(e,r,i){a(e,et(n,u,i,t))}}},t.prototype.createSummary=function(t,e,n,r){var i=nt(e.totalNumberOfRows,t,r);return n&&Object(v.d)(t)?function(t,e,n,r){var i,o=rt(t);return{template:(n+='\n      <div data-handle="min-hint" style="width: '+o.percent(o.filterMin)+'%"></div>\n      <div data-handle="max-hint" style="width: '+(100-o.percent(o.filterMax))+'%"></div>\n      <div data-handle="min" data-value="'+Object(N.k)(o.filterMin,2)+'" style="left: '+o.percent(o.filterMin)+'%" title="min filter, drag or shift click to change"></div>\n      <div data-handle=\'max\' data-value="'+Object(N.k)(o.filterMax,2)+'" style="right: '+(100-o.percent(o.filterMax))+'%" title="max filter, drag or shift click to change"></div>\n      '+Object(f.b)(o.filterMissing,0,e.idPrefix)+"\n    ")+"</div>",update:function(n,o){i||(i=function(t,e,n){var r=t.querySelector("[data-handle=min]"),i=t.querySelector("[data-handle=max]"),o=t.querySelector("[data-handle=min-hint]"),a=t.querySelector("[data-handle=max-hint]"),u=t.querySelector("input"),s=function(){var t=rt(e),n=t.unpercent(parseFloat(r.style.left)),o=t.unpercent(100-parseFloat(i.style.right));e.setFilter({filterMissing:u.checked,min:Math.abs(n-t.domain[0])<.001?NaN:n,max:Math.abs(o-t.domain[1])<.001?NaN:o})};r.onclick=function(t){if(t.shiftKey){t.preventDefault(),t.stopPropagation();var i=rt(e),a=i.unpercent(parseFloat(r.style.left)),u={attachment:r,manager:n.dialogManager,level:1,idPrefix:n.idPrefix},c=new X(u,function(t){o.style.width=i.percent(t)+"%",r.dataset.value=Object(N.k)(t,2).toString(),r.style.left=i.percent(t)+"%",s()},{value:a,min:i.domain[0],max:i.domain[1]});c.open()}},i.onclick=function(t){if(t.shiftKey){t.preventDefault(),t.stopPropagation();var r=rt(e),o=r.unpercent(100-parseFloat(i.style.right)),u={attachment:i,manager:n.dialogManager,level:1,idPrefix:n.idPrefix},c=new X(u,function(t){a.style.width=100-r.percent(t)+"%",i.dataset.value=Object(N.k)(t,2).toString(),i.style.right=100-r.percent(t)+"%",s()},{value:o,min:r.domain[0],max:r.domain[1]});c.open()}},u.onchange=function(){return s()};var c={minDelta:0,filter:function(t){return 0===t.button&&!t.shiftKey},onStart:function(t){return t.classList.add("lu-dragging")},onDrag:function(n,r){var i=t.clientWidth,u=Math.max(0,Math.min(r,i)),s=Math.round(100*u/i),c=e.getMapping().domain;if(n.dataset.value=Object(N.k)(s/100*(c[1]-c[0])+c[0],2).toString(),"min"===n.dataset.handle)return n.style.left=s+"%",void(o.style.width=s+"%");n.style.right=100-s+"%",a.style.width=100-s+"%"},onEnd:function(t){t.classList.remove("lu-dragging"),s()}};return Object($.a)(r,c),Object($.a)(i,c),function(t,n){var s=rt(e=n);o.style.width=s.percent(s.filterMin)+"%",a.style.width=100-s.percent(s.filterMax)+"%",r.dataset.value=Object(N.k)(s.filterMin,2).toString(),i.dataset.value=Object(N.k)(s.filterMax,2).toString(),r.style.left=s.percent(s.filterMin)+"%",i.style.right=100-s.percent(s.filterMax)+"%",u.checked=s.filterMissing,Object(f.d)(u.parentElement,t)}}(n,t,e)),i(o?o.missing:0,t),n.classList.toggle("lu-missing",!o),o&&r(n,{bins:o.hist.length,max:o.maxBin,hist:o.hist})}}}(t,e,i.template,i.render):function(t,e,n){if(Object(v.d)(t)){var r=t.getRange();e+="<span>"+r[0]+"</span><span>"+r[1]+"</span>"}return{template:e+"</div>",update:function(e,r){if(Object(v.d)(t)){var i=t.getRange();Array.from(e.querySelectorAll("span")).forEach(function(t,e){return t.textContent=i[e]})}e.classList.toggle("lu-missing",!r),r&&n(e,{bins:r.hist.length,max:r.maxBin,hist:r.hist})}}}(t,i.template,i.render)},t}();function et(t,e,n,r){var i,o,a=t?t.hist.length:e;if(Object(b.k)(r)){var u=(i=[]).concat.apply(i,n.map(function(t){return r.getNumbers(t)}));o=Object(N.i)(u,function(t){return t},W.c,[0,1],a)}else o=Object(N.i)(n,function(t){return r.getNumber(t)},function(t){return r.isMissing(t)},[0,1],a);return{bins:a,max:Math.max(o.maxBin,t?t.maxBin:0),hist:o.hist}}function nt(t,e,n){for(var r=Object(N.j)(t),i="",o=0;o<r;++o)i+='<div title="Bin '+o+': 0" data-x=""><div style="height: 0" ></div></div>';return{template:"<div"+(r>S.a?' class="lu-dense':"")+">"+i,render:function(t,r){var i=r.bins,o=r.max,a=r.hist,u=Array.from(t.querySelectorAll("[data-x]"));if(u.length>i)u.splice(i,u.length-i).forEach(function(t){return t.remove()});else if(u.length<i){for(var s=u.length;s<i;++s)t.insertAdjacentHTML("afterbegin",'<div title="Bin '+s+': 0" data-x=""><div style="height: 0" ></div></div>');u=Array.from(t.querySelectorAll("[data-x]"))}t.classList.toggle("lu-dense",i>S.a),u.forEach(function(t,r){var i=a[r],u=i.x0,s=i.x1,c=i.length,l=t.firstElementChild;t.title=Object(b.a)(u)+" - "+Object(b.a)(s)+" ("+c+")",t.dataset.x=Object(b.a)(u),l.style.height=Math.round(100*c/o)+"%",l.style.backgroundColor=E(e,null,n,(s+u)/2)})},guessedBins:r}}function rt(t){var e=t.getFilter(),n=t.getMapping().domain,r=isFinite(e.min)?e.min:n[0],i=isFinite(e.max)?e.max:n[1];return{filterMissing:e.filterMissing,domain:n,percent:function(t){return Math.round(100*(t-n[0])/(n[1]-n[0]))},unpercent:function(t){return t/100*(n[1]-n[0])+n[0]},filterMin:r,filterMax:i}}var it=n(32),ot=function(){function t(){this.title="Image"}return t.prototype.canRender=function(t,e){return t instanceof it.d&&e===i.a.CELL},t.prototype.create=function(t){return{template:"<div></div>",update:function(e,n){var r=Object(h.b)(e,t,n),i=t.getLink(n);e.title=i?i.alt:"",e.style.backgroundImage=r||!i?null:"url('"+i.href+"')"},render:o.h}},t.prototype.createGroup=function(){return o.g},t.prototype.createSummary=function(){return o.g},t}(),at=n(36),ut=n(58),st=function(){function t(){this.title="Interleaved"}return t.prototype.canRender=function(t){return t instanceof at.a},t.prototype.create=function(t,e){var n=Object(ut.a)(t,e,!1,i.a.CELL).cols,r=e.colWidth(t);return{template:"<div>"+n.map(function(t){return t.template}).join("")+"</div>",update:function(e,r,i,a){Object(h.b)(e,t,r)||(Object(o.f)(e,n),Object(o.e)(e,function(t,e){n[e].renderer.update(t,r,i,a)}))},render:function(e,i,o,a){Object(h.a)(e,t,i,r)||(e.save(),e.scale(1,1/n.length),n.forEach(function(t,n){t.renderer.render(e,i,n,a),e.translate(0,s.c)}),e.restore())}}},t.prototype.createGroup=function(t,e){var n=Object(ut.a)(t,e,!1,i.a.GROUP).cols;return{template:"<div>"+n.map(function(t){return t.template}).join("")+"</div>",update:function(t,e,r){Object(o.f)(t,n),Object(o.e)(t,function(t,i){n[i].groupRenderer.update(t,e,r)})}}},t.prototype.createSummary=function(t,e){var n=t.children,r=0,i=nt(e.totalNumberOfRows,t,{color:function(){return n[r++%n.length].color}}),o=i.template,a=i.render;return{template:o,update:function(t){var r=n.map(function(t){return e.statsOf(t)});if(r.some(Boolean)){t.classList.remove("lu-missing");var i=function(t){for(var e=t.find(Boolean).hist.length,n=[],r=0,i=function(e){t.forEach(function(t){var i=t?t.hist[e]:null;i?(i.length>r&&(r=i.length),n.push(i)):n.push(ct)})},o=0;o<e;++o)i(o);return{bins:e,max:r,hist:n}}(r);a(t,i)}else t.classList.add("lu-missing")}}},t}(),ct={length:0,x0:0,x1:0},lt=function(){function t(){this.title="Link"}return t.prototype.canRender=function(t,e){return t instanceof it.d&&e!==i.a.SUMMARY},t.prototype.create=function(t){var e=t.alignment||"left";return{template:"<a"+("left"!==e?' class="lu-'+e+'"':"")+' target="_blank" href=""></a>',update:function(e,n){if(!Object(h.b)(e,t,n)){var r=t.getLink(n);e.href=r?r.href:"",t.escape?Object(o.i)(e,r?r.alt:""):e.innerHTML=r?r.alt:""}},render:o.h}},t.exampleText=function(t,e){for(var n=[],r=0,i=e;r<i.length;r++){var o=i[r];if(!t.isMissing(o)){var a=t.getLink(o);if(n.push('<a target="_blank" href="'+(a?a.href:"")+'">'+(a?a.alt:"")+"</a>"),n.length>=5)break}}return n.join(", ")+(n.length<e.length?", &hellip;":"")},t.prototype.createGroup=function(e){return{template:"<div> </div>",update:function(n,r,i){n.innerHTML=""+t.exampleText(e,i)}}},t.prototype.createSummary=function(){return o.g},t}(),dt=n(71),pt=function(){function t(){this.title="Table"}return t.prototype.canRender=function(t){return Object(dt.b)(t)},t.prototype.create=function(t){return Object(dt.a)(t)&&t.dataLength?this.createFixed(t):{template:"<div></div>",update:function(e,n){Object(h.b)(e,t,n)||(e.innerHTML=t.getMapLabel(n).map(function(t){return"<div>"+t.key+"</div><div>"+t.value+"</div>"}).join(""))},render:o.h}},t.template=function(t){return"<div>"+t.labels.map(function(t){return"<div>"+t+"</div><div data-v></div>"}).join("\n")+"</div>"},t.prototype.createFixed=function(e){return{template:t.template(e),update:function(t,n){if(!Object(h.b)(t,e,n)){var r=e.getLabels(n);Object(o.d)(t,"[data-v]",function(t,e){t.innerHTML=r[e]})}},render:o.h}},t.example=function(t){return t.slice(0,5).map(function(t){return t.value}).join(", ")+(5<t.length?", &hellip;":"")},t.prototype.createGroup=function(e){return Object(dt.a)(e)&&e.dataLength?this.createFixedGroup(e):{template:"<div></div>",update:function(n,r,i){var o=ft(i.filter(function(t){return!e.isMissing(t)}).map(function(t){return e.getMapLabel(t)}));n.innerHTML=o.map(function(e){var n=e.key,r=e.values;return"<div>"+n+"</div><div>"+t.example(r)+"</div>"}).join("")}}},t.prototype.createFixedGroup=function(e){return{template:t.template(e),update:function(t,n,r){var i=r.filter(function(t){return!e.isMissing(t)}).map(function(t){return e.getLabels(t)});Object(o.d)(t,"[data-v]",function(t,e){for(var n=[],r=0,o=i;r<o.length;r++){var a=o[r][e];if(!Object(W.c)(a)&&(n.push(a),n.length>=5))break}t.innerHTML=n.join(", ")+(5<i.length?", &hellip;":"")})}}},t.prototype.createSummary=function(){return{template:"<div><div>Key</div><div>Value</div></div>",update:o.h}},t}();function ft(t){var e=new Map;return t.forEach(function(t){return t.forEach(function(t){e.has(t.key)?e.get(t.key).push(t):e.set(t.key,[t])})}),Array.from(e).sort(function(t,e){return t[0].localeCompare(e[0])}).map(function(t){return{key:t[0],values:t[1]}})}var ht=n(69),gt=function(){function t(){this.title="Table with Links"}return t.prototype.canRender=function(t,e){return t instanceof ht.a&&e!==i.a.SUMMARY},t.prototype.create=function(t){var e=t.alignment||"left";return{template:"<div></div>",update:function(n,r){Object(h.b)(n,t,r)||(n.innerHTML=t.getLinkMap(r).map(function(t){var n=t.key,r=t.value;return"<div>"+n+"</div><div"+("left"!==e?' class="lu-'+e+'"':"")+'><a href="'+r.href+'" target="_blank">'+r.alt+"</a></div>"}).join(""))},render:o.h}},t.example=function(t){for(var e=[],n=0,r=t;n<r.length;n++){var i=r[n];if(e.push('<a target="_blank" href="'+i.value.href+'">'+i.value.alt+"</a>"),e.length>=5)break}return""+e.join(", ")+(e.length<t.length)+" ? ', &hellip;': ''}"},t.prototype.createGroup=function(e){var n=e.alignment||"left";return{template:"<div></div>",update:function(r,i,o){var a=ft(o.filter(function(t){return!e.isMissing(t)}).map(function(t){return e.getLinkMap(t)}));r.innerHTML=a.map(function(e){var r=e.key,i=e.values;return"<div>"+r+"</div><div"+("left"!==n?' class="lu-'+n+'"':"")+">"+t.example(i)+"</div>"}).join("")}}},t.prototype.createSummary=function(){return o.g},t}(),mt=function(){function t(){this.title="Loading"}return t.prototype.canRender=function(){return!1},t.prototype.create=function(){return{template:"<div>Loading…</div>",update:o.h,render:o.h}},t.prototype.createGroup=function(){return this.create()},t.prototype.createSummary=function(){return this.create()},t}(),bt=function(){function t(){this.title="Bar Table"}return t.prototype.canRender=function(t,e){return Object(dt.b)(t)&&Object(b.i)(t)&&(e===i.a.CELL||e===i.a.SUMMARY&&Object(v.d)(t))},t.prototype.create=function(t,e,n,r){return{template:"<div></div>",update:function(e,n){Object(h.b)(e,t,n)||(e.innerHTML=t.getMap(n).map(function(e){var i=e.key,o=e.value;if(Object(W.c)(o))return"<div>"+i+'</div><div class="lu-missing"></div>';var a=isNaN(o)?0:Math.round(100*o*100)/100;return"<div>"+i+'</div><div title="'+Object(b.a)(o)+'"><div style="width: '+a+"%; background-color: "+E(t,n,r)+'"><span class="lu-hover-only">'+o+"</span></div></div>"}).join(""))},render:o.h}},t.prototype.createGroup=function(){return o.g},t.prototype.createSummary=function(t){return{template:"<div><div>Key</div><div><span></span><span></span>Value</div></div>",update:function(e){var n=t.getRange(),r=e.lastElementChild;r.firstElementChild.textContent=n[0],r.children[1].textContent=n[1]}}},t}(),yt=n(39),vt=function(){function t(){this.title="Default"}return t.prototype.canRender=function(t){return t instanceof yt.b},t.prototype.create=function(t){return{template:'<div class="lu-right"> </div>',update:function(e,n){Object(h.b)(e,t,n),Object(o.i)(e,t.getLabel(n))},render:o.h}},t.prototype.createGroup=function(t){return{template:"<div><div></div><div></div></div>",update:function(e,n,r){var i=e.firstElementChild,o=e.lastElementChild;if(0===r.length)return i.textContent="",void(o.textContent="");i.textContent=t.getLabel(r[0]),o.textContent=t.getLabel(r[r.length-1])}}},t.prototype.createSummary=function(){return o.g},t}(),Et=n(91),Ct=n(28);function wt(t){if(0===t.length)return"";var e="",n=!0;return t.forEach(function(t,r){Object(W.c)(t)?n=!0:n?(e+="M"+r+","+(1-t)+" ",n=!1):e+="L"+r+","+(1-t)+" "}),e}var At=function(){function t(){this.title="Sparkline"}return t.prototype.canRender=function(t,e){return Object(b.k)(t)&&e!==i.a.SUMMARY},t.prototype.create=function(t){var e=t.dataLength,n=1-t.getMapping().apply(Ct.c.CENTER);return{template:'<svg viewBox="0 0 '+(e-1)+' 1" preserveAspectRatio="none meet"><line x1="0" x2="'+(e-1)+'" y1="'+n+'" y2="'+n+'"></line><path></path></svg>',update:function(e,n){if(!Object(h.b)(e,t,n)){var r=t.getNumbers(n);e.querySelector("path").setAttribute("d",wt(r))}},render:o.h}},t.prototype.createGroup=function(t){var e=t.dataLength,n=1-t.getMapping().apply(Ct.c.CENTER);return{template:'<svg viewBox="0 0 '+e+' 1" preserveAspectRatio="none meet"><line x1="0" x2="'+(e-1)+'" y1="'+n+'" y2="'+n+'"></line><path></path></svg>',update:function(e,n,r){!function(t,e,n){var r=Array.from(t.children);r.length>e.length?r.slice(e.length).forEach(function(t){return t.remove()}):e.length>r.length&&t.insertAdjacentHTML("beforeend","<path></path>".repeat(e.length-r.length))}(e,r),Object(o.e)(e,function(e,n){var i=r[n];e.setAttribute("d",wt(t.getNumbers(i)))})}}},t.prototype.createSummary=function(){return o.g},t}(),xt=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.title="Bar Chart",e}return l.c(e,t),e.prototype.canRender=function(t,e){return Object(b.k)(t)&&Boolean(t.dataLength)&&e===i.a.CELL},e.compute=function(t,e,n){return t<e?{height:e-t,bottom:t-n[0]}:{height:t-e,bottom:e-n[0]}},e.prototype.createContext=function(t,n,r){for(var i=n.colWidth(t)/t.dataLength,a=t.getMapping().apply(Ct.c.CENTER),u="",c=0;c<t.dataLength;++c)u+='<div style="background-color: white" title=""></div>';return{templateRow:u,update:function(n,i,u,s){var c=_(0,s,t,r),l=_(1,s,t,r);Object(o.e)(n,function(t,n){var r=i[n],s=e.compute(r,a,[0,1]),d=s.bottom,p=s.height;Object(o.c)(t,{title:Object(b.a)(u[n])},{"background-color":r<a?c:l,bottom:Math.round(100*d/1)+"%",height:Math.round(100*p/1)+"%"})})},render:function(n,o,u){var c=_(0,u,t,r),l=_(1,u,t,r),d=s.c/1;o.forEach(function(t,r){n.fillStyle=t<a?c:l;var o=r*i,u=e.compute(t,a,[0,1]),s=u.bottom,p=u.height;n.fillRect(o,(1-p-s)*d,i,p*d)})}}},e.prototype.createSummary=function(){return o.g},e}(Z);n.d(e,"q",function(){return Tt}),n.d(e,"i",function(){return Ot}),n.d(e,"h",function(){return Rt}),n.d(e,"j",function(){return Mt}),n.d(e,"m",function(){return Dt}),n.d(e,"l",function(){return _t}),n.d(e,"n",function(){return Lt}),n.d(e,"f",function(){return i.default}),n.d(e,"e",function(){return i.IImposer}),n.d(e,"a",function(){return i.a}),n.d(e,"g",function(){return i.ISummaryRenderer}),n.d(e,"d",function(){return i.IGroupCellRenderer}),n.d(e,"b",function(){return i.ICellRenderer}),n.d(e,"c",function(){return i.ICellRendererFactory}),n.d(e,"k",function(){return E}),n.d(e,"p",function(){return h.b}),n.d(e,"o",function(){return h.a});var Nt=new A,Tt={actions:new a,aggregate:new c,annotate:new m,boolean:new x,boxplot:new R,brightness:new L,catdistributionbar:new B,categorical:new V,circle:new Y,default:Nt,dot:new q,group:new J,heatmap:new Q,catheatmap:new U,histogram:new tt,image:new ot,interleaving:new st,link:new lt,linkMap:new gt,loading:new mt,nested:new ut.b(!1),number:new C,mapbars:new bt,rank:new vt,selection:new Et.a,sparkline:new At,stack:new ut.b,string:new g,table:new pt,upset:new G,verticalbar:new xt};function Ot(t,e){return e[t.getRenderer()]||Nt}function Rt(t,e){return e[t.getGroupRenderer()]||Nt}function Mt(t,e){return e[t.getSummaryRenderer()]||Nt}function Dt(t,e,n){return void 0===n&&(n=i.a.CELL),Object.keys(e).filter(function(r){return e[r].canRender(t,n)}).map(function(t){var r=e[t];return{type:t,label:n===i.a.CELL?r.title:n===i.a.GROUP?r.groupTitle||r.title:r.summaryTitle||r.groupTitle||r.title}})}function _t(t,e){return Dt(t,e,i.a.GROUP)}function Lt(t,e){return Dt(t,e,i.a.SUMMARY)}},function(t,e,n){"use strict";n.d(e,"a",function(){return o});var r=n(0),i=n(1);function o(t,e,n){return void 0===t&&(t="actions"),void 0===e&&(e=[]),void 0===n&&(n=[]),{type:"actions",label:t,actions:e,groupActions:n,fixed:!0}}var a=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.actions=n.actions||[],r.groupActions=n.groupActions||[],r}return r.c(e,t),e.prototype.getLabel=function(){return""},e.prototype.getValue=function(){return""},e.prototype.compare=function(){return 0},e=r.b([Object(i.d)(),Object(i.b)("support")],e)}(n(2).e);e.b=a},function(t,e,n){"use strict";var r=n(0),i=n(18),o=n(74),a=n(1),u=n(20),s=n(2),c=n(11),l=n(10),d=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.lookup=new Map,r.currentFilter=null,r.categories=Object(c.g)(n),r.missingCategory=n.missingCategory?Object(c.h)(n.missingCategory,NaN):null,r.categories.forEach(function(t){return r.lookup.set(t.name,t)}),r.setDefaultRenderer("number"),r.setDefaultGroupRenderer("boxplot"),r}var n;return r.c(e,t),n=e,e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_MAPPING_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},Object.defineProperty(e.prototype,"dataLength",{get:function(){return this.categories.length},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"labels",{get:function(){return this.categories.map(function(t){return t.label})},enumerable:!0,configurable:!0}),e.prototype.getValue=function(t){var e=this.getCategory(t);return e?e.value:NaN},e.prototype.getCategory=function(e){var n=t.prototype.getValue.call(this,e);if(!n)return this.missingCategory;var r=String(n);return this.lookup.has(r)?this.lookup.get(r):this.missingCategory},e.prototype.getColor=function(t){return u.a.prototype.getColor.call(this,t)},e.prototype.getLabel=function(t){return u.a.prototype.getLabel.call(this,t)},e.prototype.getLabels=function(t){return u.a.prototype.getLabels.call(this,t)},e.prototype.getValues=function(t){return u.a.prototype.getValues.call(this,t)},e.prototype.getMap=function(t){return u.a.prototype.getMap.call(this,t)},e.prototype.getMapLabel=function(t){return u.a.prototype.getMapLabel.call(this,t)},e.prototype.getSet=function(t){return u.a.prototype.getSet.call(this,t)},e.prototype.getNumber=function(t){return this.getValue(t)},e.prototype.isMissing=function(t){return u.a.prototype.isMissing.call(this,t)},e.prototype.getRawNumber=function(t){return this.getNumber(t)},e.prototype.getExportValue=function(e,n){return"json"===n?this.isMissing(e)?null:{name:this.getLabel(e),value:this.getValue(e)}:t.prototype.getExportValue.call(this,e,n)},e.prototype.dump=function(t){var e=u.a.prototype.dump.call(this,t);return e.mapping=this.getMapping(),e},e.prototype.restore=function(t,e){u.a.prototype.restore.call(this,t,e),t.mapping&&this.setMapping(t.mapping)},e.prototype.getMapping=function(){return this.categories.map(function(t){return t.value})},e.prototype.setMapping=function(t){var e=Object(i.d)(t);t=t.map(function(t){return(t-e[0])/(e[1]-e[0])});var r=this.getMapping();Object(o.b)(r,t)||(this.categories.forEach(function(e,n){return e.value=t[n]||0}),this.fire([n.EVENT_MAPPING_CHANGED,s.e.EVENT_DIRTY_HEADER,s.e.EVENT_DIRTY_VALUES,s.e.EVENT_DIRTY],r,this.getMapping()))},e.prototype.isFiltered=function(){return null!=this.currentFilter},e.prototype.filter=function(t){return u.a.prototype.filter.call(this,t)},e.prototype.group=function(t){return u.a.prototype.group.call(this,t)},e.prototype.getFilter=function(){return this.currentFilter},e.prototype.setFilter=function(t){return u.a.prototype.setFilter.call(this,t)},e.prototype.compare=function(t,e){return u.a.prototype.compare.call(this,t,e)},e.prototype.getRenderer=function(){return l.c.prototype.getRenderer.call(this)},e.EVENT_MAPPING_CHANGED=l.c.EVENT_MAPPING_CHANGED,e=n=r.b([Object(a.l)("group","filterOrdinal"),Object(a.b)("categorical")],e)}(n(8).b);e.b=d},function(t,e,n){"use strict";var r=n(0),i=n(9),o=n(13),a=n(23),u=n(4),s=n(5),c=n(3),l=function(){function t(){}return t.prototype.createGroup=function(t,e,n,r){var i=this,o=this.create(t,e,n,r);return{template:"<div>"+o.template+"</div>",update:function(e,n,r){var a=i.aggregatedIndex(r,t);o.update(e.firstElementChild,r[a],a,n)}}},t.prototype.createSummary=function(){return c.g},t}(),d=n(6),p=n(15);function f(t){return"lu-stacked-"+t.id}function h(t,e,n,r,i){var o=s.d,a=0;return{cols:t.children.map(function(u){var s=a,c=u.getWidth();a+=c,a+=n?0:o;var l=r===d.a.CELL?e.renderer(u,i):null,p=r===d.a.GROUP?e.groupRenderer(u,i):null,f=r===d.a.GROUP?e.summaryRenderer(u,!1,i):null,h="",g="";switch(r){case d.a.CELL:h=l.template,g=t.getRenderer();break;case d.a.GROUP:h=p.template,g=t.getGroupRenderer();break;case d.a.SUMMARY:h=f.template,g=t.getSummaryRenderer()}return{column:u,shift:s,width:c,template:h=h.replace(/^<([^ >]+)([ >])/,'<$1 data-column-id="'+u.id+'" data-renderer="'+g+'"$2'),rendererId:g,renderer:l,groupRenderer:p,summaryRenderer:f}}),stacked:n,padding:o}}n.d(e,"c",function(){return f}),n.d(e,"a",function(){return h});var g=function(t){function e(e){void 0===e&&(e=!0);var n=t.call(this)||this;return n.stacked=e,n.title=n.stacked?"Stacked Bar":"Nested",n}return r.c(e,t),e.prototype.canRender=function(t,e){return Object(o.c)(t)&&e!==d.a.SUMMARY},e.prototype.create=function(t,e,n,r){var o=h(t,e,this.stacked,d.a.CELL,r),a=o.cols,u=o.stacked,s=o.padding,l=e.option("useGridLayout",!1),g=e.colWidth(t);return{template:"<div class='"+(l?f(t):"")+(l&&!u?" lu-grid-space":"")+"'>"+a.map(function(t){return t.template}).join("")+"</div>",update:function(e,n,r,o){if(!Object(p.b)(e,t,n)){Object(c.f)(e,a);var d=Array.from(e.children),f=t.getWidth(),h=0;a.forEach(function(t,e){var a=t.column.getWidth()/f,c=d[e];c.style.transform=u?"translate(-"+Object(i.k)(h/a*100,4)+"%,0)":null,l?c.style.gridColumnStart=(e+1).toString():(c.style.width=Object(i.k)(100*a,2)+"%",c.style.marginRight=u?null:s+"px"),t.renderer.update(c,n,r,o),u&&(h+=(1-t.column.getValue(n))*a)})}},render:function(e,n,r,i){if(!Object(p.a)(e,t,n,g)){var o=0;a.forEach(function(t){var a=t.shift-o;e.translate(a,0),t.renderer.render(e,n,r,i),e.translate(-a,0),u&&(o+=t.width*(1-t.column.getValue(n)))})}}}},e.prototype.createGroup=function(e,n,r,o){if(this.stacked&&Object(u.i)(e))return t.prototype.createGroup.call(this,e,n,r,o);var a=h(e,n,!1,d.a.GROUP,o),s=a.cols,l=a.padding,p=n.option("useGridLayout",!1);return{template:"<div class='"+(p?f(e):"")+(p?" lu-grid-space":"")+"'>"+s.map(function(t){return t.template}).join("")+"</div>",update:function(t,n,r){Object(c.f)(t,s);var o=Array.from(t.children),a=e.getWidth();s.forEach(function(t,e){var u=t.column.getWidth()/a,s=o[e];p?s.style.gridColumnStart=(e+1).toString():(s.style.width=Object(i.k)(100*u,2)+"%",s.style.marginRight=l+"px"),t.groupRenderer.update(s,n,r)})}}},e.prototype.aggregatedIndex=function(t,e){return console.assert(Object(u.i)(e)),Object(a.e)(t,e)},e}(l);e.b=g},function(t,e,n){"use strict";var r=n(18);function i(){}function o(t,e){var n=new i;if(t instanceof i)t.each(function(t,e){n.set(e,t)});else if(Array.isArray(t)){var r,o=-1,a=t.length;if(null==e)for(;++o<a;)n.set(o,t[o]);else for(;++o<a;)n.set(e(r=t[o],o,t),r)}else if(t)for(var u in t)n.set(u,t[u]);return n}i.prototype=o.prototype={constructor:i,has:function(t){return"$"+t in this},get:function(t){return this["$"+t]},set:function(t,e){return this["$"+t]=e,this},remove:function(t){var e="$"+t;return e in this&&delete this[e]},clear:function(){for(var t in this)"$"===t[0]&&delete this[t]},keys:function(){var t=[];for(var e in this)"$"===e[0]&&t.push(e.slice(1));return t},values:function(){var t=[];for(var e in this)"$"===e[0]&&t.push(this[e]);return t},entries:function(){var t=[];for(var e in this)"$"===e[0]&&t.push({key:e.slice(1),value:this[e]});return t},size:function(){var t=0;for(var e in this)"$"===e[0]&&++t;return t},empty:function(){for(var t in this)if("$"===t[0])return!1;return!0},each:function(t){for(var e in this)"$"===e[0]&&t(this[e],e.slice(1),this)}};var a=o;function u(){}var s=a.prototype;u.prototype=function(t,e){var n=new u;if(t instanceof u)t.each(function(t){n.add(t)});else if(t){var r=-1,i=t.length;if(null==e)for(;++r<i;)n.add(t[r]);else for(;++r<i;)n.add(e(t[r],r,t))}return n}.prototype={constructor:u,has:s.has,add:function(t){return this["$"+(t+="")]=t,this},remove:s.remove,clear:s.clear,values:s.keys,size:s.size,empty:s.empty,each:s.each};var c=Array.prototype,l=c.map,d=c.slice,p=n(52),f=function(t){return function(){return t}},h=function(t){return+t},g=[0,1];function m(t,e){return(e-=t=+t)?function(n){return(n-t)/e}:f(e)}function b(t,e,n,r){var i=t[0],o=t[1],a=e[0],u=e[1];return o<i?(i=n(o,i),a=r(u,a)):(i=n(i,o),a=r(a,u)),function(t){return a(i(t))}}function y(t,e,n,i){var o=Math.min(t.length,e.length)-1,a=new Array(o),u=new Array(o),s=-1;for(t[o]<t[0]&&(t=t.slice().reverse(),e=e.slice().reverse());++s<o;)a[s]=n(t[s],t[s+1]),u[s]=i(e[s],e[s+1]);return function(e){var n=Object(r.b)(t,e,1,o)-1;return u[n](a[n](e))}}function v(t,e){return e.domain(t.domain()).range(t.range()).interpolate(t.interpolate()).clamp(t.clamp())}function E(t,e){var n,r,i,o=g,a=g,u=p.a,s=!1;function c(){return n=Math.min(o.length,a.length)>2?y:b,r=i=null,f}function f(e){return(r||(r=n(o,a,s?function(t){return function(e,n){var r=t(e=+e,n=+n);return function(t){return t<=e?0:t>=n?1:r(t)}}}(t):t,u)))(+e)}return f.invert=function(t){return(i||(i=n(a,o,m,s?function(t){return function(e,n){var r=t(e=+e,n=+n);return function(t){return t<=0?e:t>=1?n:r(t)}}}(e):e)))(+t)},f.domain=function(t){return arguments.length?(o=l.call(t,h),c()):o.slice()},f.range=function(t){return arguments.length?(a=d.call(t),c()):a.slice()},f.rangeRound=function(t){return a=d.call(t),u=p.e,c()},f.clamp=function(t){return arguments.length?(s=!!t,c()):s},f.interpolate=function(t){return arguments.length?(u=t,c()):u},c()}var C=n(48),w=function(t,e,n){var i,o=t[0],a=t[t.length-1],u=Object(r.m)(o,a,null==e?10:e);switch((n=Object(C.c)(null==n?",f":n)).type){case"s":var s=Math.max(Math.abs(o),Math.abs(a));return null!=n.precision||isNaN(i=Object(C.e)(u,s))||(n.precision=i),Object(C.b)(n,s);case"":case"e":case"g":case"p":case"r":null!=n.precision||isNaN(i=Object(C.f)(u,Math.max(Math.abs(o),Math.abs(a))))||(n.precision=i-("e"===n.type));break;case"f":case"%":null!=n.precision||isNaN(i=Object(C.d)(u))||(n.precision=i-2*("%"===n.type))}return Object(C.a)(n)};function A(t){var e=t.domain;return t.ticks=function(t){var n=e();return Object(r.n)(n[0],n[n.length-1],null==t?10:t)},t.tickFormat=function(t,n){return w(e(),t,n)},t.nice=function(n){null==n&&(n=10);var i,o=e(),a=0,u=o.length-1,s=o[a],c=o[u];return c<s&&(i=s,s=c,c=i,i=a,a=u,u=i),(i=Object(r.l)(s,c,n))>0?(s=Math.floor(s/i)*i,c=Math.ceil(c/i)*i,i=Object(r.l)(s,c,n)):i<0&&(s=Math.ceil(s*i)/i,c=Math.floor(c*i)/i,i=Object(r.l)(s,c,n)),i>0?(o[a]=Math.floor(s/i)*i,o[u]=Math.ceil(c/i)*i,e(o)):i<0&&(o[a]=Math.ceil(s*i)/i,o[u]=Math.floor(c*i)/i,e(o)),t},t}function x(){var t=E(m,p.c);return t.copy=function(){return v(t,x())},A(t)}var N=function(t,e){var n,r=0,i=(t=t.slice()).length-1,o=t[r],a=t[i];return a<o&&(n=r,r=i,i=n,n=o,o=a,a=n),t[r]=e.floor(o),t[i]=e.ceil(a),t};function T(t,e){return(e=Math.log(e/t))?function(n){return Math.log(n/t)/e}:f(e)}function O(t,e){return t<0?function(n){return-Math.pow(-e,n)*Math.pow(-t,1-n)}:function(n){return Math.pow(e,n)*Math.pow(t,1-n)}}function R(t){return isFinite(t)?+("1e"+t):t<0?0:t}function M(t){return 10===t?R:t===Math.E?Math.exp:function(e){return Math.pow(t,e)}}function D(t){return t===Math.E?Math.log:10===t&&Math.log10||2===t&&Math.log2||(t=Math.log(t),function(e){return Math.log(e)/t})}function _(t){return function(e){return-t(-e)}}function L(){var t=E(T,O).domain([1,10]),e=t.domain,n=10,i=D(10),o=M(10);function a(){return i=D(n),o=M(n),e()[0]<0&&(i=_(i),o=_(o)),t}return t.base=function(t){return arguments.length?(n=+t,a()):n},t.domain=function(t){return arguments.length?(e(t),a()):e()},t.ticks=function(t){var a,u=e(),s=u[0],c=u[u.length-1];(a=c<s)&&(f=s,s=c,c=f);var l,d,p,f=i(s),h=i(c),g=null==t?10:+t,m=[];if(!(n%1)&&h-f<g){if(f=Math.round(f)-1,h=Math.round(h)+1,s>0){for(;f<h;++f)for(d=1,l=o(f);d<n;++d)if(!((p=l*d)<s)){if(p>c)break;m.push(p)}}else for(;f<h;++f)for(d=n-1,l=o(f);d>=1;--d)if(!((p=l*d)<s)){if(p>c)break;m.push(p)}}else m=Object(r.n)(f,h,Math.min(h-f,g)).map(o);return a?m.reverse():m},t.tickFormat=function(e,r){if(null==r&&(r=10===n?".0e":","),"function"!=typeof r&&(r=Object(C.a)(r)),e===1/0)return r;null==e&&(e=10);var a=Math.max(1,n*e/t.ticks().length);return function(t){var e=t/o(Math.round(i(t)));return e*n<n-.5&&(e*=n),e<=a?r(t):""}},t.nice=function(){return e(N(e(),{floor:function(t){return o(Math.floor(i(t)))},ceil:function(t){return o(Math.ceil(i(t)))}}))},t.copy=function(){return v(t,L().base(n))},t}function S(t,e){return t<0?-Math.pow(-t,e):Math.pow(t,e)}function I(){var t=1,e=E(function(e,n){return(n=S(n,t)-(e=S(e,t)))?function(r){return(S(r,t)-e)/n}:f(n)},function(e,n){return n=S(n,t)-(e=S(e,t)),function(r){return S(e+n*r,1/t)}}),n=e.domain;return e.exponent=function(e){return arguments.length?(t=+e,n(n())):t},e.copy=function(){return v(e,I().exponent(t))},A(e)}function j(){return I().exponent(.5)}n(46),n(51),n.d(e,"a",function(){return x}),n.d(e,"b",function(){return L}),n.d(e,"c",function(){return I}),n.d(e,"d",function(){return j})},function(t,e,n){"use strict";var r=n(0),i=n(1),o=n(20),a=n(2),u=function(t){function e(e,r){var i=t.call(this,e,r)||this;return i.currentFilter=null,i.setWidthImpl(30),i.categories=[{name:r.trueMarker||"✓",color:n.GROUP_TRUE.color,label:n.GROUP_TRUE.name,value:0},{name:r.trueMarker||"",color:n.GROUP_FALSE.color,label:n.GROUP_FALSE.name,value:1}],i}var n;return r.c(e,t),n=e,e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_FILTER_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},Object.defineProperty(e.prototype,"dataLength",{get:function(){return this.categories.length},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"labels",{get:function(){return this.categories.map(function(t){return t.label})},enumerable:!0,configurable:!0}),e.prototype.getValue=function(e){var n=t.prototype.getValue.call(this,e);return void 0!==n&&null!=n&&(!0===n||"true"===n||"yes"===n||"x"===n)},e.prototype.isMissing=function(){return!1},e.prototype.getCategory=function(t){var e=this.getValue(t);return this.categories[e?0:1]},e.prototype.getColor=function(t){return o.a.prototype.getColor.call(this,t)},e.prototype.getLabel=function(t){return o.a.prototype.getLabel.call(this,t)},e.prototype.getLabels=function(t){return o.a.prototype.getLabels.call(this,t)},e.prototype.getValues=function(t){return o.a.prototype.getValues.call(this,t)},e.prototype.getMap=function(t){return o.a.prototype.getMap.call(this,t)},e.prototype.getMapLabel=function(t){return o.a.prototype.getMapLabel.call(this,t)},e.prototype.getSet=function(t){var e=this.getValue(t),n=new Set;return n.add(this.categories[e?0:1]),n},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return null!=this.currentFilter&&(n.filter=this.currentFilter),n},e.prototype.restore=function(e,n){t.prototype.restore.call(this,e,n),void 0!==e.filter&&(this.currentFilter=e.filter)},e.prototype.isFiltered=function(){return null!=this.currentFilter},e.prototype.filter=function(t){return!this.isFiltered()||this.getValue(t)===this.currentFilter},e.prototype.getFilter=function(){return this.currentFilter},e.prototype.setFilter=function(t){this.currentFilter!==t&&this.fire([n.EVENT_FILTER_CHANGED,a.e.EVENT_DIRTY_VALUES,a.e.EVENT_DIRTY],this.currentFilter,this.currentFilter=t)},e.prototype.compare=function(t,e){var n=this.getValue(t),r=this.getValue(e);return n===r?0:n<r?-1:1},e.prototype.group=function(t){return this.getValue(t)?n.GROUP_TRUE:n.GROUP_FALSE},e.EVENT_FILTER_CHANGED="filterChanged",e.GROUP_TRUE={name:"True",color:"black"},e.GROUP_FALSE={name:"False",color:"white"},e=n=r.b([Object(i.l)("group","groupBy","filterBoolean"),Object(i.b)("categorical")],e)}(n(8).b);e.c=u},function(t,e,n){"use strict";n.d(e,"b",function(){return s}),n.d(e,"a",function(){return r});var r,i=n(0),o=n(1),a=n(2),u=n(7);function s(t){return void 0===t&&(t="Group Name"),{type:"group",label:t}}!function(t){t.name="name",t.count="count"}(r||(r={}));var c=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.groupSortMethod=r.name,e}var n;return i.c(e,t),n=e,Object.defineProperty(e.prototype,"frozen",{get:function(){return!1!==this.desc.frozen},enumerable:!0,configurable:!0}),e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_SORTMETHOD_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getLabel=function(){return""},e.prototype.getValue=function(){return""},e.prototype.compare=function(){return 0},e.prototype.getSortMethod=function(){return this.groupSortMethod},e.prototype.setSortMethod=function(t){this.groupSortMethod!==t&&(this.fire([n.EVENT_SORTMETHOD_CHANGED],this.groupSortMethod,this.groupSortMethod=t),this.isGroupSortedByMe().asc||this.toggleMyGroupSorting())},e.prototype.groupCompare=function(t,e){switch(this.groupSortMethod){case"count":return t.rows.length-e.rows.length;default:return t.name===u.e.name?e.name===u.e.name?0:u.b:e.name===u.e.name?-u.b:t.name.toLowerCase().localeCompare(e.name.toLowerCase())}},e.EVENT_SORTMETHOD_CHANGED="sortMethodChanged",e=n=i.b([Object(o.d)(),Object(o.l)("sortGroupBy"),Object(o.g)("sortGroup","sortGroups"),Object(o.b)("support")],e)}(a.e);e.c=c},function(t,e,n){"use strict";n.d(e,"a",function(){return p});var r=n(0),i=n(16),o=n(1),a=n(33),u=n(13),s=n(4),c=n(12),l=n(28),d=n(27);function p(t){return void 0===t&&(t="Imposition"),{type:"impositions",label:t}}var f=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.setDefaultRenderer("boxplot"),r.setDefaultGroupRenderer("boxplot"),r}var n;return r.c(e,t),n=e,Object.defineProperty(e.prototype,"label",{get:function(){var e=t.prototype.getMetaData.call(this).label,n=this._children;return"Imposition"!==e||0===n.length?e:1===n.length?n[0].label:n[0].label+" ("+n.slice(1).map(function(t){return t.label}).join(", ")+")"},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"wrapper",{get:function(){var t=this._children;return 0===t.length?null:t[0]},enumerable:!0,configurable:!0}),e.prototype.getLabel=function(t){var e=this._children;return 0===e.length?"":1===e.length?e[0].getLabel(t):e[0].getLabel(t)+" ("+e.slice(1).map(function(e){return e.label+" = "+e.getLabel(t)})+")"},e.prototype.getColor=function(t){var e=this._children;switch(e.length){case 0:return this.color;case 1:return e[0].getColor(t);default:return e[1].getColor(t)}},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_MAPPING_CHANGED,n.EVENT_COLOR_MAPPING_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getValue=function(t){var e=this.wrapper;return e?e.getValue(t):null},e.prototype.getNumber=function(t){var e=this.wrapper;return e?e.getNumber(t):NaN},e.prototype.getRawNumber=function(t){var e=this.wrapper;return e?e.getRawNumber(t):NaN},e.prototype.getExportValue=function(e,n){return"json"===n?this.isMissing(e)?null:{label:this.getLabel(e),color:this.getColor(e),value:this.getRawNumber(e)}:t.prototype.getExportValue.call(this,e,n)},e.prototype.getBoxPlotData=function(t){var e=this.wrapper;return e?e.getBoxPlotData(t):null},e.prototype.getRawBoxPlotData=function(t){var e=this.wrapper;return e?e.getRawBoxPlotData(t):null},e.prototype.getMapping=function(){var t=this.wrapper;return t?t.getMapping():new c.a},e.prototype.getOriginalMapping=function(){var t=this.wrapper;return t?t.getOriginalMapping():new c.a},e.prototype.getSortMethod=function(){var t=this.wrapper;return t?t.getSortMethod():s.c.min},e.prototype.setSortMethod=function(t){var e=this.wrapper;return e?e.setSortMethod(t):void 0},e.prototype.isMissing=function(t){var e=this.wrapper;return!e||e.isMissing(t)},e.prototype.setMapping=function(t){var e=this.wrapper;return e?e.setMapping(t):void 0},e.prototype.getColorMapping=function(){var t=this.wrapper;return t?t.getColorMapping():d.b},e.prototype.setColorMapping=function(t){var e=this.wrapper;return e?e.setColorMapping(t):void 0},e.prototype.getFilter=function(){var t=this.wrapper;return t?t.getFilter():Object(s.l)()},e.prototype.setFilter=function(t){var e=this.wrapper;return e?e.setFilter(t):void 0},e.prototype.getRange=function(){var t=this.wrapper;return t?t.getRange():["0","1"]},e.prototype.compare=function(t,e){return a.c.prototype.compare.call(this,t,e)},e.prototype.group=function(t){return a.c.prototype.group.call(this,t)},e.prototype.groupCompare=function(t,e){return a.c.prototype.groupCompare.call(this,t,e)},e.prototype.insert=function(e,n){return 0!==this._children.length||Object(s.g)(e)?this._children.length>=2?null:t.prototype.insert.call(this,e,n):null},e.prototype.insertImpl=function(e,n){return Object(s.g)(e)&&this.forward.apply(this,[e].concat(Object(i.b)(".impose",a.c.EVENT_MAPPING_CHANGED,a.c.EVENT_COLOR_MAPPING_CHANGED))),Object(c.d)(e)&&this.forward.apply(this,[e].concat(Object(i.b)(".impose",a.c.EVENT_COLOR_MAPPING_CHANGED))),t.prototype.insertImpl.call(this,e,n)},e.prototype.removeImpl=function(e,n){return Object(s.g)(e)&&this.unforward.apply(this,[e].concat(Object(i.b)(".impose",a.c.EVENT_MAPPING_CHANGED,a.c.EVENT_COLOR_MAPPING_CHANGED))),Object(c.d)(e)&&this.unforward.apply(this,[e].concat(Object(i.b)(".impose",a.c.EVENT_COLOR_MAPPING_CHANGED))),t.prototype.removeImpl.call(this,e,n)},e.EVENT_MAPPING_CHANGED=l.c.EVENT_MAPPING_CHANGED,e.EVENT_COLOR_MAPPING_CHANGED=l.c.EVENT_COLOR_MAPPING_CHANGED,e=n=r.b([Object(o.l)("filterNumber","colorMapped","editMapping"),Object(o.g)("sort","sortBoxPlot"),Object(o.c)("descending")],e)}(u.b);e.b=f},function(t,e,n){"use strict";n.d(e,"a",function(){return l});var r=n(0),i=n(16),o=n(1),a=n(13),u=n(4),s=n(10),c=n(12);function l(t){return void 0===t&&(t="Imposition"),{type:"imposition",label:t}}var d=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.setDefaultRenderer("number"),r.setDefaultGroupRenderer("boxplot"),r}var n;return r.c(e,t),n=e,Object.defineProperty(e.prototype,"label",{get:function(){var e=t.prototype.getMetaData.call(this).label,n=this._children;return"Imposition"!==e||0===n.length?e:1===n.length?n[0].label:n[0].label+" ("+n.slice(1).map(function(t){return t.label}).join(", ")+")"},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"wrapper",{get:function(){var t=this._children;return 0===t.length?null:t[0]},enumerable:!0,configurable:!0}),e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_MAPPING_CHANGED,n.EVENT_COLOR_MAPPING_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getLabel=function(t){var e=this._children;return 0===e.length?"":1===e.length?e[0].getLabel(t):e[0].getLabel(t)+" ("+e.slice(1).map(function(e){return e.label+" = "+e.getLabel(t)}).join(", ")+")"},e.prototype.getColor=function(t){var e=this._children;switch(e.length){case 0:return this.color;case 1:return e[0].getColor(t);default:return e[1].getColor(t)}},e.prototype.getValue=function(t){var e=this.wrapper;return e?e.getValue(t):NaN},e.prototype.getNumber=function(t){var e=this.wrapper;return e?e.getNumber(t):NaN},e.prototype.getRawNumber=function(t){var e=this.wrapper;return e?e.getRawNumber(t):NaN},e.prototype.getExportValue=function(e,n){return"json"===n?this.isMissing(e)?null:{label:this.getLabel(e),color:this.getColor(e),value:this.getRawNumber(e)}:t.prototype.getExportValue.call(this,e,n)},e.prototype.isMissing=function(t){var e=this.wrapper;return!e||e.isMissing(t)},e.prototype.compare=function(t,e){return s.c.prototype.compare.call(this,t,e)},e.prototype.group=function(t){return s.c.prototype.group.call(this,t)},e.prototype.groupCompare=function(t,e){return s.c.prototype.groupCompare.call(this,t,e)},e.prototype.insert=function(e,n){return 0!==this._children.length||Object(u.i)(e)?this._children.length>=2?null:t.prototype.insert.call(this,e,n):null},e.prototype.insertImpl=function(e,n){return Object(u.i)(e)&&this.forward.apply(this,[e].concat(Object(i.b)(".impose",s.c.EVENT_MAPPING_CHANGED))),Object(c.d)(e)&&this.forward.apply(this,[e].concat(Object(i.b)(".impose",s.c.EVENT_COLOR_MAPPING_CHANGED))),t.prototype.insertImpl.call(this,e,n)},e.prototype.removeImpl=function(e,n){return Object(u.i)(e)&&this.unforward.apply(this,[e].concat(Object(i.b)(".impose",s.c.EVENT_MAPPING_CHANGED))),Object(c.d)(e)&&this.unforward.apply(this,[e].concat(Object(i.b)(".impose",s.c.EVENT_COLOR_MAPPING_CHANGED))),t.prototype.removeImpl.call(this,e,n)},e.EVENT_MAPPING_CHANGED=s.c.EVENT_MAPPING_CHANGED,e.EVENT_COLOR_MAPPING_CHANGED=s.c.EVENT_COLOR_MAPPING_CHANGED,e=n=r.b([Object(o.l)("filterNumber","colorMapped","editMapping"),Object(o.c)("descending")],e)}(a.b);e.b=d},function(t,e,n){"use strict";n.d(e,"a",function(){return d});var r=n(0),i=n(16),o=n(1),a=n(13),u=n(4),s=n(12),c=n(28),l=n(27);function d(t){return void 0===t&&(t="Imposition"),{type:"impositions",label:t}}var p=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.setDefaultRenderer("numbers"),r.setDefaultGroupRenderer("numbers"),r}var n;return r.c(e,t),n=e,Object.defineProperty(e.prototype,"label",{get:function(){var e=t.prototype.getMetaData.call(this).label,n=this._children;return"Imposition"!==e||0===n.length?e:1===n.length?n[0].label:n[0].label+" ("+n.slice(1).map(function(t){return t.label}).join(", ")+")"},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"wrapper",{get:function(){var t=this._children;return 0===t.length?null:t[0]},enumerable:!0,configurable:!0}),e.prototype.getLabel=function(t){var e=this._children;return 0===e.length?"":1===e.length?e[0].getLabel(t):e[0].getLabel(t)+" ("+e.slice(1).map(function(e){return e.label+" = "+e.getLabel(t)})+")"},e.prototype.getColor=function(t){var e=this._children;switch(e.length){case 0:return this.color;case 1:return e[0].getColor(t);default:return e[1].getColor(t)}},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_MAPPING_CHANGED,n.EVENT_COLOR_MAPPING_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},Object.defineProperty(e.prototype,"labels",{get:function(){var t=this.wrapper;return t?t.labels:[]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"dataLength",{get:function(){var t=this.wrapper;return t?t.dataLength:null},enumerable:!0,configurable:!0}),e.prototype.getValue=function(t){var e=this.wrapper;return e?e.getValue(t):[]},e.prototype.getNumber=function(t){var e=this.wrapper;return e?e.getNumber(t):NaN},e.prototype.getRawNumber=function(t){var e=this.wrapper;return e?e.getRawNumber(t):NaN},e.prototype.getExportValue=function(e,n){return"json"===n?this.isMissing(e)?null:{label:this.getLabels(e),color:this.getColor(e),value:this.getRawNumbers(e)}:t.prototype.getExportValue.call(this,e,n)},e.prototype.getNumbers=function(t){var e=this.wrapper;return e?e.getNumbers(t):[]},e.prototype.getRawNumbers=function(t){var e=this.wrapper;return e?e.getRawNumbers(t):[]},e.prototype.getBoxPlotData=function(t){var e=this.wrapper;return e?e.getBoxPlotData(t):null},e.prototype.getRawBoxPlotData=function(t){var e=this.wrapper;return e?e.getRawBoxPlotData(t):null},e.prototype.getMapping=function(){var t=this.wrapper;return t?t.getMapping():new s.a},e.prototype.getOriginalMapping=function(){var t=this.wrapper;return t?t.getOriginalMapping():new s.a},e.prototype.getSortMethod=function(){var t=this.wrapper;return t?t.getSortMethod():u.b.min},e.prototype.setSortMethod=function(t){var e=this.wrapper;return e?e.setSortMethod(t):void 0},e.prototype.isMissing=function(t){var e=this.wrapper;return!e||e.isMissing(t)},e.prototype.setMapping=function(t){var e=this.wrapper;return e?e.setMapping(t):void 0},e.prototype.getColorMapping=function(){var t=this.wrapper;return t?t.getColorMapping():l.b},e.prototype.setColorMapping=function(t){var e=this.wrapper;return e?e.setColorMapping(t):void 0},e.prototype.getFilter=function(){var t=this.wrapper;return t?t.getFilter():Object(u.l)()},e.prototype.setFilter=function(t){var e=this.wrapper;return e?e.setFilter(t):void 0},e.prototype.getRange=function(){var t=this.wrapper;return t?t.getRange():["0","1"]},e.prototype.getMap=function(t){var e=this.wrapper;return e?e.getMap(t):[]},e.prototype.getMapLabel=function(t){var e=this.wrapper;return e?e.getMapLabel(t):[]},e.prototype.getLabels=function(t){var e=this.wrapper;return e?e.getLabels(t):[]},e.prototype.getValues=function(t){var e=this.wrapper;return e?e.getValues(t):[]},e.prototype.compare=function(t,e){return c.c.prototype.compare.call(this,t,e)},e.prototype.group=function(t){return c.c.prototype.group.call(this,t)},e.prototype.groupCompare=function(t,e){return c.c.prototype.groupCompare.call(this,t,e)},e.prototype.insert=function(e,n){return 0!==this._children.length||Object(u.k)(e)?this._children.length>=2?null:t.prototype.insert.call(this,e,n):null},e.prototype.insertImpl=function(e,n){return Object(u.k)(e)&&this.forward.apply(this,[e].concat(Object(i.b)(".impose",c.c.EVENT_MAPPING_CHANGED))),Object(s.d)(e)&&this.forward.apply(this,[e].concat(Object(i.b)(".impose",c.c.EVENT_COLOR_MAPPING_CHANGED))),t.prototype.insertImpl.call(this,e,n)},e.prototype.removeImpl=function(e,n){return Object(u.k)(e)&&this.unforward.apply(this,[e].concat(Object(i.b)(".impose",c.c.EVENT_MAPPING_CHANGED))),Object(s.d)(e)&&this.unforward.apply(this,[e].concat(Object(i.b)(".impose",c.c.EVENT_COLOR_MAPPING_CHANGED))),t.prototype.removeImpl.call(this,e,n)},e.EVENT_MAPPING_CHANGED=c.c.EVENT_MAPPING_CHANGED,e.EVENT_COLOR_MAPPING_CHANGED=c.c.EVENT_COLOR_MAPPING_CHANGED,e=n=r.b([Object(o.l)("filterNumber","colorMapped","editMapping"),Object(o.g)("sort","sortNumbers"),Object(o.c)("descending")],e)}(a.b);e.b=p},function(t,e,n){"use strict";n.d(e,"a",function(){return c});var r=n(0),i=n(9),o=n(1),a=n(2),u=n(36),s=n(4);function c(t){return void 0===t&&(t="Weighted Sum"),{type:"stack",label:t}}var l=function(t){function e(e,n){var r=t.call(this,e,n)||this;r.collapsed=!1;var i=r;return r.adaptChange=function(t,e){i.adaptWidthChange(this.source,t,e)},r.setDefaultRenderer("stack"),r.setDefaultGroupRenderer("stack"),r.setDefaultSummaryRenderer("default"),r}var n;return r.c(e,t),n=e,Object.defineProperty(e.prototype,"label",{get:function(){var e=t.prototype.getMetaData.call(this).label,n=this._children;if("Weighted Sum"!==e||0===n.length)return e;var r=this.getWeights();return n.map(function(t,e){return t.label+" ("+Object(i.k)(100*r[e],1)+"%)"}).join(" + ")},enumerable:!0,configurable:!0}),e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_COLLAPSE_CHANGED,n.EVENT_WEIGHTS_CHANGED,n.EVENT_MULTI_LEVEL_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.setCollapsed=function(t){this.collapsed!==t&&this.fire([n.EVENT_COLLAPSE_CHANGED,a.e.EVENT_DIRTY_HEADER,a.e.EVENT_DIRTY_VALUES,a.e.EVENT_DIRTY],this.collapsed,this.collapsed=t)},e.prototype.getCollapsed=function(){return this.collapsed},Object.defineProperty(e.prototype,"canJustAddNumbers",{get:function(){return!0},enumerable:!0,configurable:!0}),e.prototype.flatten=function(t,e,n,r){void 0===n&&(n=0),void 0===r&&(r=0);var i=null,o=n<=a.e.FLAT_ALL_COLUMNS?this._children:this._children.filter(function(t){return t.isVisible()});if(0===n||n<=a.e.FLAT_ALL_COLUMNS){var u=this.getWidth();if(this.collapsed||(u+=(o.length-1)*r),t.push(i={col:this,offset:e,width:u}),0===n)return u}var s=e;return o.forEach(function(e){s+=e.flatten(t,s,n-1,r)+r}),i&&(i.width=s-e-r),s-e-r},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return n.collapsed=this.collapsed,n},e.prototype.restore=function(e,n){this.collapsed=!0===e.collapsed,t.prototype.restore.call(this,e,n)},e.prototype.insert=function(e,n,r){return void 0===r&&(r=NaN),isNaN(r)||e.setWidth(r/(1-r)*this.getWidth()),e.on(a.e.EVENT_WIDTH_CHANGED+".stack",this.adaptChange),t.prototype.setWidth.call(this,0===this.length?e.getWidth():this.getWidth()+e.getWidth()),t.prototype.insert.call(this,e,n)},e.prototype.push=function(t,e){return void 0===e&&(e=NaN),this.insert(t,this.length,e)},e.prototype.insertAfter=function(t,e,n){void 0===n&&(n=NaN);var r=this.indexOf(e);return r<0?null:this.insert(t,r+1,n)},e.prototype.adaptWidthChange=function(e,r,o){if(!Object(i.l)(r,o,.5)){var u=this.getWeights(),s=this.getWidth(),c=r/s,l=(1-c-(o-r)/s)/(1-c),d=this._children.map(function(t){if(t===e)return o;var n=t.getWidth()*l,r=isNaN(n)||n<1?0:n;return t.setWidthImpl(r),r});t.prototype.setWidth.call(this,d.reduce(function(t,e){return t+e},0)),this.fire([n.EVENT_WEIGHTS_CHANGED,n.EVENT_MULTI_LEVEL_CHANGED,a.e.EVENT_DIRTY_HEADER,a.e.EVENT_DIRTY_VALUES,a.e.EVENT_DIRTY],u,this.getWeights())}},e.prototype.getWeights=function(){var t=this.getWidth();return this._children.map(function(e){return e.getWidth()/t})},e.prototype.setWeights=function(t){var e,r=this.getWeights(),i=t.length-this.length;if(i<0)if((e=t.reduce(function(t,e){return t+e},0))<=1)for(var o=0;o<-i;++o)t.push(1/-i*(1-e));else if(e<=100)for(o=0;o<-i;++o)t.push(1/-i*(100-e));t=t.slice(0,this.length),e=t.reduce(function(t,e){return t+e},0)/this.getWidth(),t=t.map(function(t){return t/e}),this._children.forEach(function(e,n){e.setWidthImpl(t[n])}),this.fire([n.EVENT_WEIGHTS_CHANGED,n.EVENT_MULTI_LEVEL_CHANGED,a.e.EVENT_DIRTY_HEADER,a.e.EVENT_DIRTY_VALUES,a.e.EVENT_DIRTY],r,t)},e.prototype.removeImpl=function(e,n){return e.on(a.e.EVENT_WIDTH_CHANGED+".stack",null),t.prototype.setWidth.call(this,0===this.length?100:this.getWidth()-e.getWidth()),t.prototype.removeImpl.call(this,e,n)},e.prototype.setWidth=function(e){var n=e/this.getWidth();this._children.forEach(function(t){t.setWidthImpl(t.getWidth()*n)}),t.prototype.setWidth.call(this,e)},e.prototype.compute=function(t){var e=this.getWidth();return this._children.reduce(function(n,r){return n+r.getValue(t)*(r.getWidth()/e)},0)},e.prototype.getRenderer=function(){return this.getCollapsed()&&this.isLoaded()?n.COLLAPSED_RENDERER:t.prototype.getRenderer.call(this)},e.prototype.isMissing=function(t){return this._children.some(function(e){return Object(s.i)(e)&&e.isMissing(t)})},e.prototype.getExportValue=function(e,n){return"json"===n?{value:this.getRawNumber(e),children:this.children.map(function(t){return t.getExportValue(e,n)})}:t.prototype.getExportValue.call(this,e,n)},e.EVENT_COLLAPSE_CHANGED="collapseChanged",e.EVENT_WEIGHTS_CHANGED="weightsChanged",e.EVENT_MULTI_LEVEL_CHANGED="nestedChildRatio",e.COLLAPSED_RENDERER="number",e=n=r.b([Object(o.l)("editWeights","collapse")],e)}(u.a);e.b=l},function(t,e,n){"use strict";n.d(e,"c",function(){return c});var r=n(0),i=n(18),o=n(1),a=n(2),u=n(36),s=n(4);function c(t){return void 0===t&&(t="Reduce"),{type:"reduce",label:t}}var l=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.reduce=n.reduce||s.b.max,r.setDefaultRenderer("interleaving"),r.setDefaultGroupRenderer("interleaving"),r.setDefaultSummaryRenderer("interleaving"),r}var n;return r.c(e,t),n=e,Object.defineProperty(e.prototype,"label",{get:function(){var e=t.prototype.getMetaData.call(this).label;return"Reduce"!==e?e:""+this.reduce[0].toUpperCase()+this.reduce.slice(1)+"("+this.children.map(function(t){return t.label}).join(", ")+")"},enumerable:!0,configurable:!0}),e.prototype.getColor=function(t){var e=this._children;if(0===e.length||this.reduce===s.b.q1||this.reduce===s.b.q3||this.reduce===s.b.mean)return this.color;var n=this.compute(t),r=e.find(function(e){return e.getValue(t)===n});return r?r.color:this.color},e.prototype.compute=function(t){var e=this._children.map(function(e){return e.getValue(t)}).filter(function(t){return!isNaN(t)});if(0===e.length)return NaN;switch(this.reduce){case s.b.mean:return e.reduce(function(t,e){return t+e},0)/e.length;case s.b.max:return Math.max.apply(Math,e);case s.b.min:return Math.min.apply(Math,e);case s.b.median:return Object(i.h)(e);case s.b.q1:return Object(i.j)(e.sort(function(t,e){return t-e}),.25);case s.b.q3:return Object(i.j)(e.sort(function(t,e){return t-e}),.75)}},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_REDUCE_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getReduce=function(){return this.reduce},e.prototype.setReduce=function(t){this.reduce!==t&&this.fire([n.EVENT_REDUCE_CHANGED,a.e.EVENT_DIRTY_VALUES,a.e.EVENT_DIRTY],this.reduce,this.reduce=t)},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return n.reduce=this.reduce,n},e.prototype.restore=function(e,n){this.reduce=e.reduce||this.reduce,t.prototype.restore.call(this,e,n)},Object.defineProperty(e.prototype,"canJustAddNumbers",{get:function(){return!0},enumerable:!0,configurable:!0}),e.prototype.getExportValue=function(e,n){return"json"===n?{value:this.getRawNumber(e),children:this.children.map(function(t){return t.getExportValue(e,n)})}:t.prototype.getExportValue.call(this,e,n)},e.EVENT_REDUCE_CHANGED="reduceChanged",e=n=r.b([Object(o.l)("reduce")],e)}(u.a);e.d=l},function(t,e,n){"use strict";n.d(e,"c",function(){return c});var r=n(0),i=n(1),o=n(2),a=n(36),u=n(4),s="let s = 0;\ncol.forEach((c) => s += c.v);\nreturn s / col.length";function c(t){return void 0===t&&(t="script"),{type:"script",label:t,script:s}}var l=function(){function t(t,e,n){this.c=t,this.v=e,this.raw=n}return Object.defineProperty(t.prototype,"type",{get:function(){return this.c.desc.type},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"name",{get:function(){return this.c.getMetaData().label},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"id",{get:function(){return this.c.id},enumerable:!0,configurable:!0}),t}(),d=function(){function t(t,e){var n=this;this.children=t,this.allFactory=e,this.lookup=new Map,this._all=null,t.forEach(function(t){n.lookup.set("ID@"+t.id,t),n.lookup.set("NAME@"+t.name,t)})}return t.prototype.byName=function(t){return this.lookup.get("NAME@"+t)},t.prototype.byID=function(t){return this.lookup.get("ID@"+t)},t.prototype.byIndex=function(t){return this.children[t]},t.prototype.forEach=function(t){return this.children.forEach(t)},Object.defineProperty(t.prototype,"length",{get:function(){return this.children.length},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"all",{get:function(){return null==this._all&&(this._all=this.allFactory?this.allFactory():null),this._all},enumerable:!0,configurable:!0}),t}(),p=function(t){function e(e,r){var i=t.call(this,e,r)||this;return i.script=n.DEFAULT_SCRIPT,i.f=null,i.script=r.script||i.script,i.setDefaultRenderer("number"),i.setDefaultGroupRenderer("number"),i.setDefaultSummaryRenderer("number"),i}var n;return r.c(e,t),n=e,e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_SCRIPT_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.setScript=function(t){this.script!==t&&(this.f=null,this.fire([n.EVENT_SCRIPT_CHANGED,o.e.EVENT_DIRTY_VALUES,o.e.EVENT_DIRTY],this.script,this.script=t))},e.prototype.getScript=function(){return this.script},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return n.script=this.script,n},e.prototype.restore=function(e,n){this.script=e.script||this.script,t.prototype.restore.call(this,e,n)},e.prototype.compute=function(t){var e=this;null==this.f&&(this.f=new Function("children","values","raws","col","row","index",function(t){var e=t.trim();return e.includes("return")||(e="return ("+e+");"),"\n  const max = function(arr) { return Math.max.apply(Math, arr); };\n  const min = function(arr) { return Math.min.apply(Math, arr); };\n  const extent = function(arr) { return [min(arr), max(arr)]; };\n  const clamp = function(v, minValue, maxValue) { return v < minValue ? minValue : (v > maxValue ? maxValue : v); };\n  const normalize = function(v, minMax, max) {\n    if (Array.isArray(minMax)) {\n      minMax = minMax[0];\n      max = minMax[1];\n    }\n    return (v - minMax) / (max - minMax);\n  };\n  const denormalize = function(v, minMax, max) {\n    if (Array.isArray(minMax)) {\n      minMax = minMax[0];\n      max = minMax[1];\n    }\n    return v * (max - minMax) + minMax;\n  };\n  const linear = function(v, source, target) {\n    target = target || [0, 1];\n    return denormalize(normalize(v, source), target);\n  };\n  const v = (function custom() {\n    "+e+"\n  })();\n\n  return typeof v === 'number' ? v : NaN"}(this.script)));var n=this._children,r=this._children.map(function(e){return e.getValue(t)}),i=this._children.map(function(e){return Object(u.i)(e)?e.getRawNumber(t):null}),o=new d(n.map(function(t,e){return new l(t,r[e],i[e])}),function(){var n=e.findMyRanker().flatColumns;return new d(n.map(function(e){return new l(e,e.getValue(t),Object(u.i)(e)?e.getRawNumber(t):null)}))});return this.f.call(this,n,r,i,o,t.v,t.i)},e.prototype.getExportValue=function(e,n){return"json"===n?{value:this.getRawNumber(e),children:this.children.map(function(t){return t.getExportValue(e,n)})}:t.prototype.getExportValue.call(this,e,n)},e.EVENT_SCRIPT_CHANGED="scriptChanged",e.DEFAULT_SCRIPT=s,e=n=r.b([Object(i.l)("script")],e)}(a.a);e.d=p},function(t,e,n){"use strict";n.d(e,"b",function(){return o});var r=n(0),i=n(1);function o(t){return void 0===t&&(t="Selection Checkboxes"),{type:"selection",label:t,fixed:!0}}var a=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.setDefaultWidth(20),r}var n;return r.c(e,t),n=e,Object.defineProperty(e.prototype,"frozen",{get:function(){return!1!==this.desc.frozen},enumerable:!0,configurable:!0}),e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_SELECT])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.setValue=function(t,e){return this.getValue(t)===e||this.setImpl(t,e)},e.prototype.setValues=function(t,e){if(0!==t.length)return this.desc.setterAll&&this.desc.setterAll(t,e),this.fire(n.EVENT_SELECT,t[0],e,t),!0},e.prototype.setImpl=function(t,e){return this.desc.setter&&this.desc.setter(t,e),this.fire(n.EVENT_SELECT,t,e),!0},e.prototype.toggleValue=function(t){var e=this.getValue(t);return this.setImpl(t,!e),!e},e.prototype.compare=function(t,e){var n=!0===this.getValue(t),r=!0===this.getValue(e);return n===r?0:n<r?-1:1},e.prototype.group=function(t){return this.getValue(t)?n.SELECTED_GROUP:n.NOT_SELECTED_GROUP},e.SELECTED_GROUP={name:"Selected",color:"orange"},e.NOT_SELECTED_GROUP={name:"Unselected",color:"gray"},e.EVENT_SELECT="select",e=n=r.b([Object(i.d)(),Object(i.l)("sort","sortBy","group","groupBy","invertSelection"),Object(i.b)("support")],e)}(n(8).b);e.c=a},function(t,e,n){"use strict";var r=n(0),i=n(1),o=n(23),a=n(35),u=n(32),s=n(21),c=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.patternFunction=null,r.setDefaultWidth(200),r.alignment=n.alignment||s.a.left,r.escape=!1!==n.escape,r.pattern=n.pattern||"",r.patternTemplates=n.patternTemplates||[],r.setDefaultRenderer("map"),r}return r.c(e,t),e.prototype.setPattern=function(t){u.d.prototype.setPattern.call(this,t)},e.prototype.getPattern=function(){return this.pattern},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([u.d.EVENT_PATTERN_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getValue=function(t){return this.getLinkMap(t).map(function(t){var e=t.key,n=t.value;return{key:e,value:n?n.href:""}})},e.prototype.getLabels=function(t){return this.getLinkMap(t).map(function(t){var e=t.key,n=t.value;return{key:e,value:n?n.alt:""}})},e.prototype.getLinkMap=function(e){var n=this;return t.prototype.getValue.call(this,e).map(function(t){var r=t.key,i=t.value;return{key:r,value:n.transformValue(i,e,r)}})},e.prototype.transformValue=function(t,e,n){return null==t||""===t?null:"string"==typeof t?this.pattern?(this.patternFunction||(this.patternFunction=Object(o.f)(this.pattern,"item","key")),{alt:t,href:this.patternFunction.call(this,t,e.v,n)}):{alt:t,href:t}:t},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return this.pattern!==this.desc.pattern&&(n.pattern=this.pattern),n},e.prototype.restore=function(e,n){e.pattern&&(this.pattern=e.pattern),t.prototype.restore.call(this,e,n)},e.EVENT_PATTERN_CHANGED=u.d.EVENT_PATTERN_CHANGED,e=r.b([Object(i.l)("search","editPattern")],e)}(a.a);e.a=c},function(t,e,n){"use strict";var r=n(0),i=n(9),o=n(10),a=n(50),u=function(t){function e(e,n,r){void 0===n&&(n=[]),void 0===r&&(r={});var i=t.call(this,n,r)||this;i._data=e,i.options={filterGlobally:!1,jumpToSearchResult:!1,maxNestedSortingCriteria:1/0,maxGroupColumns:1/0},i.filter=null,Object.assign(i.options,r),i._dataRows=s(e);var o=i;return i.reorderAll=function(){var t=this.source;o.getRankings().forEach(function(e){e!==t&&e.dirtyOrder()})},i}return r.c(e,t),e.prototype.setFilter=function(t){this.filter=t,this.reorderAll()},e.prototype.getFilter=function(){return this.filter},e.prototype.getTotalNumberOfRows=function(){return this.data.length},e.prototype.getMaxGroupColumns=function(){return this.options.maxGroupColumns},e.prototype.getMaxNestedSortingCriteria=function(){return this.options.maxNestedSortingCriteria},Object.defineProperty(e.prototype,"data",{get:function(){return this._data},enumerable:!0,configurable:!0}),e.prototype.setData=function(t){this._data=t,this._dataRows=s(t),this.reorderAll()},e.prototype.clearData=function(){this.setData([])},e.prototype.appendData=function(t){var e,n;(e=this._data).push.apply(e,t),(n=this._dataRows).push.apply(n,s(t)),this.reorderAll()},e.prototype.cloneRanking=function(e){var n=t.prototype.cloneRanking.call(this,e);return this.options.filterGlobally&&n.on(o.c.EVENT_FILTER_CHANGED+".reorderAll",this.reorderAll),n},e.prototype.cleanUpRanking=function(e){this.options.filterGlobally&&e.on(o.c.EVENT_FILTER_CHANGED+".reorderAll",null),t.prototype.cleanUpRanking.call(this,e)},e.prototype.sortImpl=function(t){var e=this;if(0===this._data.length)return[];var n=this._data.map(function(t,e){return{v:t,i:e,group:null}}),r=null;if(this.options.filterGlobally){var i=this.getRankings().filter(function(t){return t.isFiltered()});i.length>0&&(r=function(t){return i.every(function(e){return e.filter(t)})})}else t.isFiltered()&&(r=function(e){return t.filter(e)});if((r||this.filter)&&(n=n.filter(function(t){return(!e.filter||e.filter(t))&&(!r||r(t))})),0===n.length)return[];if(n.forEach(function(e){return e.group=t.grouper(e)||a.a}),1===new Set(n.map(function(t){return t.group.name})).size){var o=n[0].group;n.sort(function(e,n){return t.comparator(e,n)});var u=n.map(function(t){return t.i});return[Object.assign({order:u},o)]}n.sort(function(e,n){var r=e.group,i=n.group;return r.name!==i.name?r.name.toLowerCase().localeCompare(i.name.toLowerCase()):t.comparator(e,n)});var s=[Object.assign({order:[],rows:[]},n[0].group)],c=s[0];return n.forEach(function(t){var e=t.group;e.name===c.name?(c.order.push(t.i),c.rows.push(t)):(c=Object.assign({order:[t.i],rows:[t]},e),s.push(c))}),s.sort(function(e,n){return t.groupComparator(e,n)}),s},e.prototype.viewRaw=function(t){var e=this;return t.map(function(t){return e._data[t]})},e.prototype.viewRawRows=function(t){var e=this;return t.map(function(t){return e._dataRows[t]})},e.prototype.view=function(t){return this.viewRaw(t)},e.prototype.fetch=function(t){var e=this;return t.map(function(t){return t.map(function(t){return e._dataRows[t]})})},e.prototype.stats=function(t){var e=this,n=null,r=function(){return null==n&&(n=e.viewRawRows(t)),n};return{stats:function(t){return Object(i.i)(r(),function(e){return t.getNumber(e)},function(e){return t.isMissing(e)},[0,1])},hist:function(t){return Object(i.h)(r(),function(e){return t.getCategory(e)},t.categories)}}},e.prototype.mappingSample=function(t){var e=this,n=this._dataRows.length;if(n<=120)return this._dataRows.map(t.getRawNumber.bind(t));for(var r=[],i=0;i<120;++i){for(var o=Math.floor(Math.random()*(n-1));r.indexOf(o)>=0;)o=Math.floor(Math.random()*(n-1));r.push(o)}return r.map(function(n){return t.getRawNumber(e._dataRows[n])})},e.prototype.searchAndJump=function(t,e){for(var n="string"==typeof(t="string"==typeof t?t.toLowerCase():t)?function(e){return e.toLowerCase().indexOf(t)>=0}:t.test.bind(t),r=[],i=0;i<this._dataRows.length;++i)n(e.getLabel(this._dataRows[i]))&&r.push(i);this.jumpToNearest(r)},e}(n(90).a);function s(t){return t.map(function(t,e){return{v:t,i:e}})}e.a=u},function(t,e,n){"use strict";function r(t){return"function"==typeof t.getMap&&"function"==typeof t.getMapLabel}function i(t){return"function"==typeof t.getLabels&&"function"==typeof t.getValues&&r(t)}n.d(e,"b",function(){return r}),n.d(e,"a",function(){return i})},function(t,e,n){"use strict";n.d(e,"a",function(){return i});var r=n(0);function i(t){return void 0===t&&(t="Nested"),{type:"nested",label:t}}var o=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.c(e,t),e.prototype.compare=function(t,e){for(var n=0,r=this.children;n<r.length;n++){var i=r[n].compare(t,e);if(0!==i)return i}return 0},e.prototype.getLabel=function(t){return this.children.map(function(e){return e.getLabel(t)}).join(";")},e.prototype.getValue=function(t){return this.children.map(function(e){return e.getValue(t)}).join(";")},e}(n(85).a);e.b=o},function(t,e,n){"use strict";function r(t){return t&&void 0!==t.name}n.d(e,"b",function(){return r})},function(t,e,n){"use strict";n.d(e,"a",function(){return i}),n.d(e,"c",function(){return o}),n.d(e,"b",function(){return a}),n.d(e,"d",function(){return u});var r=n(92),i="function"==typeof r?r:n.n(r).a;function o(t){return function(e,n){if(e in t)return t[e];if(e.indexOf(".")>0){var r=e.substring(0,e.indexOf("."));if(e=e.substring(e.indexOf(".")+1),r in t&&e in t[r])return t[r][e]}return n}}function a(t,e){return t.length===e.length&&t.every(function(t,n){return t===e[n]})}function u(t){return t.replace(/[\s!#$%&'()*+,.\/:;<=>?@\[\\\]^`{|}~]+/g,"_")}},function(t,e,n){"use strict";var r=n(0),i=n(47),o=n(74),a=n(9),u=n(16),s=n(4),c=n(11),l=n(73),d=n(14),p=n(19),f=n(55),h=n(53),g=n(77),m=function(){function t(t){void 0===t&&(t=document);var e=this;this.escKeyListener=function(t){27===t.which&&e.removeLast()},this.openDialogs=[],this.node=t.createElement("div"),this.node.classList.add("lu-backdrop"),this.node.innerHTML="<div></div>",this.node.onclick=function(){e.removeAll()}}return t.prototype.setHighlight=function(t){this.node.firstElementChild.style.clipPath="polygon(\n      0% 0%,\n      0% 100%,\n      "+t.left+"px 100%,\n      "+t.left+"px "+t.top+"px,\n      "+(t.left+t.width)+"px "+t.top+"px,\n      "+(t.left+t.width)+"px "+(t.top+t.height)+"px,\n      "+t.left+"px "+(t.top+t.height)+"px,\n      "+t.left+"px 100%,\n      100% 100%,\n      100% 0%\n    )"},t.prototype.setHighlightColumn=function(t){var e=this.node.parentElement;if(e){var n=e.querySelector('.lu-header[data-col-id="'+t.id+'"]');if(n){var r=n.getBoundingClientRect(),i=e.getBoundingClientRect();this.setHighlight({left:r.left-i.left,top:r.top-i.top,width:r.width,height:i.height})}else this.clearHighlight()}else this.clearHighlight()},t.prototype.clearHighlight=function(){this.node.firstElementChild.style.clipPath=null},t.prototype.removeLast=function(){0!==this.openDialogs.length&&this.remove(this.openDialogs[this.openDialogs.length-1])},t.prototype.removeAll=function(){0!==this.openDialogs.length&&(this.openDialogs.splice(0,this.openDialogs.length).forEach(function(t){return t.destroy()}),this.takeDown())},t.prototype.remove=function(t){var e=this.openDialogs.indexOf(t);return!(e<0||(this.openDialogs.splice(e,this.openDialogs.length-e).reverse().forEach(function(t){return t.destroy()}),0===this.openDialogs.length&&this.takeDown(),0))},t.prototype.removeAboveLevel=function(t){var e=this;this.openDialogs.filter(function(e){return e.level>=t}).reverse().forEach(function(t){return e.remove(t)})},t.prototype.removeLike=function(t){var e=this.openDialogs.find(function(e){return t.equals(e)});return!!e&&(this.remove(e),!0)},t.prototype.setUp=function(){this.node.ownerDocument.addEventListener("keyup",this.escKeyListener,{passive:!0}),this.node.style.display="block"},t.prototype.takeDown=function(){this.clearHighlight(),this.node.ownerDocument.removeEventListener("keyup",this.escKeyListener),this.node.style.display=null},t.prototype.push=function(t){this.removeAboveLevel(t.level),0===this.openDialogs.length&&this.setUp(),this.openDialogs.push(t)},t}(),b=function(t){function e(e,n,r){var a=t.call(this)||this;a.data=e,a.histCache=new Map,a.rankings=[],a.slopeGraphs=[],a.updateAbles=[],a.zoomFactor=1,a.idPrefix="lu"+Math.random().toString(36).slice(-8).substr(0,3),a.enabledHighlightListening=!1,a.options=r,a.node=n.ownerDocument.createElement("main"),a.node.id=a.idPrefix,a.node.classList.toggle("lu-whole-hover",r.expandLineOnHover),n.appendChild(a.node);var u=function(t){var e=a.histCache.get(t.id);return null==e||e instanceof Promise?null:e},s=new m(n.ownerDocument);return n.appendChild(s.node),a.ctx={idPrefix:a.idPrefix,document:n.ownerDocument,provider:e,dialogManager:s,toolbar:a.options.toolbar,option:Object(o.c)(Object.assign({useGridLayout:!0},a.options)),statsOf:u,renderer:function(t,e){return Object(f.i)(t,a.options.renderers).create(t,a.ctx,u(t),e)},groupRenderer:function(t,e){return Object(f.h)(t,a.options.renderers).createGroup(t,a.ctx,u(t),e)},summaryRenderer:function(t,e,n){return Object(f.j)(t,a.options.renderers).createSummary(t,a.ctx,e,n)},totalNumberOfRows:0,createRenderer:function(t,e){return{single:this.renderer(t,e),group:this.groupRenderer(t,e),summary:r.summaryHeader?this.summaryRenderer(t,!1,e):null,singleId:t.getRenderer(),groupId:t.getGroupRenderer(),summaryId:t.getSummaryRenderer(),singleTemplate:null,groupTemplate:null,summaryTemplate:null}},getPossibleRenderer:function(t){return{item:Object(f.m)(t,a.options.renderers),group:Object(f.l)(t,a.options.renderers),summary:Object(f.n)(t,a.options.renderers)}},colWidth:function(t){return t.isVisible()?t.getWidth():0}},a.table=new i.MultiTableRowRenderer(a.node,"#"+a.idPrefix),a.style.addRule("lineup_groupPadding","\n       #"+a.idPrefix+" .lu-row[data-agg=group],\n       #"+a.idPrefix+" .lu-row[data-meta~=last] {\n        margin-bottom: "+r.groupPadding+"px;\n       }",!1),a.style.addRule("lineup_rowPadding","\n       #"+a.idPrefix+" .lu-row[data-lod] {\n         padding-top: 0;\n       }",!1),a.style.addRule("lineup_rowPadding2","\n        #"+a.idPrefix+" .lu-row,\n        #"+a.idPrefix+" .lu-row[data-lod]:hover,\n        #"+a.idPrefix+" .lu-row[data-lod].le-highlighted,\n        #"+a.idPrefix+" .lu-row[data-lod].lu-selected,\n        #"+a.idPrefix+" .lu-row[data-lod]:hover + .lu-row,\n        #"+a.idPrefix+" .lu-row[data-lod].le-highlighted + .lu-row,\n        #"+a.idPrefix+" .lu-row[data-lod].lu-selected + .lu-row {\n          padding-top: "+r.rowPadding+"px;\n        }",!1),a.style.addRule("lineup_rotation","\n       #"+a.idPrefix+".lu-rotated-label .lu-label.lu-rotated {\n           transform: rotate("+-a.options.labelRotation+"deg);\n       }"),a.initProvider(e),a}return r.c(e,t),Object.defineProperty(e.prototype,"style",{get:function(){return this.table.style},enumerable:!0,configurable:!0}),e.prototype.zoomOut=function(){this.zoomFactor=Math.max(this.zoomFactor-.1,.5),this.updateZoomFactor(),this.update()},e.prototype.zoomIn=function(){this.zoomFactor=Math.min(this.zoomFactor+.1,2),this.updateZoomFactor(),this.update()},e.prototype.updateZoomFactor=function(){this.node.querySelector("main").style.fontSize=100*this.zoomFactor+"%"},e.prototype.pushUpdateAble=function(t){this.updateAbles.push(t)},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([e.EVENT_HIGHLIGHT_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.setDataProvider=function(t){this.takeDownProvider(),this.data=t,this.ctx.provider=t,this.initProvider(t)},e.prototype.takeDownProvider=function(){var t=this;this.data.on(p.e.EVENT_SELECTION_CHANGED+".body",null),this.data.on(p.e.EVENT_ADD_RANKING+".body",null),this.data.on(p.e.EVENT_REMOVE_RANKING+".body",null),this.data.on(p.e.EVENT_GROUP_AGGREGATION_CHANGED+".body",null),this.data.on(p.e.EVENT_JUMP_TO_NEAREST+".body",null),this.rankings.forEach(function(e){return t.table.remove(e)}),this.rankings.splice(0,this.rankings.length),this.slopeGraphs.forEach(function(e){return t.table.remove(e)}),this.slopeGraphs.splice(0,this.slopeGraphs.length)},e.prototype.initProvider=function(t){var e=this;t.on(p.e.EVENT_SELECTION_CHANGED+".body",function(){return e.updateSelection(t.getSelection())}),t.on(p.e.EVENT_ADD_RANKING+".body",function(t){e.addRanking(t)}),t.on(p.e.EVENT_REMOVE_RANKING+".body",function(t){e.removeRanking(t)}),t.on(p.e.EVENT_GROUP_AGGREGATION_CHANGED+".body",function(t){e.update(e.rankings.filter(function(e){return e.ranking===t}))}),t.on(p.e.EVENT_JUMP_TO_NEAREST+".body",function(t){e.setHighlightToNearest(t,!0)}),this.data.getRankings().forEach(function(t){return e.addRanking(t)})},e.prototype.updateSelection=function(t){var e=new Set(t);this.rankings.forEach(function(t){return t.updateSelection(e)}),this.slopeGraphs.forEach(function(t){return t.updateSelection(e)})},e.prototype.updateHist=function(t,e){var n=this;this.options.summaryHeader&&((t?[t]:this.rankings).forEach(function(t){var r=t.ranking,i=r.getOrder(),o=e?[e]:r.flatColumns,a=null==i?null:n.data.stats(i);o.filter(function(t){return t.isVisible()&&Object(s.i)(t)}).forEach(function(t){n.histCache.set(t.id,null==a?null:a.stats(t))}),o.filter(function(t){return Object(c.c)(t)&&t.isVisible()}).forEach(function(t){n.histCache.set(t.id,null==a?null:a.hist(t))}),e?t.updateHeaderOf(e):t.updateHeaders()}),this.updateAbles.forEach(function(t){return t(n.ctx)}))},e.prototype.addRanking=function(t){var e=this;if(this.rankings.length>0){var n=this.table.pushSeparator(function(n,r){return new g.b(n,r,t.id+"S",e.ctx,{mode:"band"===e.options.defaultSlopeGraphMode?g.a.BAND:g.a.ITEM})});this.slopeGraphs.push(n)}var r=this.table.pushTable(function(n,r,i,o){return new h.c(t,n,r,i,o,e.ctx,{animation:e.options.animated,customRowUpdate:e.options.customRowUpdate||function(){},levelOfDetail:e.options.levelOfDetail||function(){return"high"},flags:e.options.flags})});r.on(h.c.EVENT_WIDTH_CHANGED,function(){e.updateRotatedHeaderState(),e.table.widthChanged()}),r.on(h.c.EVENT_UPDATE_DATA,function(){return e.update([r])}),r.on(h.c.EVENT_UPDATE_HIST,function(t){return e.updateHist(r,t)}),this.forward(r,h.c.EVENT_HIGHLIGHT_CHANGED),this.enabledHighlightListening&&r.enableHighlightListening(!0),t.on(Object(u.b)(".renderer",d.b.EVENT_ORDER_CHANGED),function(){return e.updateHist(r)}),this.rankings.push(r),this.update([r])},e.prototype.updateRotatedHeaderState=function(){if(0!==this.options.labelRotation){var t=this.node.querySelector(".lu-label.lu-rotated");this.node.classList.toggle("lu-rotated-label",Boolean(t))}},e.prototype.removeRanking=function(t){if(!t)return this.rankings.splice(0,this.rankings.length),this.slopeGraphs.splice(0,this.slopeGraphs.length),void this.table.clear();var e=this.rankings.findIndex(function(e){return e.ranking===t});if(!(e<0)){var n=this.rankings.splice(e,1)[0],r=this.slopeGraphs.splice(0===e?e:e-1,1)[0];this.table.remove(n),r&&this.table.remove(r)}},e.prototype.update=function(t){var e=this;if(void 0===t&&(t=this.rankings),0!==(t=t.filter(function(t){return!t.hidden&&t.body.parentElement})).length){var n=t.map(function(t){return t.ranking.getOrder()}),r=this.data.fetch(n);this.ctx.totalNumberOfRows=Math.max.apply(Math,r.map(function(t){return t.length}));var o=r.map(function(t){return t.map(function(t){return t})});0===this.histCache.size&&this.updateHist();var u=function(t){return Object(a.k)(t,2)},s=u(this.zoomFactor*this.options.rowPadding),c=u(this.zoomFactor*this.options.groupPadding);t.forEach(function(t,n){var r=t.groupData(o[n]),a=function(t,n){if(e.options.dynamicHeight){var r=e.options.dynamicHeight(n,t),i=function(t,e){return"number"==typeof t?t:t(e)};if(r)return{defaultHeight:u(e.zoomFactor*r.defaultHeight),height:function(t){return u(e.zoomFactor*i(r.height,t))},padding:function(t){return u(e.zoomFactor*i(r.padding,t))}}}var o=u(e.zoomFactor*e.options.rowHeight),a=u(e.zoomFactor*e.options.groupHeight);return{defaultHeight:o,height:function(t){return Object(l.b)(t)?a:o},padding:s}}(t.ranking,r),d=a.height,p=a.defaultHeight,f=a.padding,h=Object(i.nonUniformContext)(r.map(d),p,function(t){var e="number"==typeof f?f:f(r[t]||null);return t>=0&&r[t]&&(Object(l.b)(r[t])||"last"===r[t].meta||"first last"===r[t].meta)?c+e:e});t.render(r,h)}),this.updateSlopeGraphs(t),this.updateRotatedHeaderState(),this.table.widthChanged()}},e.prototype.updateSlopeGraphs=function(t){var e=this;void 0===t&&(t=this.rankings);var n=new Set(t.map(function(t){return e.rankings.indexOf(t)}));this.slopeGraphs.forEach(function(t,r){if(!t.hidden){var i=r,o=r+1;if(n.has(i)||n.has(o)){var a=e.rankings[i],u=e.rankings[o];t.rebuild(a.currentData,a.context,u.currentData,u.context)}}})},e.prototype.setHighlight=function(t,n){var r=this.rankings.map(function(e){return e.setHighlight(t)});return this.fire(e.EVENT_HIGHLIGHT_CHANGED,t),!(0===this.rankings.length||t<0)&&(n?this.rankings[0].scrollIntoView(t):r[0])},e.prototype.setHighlightToNearest=function(t,e){if(0===this.rankings.length)return!1;var n=this.rankings[0].findNearest(t);return n>=0&&this.setHighlight(n,e)},e.prototype.getHighlight=function(){for(var t=0,e=this.rankings;t<e.length;t++){var n=e[t].getHighlight();if(n>=0)return n}return-1},e.prototype.enableHighlightListening=function(t){for(var e=0,n=this.rankings;e<n.length;e++)n[e].enableHighlightListening(t);this.enabledHighlightListening=t},e.prototype.destroy=function(){this.takeDownProvider(),this.table.destroy(),this.node.remove()},e.EVENT_HIGHLIGHT_CHANGED=h.c.EVENT_HIGHLIGHT_CHANGED,e}(u.a);e.a=b},function(t,e,n){"use strict";var r=n(16),i=n(65),o=n(67),a=n(72),u=n(66),s=n(63),c=n(39),l=n(68),d=n(61),p=n(26),f=n(1),h=n(19),g=n(43),m=n(14),b=n(49),y=n(42),v=n(0),E=n(17),C=function(t){function e(e,n,r,i,o){var a=t.call(this,r,{fullDialog:Boolean(o),resetPossible:!1})||this;return a.column=e,a.addons=n,a.ctx=i,a.onClick=o,a}return v.c(e,t),e.prototype.build=function(t){for(var e=0,n=this.addons;e<n.length;e++){var r=n[e];this.node.insertAdjacentHTML("beforeend","<strong>"+r.title+"</strong>"),r.append(this.column,t,this.dialog,this.ctx)}},e.prototype.submit=function(){return this.onClick&&this.onClick(),!0},e}(E.c),w=function(){function t(t,e){this.ctx=t,this.node=e.createElement("aside"),this.node.classList.add("lu-hierarchy"),this.node.innerHTML='\n      <section class="lu-hierarchy-group">\n      </section>\n      <section class="lu-hierarchy-sort">\n      </section>\n      <section class="lu-hierarchy-sortgroup">\n      </section>\n    ';var n={doc:e,placeholder:"Add Sort Criteria...",formatItem:function(t,e){return e.dataset.typeCat=Object(f.e)(t.col).name,e.dataset.type=t.col.desc.type,t.text}};this.groupAdder=new g.d(Object.assign({},n,{placeholder:"Add Grouping Criteria..."})),this.groupSortAdder=new g.d(Object.assign({},n,{placeholder:"Add Grouping Sort Criteria..."})),this.sortAdder=new g.d(n)}return t.prototype.update=function(t){t?(this.node.style.display=null,this.renderGroups(t,this.node.firstElementChild),this.renderSorting(t,this.node.children[1]),this.renderGroupSorting(t,this.node.lastElementChild)):this.node.style.display="none"},t.prototype.render=function(t,e,n,r,i,o){var a=this,u=new Map(Array.from(t.children).map(function(t){return[t.dataset.id,t]}));t.innerHTML="",e.forEach(function(e){var s=n(e),c=u.get(s.id);if(c)return t.appendChild(c),void Object(b.e)(c,s);var l=Object(y.c)(s,i,a.ctx);t.insertAdjacentHTML("beforeend",'<div data-id="'+s.id+'" class="lu-toolbar">\n      <div class="lu-label">'+s.label+"</div>\n      "+(l.length>0?'<i title="Customize" class="lu-action"><span aria-hidden="true">Customize</span> </i>':"")+'\n      <i title="Move Up" class="lu-action"><span aria-hidden="true">Move Up</span> </i>\n      <i title="Move Down" class="lu-action"><span aria-hidden="true">Move Down</span> </i>\n      <i title="Remove from hierarchy" class="lu-action"><span aria-hidden="true">Remove from hierarchy</span> </i>\n      </div>');var d=t.lastElementChild;function p(t){t.preventDefault(),t.stopPropagation()}d.querySelector('i[title="Move Down"]').onclick=function(t){p(t),o(e,1)},d.querySelector('i[title="Move Up"]').onclick=function(t){p(t),o(e,-1)},d.querySelector("i[title^=Remove]").onclick=function(t){p(t),o(e,0)},l.length>0&&(d.querySelector("i[title=Customize]").onclick=function(t){p(t),a.customize(s,l,t)}),r(e,d),Object(b.e)(d,s)})},t.prototype.renderGroups=function(t,e){var n=t.getGroupCriteria();0!==n.length?(this.render(e,n,function(t){return t},function(t,e){e.insertAdjacentHTML("afterbegin",'<i title="Group" class="lu-action" data-group="true"><span aria-hidden="true">Group</span> </i>')},"group",function(t,e){if(0!==e){var n=t.isGroupedBy();t.findMyRanker().groupBy(t,n+e)}else t.groupByMe()}),this.addGroupAdder(t,n,e)):e.innerHTML=""},t.prototype.renderSorting=function(t,e){var n=t.getSortCriteria();if(0!==n.length){var r=function(t,e){var n=t.col,r=n.isSortedByMe();isFinite(e)?0!==e?n.sortByMe("asc"===r.asc,r.priority+e):n.sortByMe("asc"===r.asc,-1):n.sortByMe("desc"===r.asc,r.priority)};this.render(e,n,function(t){return t.col},function(t,e){e.insertAdjacentHTML("afterbegin",'\n      <i title="Sort" class="lu-action" data-sort="'+(t.asc?"asc":"desc")+'"><span aria-hidden="true">Toggle Sorting</span> </i>'),e.querySelector("i[title=Sort]").onclick=function(e){e.preventDefault(),e.stopPropagation(),r(t,1/0)}},"sort",r),this.addSortAdder(t,n,e)}else e.innerHTML=""},t.prototype.renderGroupSorting=function(t,e){var n=t.getGroupSortCriteria();if(0!==n.length){var r=function(t,e){var n=t.col,r=n.isGroupSortedByMe();isFinite(e)?0!==e?n.groupSortByMe("asc"===r.asc,r.priority+e):n.groupSortByMe("asc"===r.asc,-1):n.groupSortByMe("desc"===r.asc,r.priority)};this.render(e,n,function(t){return t.col},function(t,e){e.insertAdjacentHTML("afterbegin",'\n      <i title="Sort Group" class="lu-action" data-sort="'+(t.asc?"asc":"desc")+'"><span aria-hidden="true">Toggle Sorting</span> </i>'),e.querySelector('i[title="Sort Group"]').onclick=function(e){e.preventDefault(),e.stopPropagation(),r(t,1/0)}},"sortGroup",r),this.addGroupSortAdder(t,n,e)}else e.innerHTML=""},t.prototype.addAdder=function(t,e,n,r,i,o,a){var u=this,s=new Set(r);if(t.data=e.children.filter(function(t){return!Object(f.k)(t)&&!s.has(t)&&o(t)}).map(function(t){return{col:t,id:t.id,text:t.label}}),t.on(g.d.EVENT_SELECT,function(e){var r=Object(y.c)(e.col,n,u.ctx);r.length>0?u.customize(e.col,r,{currentTarget:t.node},function(){return a(e.col)}):a(e.col)}),!(t.data.length<=0)){var c=i.ownerDocument.createElement("footer");c.appendChild(t.node),i.appendChild(c)}},t.prototype.addSortAdder=function(t,e,n){var r=this;this.addAdder(this.sortAdder,t,"sort",e.map(function(t){return t.col}),n,function(t){return Object(y.f)(t,r.ctx)},function(n){t.sortBy(n,Object(f.j)(n),e.length)})},t.prototype.addGroupAdder=function(t,e,n){var r=this;this.addAdder(this.groupAdder,t,"group",e,n,function(t){return Object(y.d)(t,r.ctx)},function(n){t.groupBy(n,e.length)})},t.prototype.addGroupSortAdder=function(t,e,n){var r=this;this.addAdder(this.groupSortAdder,t,"sortGroup",e.map(function(t){return t.col}),n,function(t){return Object(y.e)(t,r.ctx)},function(n){t.groupSortBy(n,Object(f.j)(n),e.length)})},t.prototype.customize=function(t,e,n,r){new C(t,e,Object(y.b)(this.ctx,0,n),this.ctx,r).open()},t}(),A=n(2),x=n(10),N=function(){function t(t,e,n){var r=this;this.column=t,this.ctx=e,this.node=n.createElement("article"),this.node.classList.add("lu-side-panel-entry"),this.node.dataset.colId=t.id,this.node.dataset.type=t.desc.type,this.summary=e.summaryRenderer(t,!0),this.column.on([x.c.EVENT_FILTER_CHANGED+".panel",A.e.EVENT_DIRTY_HEADER+".panel"],function(){r.update()}),this.init(),this.update()}return t.prototype.init=function(){this.node.innerHTML='\n      <header><div class="lu-label"></div><div class="lu-toolbar"></div></header>'+this.summary.template,Object(b.b)(this.node.querySelector(".lu-toolbar"),0,this.column,this.ctx),Object(b.d)(this.node.querySelector("header"),this.column,this.ctx);var t=this.node.lastElementChild;t.classList.add("lu-summary"),t.dataset.renderer=this.column.getSummaryRenderer(),t.dataset.interactive=""},t.prototype.update=function(t){void 0===t&&(t=this.ctx),this.ctx=t,Object(b.e)(this.node,this.column),this.summary.update(this.node.querySelector(".lu-summary"),t.statsOf(this.column))},t.prototype.destroy=function(){this.column.on([x.c.EVENT_FILTER_CHANGED+".panel",A.e.EVENT_DIRTY_HEADER+".panel"],null),this.node.remove()},t}(),T=function(t){function e(e,n){var r=t.call(this,n,{fullDialog:!0})||this;return r.ranking=e,r.before=e.getLabel(),r}return v.c(e,t),e.prototype.build=function(t){t.classList.add("lu-rename-dialog"),t.insertAdjacentHTML("beforeend",'\n      <input type="text" value="'+this.ranking.getLabel()+'" required autofocus placeholder="name">')},e.prototype.reset=function(){this.findInput('input[type="text"]').value=this.before,this.ranking.setLabel(this.before)},e.prototype.submit=function(){var t=this.findInput('input[type="text"]').value;return this.ranking.setLabel(t),!0},e}(E.c),O=function(t){function e(e,n,r){var i=t.call(this,n)||this;return i.ranking=e,i.ctx=r,i}return v.c(e,t),e.prototype.addIcon=function(t,e,n){t.insertAdjacentHTML("beforeend",'<i title="'+e+'" class="lu-action"><span>'+e+"</span> </i>"),t.lastElementChild.onclick=function(t){t.stopPropagation(),n(t)}},e.prototype.build=function(t){var e=this;t.classList.add("lu-more-options"),this.addIcon(t,"Rename",function(t){new T(e.ranking,Object(y.b)(e.ctx,e.level+1,t)).open()}),this.addIcon(t,"Remove",function(){e.destroy(),e.ctx.provider.removeRanking(e.ranking)})},e}(E.c),R=function(){function t(t,e,n,r){this.ranking=t,this.ctx=e,this.options=r,this.entries=new Map,this.node=n.createElement("section"),this.header=n.createElement("div"),this.dropdown=n.createElement("div"),this.dropdown.innerHTML=this.header.innerHTML="<span>"+t.getLabel()+'</span><i class="lu-action" title="More &hellip;"><span aria-hidden="true">More &hellip;"></span></i>',this.header.lastElementChild.onclick=this.dropdown.lastElementChild.onclick=function(n){n.stopPropagation(),new O(t,Object(y.b)(e,1,n),e).open()},this.hierarchy=this.options.hierarchy?new w(e,n):null,this.init()}return t.prototype.init=function(){var t=this;this.node.innerHTML="\n      <div><main></main></div>\n    ",this.hierarchy&&this.node.insertBefore(this.hierarchy.node,this.node.firstChild),this.hierarchy&&this.ranking.on(Object(r.b)(".panel",m.b.EVENT_GROUP_CRITERIA_CHANGED,m.b.EVENT_SORT_CRITERIA_CHANGED,m.b.EVENT_GROUP_SORT_CRITERIA_CHANGED),function(){t.updateHierarchy()}),this.ranking.on(Object(r.b)(".panel",m.b.EVENT_ADD_COLUMN,m.b.EVENT_REMOVE_COLUMN,m.b.EVENT_MOVE_COLUMN),function(){t.updateList(),t.updateHierarchy()}),this.ranking.on(Object(r.b)(".panel",m.b.EVENT_LABEL_CHANGED),function(){t.dropdown.firstElementChild.textContent=t.header.firstElementChild.textContent=t.ranking.getLabel()})},Object.defineProperty(t.prototype,"active",{get:function(){return this.node.classList.contains("lu-active")},set:function(t){this.node.classList.toggle("lu-active",t),this.header.classList.toggle("lu-active",t),this.dropdown.classList.toggle("lu-active",t),t||(this.updateList(),this.updateHierarchy())},enumerable:!0,configurable:!0}),t.prototype.update=function(t){this.ctx=t,this.updateList(),this.updateHierarchy()},t.prototype.updateHierarchy=function(){this.hierarchy&&this.active&&this.hierarchy.update(this.ranking)},t.prototype.updateList=function(){var t=this;if(this.active){var e=this.node.querySelector("main"),n=this.ranking.flatColumns.filter(function(t){return!Object(f.k)(t)});if(0===n.length)return e.innerHTML="",this.entries.forEach(function(t){return t.destroy()}),void this.entries.clear();e.innerHTML="";var r=new Map(this.entries);this.entries.clear(),n.forEach(function(n){var i=r.get(n.id);if(i)return i.update(t.ctx),e.appendChild(i.node),t.entries.set(n.id,i),void r.delete(n.id);var o=new N(n,t.ctx,e.ownerDocument);e.appendChild(o.node),t.entries.set(n.id,o)}),r.forEach(function(t){return t.destroy()})}},t.prototype.destroy=function(){this.header.remove(),this.node.remove(),this.ranking.on(Object(r.b)(".panel",m.b.EVENT_GROUP_CRITERIA_CHANGED,m.b.EVENT_SORT_CRITERIA_CHANGED,m.b.EVENT_GROUP_SORT_CRITERIA_CHANGED,m.b.EVENT_ADD_COLUMN,m.b.EVENT_MOVE_COLUMN,m.b.EVENT_REMOVE_COLUMN,m.b.EVENT_LABEL_CHANGED),null),this.entries.forEach(function(t){return t.destroy()}),this.entries.clear()},t}(),M=function(t){function e(e,n){var r=t.call(this,n)||this;return r.items=e,r}return v.c(e,t),e.prototype.build=function(t){t.classList.add("lu-more-options","lu-choose-options");for(var e=0,n=this.items;e<n.length;e++){var r=n[e];t.appendChild(r)}},e}(E.c);function D(t){return null!=t.desc}var _=function(){function t(t,e,n){void 0===n&&(n={}),this.ctx=t,this.options={additionalDescs:[Object(i.a)("Weighted Sum"),Object(o.c)("Scripted Formula"),Object(a.a)("Nested"),Object(u.c)(),Object(s.a)(),Object(c.a)(),Object(l.b)(),Object(d.b)(),Object(p.b)()],chooser:!0,hierarchy:!0,placeholder:"Add Column...",formatItem:function(t,e){return e.dataset.typeCat=D(t)?t.category.name:t.children[0].category.name,D(t)&&(e.dataset.type=t.desc.type),t.text},collapseable:!0},this.chooser=null,this.descs=[],this.rankings=[],Object.assign(this.options,n),this.node=e.createElement("aside"),this.node.classList.add("lu-side-panel"),this.search=this.options.chooser?new g.d(this.options):null,this.data=t.provider,this.init(),this.update(t)}return t.prototype.init=function(){var t=this;this.node.innerHTML='\n      <aside class="lu-stats"></aside>\n      <header>\n        <i class="lu-action" title="Choose &hellip;"><span aria-hidden="true">Choose &hellip;</span></i>\n      </header>\n      <main></main>\n    ',this.node.querySelector("header > i").onclick=function(e){e.stopPropagation(),new M(t.rankings.map(function(t){return t.dropdown}),Object(y.b)(t.ctx,1,e)).open()},this.options.collapseable&&(this.node.insertAdjacentHTML("beforeend",'<div class="lu-collapser" title="Collapse Panel"></div>'),this.node.lastElementChild.onclick=function(){return t.collapsed=!t.collapsed},this.collapsed="collapsed"===this.options.collapseable),this.initChooser(),this.changeDataStorage(null,this.data)},t.prototype.initChooser=function(){var t=this;this.search&&(this.chooser=this.node.ownerDocument.createElement("header"),this.chooser.innerHTML="<form></form>",this.chooser.firstElementChild.appendChild(this.search.node),this.search.on(g.d.EVENT_SELECT,function(e){var n=t.data.create(e.desc);if(n){var r=t.active;r&&r.ranking.push(n)}}))},Object.defineProperty(t.prototype,"active",{get:function(){return this.rankings.find(function(t){return t.active})},enumerable:!0,configurable:!0}),t.prototype.changeDataStorage=function(t,e){var n,i=this;t&&t.on(Object(r.b)(".panel",h.e.EVENT_ADD_RANKING,h.e.EVENT_REMOVE_RANKING,h.e.EVENT_ADD_DESC,h.e.EVENT_CLEAR_DESC,h.e.EVENT_ORDER_CHANGED,h.e.EVENT_SELECTION_CHANGED),null),this.data=e;var o=function(t){return{desc:t,category:Object(f.f)(t,e.columnTypes),id:t.type+"@"+t.label,text:t.label}};(n=this.descs).splice.apply(n,[0,this.descs.length].concat(e.getColumns().concat(this.options.additionalDescs).map(o))),e.on(h.e.EVENT_ADD_DESC+".panel",function(t){i.descs.push(o(t)),i.updateChooser()}),e.on(h.e.EVENT_CLEAR_DESC+".panel",function(){i.descs.splice(0,i.descs.length),i.updateChooser()}),e.on(Object(r.b)(".panel",h.e.EVENT_SELECTION_CHANGED,h.e.EVENT_ORDER_CHANGED),function(){i.updateStats()}),e.on(Object(r.b)(".panel",h.e.EVENT_ADD_RANKING),function(t,e){i.createEntry(t,e),i.makeActive(e)}),e.on(Object(r.b)(".panel",h.e.EVENT_REMOVE_RANKING),function(t,e){if(e<0)return i.rankings.splice(0,i.rankings.length).forEach(function(t){return t.destroy()}),i.node.querySelector("header").dataset.count="0",void i.makeActive(-1);var n=i.rankings.splice(e,1)[0];i.node.querySelector("header").dataset.count=String(i.rankings.length),n.destroy(),n.active&&i.makeActive(0===i.rankings.length?-1:Math.max(e-1,0))}),this.rankings.splice(0,this.rankings.length).forEach(function(t){return t.destroy()}),e.getRankings().forEach(function(t,e){i.createEntry(t,e)}),this.rankings.length>0&&this.makeActive(0),this.updateStats()},t.prototype.createEntry=function(t,e){var n=this,r=new R(t,this.ctx,this.node.ownerDocument,this.options),i=this.node.querySelector("header"),o=this.node.querySelector("main");i.insertBefore(r.header,i.children[e+1]),i.dataset.count=String(this.rankings.length+1),r.header.onclick=function(t){t.preventDefault(),t.stopPropagation(),n.makeActive(n.rankings.indexOf(r))},r.dropdown.onclick=r.header.onclick=function(t){t.preventDefault(),t.stopPropagation(),n.ctx.dialogManager.removeAboveLevel(0),n.makeActive(n.rankings.indexOf(r))},o.insertBefore(r.node,o.children[e]),this.rankings.splice(e,0,r)},Object.defineProperty(t.prototype,"collapsed",{get:function(){return this.node.classList.contains("lu-collapsed")},set:function(t){this.node.classList.toggle("lu-collapsed",t),t||(this.updateChooser(),this.updateStats(),this.updateRanking())},enumerable:!0,configurable:!0}),t.prototype.makeActive=function(t){this.rankings.forEach(function(e,n){return e.active=t===n});var e=this.active;if(e&&this.chooser){e.node.insertAdjacentElement("afterbegin",this.chooser);var n=this.node.closest(".lu"),r=n?n.querySelector('article[data-ranking="'+e.ranking.id+'"]'):null;r&&r.scrollIntoView()}this.updateRanking()},t.prototype.updateRanking=function(){var t=this.active;t&&!this.collapsed&&t.update(this.ctx)},t.prototype.update=function(t){var e=this.data;this.ctx=t,t.provider!==e&&this.changeDataStorage(e,t.provider),this.updateChooser(),this.updateStats();var n=this.active;n&&n.update(t)},t.prototype.updateStats=function(){if(!this.collapsed){var t=this.node.querySelector("aside.lu-stats"),e=this.data.getSelection(),n=this.data.getFirstRanking(),r=n?n.getGroups().reduce(function(t,e){return t+e.order.length},0):0;t.innerHTML="Showing <strong>"+r+"</strong> of "+this.data.getTotalNumberOfRows()+" items"+(e.length>0?"; <span>"+e.length+" selected</span>":"")}},t.prototype.destroy=function(){this.node.remove(),this.data&&(this.rankings.forEach(function(t){return t.destroy()}),this.rankings.length=0,this.data.on(Object(r.b)(".panel",h.e.EVENT_ADD_RANKING,h.e.EVENT_REMOVE_RANKING,h.e.EVENT_ADD_DESC),null))},t.groupByType=function(t){var e=new Map;return t.forEach(function(t){e.has(t.category)?e.get(t.category).push(t):e.set(t.category,[t])}),Array.from(e).map(function(t){var e=t[0],n=t[1];return{text:e.label,order:e.order,children:n.sort(function(t,e){return t.text.localeCompare(e.text)})}}).sort(function(t,e){return t.order-e.order})},t.prototype.updateChooser=function(){this.search&&!this.collapsed&&(this.search.data=t.groupByType(this.descs))},t}();e.b=_},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r,i=n(47),o=n(73),a=n(5),u=function(){function t(t,e,n){this.left=t,this.right=e,this.dataIndices=n}return t.prototype.isSelected=function(t){return 1===this.dataIndices.length?t.has(this.dataIndices[0]):this.dataIndices.some(function(e){return t.has(e)})},t.prototype.update=function(t,e){t.setAttribute("data-i",String(this.dataIndices[0])),t.setAttribute("class","lu-slope"),t.setAttribute("d","M0,"+this.left+"L"+e+","+this.right)},t}(),s=function(){function t(t,e,n){this.left=t,this.right=e,this.dataIndices=n}return t.prototype.isSelected=function(t){return this.dataIndices.some(function(e){return t.has(e)})},t.prototype.update=function(t,e){t.setAttribute("class","lu-group-slope"),t.setAttribute("d","M0,"+this.left[0]+"L"+e+","+this.right[0]+"L"+e+","+this.right[1]+"L0,"+this.left[1]+"Z")},t}();!function(t){t.ITEM="item",t.BAND="band"}(r||(r={}));var c=function(){function t(t,e,n,i,o){void 0===o&&(o={}),this.header=t,this.body=e,this.id=n,this.ctx=i,this.leftSlopes=[],this.rightSlopes=[],this.pool=[],this.scrollListener=null,this.width=a.g,this.height=0,this.current=null,this._mode=r.ITEM,this.node=t.ownerDocument.createElementNS("http://www.w3.org/2000/svg","svg"),this.node.innerHTML='<g transform="translate(0,0)"></g>',t.classList.add("lu-slopegraph-header"),this._mode=o.mode===r.BAND?r.BAND:r.ITEM,this.initHeader(t),e.classList.add("lu-slopegraph"),this.body.style.height="1px",e.appendChild(this.node)}return t.prototype.init=function(){var t=this;this.hide();var e=this.body.parentElement.__le_scroller__,n=e.asInfo();e.push("animation",this.scrollListener=function(e){Math.abs(n.top-e.top)<5||(n=e,t.onScrolledVertically(e.top,e.height))})},t.prototype.initHeader=function(t){var e=this;t.innerHTML='<i title="Item" class="'+(this._mode===r.ITEM?"active":"")+'"><span aria-hidden="true">Item</span></i>\n        <i title="Band" class="'+(this._mode===r.BAND?"active":"")+'"><span aria-hidden="true">Band</span></i>';var n=Array.from(t.children);n.forEach(function(t,i){t.onclick=function(o){o.preventDefault(),o.stopPropagation(),t.classList.contains("active")||(e.mode=0===i?r.ITEM:r.BAND,n.forEach(function(t,e){return t.classList.toggle("active",e===i)}))}})},Object.defineProperty(t.prototype,"mode",{get:function(){return this._mode},set:function(t){t!==this._mode&&(this._mode=t,this.current&&this.rebuild(this.current.left,this.current.leftContext,this.current.right,this.current.rightContext))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"hidden",{get:function(){return this.header.classList.contains("loading")},set:function(t){this.header.classList.toggle("loading",t),this.body.classList.toggle("loading",t)},enumerable:!0,configurable:!0}),t.prototype.hide=function(){this.hidden=!0},t.prototype.show=function(){var t=this.hidden;this.hidden=!1,t&&this.revalidate()},t.prototype.destroy=function(){this.header.remove(),this.scrollListener&&this.body.parentElement.__le_scroller__.remove(this.scrollListener),this.body.remove()},t.prototype.rebuild=function(t,e,n,r){this.current={left:t,leftContext:e,right:n,rightContext:r};var i=this.prepareRightSlopes(n,r);this.computeSlopes(t,e,i),this.revalidate()},t.prototype.computeSlopes=function(t,e,n){var i=this,a=this.mode,c=new Map,l=0;this.leftSlopes=t.map(function(d,p){var f=e.exceptionsLookup.get(p)||e.defaultRowHeight,h=f-0,g=[],m=l;l+=f;var b,y=0,v=function(t,e,n,r){void 0===n&&(n=1),void 0===r&&(r=0),g.push(t),e.ref.forEach(function(e){return i.rightSlopes[e].push(t)}),e.offset+=n*e.heightPerRow,y+=n*r};if(Object(o.b)(d))b=d;else{var E=d,C=E.i,w=n.get(C);if(!w)return g;if(a===r.ITEM){var A=new u(m+h/2,w.start+w.offset+w.heightPerRow/2,[C]);return v(A,w),g}if(c.has(E.group))return c.get(E.group);var x=function(n,r){for(var i=0,a=0,u=[],s=n;s<t.length;++s){var c=t[s];if(Object(o.b)(c)||c.group!==r)break;i++,a+=e.exceptionsLookup.get(s)||e.defaultRowHeight,u.push(c)}var l=a-e.padding(n+i-1);return{gr:Object.assign({rows:u},r),padded:l,height:a}}(p,E.group);b=x.gr,f=x.height,h=x.padded,c.set(E.group,g)}var N=new Set(b.rows.map(function(t){return t.i})),T=h/b.rows.length;return b.rows.forEach(function(t){if(N.has(t.i)){N.delete(t.i);var e=n.get(t.i);if(e){var i=e.rows.filter(function(t){return N.delete(t)});i.push(t.i);var o,c=i.length;o=1===c?new u(m+y+T/2,e.start+e.offset+e.heightPerRow/2,[t.i]):a===r.ITEM?new u(m+y+T*c/2,e.start+e.offset+e.heightPerRow*c/2,i):new s([m+y,m+y+T*c],[e.start+e.offset,e.start+e.offset+e.heightPerRow*c],i),v(o,e,c,T)}}}),g})},t.prototype.prepareRightSlopes=function(t,e){var n=new Map,i=this.mode,a=new Map,u=0;return this.rightSlopes=t.map(function(t,s){var c=e.exceptionsLookup.get(s)||e.defaultRowHeight,l=c-0,d=u;u+=c;var p=[],f={start:d,offset:0,ref:[s]};if(Object(o.b)(t)){var h=Object.assign(f,{rows:t.rows.map(function(t){return t.i}),heightPerRow:l/t.rows.length,group:t});return t.rows.forEach(function(t){return n.set(t.i,h)}),p}var g=t,m=t.i,b=Object.assign(f,{rows:[m],heightPerRow:l,group:g.group});return i===r.ITEM?(n.set(m,b),p):(a.has(g.group)?((b=a.get(g.group)).rows.push(m),b.ref.push(s)):(b.heightPerRow=c,a.set(g.group,b)),n.set(m,b),p)}),n},t.prototype.revalidate=function(){if(this.current&&!this.hidden){var t=this.body.parentElement;this.onScrolledVertically(t.scrollTop,t.clientHeight)}},t.prototype.highlight=function(t){var e=this.body.querySelector("[data-i].le-highlighted");if(e&&e.classList.remove("le-highlighted"),!(t<0)){var n=this.body.querySelector('[data-i="'+t+'"]');return n&&n.classList.add("le-highlighted"),null!=n}},t.prototype.onScrolledVertically=function(t,e){if(this.current){var n=this.current,r=n.leftContext,o=n.rightContext,a=Object(i.range)(t,e,r.defaultRowHeight,r.exceptions,r.numberOfRows),u=Object(i.range)(t,e,o.defaultRowHeight,o.exceptions,o.numberOfRows),s=Math.min(a.firstRowPos,u.firstRowPos),c=Math.max(a.endPos,u.endPos);this.body.style.transform="translate(0, "+s.toFixed(0)+"px)",this.body.style.height=(c-s).toFixed(0)+"px",this.node.firstElementChild.setAttribute("transform","translate(0,-"+s.toFixed(0)+")"),this.choose(a.first,a.last,u.first,u.last)}},t.prototype.choose=function(t,e,n,r){for(var i=new Set,o=t;o<=e;++o)this.leftSlopes[o].forEach(function(t){return i.add(t)});for(o=n;o<=r;++o)this.rightSlopes[o].forEach(function(t){return i.add(t)});this.render(i)},t.prototype.render=function(t){var e=this.node.firstElementChild,n=e.ownerSVGElement.getBoundingClientRect().width,r=this.matchLength(t,e),i=this.ctx.provider,o={has:function(t){return i.isSelected(t)}},a=0;t.forEach(function(t){var i=r[a++];t.update(i,n),i.__data__=t;var u=t.isSelected(o);i.classList.toggle("lu-selected",u),u&&e.appendChild(i)})},t.prototype.matchLength=function(t,e){for(var n=this,r=Array.from(e.children),i=t.size;i<r.length;++i){var o=r[i];this.pool.push(o),o.remove()}var a=function(t){var i=u.pool.pop();if(i)e.appendChild(i),r.push(i);else{var o=e.ownerDocument.createElementNS("http://www.w3.org/2000/svg","path");o.onclick=function(t){var e=o.__data__,r=n.ctx.provider,i=e.dataIndices;if(t.ctrlKey)i.forEach(function(t){return r.toggleSelection(t,!0)});else{var a=r.isSelected(i[0]);r.setSelection(a?[]:i)}},e.appendChild(o),r.push(o)}},u=this;for(i=r.length;i<t.size;++i)a();return r},t.prototype.updateSelection=function(t){var e=this.node.firstElementChild;Array.from(e.children).forEach(function(n){var r=n.__data__.isSelected(t);n.classList.toggle("lu-selected",r),r&&e.appendChild(n)})},t}();e.b=c},function(t,e,n){"use strict";function r(t,e){void 0===e&&(e={});var n=Object.assign({container:t.parentElement,filter:function(){return!0},onStart:function(){},onDrag:function(){},onEnd:function(){},minDelta:2},e),r=function(t,e){return t-e.getBoundingClientRect().left-e.clientLeft},i=0,o=0,a=0,u=function(e){if(n.filter(e)){e.stopPropagation(),e.preventDefault();var i=r(e.clientX,n.container)-a;Math.abs(o-i)<n.minDelta||(o=i,n.onDrag(t,i,o-i,e))}},s=function(e){if(n.filter(e)){e.stopPropagation(),e.preventDefault();var o=r(e.clientX,n.container)-a;n.container.removeEventListener("mousemove",u),n.container.removeEventListener("mouseup",s),n.container.removeEventListener("mouseleave",s),Math.abs(i-o)<2||n.onEnd(t,o,i-o,e)}};t.onmousedown=function(e){n.filter(e)&&(e.stopPropagation(),e.preventDefault(),a=r(e.clientX,t),i=o=r(e.clientX,n.container)-a,n.container.addEventListener("mousemove",u),n.container.addEventListener("mouseup",s),n.container.addEventListener("mouseleave",s),n.onStart(t,i,0,e))}}n.d(e,"a",function(){return r})},function(t,e,n){"use strict";var r=n(0),i=n(2),o=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.annotations=new Map,e}return r.c(e,t),e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([e.EVENT_VALUE_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getValue=function(e){return this.annotations.has(e.i)?this.annotations.get(e.i):t.prototype.getValue.call(this,e)},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return n.annotations={},this.annotations.forEach(function(t,e){n.annotations[e]=t}),n},e.prototype.restore=function(e,n){var r=this;t.prototype.restore.call(this,e,n),e.annotations&&Object.keys(e.annotations).forEach(function(t){r.annotations.set(Number(t),e.annotations[t])})},e.prototype.setValue=function(t,n){var r=this.getValue(t);return r===n||(""===n||null==n?this.annotations.delete(t.i):this.annotations.set(t.i,n),this.fire([e.EVENT_VALUE_CHANGED,i.e.EVENT_DIRTY_VALUES,i.e.EVENT_DIRTY],t.i,r,n),!0)},e.EVENT_VALUE_CHANGED="valueChanged",e}(n(21).d);e.a=o},function(t,e,n){"use strict";var r=n(0),i=n(37),o=n(81),a=n(82),u=n(76),s=n(87),c=n(86),l=n(25);n.d(e,"a",function(){return l.ITaggleOptions});var d=function(t){function e(e,n,r){void 0===r&&(r={});var l=t.call(this,e,n,r&&!0===r.ignoreUnsupportedBrowser)||this;if(l.options=Object(i.a)(),Object(o.a)(l.options,r,{violationChanged:function(t,e){return l.setViolation(e)}}),!l.isBrowserSupported)return l.spaceFilling=null,l.renderer=null,l.panel=null,l;l.node.classList.add("lu-taggle","lu"),l.renderer=new c.a(n,l.node,l.options),l.panel=new u.b(l.renderer.ctx,l.node.ownerDocument,{collapseable:!l.options.sidePanelCollapsed||"collapsed",hierarchy:l.options.hierarchyIndicator}),l.renderer.pushUpdateAble(function(t){return l.panel.update(t)}),l.node.insertBefore(l.panel.node,l.node.firstChild),l.panel.node.insertAdjacentHTML("afterbegin",'<div class="lu-rule-button-chooser"><label>\n            <input type="checkbox">\n            <span>Overview</span>\n            <div></div>\n          </label></div>');var d=Object(s.a)(l.options);l.spaceFilling=l.node.querySelector(".lu-rule-button-chooser");var p=l.spaceFilling.querySelector("input");return p.onchange=function(){var t=l.spaceFilling.classList.toggle("chosen");self.setTimeout(function(){return l.renderer.switchRule(t?d:null)})},l.options.overviewMode&&(p.checked=!0,l.spaceFilling.classList.toggle("chosen"),l.renderer.switchRule(d)),l.forward(l.renderer,a.a.EVENT_HIGHLIGHT_CHANGED+".main"),l}return r.c(e,t),e.prototype.setViolation=function(t){t=t||"",this.spaceFilling&&(this.spaceFilling.classList.toggle("violated",Boolean(t)),this.spaceFilling.lastElementChild.innerHTML=t.replace(/\n/g,"<br>"))},e.prototype.destroy=function(){this.node.classList.remove("lu-taggle","lu"),this.renderer&&this.renderer.destroy(),this.panel&&this.panel.destroy(),t.prototype.destroy.call(this)},e.prototype.update=function(){this.renderer&&this.renderer.update()},e.prototype.setHighlight=function(t,e){return void 0===e&&(e=!0),null!=this.renderer&&this.renderer.setHighlight(t,e)},e.prototype.getHighlight=function(){return this.renderer?this.renderer.getHighlight():-1},e.prototype.enableHighlightListening=function(t){this.renderer&&this.renderer.enableHighlightListening(t)},e.prototype.setDataProvider=function(e,n){t.prototype.setDataProvider.call(this,e,n),this.renderer&&(this.renderer.setDataProvider(e),this.update(),this.panel.update(this.renderer.ctx))},e}(a.a);e.b=d},function(t,e,n){"use strict";n.d(e,"a",function(){return i});var r="[object Object]";function i(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];for(var n=null,o=0,a=t;o<a.length;o++){var u=a[o],s=Object.keys(u);if(null!=n)for(var c=0,l=s;c<l.length;c++){var d=l[c],p=u[d];Object.prototype.toString.call(p)===r&&Object.getPrototypeOf(p)===Object.prototype?(void 0===n[d]&&(n[d]={}),n[d]=i(n[d],p)):Array.isArray(p)?(void 0===n[d]&&(n[d]=[]),n[d]=p.concat(n[d])):n[d]=p}else n=u}return n}},function(t,e,n){"use strict";var r=n(0),i=n(16),o=n(19),a=n(98),u=57,s=64,c=16;function l(){var t=Object(a.detect)();if(!t)return"browser cannot be detected";var e="unsupported browser detected:";switch(t.name){case"firefox":var n=parseInt(t.version.slice(0,t.version.indexOf(".")),10);return n<=u&&52!==n?e+" Firefox "+t.version+" (&lt; "+u+")":null;case"edge":return parseInt(t.version.slice(0,t.version.indexOf(".")),10)<=c?e+" Edge "+t.version+" (&lt; "+c+")":null;case"chrome":return parseInt(t.version.slice(0,t.version.indexOf(".")),10)<=s?e+" Chrome "+t.version+" (&lt; "+s+")":null;case"ie":return e+" Internet Explorer"}return console.warn("unknown browser detected",t,"assuming fine..."),null}n.d(e,"a",function(){return d});var d=function(t){function e(e,n,r){var i=t.call(this)||this;i.node=e,i.data=n,i.highlightListeners=0;var a=l();return i.isBrowserSupported=r||!a,i.isBrowserSupported||(i.node.classList.add("lu-unsupported-browser"),i.node.innerHTML="<span>"+a+'</span>\n      <div class="lu-unsupported-browser-hint">\n        <a href="https://www.mozilla.org/en-US/firefox/" rel="noopener" target="_blank" data-browser="firefox" data-version="'+u+'"></a>\n        <a href="https://www.google.com/chrome/index.html" rel="noopener" target="_blank" data-browser="chrome" data-version="'+s+'" title="best support"></a>\n        <a href="https://www.microsoft.com/en-us/windows/microsoft-edge" rel="noopener" target="_blank" data-browser="edge" data-version="'+c+'"></a>\n      </div><span>use the <code>ignoreUnsupportedBrowser=true</code> option to ignore this error at your own risk</span>'),i.forward(i.data,o.e.EVENT_SELECTION_CHANGED+".main"),i}return r.c(e,t),e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([e.EVENT_HIGHLIGHT_CHANGED,e.EVENT_SELECTION_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.destroy=function(){this.node.innerHTML=""},e.prototype.dump=function(){return this.data.dump()},e.prototype.setDataProvider=function(t,e){this.data&&this.unforward(this.data,o.e.EVENT_SELECTION_CHANGED+".taggle"),this.data=t,e&&this.data.restore(e),this.forward(this.data,o.e.EVENT_SELECTION_CHANGED+".taggle")},e.prototype.getSelection=function(){return this.data.getSelection()},e.prototype.setSelection=function(t){this.data.setSelection(t)},e.prototype.sortBy=function(t,e){void 0===e&&(e=!1);var n=this.data.find(t);return n&&n.sortByMe(e),null!=n},e.prototype.listenersChanged=function(n,r){t.prototype.listenersChanged.call(this,n,r),n.startsWith(e.EVENT_HIGHLIGHT_CHANGED)&&(r?(this.highlightListeners+=1,1===this.highlightListeners&&this.enableHighlightListening(!0)):(this.highlightListeners-=1,0===this.highlightListeners&&this.enableHighlightListening(!1)))},e.prototype.enableHighlightListening=function(t){},e.EVENT_SELECTION_CHANGED=o.e.EVENT_SELECTION_CHANGED,e.EVENT_HIGHLIGHT_CHANGED="highlightChanged",e}(i.a)},function(t,e,n){"use strict";n.r(e);var r=n(38);n.d(e,"Taggle",function(){return r.e}),n.d(e,"ITaggleOptions",function(){return r.c}),n.d(e,"TaggleRenderer",function(){return r.d}),n.d(e,"IRuleInstance",function(){return r.b}),n.d(e,"IRule",function(){return r.a}),n.d(e,"spaceFillingRule",function(){return r.f});var i=n(75);n.d(e,"EngineRenderer",function(){return i.a});var o=n(53);n.d(e,"EngineRanking",function(){return o.c}),n.d(e,"IEngineRankingContext",function(){return o.IEngineRankingContext}),n.d(e,"IEngineRankingOptions",function(){return o.IEngineRankingOptions});var a=n(77);n.d(e,"SlopeGraph",function(){return a.b}),n.d(e,"EMode",function(){return a.a});var u=n(94);n.d(e,"IDialogOptions",function(){return u.c}),n.d(e,"ADialog",function(){return u.a}),n.d(e,"IDialogContext",function(){return u.b});var s=n(95);n.d(e,"SidePanel",function(){return s.f}),n.d(e,"ISidePanelOptions",function(){return s.d}),n.d(e,"SearchBox",function(){return s.e}),n.d(e,"IGroupSearchItem",function(){return s.a}),n.d(e,"IItem",function(){return s.b}),n.d(e,"ISearchBoxOptions",function(){return s.c});var c=n(42);n.d(e,"dialogContext",function(){return c.b}),n.d(e,"toolbarActions",function(){return c.g}),n.d(e,"getToolbarDialogAddons",function(){return c.c}),n.d(e,"isSortAble",function(){return c.f}),n.d(e,"isGroupAble",function(){return c.d}),n.d(e,"isGroupSortAble",function(){return c.e});var l=n(96);for(var d in l)["Taggle","ITaggleOptions","TaggleRenderer","IRuleInstance","IRule","spaceFillingRule","EngineRenderer","EngineRanking","IEngineRankingContext","IEngineRankingOptions","SlopeGraph","EMode","LineUp","ILineUpOptions","IDialogOptions","ADialog","IDialogContext","SidePanel","ISidePanelOptions","SearchBox","IGroupSearchItem","IItem","ISearchBoxOptions","dialogContext","toolbarActions","default","getToolbarDialogAddons","isSortAble","isGroupAble","isGroupSortAble","default"].indexOf(d)<0&&function(t){n.d(e,t,function(){return l[t]})}(d);var p=n(41);n.d(e,"LineUp",function(){return p.b}),n.d(e,"ILineUpOptions",function(){return p.a})},function(t,e,n){"use strict";var r=n(19),i=n(70),o=n(0),a=n(50),u=function(t){function e(e,n,r){void 0===n&&(n=[]),void 0===r&&(r={});var i=t.call(this,n,r)||this;return i.server=e,i.options={maxCacheSize:1e3},i.cache=new Map,Object.assign(i.options,r),i}return o.c(e,t),e.prototype.getTotalNumberOfRows=function(){return this.cache.size},e.prototype.sortImpl=function(t){return this.server.sort(t).then(function(t){return[Object.assign({order:t},a.a)]})},e.prototype.loadFromServer=function(t){return this.server.view(t).then(function(e){return e.map(function(e,n){return{v:e,dataIndex:t[n]}})})},e.prototype.view=function(t){if(0===t.length)return Promise.resolve([]);var e=this.fetch([t])[0];return Promise.all(e).then(function(t){return t.map(function(t){return t.v})})},e.prototype.computeMissing=function(t){var e=new Set,n=e.add.bind(e);return t.forEach(function(t){return t.forEach(n)}),this.cache.forEach(function(t,n){return e.delete(n)}),this.cache.size,e.size,this.options.maxCacheSize,Array.from(e)},e.prototype.loadInCache=function(t){var e=this;if(0!==t.length){var n=this.loadFromServer(t);t.forEach(function(r,i){var o=t[i];e.cache.set(o,n.then(function(t){return{v:t[i],i:o}}))})}},e.prototype.fetch=function(t){var e=this,n=this.computeMissing(t);return this.loadInCache(n),t.map(function(t){return t.map(function(t){return e.cache.get(t)})})},e.prototype.mappingSample=function(t){return this.server.mappingSample(t.desc.column)},e.prototype.searchAndJump=function(t,e){var n=this;this.server.search(t,e.desc.column).then(function(t){n.jumpToNearest(t)})},e.prototype.stats=function(t){return this.server.stats(t)},e}(n(90).a),s=n(44);n.d(e,"a",function(){return r.e}),n.d(e,"e",function(){return r.IStatsBuilder}),n.d(e,"c",function(){return r.IDataProviderOptions}),n.d(e,"b",function(){return r.IDataProvider}),n.d(e,"d",function(){return r.c}),n.d(e,"f",function(){return i.a}),n.d(e,"g",function(){return u}),n.d(e,"h",function(){return s.b}),n.d(e,"j",function(){return s.d}),n.d(e,"i",function(){return s.c}),n.d(e,"k",function(){return s.e})},function(t,e,n){"use strict";var r=n(0),i=n(9),o=n(1),a=n(2),u=n(13),s=n(4),c=n(65),l=function(t){function e(e,n){var r=t.call(this,e,n)||this;r.collapsed=!1;var i=r;return r.adaptChange=function(t,e){i.adaptWidthChange(t,e)},r}var n;return r.c(e,t),n=e,e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([n.EVENT_COLLAPSE_CHANGED,n.EVENT_MULTI_LEVEL_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.setCollapsed=function(t){this.collapsed!==t&&this.fire([c.b.EVENT_COLLAPSE_CHANGED,a.e.EVENT_DIRTY_HEADER,a.e.EVENT_DIRTY_VALUES,a.e.EVENT_DIRTY],this.collapsed,this.collapsed=t)},e.prototype.getCollapsed=function(){return this.collapsed},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return n.collapsed=this.collapsed,n},e.prototype.restore=function(e,n){this.collapsed=!0===e.collapsed,t.prototype.restore.call(this,e,n)},e.prototype.flatten=function(t,e,n,r){return void 0===n&&(n=0),void 0===r&&(r=0),c.b.prototype.flatten.call(this,t,e,n,r)},e.prototype.insert=function(e,n){return e.on(a.e.EVENT_WIDTH_CHANGED+".stack",this.adaptChange),t.prototype.setWidth.call(this,0===this.length?e.getWidth():this.getWidth()+e.getWidth()),t.prototype.insert.call(this,e,n)},e.prototype.adaptWidthChange=function(e,r){if(!Object(i.l)(e,r,.5)){var o=this.getWidth(),u=o+(r-e);this.fire([n.EVENT_MULTI_LEVEL_CHANGED,a.e.EVENT_DIRTY_HEADER,a.e.EVENT_DIRTY],o,u),t.prototype.setWidth.call(this,u)}},e.prototype.removeImpl=function(e,n){return e.on(a.e.EVENT_WIDTH_CHANGED+".stack",null),t.prototype.setWidth.call(this,0===this.length?100:this.getWidth()-e.getWidth()),t.prototype.removeImpl.call(this,e,n)},e.prototype.setWidth=function(e){var r=this.getWidth(),o=e/r;this._children.forEach(function(t){t.setWidthImpl(t.getWidth()*o)}),Object(i.l)(r,e,.5)||this.fire([n.EVENT_MULTI_LEVEL_CHANGED,a.e.EVENT_DIRTY_HEADER,a.e.EVENT_DIRTY],r,e),t.prototype.setWidth.call(this,e)},e.prototype.getRenderer=function(){return this.getCollapsed()?n.EVENT_COLLAPSE_CHANGED:t.prototype.getRenderer.call(this)},e.prototype.isMissing=function(t){return!!this.getCollapsed()&&this._children.some(function(e){return(Object(s.i)(e)||Object(u.c)(e))&&e.isMissing(t)})},e.prototype.getExportValue=function(e,n){return"json"===n?{children:this.children.map(function(t){return t.getExportValue(e,n)})}:t.prototype.getExportValue.call(this,e,n)},e.EVENT_COLLAPSE_CHANGED=c.b.EVENT_COLLAPSE_CHANGED,e.EVENT_MULTI_LEVEL_CHANGED=c.b.EVENT_MULTI_LEVEL_CHANGED,e=n=r.b([Object(o.l)("collapse")],e)}(u.b);e.a=l},function(t,e,n){"use strict";var r=n(0),i=n(16),o=n(54),a=n(73),u=n(19),s=n(75),c=function(t){function e(n,r,i){var a=t.call(this)||this;return a.data=n,a.isDynamicLeafHeight=!1,a.rule=null,a.levelOfDetail=null,a.resizeListener=function(){return Object(o.a)(function(){return a.update()},100)},a.options={violationChanged:function(){},rowPadding:2},a.renderer=new s.a(n,r,Object.assign({},i,{dynamicHeight:function(t,e){var n=a.dynamicHeight(t);return n||(i.dynamicHeight?i.dynamicHeight(t,e):null)},levelOfDetail:function(t){return a.levelOfDetail?a.levelOfDetail(t):"high"}})),a.renderer.style.addRule("taggle_lod_rule","\n      #"+a.renderer.idPrefix+" [data-lod=low][data-agg=detail]:hover {\n        /* show regular height for hovered rows in low + medium LOD */\n        height: "+i.rowHeight+"px !important;\n      }\n    "),a.data.on(u.e.EVENT_SELECTION_CHANGED+".rule",function(){a.isDynamicLeafHeight&&a.update()}),a.forward(a.renderer,e.EVENT_HIGHLIGHT_CHANGED+".main"),window.addEventListener("resize",a.resizeListener,{passive:!0}),a}return r.c(e,t),Object.defineProperty(e.prototype,"ctx",{get:function(){return this.renderer.ctx},enumerable:!0,configurable:!0}),e.prototype.pushUpdateAble=function(t){this.renderer.pushUpdateAble(t)},e.prototype.dynamicHeight=function(t){var e=this;if(!this.rule)return this.levelOfDetail=null,null;var n=this.renderer?this.renderer.node.querySelector("main").clientHeight:100,r=this.rule.apply(t,n,new Set(this.data.getSelection()));this.isDynamicLeafHeight="function"==typeof r.item,this.options.violationChanged(this.rule,r.violation||"");var i=function(t){return Object(a.b)(t)?"number"==typeof r.group?r.group:r.group(t):"number"==typeof r.item?r.item:r.item(t)};return this.levelOfDetail=function(n){var r=t[n];return e.rule?e.rule.levelOfDetail(r,i(r)):"high"},{defaultHeight:"number"==typeof r.item?r.item:NaN,height:i,padding:0}},e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([e.EVENT_HIGHLIGHT_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.zoomOut=function(){this.renderer.zoomOut()},e.prototype.zoomIn=function(){this.renderer.zoomIn()},e.prototype.switchRule=function(t){this.rule!==t&&(this.rule=t,this.update())},e.prototype.destroy=function(){this.renderer.destroy(),window.removeEventListener("resize",this.resizeListener)},e.prototype.update=function(){this.renderer.update()},e.prototype.setDataProvider=function(t){var e=this;this.data&&this.data.on(u.e.EVENT_SELECTION_CHANGED+".rule",null),this.data=t,this.data.on(u.e.EVENT_SELECTION_CHANGED+".rule",function(){e.isDynamicLeafHeight&&e.update()}),this.renderer.setDataProvider(t),this.update()},e.prototype.setHighlight=function(t,e){return this.renderer.setHighlight(t,e)},e.prototype.getHighlight=function(){return this.renderer.getHighlight()},e.prototype.enableHighlightListening=function(t){this.renderer.enableHighlightListening(t)},e.EVENT_HIGHLIGHT_CHANGED=s.a.EVENT_HIGHLIGHT_CHANGED,e}(i.a);e.a=c},function(t,e,n){"use strict";n.d(e,"a",function(){return i});var r=n(73);function i(t){return{apply:function(e,n,i){var o=function(e,n,i){var o=n-t.rowHeight-5,a=e.filter(function(t){return!Object(r.b)(t)}),u=e.length-a.length,s=a.reduce(function(t,e){return t+("last"===e.meta||"first last"===e.meta?1:0)},0),c=a.reduce(function(t,e){return t+(i.has(e.i)?1:0)},0),l=a.length-c,d=u+s;if(l<=0)return{height:t.rowHeight,violation:""};var p=(o-u*t.groupHeight-d*t.groupPadding-c*t.rowHeight)/l;return p<1?{height:1,violation:"Not possible to fit all rows on the screen. Set filters or aggregate groups to make it fit again."}:p>t.rowHeight?{height:t.rowHeight,violation:""}:{height:p,violation:""}}(e,n,i),a=o.violation,u=o.height;return{item:function(e){return i.has(e.i)?t.rowHeight:u},group:t.groupHeight,violation:a}},levelOfDetail:function(e,n){return n>=.9*(Object(r.b)(e)?t.groupHeight:t.rowHeight)?"high":"low"}}}},function(t,e,n){"use strict";var r=function(){function t(t){void 0===t&&(t=[]),this[Symbol.toStringTag]=Symbol("OrderedSet"),this.set=new Set,this.list=[],this.addAll(t)}return Object.defineProperty(t.prototype,"size",{get:function(){return this.set.size},enumerable:!0,configurable:!0}),t.prototype.clear=function(){this.set.clear(),this.list.splice(0,this.list.length)},t.prototype.addAll=function(t){var e=this;return t.forEach(function(t){return e.add(t)}),this},t.prototype.add=function(t){return this.set.has(t)?this:(this.set.add(t),this.list.push(t),this)},t.prototype.has=function(t){return this.set.has(t)},t.prototype.delete=function(t){if(!this.set.delete(t))return!1;var e=this.list.indexOf(t);return console.assert(e>=0),this.list.splice(e,1),!0},t.prototype.deleteAll=function(t){var e=this;return t.reduce(function(t,n){return e.delete(n)&&t},!0)},t.prototype.forEach=function(t,e){this.list.forEach(function(e){t.call(this,e,e,this)},e)},t.prototype[Symbol.iterator]=function(){return this.list[Symbol.iterator]()},t}();e.a=r},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r,i=n(0),o=n(51),a=n(18),u=n(1),s=n(31),c=n(7);!function(t){t.min="min",t.max="max",t.median="median"}(r||(r={}));var l=function(t){function e(e,n){var i=t.call(this,e,n)||this;return i.format=Object(o.a)(n.dateFormat||"%x"),i.parse=n.dateParse?Object(o.b)(n.dateParse):Object(o.b)(n.dateFormat||"%x"),i.sort=n.sort||r.median,i.setDefaultRenderer("default"),i}var n;return i.c(e,t),n=e,e.prototype.createEventList=function(){return t.prototype.createEventList.call(this).concat([s.d.EVENT_SPLICE_CHANGED])},e.prototype.on=function(e,n){return t.prototype.on.call(this,e,n)},e.prototype.getValue=function(e){var n=this;return t.prototype.getValue.call(this,e).map(function(t){return Object(c.c)(t)?null:t instanceof Date?t:n.parse(String(t))})},e.prototype.getLabels=function(t){var e=this;return this.getValue(t).map(function(t){return t instanceof Date?e.format(t):""})},e.prototype.getSortMethod=function(){return this.sort},e.prototype.setSortMethod=function(t){this.sort!==t&&(this.fire([n.EVENT_SORTMETHOD_CHANGED],this.sort,this.sort=t),this.isSortedByMe().asc||this.sortByMe())},e.prototype.dump=function(e){var n=t.prototype.dump.call(this,e);return n.sortMethod=this.getSortMethod(),n},e.prototype.restore=function(e,n){t.prototype.restore.call(this,e,n),e.sortMethod&&(this.sort=e.sortMethod)},e.prototype.compare=function(t,e){var n=this.getValue(t).filter(Boolean),r=this.getValue(e).filter(Boolean);return n===r?0:0===n.length?0===r.length?0:c.a:0===r.length?-c.a:d(n,this.sort)-d(r,this.sort)},e.EVENT_SORTMETHOD_CHANGED="sortMethodChanged",e=n=i.b([Object(u.g)("sort","sortDates")],e)}(s.d);function d(t,e){switch(e){case r.min:return Object(a.i)(t,function(t){return t.getTime()});case r.max:return Object(a.f)(t,function(t){return t.getTime()});case r.median:return Object(a.h)(t,function(t){return t.getTime()})}}e.b=l},function(t,e,n){"use strict";var r=n(0),i=n(26),o=n(39),a=n(68),u=n(1),s=n(14),c=n(19);function l(t,e,n){var r=n.column;return function(t){return"string"==typeof t&&t.indexOf(".")>=0}(r)?function(t,e){return e.hasOwnProperty(t)?e[t]:t.split(".").reduce(function(t,e){return void 0===t?t:/\d+/.test(e)?t[+e]:t[e]},e)}(r,t.v):t.v[r]}var d=function(t){function e(e,n){void 0===e&&(e=[]),void 0===n&&(n={});var r=t.call(this,n)||this;return r.columns=e,r.rankingIndex=0,r.ranks=new Map,e.forEach(function(t){t.accessor=t.accessor||l,t.label=t.label||t.column}),r}return r.c(e,t),e.prototype.rankAccessor=function(t,e,n,r){for(var i=0,o=0,a=this.ranks.get(r.id)||[];o<a.length;o++){var u=a[o],s=u.order.indexOf(t.i);if(s>=0)return i+s+1;i+=u.order.length}return-1},e.prototype.getMaxNestedSortingCriteria=function(){return 1},e.prototype.getMaxGroupColumns=function(){return 1},e.prototype.cloneRanking=function(t){var e=this,n=this.nextRankingId(),r=new s.b(n,this.getMaxNestedSortingCriteria(),this.getMaxGroupColumns());return t&&(this.ranks.set(n,this.ranks.get(t.id)),t.children.forEach(function(t){e.push(r,t.desc)})),r},e.prototype.cleanUpRanking=function(t){this.ranks.delete(t.id)},e.prototype.sort=function(t){var e=this,n=this.sortImpl(t);return Array.isArray(n)?(this.ranks.set(t.id,n),n):n.then(function(n){return e.ranks.set(t.id,n),n})},e.prototype.pushDesc=function(t){var e=t;e.accessor=e.accessor||l,e.label=t.label||e.column,this.columns.push(t),this.fire(c.e.EVENT_ADD_DESC,e)},e.prototype.clearColumns=function(){this.clearRankings(),this.columns.splice(0,this.columns.length),this.fire(c.e.EVENT_CLEAR_DESC)},e.prototype.getColumns=function(){return this.columns.slice()},e.prototype.findDesc=function(t){return this.columns.filter(function(e){return e.column===t})[0]},e.prototype.toDescRef=function(t){return void 0!==t.column?t.type+"@"+t.column:t},e.prototype.deriveDefault=function(t){var e=this;void 0===t&&(t=!0);var n=this.pushRanking();return t&&(this.getMaxGroupColumns()>0&&n.push(this.create(Object(i.b)())),n.push(this.create(Object(o.a)())),this.multiSelections&&n.push(this.create(Object(a.b)()))),this.getColumns().forEach(function(t){var r=e.create(t);r&&!Object(u.k)(r)&&n.push(r)}),n},e.prototype.fromDescRef=function(t){if("string"==typeof t)return this.columns.find(function(e){return e.type+"@"+e.column===t});var e=this.columns.find(function(e){return t.column===e.column&&t.label===e.label&&t.type===e.type});return e||t},e.prototype.restore=function(e){t.prototype.restore.call(this,e),this.rankingIndex=1+Math.max.apply(Math,this.getRankings().map(function(t){return+t.id.substring(4)}))},e.prototype.nextRankingId=function(){return"rank"+this.rankingIndex++},e}(c.e);e.a=d},function(t,e,n){"use strict";n.d(e,"b",function(){return a});var r=n(68),i=n(3),o=function(){function t(){this.title="Default"}return t.prototype.canRender=function(t){return t instanceof r.c},t.prototype.create=function(t,e){return{template:"<div></div>",update:function(n,r,i){n.onclick=function(n){if(n.preventDefault(),n.stopPropagation(),n.shiftKey){var o=t.findMyRanker().id;if(a(e.provider,o,r.i,i,n.ctrlKey))return}t.toggleValue(r)}},render:i.h}},t.prototype.createGroup=function(t){return{template:"<div></div>",update:function(e,n,r){r.reduce(function(e,n){return t.getValue(n)?e+1:e},0)>=r.length/2?e.classList.add("lu-group-selected"):e.classList.remove("lu-group-selected"),e.onclick=function(n){n.preventDefault(),n.stopPropagation();var i=e.classList.toggle("lu-group-selected");t.setValues(r,i)}}}},t.prototype.createSummary=function(t,e){return{template:'<div title="(Un)Select All" data-icon="unchecked"></div>',update:function(n){n.onclick=function(r){r.stopPropagation(),"unchecked"===n.dataset.icon?(e.provider.selectAllOf(t.findMyRanker()),n.dataset.icon="checked"):(e.provider.setSelection([]),n.dataset.icon="unchecked")}}}},t}();function a(t,e,n,r,i){var o=t.getRankings().find(function(t){return t.id===e});if(!o)return!1;var a=t.getSelection();if(0===a.length||a.includes(n))return!1;var u=o.getOrder(),s=new Map(o.getOrder().map(function(t,e){return[t,e]})),c=a.map(function(t){var e=s.has(t)?s.get(t):1/0;return{s:t,index:e,distance:Math.abs(r-e)}}).sort(function(t,e){return t.distance-e.distance})[0];if(!isFinite(c.distance))return!1;if(i||(a.splice(0,a.length),a.push(c.s)),c.index<r)for(var l=c.index+1;l<=r;++l)a.push(u[l]);else for(l=r;l<=c.index;++l)a.push(u[l]);return t.setSelection(a),!0}e.a=o},function(t,e,n){"use strict";var r=Array.isArray,i=Object.keys,o=Object.prototype.hasOwnProperty;t.exports=function t(e,n){if(e===n)return!0;if(e&&n&&"object"==typeof e&&"object"==typeof n){var a,u,s,c=r(e),l=r(n);if(c&&l){if((u=e.length)!=n.length)return!1;for(a=u;0!=a--;)if(!t(e[a],n[a]))return!1;return!0}if(c!=l)return!1;var d=e instanceof Date,p=n instanceof Date;if(d!=p)return!1;if(d&&p)return e.getTime()==n.getTime();var f=e instanceof RegExp,h=n instanceof RegExp;if(f!=h)return!1;if(f&&h)return e.toString()==n.toString();var g=i(e);if((u=g.length)!==i(n).length)return!1;for(a=u;0!=a--;)if(!o.call(n,g[a]))return!1;for(a=u;0!=a--;)if(!t(e[s=g[a]],n[s]))return!1;return!0}return e!=e&&n!=n}},function(t,e){},function(t,e,n){"use strict";var r=n(17);n.d(e,"c",function(){return r.IDialogOptions}),n.d(e,"a",function(){return r.c}),n.d(e,"b",function(){return r.IDialogContext})},function(t,e,n){"use strict";var r=n(76);n.d(e,"f",function(){return r.b}),n.d(e,"d",function(){return r.ISidePanelOptions});var i=n(43);n.d(e,"e",function(){return i.d}),n.d(e,"a",function(){return i.IGroupSearchItem}),n.d(e,"b",function(){return i.IItem}),n.d(e,"c",function(){return i.ISearchBoxOptions})},function(t,e){},function(t,e,n){"use strict";var r=n(0),i=n(44),o=n(70),a=n(41),u=n(80),s=function(){function t(t,e){this.desc={column:e,type:t,label:e?e[0].toUpperCase()+e.slice(1):t}}return t.prototype.label=function(t){return this.desc.label=t,this},t.prototype.description=function(t){return this.desc.description=t,this},t.prototype.frozen=function(){return this.desc.frozen=!0,this},t.prototype.renderer=function(t,e,n){return t&&(this.desc.renderer=t),e&&(this.desc.groupRenderer=e),n&&(this.desc.summaryRenderer=n),this},t.prototype.custom=function(t,e){return this.desc[t]=e,this},t.prototype.width=function(t){return this.desc.width=t,this},t.prototype.color=function(t){return this.desc.color=t,this},t.prototype.asArray=function(t){console.assert(["boolean","categorical","date","number","string","link"].includes(this.desc.type)),this.desc.type+="s";var e=this.desc;return Array.isArray(t)?(e.labels=t,e.dataLength=t.length):"number"==typeof t&&(e.dataLength=t),this},t.prototype.asMap=function(){return console.assert(["categorical","date","number","string","link"].includes(this.desc.type)),this.desc.type+="Map",this},t.prototype.build=function(t){return this.desc},t}(),c=s;function l(t,e){return new s(t,e)}var d=function(){function t(){this.options={renderers:{},toolbar:{}}}return t.prototype.animated=function(t){return this.options.animated=t,this},t.prototype.labelRotation=function(t){return this.options.labelRotation=t,this},t.prototype.sidePanel=function(t,e){return void 0===e&&(e=!1),this.options.sidePanel=t,this.options.sidePanelCollapsed=e,this},t.prototype.hierarchyIndicator=function(t){return this.options.hierarchyIndicator=t,this},t.prototype.defaultSlopeGraphMode=function(t){return this.options.defaultSlopeGraphMode=t,this},t.prototype.summaryHeader=function(t){return this.options.summaryHeader=t,this},t.prototype.expandLineOnHover=function(t){return this.options.expandLineOnHover=t,this},t.prototype.overviewMode=function(){return this.options.overviewMode=!0,this},t.prototype.ignoreUnsupportedBrowser=function(){return this.options.ignoreUnsupportedBrowser=!0,this},t.prototype.registerRenderer=function(t,e){return this.options.renderers[t]=e,this},t.prototype.registerToolbarAction=function(t,e){return this.options.toolbar[t]=e,this},t.prototype.registerToolbarDialogAddon=function(t,e){return this.options.toolbar[t]=e,this},t.prototype.rowHeight=function(t,e){return this.options.rowHeight=t,this.options.rowPadding=e,this},t.prototype.groupRowHeight=function(t,e){return this.options.groupHeight=t,this.options.groupPadding=e,this},t.prototype.dynamicHeight=function(t){return this.options.dynamicHeight=t,this},t}(),p=n(62),f=n(64),h=n(63),g=n(72),m=n(65),b=n(66),y=n(67),v=n(68),E=n(61),C=n(26),w=n(39),A=function(){function t(){this.columns=[],this.sort=[],this.groups=[]}return t.prototype.sortBy=function(t,e){if(void 0===e&&(e=!0),t.includes(":")){var n=t.indexOf(":");e=t.slice(n+1),t=t.slice(0,n)}return this.sort.push({column:t,asc:!0===e||"a"===String(e)[0]}),this},t.prototype.groupBy=function(){for(var t,e=[],n=0;n<arguments.length;n++)e[n]=arguments[n];return(t=this.groups).push.apply(t,e),this},t.prototype.column=function(t){if("string"==typeof t){switch(t){case"_aggregate":return this.aggregate();case"_selection":return this.selection();case"_group":return this.group();case"_rank":return this.rank();case"_*":return this.supportTypes();case"*":return this.allColumns()}return this.columns.push(t),this}var e=t.label||null;switch(t.type){case"impose":return this.impose(e,t.column,t.categoricalColumn);case"min":case"max":case"median":case"mean":return console.assert(t.columns.length>=2),this.reduce.apply(this,[e,t.type,t.columns[0],t.columns[1]].concat(t.columns.slice(2)));case"nested":return console.assert(t.columns.length>=1),this.nested.apply(this,[e,t.columns[0]].concat(t.columns.slice(1)));case"script":return console.assert(t.columns.length>=2),this.scripted.apply(this,[e,t.code,t.columns[0],t.columns[1]].concat(t.columns.slice(2)));case"weightedSum":console.assert(t.columns.length>=2),console.assert(t.columns.length===t.weights.length);var n=[];return t.columns.slice(2).forEach(function(e,r){n.push(e),n.push(t.weights[r+2])}),this.weightedSum.apply(this,[e,t.columns[0],t.weights[0],t.columns[1],t.weights[1]].concat(n))}return console.error("invalid column type: ",t),this},t.prototype.impose=function(t,e,n){return this.columns.push({desc:function(n){var r=n.getColumns().find(function(t){return t.label===e||t.column===e});switch(r?r.type:""){case"boxplot":return Object(p.a)(t||void 0);case"numbers":return Object(f.a)(t||void 0);default:return Object(h.a)(t||void 0)}},columns:[e,n]}),this},t.prototype.nested=function(t,e){for(var n=[],r=2;r<arguments.length;r++)n[r-2]=arguments[r];return this.columns.push({desc:Object(g.a)(t||void 0),columns:[e].concat(n)}),this},t.prototype.weightedSum=function(t,e,n,r,i){for(var o=[],a=5;a<arguments.length;a++)o[a-5]=arguments[a];var u=[n,i].concat(o.filter(function(t,e){return e%2==1}));return this.columns.push({desc:Object(m.a)(t||void 0),columns:[e,r].concat(o.filter(function(t,e){return e%2==0})),post:function(t){t.setWeights(u)}}),this},t.prototype.reduce=function(t,e,n,r){for(var i=[],o=4;o<arguments.length;o++)i[o-4]=arguments[o];return this.columns.push({desc:Object(b.c)(t||void 0),columns:[n,r].concat(i),post:function(t){t.setReduce(e)}}),this},t.prototype.scripted=function(t,e,n,r){for(var i=[],o=4;o<arguments.length;o++)i[o-4]=arguments[o];return this.columns.push({desc:Object(y.c)(t||void 0),columns:[n,r].concat(i),post:function(t){t.setScript(e)}}),this},t.prototype.selection=function(){return this.columns.push({desc:Object(v.b)(),columns:[]}),this},t.prototype.group=function(){return this.columns.push({desc:Object(E.b)(),columns:[]}),this},t.prototype.aggregate=function(){return this.columns.push({desc:Object(C.b)(),columns:[]}),this},t.prototype.rank=function(){return this.columns.push({desc:Object(w.a)(),columns:[]}),this},t.prototype.supportTypes=function(){return this.aggregate().rank().selection()},t.prototype.allColumns=function(){return this.columns.push(t.ALL_MAGIC_FLAG),this},t.prototype.build=function(e){var n=e.pushRanking(),r=e.getColumns(),i=function(t){return r.find(function(e){return e.label===t||e.column===t})};this.columns.forEach(function(o){if(o!==t.ALL_MAGIC_FLAG)if("string"!=typeof o){var a=e.create("function"==typeof o.desc?o.desc(e):o.desc);n.push(a),o.columns.forEach(function(t){var n=i(t),r=n?e.create(n):null;r?a.push(r):console.warn("invalid column: ",t)}),o.post&&o.post(a)}else!function(t){var r=i(t);r?e.push(n,r):console.warn("invalid column: ",t)}(o);else r.forEach(function(t){return e.push(n,t)})});var o=n.children,a=[];this.groups.forEach(function(t){var r=o.find(function(e){return e.desc.label===t||e.desc.column===t});if(r)a.push(r);else{var u=i(t);u&&e.push(n,u)||console.warn("invalid group criteria column: ",t)}}),a.length>0&&n.setGroupCriteria(a);var u=[];return this.sort.forEach(function(t){var r=t.column,a=t.asc,s=o.find(function(t){return t.desc.label===r||t.desc.column===r});if(s)u.push({col:s,asc:a});else{var c=i(r);c&&e.push(n,c)||console.warn("invalid sort criteria column: ",r)}}),u.length>0&&n.setSortCriteria(u),n},t.ALL_MAGIC_FLAG="*",t}(),x=A;function N(){return new A}var T=function(t){function e(e){var n=t.call(this)||this;return n.data=e,n.columns=[],n.providerOptions={columnTypes:{}},n.rankBuilders=[],n._deriveColors=!1,n}return r.c(e,t),e.prototype.singleSelection=function(){return this.providerOptions.multiSelection=!1,this},e.prototype.filterGlobally=function(){return this.providerOptions.filterGlobally=!0,this},e.prototype.noCriteriaLimits=function(){return this.providerOptions.maxGroupColumns=1/0,this.providerOptions.maxNestedSortingCriteria=1/0,this},e.prototype.limitCriteria=function(t,e){return this.providerOptions.maxGroupColumns=t,this.providerOptions.maxNestedSortingCriteria=e,this},e.prototype.deriveColumns=function(){for(var t,e=[],n=0;n<arguments.length;n++)e[n]=arguments[n];return(t=this.columns).push.apply(t,Object(i.d)(this.data,{columns:e})),this},e.prototype.deriveColors=function(){return this._deriveColors=!0,this},e.prototype.registerColumnType=function(t,e){return this.providerOptions.columnTypes[t]=e,this},e.prototype.column=function(t){return this.columns.push(t instanceof c?t.build.bind(t):t),this},e.prototype.restore=function(t){return this.rankBuilders.push(function(e){return e.restore(t)}),this},e.prototype.defaultRanking=function(t){return void 0===t&&(t=!0),this.rankBuilders.push(function(e){return e.deriveDefault(t)}),this},e.prototype.ranking=function(t){return this.rankBuilders.push(t instanceof x?t.build.bind(t):t),this},e.prototype.buildData=function(){var t=this,e=this.columns.map(function(e){return"function"==typeof e?e(t.data):e});this._deriveColors&&Object(i.c)(e);var n=new o.a(this.data,e,this.providerOptions);return 0===this.rankBuilders.length&&this.defaultRanking(),this.rankBuilders.forEach(function(t){return t(n)}),n},e.prototype.build=function(t){return new a.b(t,this.buildData(),this.options)},e.prototype.buildTaggle=function(t){return new u.b(t,this.buildData(),this.options)},e}(d);function O(t){return new T(t)}var R=function(t){function e(e){return t.call(this,"categorical",e)||this}return r.c(e,t),e.prototype.asOrdinal=function(){return this.desc.type="ordinal",this},e.prototype.categories=function(t){return this.desc.categories=t,this},e.prototype.missingCategory=function(t){return this.desc.missingCategory=t,this},e.prototype.asSet=function(t){return t&&(this.desc.separator=t),this.desc.type="set",this},e.prototype.derive=function(t){var e=this,n=new Set,r="set"===this.desc.type,o=this.desc.separator||";",a=function(t){return"string"==typeof t&&""!==t?t:null!=t&&"string"==typeof t.value&&""!==t.value?t.value:null};t.forEach(function(t){var i=t[e.desc.column];Array.isArray(i)?i.forEach(function(t){return n.add(a(t))}):null!=i&&""!==i&&(r?[i.toString()]:i.toString().split(o)).forEach(function(t){return n.add(a(t))})}),this.categories(Object(i.b)(n))},e.prototype.build=function(e){return this.desc.categories||this.derive(e),t.prototype.build.call(this,e)},e}(c);function M(t,e){var n=new R(t);return e&&n.categories(e),n}var D=n(18),_=function(t){function e(e){return t.call(this,"number",e)||this}return r.c(e,t),e.prototype.mapping=function(t,e,n){return"linear"===t?(this.desc.domain=e,void(n&&(this.desc.range=n))):(this.desc.map={type:t,domain:e,range:n||[0,1]},this)},e.prototype.colorMapping=function(t){return this.desc.colorMapping=t,this},e.prototype.scripted=function(t,e){return this.desc.map={domain:e,code:t,type:"script"},this},e.prototype.asArray=function(e,n){return n&&(this.desc.sort=n),t.prototype.asArray.call(this,e)},e.prototype.asMap=function(e){return e&&(this.desc.sort=e),t.prototype.asMap.call(this)},e.prototype.asBoxPlot=function(t){return t&&(this.desc.sort=t),this.desc.type="boxplot",this},e.prototype.derive=function(t){var e=this.desc.column,n=function(t,e){var n=[];return(Array.isArray(t)?t:[t]).forEach(function(t){"number"!=typeof t||isNaN(t)||n.push(t),null==t||"number"!=typeof t.value||isNaN(t.value)||n.push(t.value),null==t||"number"!=typeof t[e]||isNaN(t[e])||n.push(t[e])}),n};return[Object(D.i)(t,function(t){var r=t[e],i=n(r,"min");return 0===i.length?1/0:Object(D.i)(i)}),Object(D.f)(t,function(t){var r=t[e],i=n(r,"max");return 0===i.length?-1/0:Object(D.f)(i)})]},e.prototype.build=function(e){if(this.desc.map||this.desc.domain){var n=this.desc.domain||this.desc.map.domain;if(isNaN(n[0])||isNaN(n[1])){var r=this.derive(e);isNaN(n[0])&&(n[0]=r[0]),isNaN(n[1])&&(n[1]=r[1])}}else this.mapping("linear",this.derive(e));return t.prototype.build.call(this,e)},e}(c);function L(t,e){var n=new _(t);return e&&n.mapping("linear",e),n}var S=function(t){function e(e){return t.call(this,"string",e)||this}return r.c(e,t),e.prototype.editable=function(){return this.desc.type="annotate",this},e.prototype.alignment=function(t){return this.desc.alignment=t,this},e.prototype.html=function(){return this.desc.escape=!1,this},e.prototype.pattern=function(t,e){return this.desc.type="link",this.desc.pattern=t,e&&(this.desc.patternTemplates=e),this},e}(c);function I(t){return new S(t)}var j=function(t){function e(e){return t.call(this,"date",e)||this}return r.c(e,t),e.prototype.format=function(t,e){return this.desc.dateFormat=t,e&&(this.desc.dateParse=e),this},e}(c);function k(t){return new j(t)}var V=function(t){function e(e){return t.call(this,"hierarchy",e)||this}return r.c(e,t),e.prototype.hierarchy=function(t,e){return this.desc.hierarchy=t,e&&(this.desc.hierarchySeparator=e),this},e.prototype.build=function(e){return console.assert(Boolean(this.desc.hierarchy)),t.prototype.build.call(this,e)},e}(c);function H(t,e){var n=new V(t);return e&&n.hierarchy(e),n}var P=function(t){function e(){return t.call(this,"actions","")||this}return r.c(e,t),e.prototype.action=function(t){return this.actions([t])},e.prototype.actions=function(t){var e;return this.desc.actions||(this.desc.actions=[]),(e=this.desc.actions).push.apply(e,t),this},e.prototype.groupAction=function(t){return this.groupActions([t])},e.prototype.groupActions=function(t){var e;return this.desc.groupActions||(this.desc.groupActions=[]),(e=this.desc.groupActions).push.apply(e,t),this},e}(c);function G(){return new P}function F(t,e){var n=t.column,r={column:n,type:t.type,label:n?n[0].toUpperCase()+n.slice(1):t.type};if(["label","description","frozen","color","width","renderer","groupRenderer","summaryRenderer","visible","fixed"].forEach(function(e){t.hasOwnProperty(e)&&(r[e]=t[e])}),t.custom&&Object.assign(r,t.custom),t.asMap&&(console.assert(["categorical","date","number","string","link"].includes(r.type)),r.type+="Map"),null!=t.asArray){console.assert(["boolean","categorical","date","number","string","link"].includes(r.type)),r.type+="s";var i=r,o=t.asArray;Array.isArray(o)?(i.labels=o,i.dataLength=o.length):"number"==typeof o&&(i.dataLength=o)}return r}function U(t,e){var n=N();if(t.sortBy&&(r=Array.isArray(t.sortBy)?t.sortBy:[t.sortBy]).forEach(function(t){"string"==typeof t?n.sortBy(t):n.sortBy(t.column,t.asc)}),t.groupBy){var r=Array.isArray(t.groupBy)?t.groupBy:[t.groupBy];n.groupBy.apply(n,r)}return t.columns&&t.columns.forEach(function(t){return n.column(t)}),n.build(e)}var B=n(74);function z(t,e){var n={};return e.forEach(function(e){t.hasOwnProperty(e)&&(n[e]=t[e])}),n}function Y(t,e,n){return n.every(function(t){return!e(t)})?null:z(t,n)}var W=["singleSelection","filterGlobally","noCriteriaLimits","maxGroupColumns","maxNestedSortingCriteria","columnTypes"],q=["animated","sidePanel","sidePanelCollapsed","hierarchyIndicator","defaultSlopeGraphMode","summaryHeader","expandLineOnHover","overviewMode","renderers","toolbar","rowHeight","rowPadding","groupHeight","groupPadding","dynamicHeight","labelRotation","ignoreUnsupportedBrowser"],K=function(){function t(t){var e=this;this.adapter=t,this.data=null,this.instance=null,this.prevRankings=null,this.prevColumns=null,this.prevHighlight=null,this.onSelectionChanged=function(t){e.props.onSelectionChanged&&!Object(B.a)(e.props.selection,t)&&e.props.onSelectionChanged(t)},this.onHighlightChanged=function(t){(null!=e.prevHighlight?e.prevHighlight:-1)!==t&&(e.prevHighlight=t,e.props.onHighlightChanged&&e.props.onHighlightChanged(t))}}return Object.defineProperty(t.prototype,"props",{get:function(){return this.adapter.props()},enumerable:!0,configurable:!0}),t.prototype.componentDidMount=function(){this.data=this.buildProvider(),this.instance=this.adapter.createInstance(this.data,z(this.props,q)),this.instance.on(a.b.EVENT_HIGHLIGHT_CHANGED,this.onHighlightChanged)},t.prototype.resolveColumnDescs=function(t){var e=this.adapter.columnDescs(t),n=0===e.length||Boolean(this.props.deriveColumns),r=Array.isArray(this.props.deriveColumns)?this.props.deriveColumns:[];return{columns:e,deriveColors:Boolean(this.props.deriveColors),deriveColumns:n,deriveColumnNames:r}},t.prototype.resolveRankings=function(){var t=this.adapter.rankingBuilders();return{builders:t,restore:this.props.restore,derive:0===t.length&&!this.props.restore||Boolean(this.props.defaultRanking),supportTypes:"noSupportTypes"!==this.props.defaultRanking}},t.prototype.buildColumns=function(t,e){this.prevColumns=e;var n=e.columns.map(function(t){return Object.assign({},t)});return e.deriveColumns&&n.push.apply(n,Object(i.d)(t,{columns:e.deriveColumnNames})),e.deriveColors&&Object(i.c)(n),n},t.prototype.buildRankings=function(t,e){t.clearRankings(),this.prevRankings=e,e.derive&&t.deriveDefault(e.supportTypes),e.restore&&t.restore(e.restore),e.builders.forEach(function(e){return U(e,t)})},t.prototype.buildProvider=function(){var t=this.buildColumns(this.props.data,this.resolveColumnDescs(this.props.data)),e=new o.a(this.props.data,t,z(this.props,W));return this.buildRankings(e,this.resolveRankings()),e.setSelection(this.props.selection||[]),e.on(o.a.EVENT_SELECTION_CHANGED,this.onSelectionChanged),e},t.prototype.updateLineUp=function(t,e){var n=Y(this.props,t,q);return n?(this.instance&&this.instance.destroy(),this.instance=this.adapter.createInstance(this.data,n),this.prevHighlight=null==this.props.highlight?-1:this.props.highlight,this.instance.setHighlight(this.prevHighlight),this.instance.on(a.b.EVENT_HIGHLIGHT_CHANGED,this.onHighlightChanged),!0):(e&&this.instance.setDataProvider(this.data),!!(e||null!=this.props.highlight&&this.prevHighlight!==this.props.highlight)&&(this.prevHighlight=null==this.props.highlight?-1:this.props.highlight,this.instance.on(a.b.EVENT_HIGHLIGHT_CHANGED,null),this.instance.setHighlight(this.prevHighlight),this.instance.on(a.b.EVENT_HIGHLIGHT_CHANGED,this.onHighlightChanged),!0))},t.prototype.updateProvider=function(t){var e=this;if(Y(this.props,t,W)||!this.data||t("data"))return this.data=this.buildProvider(),!0;var n=this.resolveRankings(),r=this.resolveColumnDescs(this.props.data),i=!Object(B.a)(this.prevColumns,r);if(i){var a=this.buildColumns(this.props.data,r);this.data.clearColumns(),a.forEach(function(t){return e.data.pushDesc(t)})}return!i&&Object(B.a)(n,this.prevRankings)||this.buildRankings(this.data,n),this.data.on(o.a.EVENT_SELECTION_CHANGED,null),this.data.setSelection(this.props.selection||[]),this.data.on(o.a.EVENT_SELECTION_CHANGED,this.onSelectionChanged),!1},t.prototype.componentDidUpdate=function(t){var e=this.updateProvider(t);this.updateLineUp(t,e)},t.prototype.componentWillUnmount=function(){this.instance&&(this.instance.destroy(),this.instance=null),this.data=null},t}(),J={buildString:function(t){var e=F(r.a({},t,{type:"string"}));return["pattern","patternTemplate","alignment"].forEach(function(n){t.hasOwnProperty(n)&&(e[n]=t[n])}),t.editable&&(e.type="annotate"),t.pattern&&(e.type="link"),t.html&&(e.escape=!1),e},buildNumber:function(t,e){var n=F(r.a({},t,{type:"number"})),i=t.domain?t.domain:Object(D.d)(e,function(t){return t[n.column]});return["sort","colorMapping"].forEach(function(e){t.hasOwnProperty(e)&&(n[e]=t[e])}),t.scripted?n.map={domain:i,code:t.scripted,type:"script"}:t.mapping&&"linear"!==t.mapping?n.map={type:t.mapping,domain:i,range:t.range||[0,1]}:(n.domain=i,t.range&&(n.range=t.range)),n},buildHierarchy:function(t){var e=F(r.a({},t,{type:"hierarchy"}));return["hierarchy","hierarchySeparator"].forEach(function(n){t.hasOwnProperty(n)&&(e[n]=t[n])}),e},buildDate:function(t){var e=F(r.a({},t,{type:"date"}));return["dateFormat","dateParse"].forEach(function(n){t.hasOwnProperty(n)&&(e[n]=t[n])}),e},buildCategorical:function(t,e){var n=F(r.a({},t,{type:"categorical"}));if(t.asOrdinal&&(n.type="ordinal"),t.missingCategory&&(n.missingCategory=t.missingCategory),t.asSet&&("string"==typeof t.asSet&&(n.separator=t.asSet),n.type="set"),t.categories)n.categories=t.categories;else{var i=new Set(e.map(function(t){return t[n.column]}));n.categories=Array.from(i).sort()}return n},buildActions:function(t){var e=F(r.a({},t,{type:"actions"}));return["actions","groupActions"].forEach(function(n){t.hasOwnProperty(n)&&(e[n]=t[n])}),e},build:F,buildGeneric:function(t){return t.column},buildWeightedSumRanking:function(t,e){var n={type:"weightedSum",columns:e.map(function(t){return t.column}),weights:e.map(function(t){return t.weight})};return t.label&&(n.label=t.label),n},buildSupportRanking:function(t){return"_"+t.type},buildScriptRanking:function(t,e){var n={type:"script",code:t.code,columns:e};return t.label&&(n.label=t.label),n},buildReduceRanking:function(t,e){var n={type:t.type,columns:e};return t.label&&(n.label=t.label),n},buildRanking:U,buildNestedRanking:function(t,e){var n={type:"nested",columns:e};return t.label&&(n.label=t.label),n},buildImposeRanking:function(t){return Object.assign({type:"impose"},t)},buildAllColumnsRanking:function(){return"*"},equal:B.a,isSame:Y,isTypeInstance:function(t,e){for(var n=t;n&&n!==e;)n=n.__proto__;return n===e},Adapter:K};function Z(t,e){for(var n,r=[],i=2;i<arguments.length;i++)r[i-2]=arguments[i];return(n=O(e)).deriveColumns.apply(n,r).deriveColors().defaultRanking().buildTaggle(t)}function Q(t,e){for(var n,r=[],i=2;i<arguments.length;i++)r[i-2]=arguments[i];return(n=O(e)).deriveColumns.apply(n,r).deriveColors().defaultRanking().build(t)}n.d(e,"t",function(){return Z}),n.d(e,"s",function(){return Q}),n.d(e,"C",function(){return O}),n.d(e,"w",function(){return l}),n.d(e,"v",function(){return M}),n.d(e,"z",function(){return L}),n.d(e,"B",function(){return I}),n.d(e,"x",function(){return k}),n.d(e,"y",function(){return H}),n.d(e,"u",function(){return G}),n.d(e,"A",function(){return N}),n.d(e,"c",function(){}),n.d(e,"d",function(){}),n.d(e,"f",function(){}),n.d(e,"g",function(){}),n.d(e,"j",function(){}),n.d(e,"o",function(){}),n.d(e,"b",function(){}),n.d(e,"r",function(){}),n.d(e,"a",function(){}),n.d(e,"e",function(){}),n.d(e,"k",function(){}),n.d(e,"h",function(){}),n.d(e,"i",function(){}),n.d(e,"l",function(){}),n.d(e,"m",function(){}),n.d(e,"n",function(){}),n.d(e,"p",function(){}),n.d(e,"q",function(){}),n.d(e,"D",function(){return J})},function(t,e){function r(t){var e=a([["iOS",/iP(hone|od|ad)/],["Android OS",/Android/],["BlackBerry OS",/BlackBerry|BB10/],["Windows Mobile",/IEMobile/],["Amazon OS",/Kindle/],["Windows 3.11",/Win16/],["Windows 95",/(Windows 95)|(Win95)|(Windows_95)/],["Windows 98",/(Windows 98)|(Win98)/],["Windows 2000",/(Windows NT 5.0)|(Windows 2000)/],["Windows XP",/(Windows NT 5.1)|(Windows XP)/],["Windows Server 2003",/(Windows NT 5.2)/],["Windows Vista",/(Windows NT 6.0)/],["Windows 7",/(Windows NT 6.1)/],["Windows 8",/(Windows NT 6.2)/],["Windows 8.1",/(Windows NT 6.3)/],["Windows 10",/(Windows NT 10.0)/],["Windows ME",/Windows ME/],["Open BSD",/OpenBSD/],["Sun OS",/SunOS/],["Linux",/(Linux)|(X11)/],["Mac OS",/(Mac_PowerPC)|(Macintosh)/],["QNX",/QNX/],["BeOS",/BeOS/],["OS/2",/OS\/2/],["Search Bot",/(nuhk)|(Googlebot)|(Yammybot)|(Openbot)|(Slurp)|(MSNBot)|(Ask Jeeves\/Teoma)|(ia_archiver)/]]).filter(function(e){return e.rule&&e.rule.test(t)})[0];return e?e.name:null}function i(){return void 0!==n&&n.version&&{name:"node",version:n.version.slice(1),os:n.platform}}function o(t){var e=a([["aol",/AOLShield\/([0-9\._]+)/],["edge",/Edge\/([0-9\._]+)/],["yandexbrowser",/YaBrowser\/([0-9\._]+)/],["vivaldi",/Vivaldi\/([0-9\.]+)/],["kakaotalk",/KAKAOTALK\s([0-9\.]+)/],["samsung",/SamsungBrowser\/([0-9\.]+)/],["chrome",/(?!Chrom.*OPR)Chrom(?:e|ium)\/([0-9\.]+)(:?\s|$)/],["phantomjs",/PhantomJS\/([0-9\.]+)(:?\s|$)/],["crios",/CriOS\/([0-9\.]+)(:?\s|$)/],["firefox",/Firefox\/([0-9\.]+)(?:\s|$)/],["fxios",/FxiOS\/([0-9\.]+)/],["opera",/Opera\/([0-9\.]+)(?:\s|$)/],["opera",/OPR\/([0-9\.]+)(:?\s|$)$/],["ie",/Trident\/7\.0.*rv\:([0-9\.]+).*\).*Gecko$/],["ie",/MSIE\s([0-9\.]+);.*Trident\/[4-7].0/],["ie",/MSIE\s(7\.0)/],["bb10",/BB10;\sTouch.*Version\/([0-9\.]+)/],["android",/Android\s([0-9\.]+)/],["ios",/Version\/([0-9\._]+).*Mobile.*Safari.*/],["safari",/Version\/([0-9\._]+).*Safari/],["facebook",/FBAV\/([0-9\.]+)/],["instagram",/Instagram\ ([0-9\.]+)/],["ios-webview",/AppleWebKit\/([0-9\.]+).*Mobile/]]);if(!t)return null;var n=e.map(function(e){var n=e.rule.exec(t),r=n&&n[1].split(/[._]/).slice(0,3);return r&&r.length<3&&(r=r.concat(1==r.length?[0,0]:[0])),n&&{name:e.name,version:r.join(".")}}).filter(Boolean)[0]||null;return n&&(n.os=r(t)),/alexa|bot|crawl(er|ing)|facebookexternalhit|feedburner|google web preview|nagios|postrank|pingdom|slurp|spider|yahoo!|yandex/i.test(t)&&((n=n||{}).bot=!0),n}function a(t){return t.map(function(t){return{name:t[0],rule:t[1]}})}t.exports={detect:function(){return"undefined"!=typeof navigator?o(navigator.userAgent):i()},detectOS:r,getNodeVersion:i,parseUserAgent:o}},function(t,e,n){"use strict";n.r(e),n.d(e,"version",function(){return g}),n.d(e,"buildId",function(){return m}),n.d(e,"license",function(){return b}),n.d(e,"createLocalDataProvider",function(){return y}),n.d(e,"createLineUp",function(){return v}),n.d(e,"createTaggle",function(){return E}),n(101);var r=n(70),i=n(41),o=n(80),a=n(84);n.d(e,"DataProvider",function(){return a.a}),n.d(e,"IStatsBuilder",function(){return a.e}),n.d(e,"IDataProviderOptions",function(){return a.c}),n.d(e,"IDataProvider",function(){return a.b}),n.d(e,"IExportOptions",function(){return a.d}),n.d(e,"LocalDataProvider",function(){return a.f}),n.d(e,"RemoteDataProvider",function(){return a.g}),n.d(e,"cleanCategories",function(){return a.h}),n.d(e,"deriveColumnDescriptions",function(){return a.j}),n.d(e,"deriveColors",function(){return a.i}),n.d(e,"exportRanking",function(){return a.k});var u=n(55);n.d(e,"IRenderContext",function(){return u.f}),n.d(e,"IImposer",function(){return u.e}),n.d(e,"ERenderMode",function(){return u.a}),n.d(e,"ISummaryRenderer",function(){return u.g}),n.d(e,"IGroupCellRenderer",function(){return u.d}),n.d(e,"ICellRenderer",function(){return u.b}),n.d(e,"ICellRendererFactory",function(){return u.c}),n.d(e,"colorOf",function(){return u.k}),n.d(e,"renderMissingDOM",function(){return u.p}),n.d(e,"renderMissingCanvas",function(){return u.o}),n.d(e,"renderers",function(){return u.q}),n.d(e,"chooseRenderer",function(){return u.i}),n.d(e,"chooseGroupRenderer",function(){return u.h}),n.d(e,"chooseSummaryRenderer",function(){return u.j}),n.d(e,"possibleRenderer",function(){return u.m}),n.d(e,"possibleGroupRenderer",function(){return u.l}),n.d(e,"possibleSummaryRenderer",function(){return u.n});var s=n(22);n.d(e,"isSupportType",function(){return s.Ac}),n.d(e,"Category",function(){return s.l}),n.d(e,"SupportType",function(){return s.Nb}),n.d(e,"SortByDefault",function(){return s.Ib}),n.d(e,"Categories",function(){return s.k}),n.d(e,"toolbar",function(){return s.Kc}),n.d(e,"dialogAddons",function(){return s.hc}),n.d(e,"isMissingValue",function(){return s.uc}),n.d(e,"isUnknown",function(){return s.Bc}),n.d(e,"FIRST_IS_NAN",function(){return s.y}),n.d(e,"FIRST_IS_MISSING",function(){return s.x}),n.d(e,"missingGroup",function(){return s.Cc}),n.d(e,"ActionColumn",function(){return s.a}),n.d(e,"AggregateGroupColumn",function(){return s.b}),n.d(e,"createAggregateDesc",function(){return s.Sb}),n.d(e,"IAggregateGroupColumnDesc",function(){return s.B}),n.d(e,"AnnotateColumn",function(){return s.c}),n.d(e,"ArrayColumn",function(){return s.d}),n.d(e,"IArrayColumnDesc",function(){return s.C}),n.d(e,"IArrayDesc",function(){return s.D}),n.d(e,"ISplicer",function(){return s.jb}),n.d(e,"BooleanColumn",function(){return s.e}),n.d(e,"IBooleanColumnDesc",function(){return s.E}),n.d(e,"IBooleanDesc",function(){return s.F}),n.d(e,"BooleansColumn",function(){return s.f}),n.d(e,"BoxPlotColumn",function(){return s.g}),n.d(e,"IBoxPlotColumnDesc",function(){return s.G}),n.d(e,"IBoxPlotDesc",function(){return s.H}),n.d(e,"CategoricalColumn",function(){return s.h}),n.d(e,"CategoricalMapColumn",function(){return s.i}),n.d(e,"CategoricalsColumn",function(){return s.j}),n.d(e,"Column",function(){return s.m}),n.d(e,"IFlatColumn",function(){return s.P}),n.d(e,"IColumnParent",function(){return s.N}),n.d(e,"IColumnMetaData",function(){return s.M}),n.d(e,"IColumnDesc",function(){return s.L}),n.d(e,"CompositeColumn",function(){return s.n}),n.d(e,"isMultiLevelColumn",function(){return s.vc}),n.d(e,"IMultiLevelColumn",function(){return s.V}),n.d(e,"CompositeNumberColumn",function(){return s.o}),n.d(e,"DateColumn",function(){return s.q}),n.d(e,"DatesMapColumn",function(){return s.r}),n.d(e,"DummyColumn",function(){return s.s}),n.d(e,"GroupColumn",function(){return s.z}),n.d(e,"createGroupDesc",function(){return s.Tb}),n.d(e,"EGroupSortMethod",function(){return s.v}),n.d(e,"HierarchyColumn",function(){return s.A}),n.d(e,"ICategoryNode",function(){return s.K}),n.d(e,"IPartialCategoryNode",function(){return s.cb}),n.d(e,"IHierarchyColumnDesc",function(){return s.Q}),n.d(e,"IHierarchyDesc",function(){return s.R}),n.d(e,"ICategoryInternalNode",function(){return s.J}),n.d(e,"ICutOffNode",function(){return s.O}),n.d(e,"resolveInnerNodes",function(){return s.Gc}),n.d(e,"isHierarchical",function(){return s.rc}),n.d(e,"deriveHierarchy",function(){return s.gc}),n.d(e,"ImpositionBoxPlotColumn",function(){return s.nb}),n.d(e,"ImpositionCompositeColumn",function(){return s.ob}),n.d(e,"ImpositionCompositesColumn",function(){return s.pb}),n.d(e,"MapColumn",function(){return s.tb}),n.d(e,"MultiLevelCompositeColumn",function(){return s.ub}),n.d(e,"NestedColumn",function(){return s.vb}),n.d(e,"NumberColumn",function(){return s.wb}),n.d(e,"INumberColumnDesc",function(){return s.X}),n.d(e,"INumberColumn",function(){return s.W}),n.d(e,"isNumberColumn",function(){return s.wc}),n.d(e,"NumberMapColumn",function(){return s.xb}),n.d(e,"INumberMapDesc",function(){return s.Z}),n.d(e,"INumberMapColumnDesc",function(){return s.Y}),n.d(e,"NumbersColumn",function(){return s.yb}),n.d(e,"INumbersDesc",function(){return s.bb}),n.d(e,"INumbersColumnDesc",function(){return s.ab}),n.d(e,"OrdinalColumn",function(){return s.zb}),n.d(e,"ICategoricalNumberColumnDesc",function(){return s.I}),n.d(e,"RankColumn",function(){return s.Ab}),n.d(e,"Ranking",function(){return s.Bb}),n.d(e,"ISortCriteria",function(){return s.ib}),n.d(e,"ReduceColumn",function(){return s.Cb}),n.d(e,"createReduceDesc",function(){return s.ac}),n.d(e,"IReduceDesc",function(){return s.eb}),n.d(e,"IReduceColumnDesc",function(){return s.db}),n.d(e,"ScriptColumn",function(){return s.Eb}),n.d(e,"createScriptDesc",function(){return s.bc}),n.d(e,"IScriptDesc",function(){return s.gb}),n.d(e,"IScriptColumnDesc",function(){return s.fb}),n.d(e,"SelectionColumn",function(){return s.Gb}),n.d(e,"createSelectionDesc",function(){return s.cc}),n.d(e,"ISelectionColumnDesc",function(){return s.hb}),n.d(e,"SetColumn",function(){return s.Hb}),n.d(e,"StackColumn",function(){return s.Jb}),n.d(e,"createStackDesc",function(){return s.dc}),n.d(e,"StringColumn",function(){return s.Kb}),n.d(e,"EAlignment",function(){return s.u}),n.d(e,"IStringDesc",function(){return s.lb}),n.d(e,"IStringColumnDesc",function(){return s.kb}),n.d(e,"StringMapColumn",function(){return s.Lb}),n.d(e,"StringsColumn",function(){return s.Mb}),n.d(e,"ValueColumn",function(){return s.Ob}),n.d(e,"IValueColumnDesc",function(){return s.mb}),n.d(e,"LinkColumn",function(){return s.qb}),n.d(e,"ILinkDesc",function(){return s.U}),n.d(e,"ILinkColumnDesc",function(){return s.T}),n.d(e,"ILink",function(){return s.S}),n.d(e,"LinkMapColumn",function(){return s.rb}),n.d(e,"LinksColumn",function(){return s.sb}),n.d(e,"defineColumn",function(){return s.fc}),n.d(e,"models",function(){return s.Dc}),n.d(e,"defaultGroup",function(){return s.ec}),n.d(e,"isGroup",function(){return s.qc}),n.d(e,"isSetColumn",function(){return s.zc}),n.d(e,"toCategory",function(){return s.Jc}),n.d(e,"compareCategory",function(){return s.Qb}),n.d(e,"groupCompareCategory",function(){return s.jc}),n.d(e,"toCategories",function(){return s.Ic}),n.d(e,"isCategoricalColumn",function(){return s.mc}),n.d(e,"isEqualCategoricalFilter",function(){return s.oc}),n.d(e,"isCategoryIncluded",function(){return s.nc}),n.d(e,"DEFAULT_FORMATTER",function(){return s.p}),n.d(e,"compareBoxPlot",function(){return s.Pb}),n.d(e,"getBoxPlotNumber",function(){return s.ic}),n.d(e,"ESortMethod",function(){return s.w}),n.d(e,"isBoxPlotColumn",function(){return s.lc}),n.d(e,"EAdvancedSortMethod",function(){return s.t}),n.d(e,"isNumbersColumn",function(){return s.yc}),n.d(e,"numberCompare",function(){return s.Fc}),n.d(e,"noNumberFilter",function(){return s.Ec}),n.d(e,"isEqualNumberFilter",function(){return s.pc}),n.d(e,"isNumberIncluded",function(){return s.xc}),n.d(e,"isMapColumn",function(){return s.tc}),n.d(e,"isArrayColumn",function(){return s.kc}),n.d(e,"isMapAbleColumn",function(){return s.sc}),n.d(e,"ScaleMappingFunction",function(){return s.Db}),n.d(e,"ScriptMappingFunction",function(){return s.Fb}),n.d(e,"createMappingFunction",function(){return s.Xb}),n.d(e,"restoreMapping",function(){return s.Hc}),n.d(e,"createActionDesc",function(){return s.Rb}),n.d(e,"createImpositionBoxPlotDesc",function(){return s.Ub}),n.d(e,"createImpositionDesc",function(){return s.Vb}),n.d(e,"createImpositionsDesc",function(){return s.Wb}),n.d(e,"createNestedDesc",function(){return s.Yb}),n.d(e,"createRankDesc",function(){return s.Zb});var c=n(97);n.d(e,"asTaggle",function(){return c.t}),n.d(e,"asLineUp",function(){return c.s}),n.d(e,"builder",function(){return c.C}),n.d(e,"buildColumn",function(){return c.w}),n.d(e,"buildRanking",function(){return c.A}),n.d(e,"IBuilderAdapterCategoricalColumnDescProps",function(){return c.c}),n.d(e,"IBuilderAdapterColumnDescProps",function(){return c.d}),n.d(e,"IBuilderAdapterDateColumnDescProps",function(){return c.f}),n.d(e,"IBuilderAdapterHierarchyColumnDescProps",function(){return c.g}),n.d(e,"IBuilderAdapterNumberColumnDescProps",function(){return c.j}),n.d(e,"IBuilderAdapterStringColumnDescProps",function(){return c.o}),n.d(e,"IBuilderAdapterActionsColumnDescProps",function(){return c.b}),n.d(e,"IChangeDetecter",function(){return c.r}),n.d(e,"IBuilderAdapter",function(){return c.a}),n.d(e,"IBuilderAdapterDataProps",function(){return c.e}),n.d(e,"IBuilderAdapterProps",function(){return c.k}),n.d(e,"IBuilderAdapterImposeColumnProps",function(){return c.h}),n.d(e,"IBuilderAdapterNestedColumnProps",function(){return c.i}),n.d(e,"IBuilderAdapterRankingProps",function(){return c.l}),n.d(e,"IBuilderAdapterReduceColumnProps",function(){return c.m}),n.d(e,"IBuilderAdapterScriptColumnProps",function(){return c.n}),n.d(e,"IBuilderAdapterSupportColumnProps",function(){return c.p}),n.d(e,"IBuilderAdapterWeightedSumColumnProps",function(){return c.q}),n.d(e,"builderAdapter",function(){return c.D}),n.d(e,"buildCategoricalColumn",function(){return c.v}),n.d(e,"buildNumberColumn",function(){return c.z}),n.d(e,"buildStringColumn",function(){return c.B}),n.d(e,"buildDateColumn",function(){return c.x}),n.d(e,"buildHierarchicalColumn",function(){return c.y}),n.d(e,"buildActionsColumn",function(){return c.u});var l=n(83);for(var d in l)["default","IBoxPlotData","IAdvancedBoxPlotData","IStatistics","ICategoricalBin","ICategoricalStatistics","INumberBin","version","buildId","license","createLocalDataProvider","createLineUp","createTaggle","DataProvider","IStatsBuilder","IDataProviderOptions","IDataProvider","IExportOptions","LocalDataProvider","RemoteDataProvider","cleanCategories","deriveColumnDescriptions","deriveColors","exportRanking","IRenderContext","IImposer","ERenderMode","ISummaryRenderer","IGroupCellRenderer","ICellRenderer","ICellRendererFactory","colorOf","renderMissingDOM","renderMissingCanvas","renderers","chooseRenderer","chooseGroupRenderer","chooseSummaryRenderer","possibleRenderer","possibleGroupRenderer","possibleSummaryRenderer","isSupportType","Category","SupportType","SortByDefault","Categories","toolbar","dialogAddons","isMissingValue","isUnknown","FIRST_IS_NAN","FIRST_IS_MISSING","missingGroup","ActionColumn","AggregateGroupColumn","createAggregateDesc","IAggregateGroupColumnDesc","AnnotateColumn","ArrayColumn","IArrayColumnDesc","IArrayDesc","ISplicer","BooleanColumn","IBooleanColumnDesc","IBooleanDesc","BooleansColumn","BoxPlotColumn","IBoxPlotColumnDesc","IBoxPlotDesc","CategoricalColumn","CategoricalMapColumn","CategoricalsColumn","Column","IFlatColumn","IColumnParent","IColumnMetaData","IColumnDesc","CompositeColumn","isMultiLevelColumn","IMultiLevelColumn","CompositeNumberColumn","DateColumn","DatesMapColumn","DummyColumn","GroupColumn","createGroupDesc","EGroupSortMethod","HierarchyColumn","ICategoryNode","IPartialCategoryNode","IHierarchyColumnDesc","IHierarchyDesc","ICategoryInternalNode","ICutOffNode","resolveInnerNodes","isHierarchical","deriveHierarchy","ImpositionBoxPlotColumn","ImpositionCompositeColumn","ImpositionCompositesColumn","MapColumn","MultiLevelCompositeColumn","NestedColumn","NumberColumn","INumberColumnDesc","INumberColumn","isNumberColumn","NumberMapColumn","INumberMapDesc","INumberMapColumnDesc","NumbersColumn","INumbersDesc","INumbersColumnDesc","OrdinalColumn","ICategoricalNumberColumnDesc","RankColumn","Ranking","ISortCriteria","ReduceColumn","createReduceDesc","IReduceDesc","IReduceColumnDesc","ScriptColumn","createScriptDesc","IScriptDesc","IScriptColumnDesc","SelectionColumn","createSelectionDesc","ISelectionColumnDesc","SetColumn","StackColumn","createStackDesc","StringColumn","EAlignment","IStringDesc","IStringColumnDesc","StringMapColumn","StringsColumn","ValueColumn","IValueColumnDesc","LinkColumn","ILinkDesc","ILinkColumnDesc","ILink","LinkMapColumn","LinksColumn","defineColumn","models","defaultGroup","isGroup","isSetColumn","toCategory","compareCategory","groupCompareCategory","toCategories","isCategoricalColumn","isEqualCategoricalFilter","isCategoryIncluded","DEFAULT_FORMATTER","compareBoxPlot","getBoxPlotNumber","ESortMethod","isBoxPlotColumn","EAdvancedSortMethod","isNumbersColumn","numberCompare","noNumberFilter","isEqualNumberFilter","isNumberIncluded","isMapColumn","isArrayColumn","isMapAbleColumn","ScaleMappingFunction","ScriptMappingFunction","createMappingFunction","restoreMapping","createActionDesc","createImpositionBoxPlotDesc","createImpositionDesc","createImpositionsDesc","createNestedDesc","createRankDesc","asTaggle","asLineUp","builder","buildColumn","buildRanking","IBuilderAdapterCategoricalColumnDescProps","IBuilderAdapterColumnDescProps","IBuilderAdapterDateColumnDescProps","IBuilderAdapterHierarchyColumnDescProps","IBuilderAdapterNumberColumnDescProps","IBuilderAdapterStringColumnDescProps","IBuilderAdapterActionsColumnDescProps","IChangeDetecter","IBuilderAdapter","IBuilderAdapterDataProps","IBuilderAdapterProps","IBuilderAdapterImposeColumnProps","IBuilderAdapterNestedColumnProps","IBuilderAdapterRankingProps","IBuilderAdapterReduceColumnProps","IBuilderAdapterScriptColumnProps","IBuilderAdapterSupportColumnProps","IBuilderAdapterWeightedSumColumnProps","builderAdapter","buildCategoricalColumn","buildNumberColumn","buildStringColumn","buildDateColumn","buildHierarchicalColumn","buildActionsColumn","default"].indexOf(d)<0&&function(t){n.d(e,t,function(){return l[t]})}(d);var p=n(25);n.d(e,"DENSE_HISTOGRAM",function(){return p.a}),n.d(e,"MIN_LABEL_WIDTH",function(){return p.e}),n.d(e,"HOVER_DELAY_SHOW_DETAIL",function(){return p.b});var f=n(37);n.d(e,"defaultOptions",function(){return f.a}),n.d(e,"default",function(){return i.b});var h=n(9);n.d(e,"IBoxPlotData",function(){return h.IBoxPlotData}),n.d(e,"IAdvancedBoxPlotData",function(){return h.IAdvancedBoxPlotData}),n.d(e,"IStatistics",function(){return h.IStatistics}),n.d(e,"ICategoricalBin",function(){return h.ICategoricalBin}),n.d(e,"ICategoricalStatistics",function(){return h.ICategoricalStatistics}),n.d(e,"INumberBin",function(){return h.INumberBin});var g="3.1.7",m="20181025-154657",b="BSD-3-Clause";function y(t,e,n){return void 0===n&&(n={}),new r.a(t,e,n)}function v(t,e,n){return void 0===n&&(n={}),new i.b(t,e,n)}function E(t,e,n){return void 0===n&&(n={}),new o.b(t,e,n)}},function(t,r){
/*! *****************************************************************************
Copyright (C) Microsoft. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
var i;!function(t){!function(r){var i="object"==typeof e?e:"object"==typeof self?self:"object"==typeof this?this:Function("return this;")(),o=a(t);function a(t,e){return function(n,r){"function"!=typeof t[n]&&Object.defineProperty(t,n,{configurable:!0,writable:!0,value:r}),e&&e(n,r)}}void 0===i.Reflect?i.Reflect=t:o=a(i.Reflect,o),function(t){var e=Object.prototype.hasOwnProperty,r="function"==typeof Symbol,i=r&&void 0!==Symbol.toPrimitive?Symbol.toPrimitive:"@@toPrimitive",o=r&&void 0!==Symbol.iterator?Symbol.iterator:"@@iterator",a="function"==typeof Object.create,u={__proto__:[]}instanceof Array,s=!a&&!u,c={create:a?function(){return S(Object.create(null))}:u?function(){return S({__proto__:null})}:function(){return S({})},has:s?function(t,n){return e.call(t,n)}:function(t,e){return e in t},get:s?function(t,n){return e.call(t,n)?t[n]:void 0}:function(t,e){return t[e]}},l=Object.getPrototypeOf(Function),d="object"==typeof n&&n.env&&"true"===n.env.REFLECT_METADATA_USE_MAP_POLYFILL,p=d||"function"!=typeof Map||"function"!=typeof Map.prototype.entries?function(){var t={},e=[],n=function(){function t(t,e,n){this._index=0,this._keys=t,this._values=e,this._selector=n}return t.prototype["@@iterator"]=function(){return this},t.prototype[o]=function(){return this},t.prototype.next=function(){var t=this._index;if(t>=0&&t<this._keys.length){var n=this._selector(this._keys[t],this._values[t]);return t+1>=this._keys.length?(this._index=-1,this._keys=e,this._values=e):this._index++,{value:n,done:!1}}return{value:void 0,done:!0}},t.prototype.throw=function(t){throw this._index>=0&&(this._index=-1,this._keys=e,this._values=e),t},t.prototype.return=function(t){return this._index>=0&&(this._index=-1,this._keys=e,this._values=e),{value:t,done:!0}},t}();return function(){function e(){this._keys=[],this._values=[],this._cacheKey=t,this._cacheIndex=-2}return Object.defineProperty(e.prototype,"size",{get:function(){return this._keys.length},enumerable:!0,configurable:!0}),e.prototype.has=function(t){return this._find(t,!1)>=0},e.prototype.get=function(t){var e=this._find(t,!1);return e>=0?this._values[e]:void 0},e.prototype.set=function(t,e){var n=this._find(t,!0);return this._values[n]=e,this},e.prototype.delete=function(e){var n=this._find(e,!1);if(n>=0){for(var r=this._keys.length,i=n+1;i<r;i++)this._keys[i-1]=this._keys[i],this._values[i-1]=this._values[i];return this._keys.length--,this._values.length--,e===this._cacheKey&&(this._cacheKey=t,this._cacheIndex=-2),!0}return!1},e.prototype.clear=function(){this._keys.length=0,this._values.length=0,this._cacheKey=t,this._cacheIndex=-2},e.prototype.keys=function(){return new n(this._keys,this._values,r)},e.prototype.values=function(){return new n(this._keys,this._values,i)},e.prototype.entries=function(){return new n(this._keys,this._values,a)},e.prototype["@@iterator"]=function(){return this.entries()},e.prototype[o]=function(){return this.entries()},e.prototype._find=function(t,e){return this._cacheKey!==t&&(this._cacheIndex=this._keys.indexOf(this._cacheKey=t)),this._cacheIndex<0&&e&&(this._cacheIndex=this._keys.length,this._keys.push(t),this._values.push(void 0)),this._cacheIndex},e}();function r(t,e){return t}function i(t,e){return e}function a(t,e){return[t,e]}}():Map,f=d||"function"!=typeof Set||"function"!=typeof Set.prototype.entries?function(){function t(){this._map=new p}return Object.defineProperty(t.prototype,"size",{get:function(){return this._map.size},enumerable:!0,configurable:!0}),t.prototype.has=function(t){return this._map.has(t)},t.prototype.add=function(t){return this._map.set(t,t),this},t.prototype.delete=function(t){return this._map.delete(t)},t.prototype.clear=function(){this._map.clear()},t.prototype.keys=function(){return this._map.keys()},t.prototype.values=function(){return this._map.values()},t.prototype.entries=function(){return this._map.entries()},t.prototype["@@iterator"]=function(){return this.keys()},t.prototype[o]=function(){return this.keys()},t}():Set,h=new(d||"function"!=typeof WeakMap?function(){var t=16,n=c.create(),r=i();return function(){function t(){this._key=i()}return t.prototype.has=function(t){var e=o(t,!1);return void 0!==e&&c.has(e,this._key)},t.prototype.get=function(t){var e=o(t,!1);return void 0!==e?c.get(e,this._key):void 0},t.prototype.set=function(t,e){var n=o(t,!0);return n[this._key]=e,this},t.prototype.delete=function(t){var e=o(t,!1);return void 0!==e&&delete e[this._key]},t.prototype.clear=function(){this._key=i()},t}();function i(){var t;do{t="@@WeakMap@@"+u()}while(c.has(n,t));return n[t]=!0,t}function o(t,n){if(!e.call(t,r)){if(!n)return;Object.defineProperty(t,r,{value:c.create()})}return t[r]}function a(t,e){for(var n=0;n<e;++n)t[n]=255*Math.random()|0;return t}function u(){var e=function(t){return"function"==typeof Uint8Array?"undefined"!=typeof crypto?crypto.getRandomValues(new Uint8Array(t)):"undefined"!=typeof msCrypto?msCrypto.getRandomValues(new Uint8Array(t)):a(new Uint8Array(t),t):a(new Array(t),t)}(t);e[6]=79&e[6]|64,e[8]=191&e[8]|128;for(var n="",r=0;r<t;++r){var i=e[r];4!==r&&6!==r&&8!==r||(n+="-"),i<16&&(n+="0"),n+=i.toString(16).toLowerCase()}return n}}():WeakMap);function g(t,e,n){var r=h.get(t);if(C(r)){if(!n)return;r=new p,h.set(t,r)}var i=r.get(e);if(C(i)){if(!n)return;i=new p,r.set(e,i)}return i}function m(t,e,n){var r=g(e,n,!1);return!C(r)&&function(t){return!!t}(r.has(t))}function b(t,e,n){var r=g(e,n,!1);if(!C(r))return r.get(t)}function y(t,e,n,r){var i=g(n,r,!0);i.set(t,e)}function v(t,e){var n=[],r=g(t,e,!1);if(C(r))return n;for(var i=r.keys(),a=function(t){var e=R(t,o);if(!T(e))throw new TypeError;var n=e.call(t);if(!A(n))throw new TypeError;return n}(i),u=0;;){var s=D(a);if(!s)return n.length=u,n;var c=M(s);try{n[u]=c}catch(t){try{_(a)}finally{throw t}}u++}}function E(t){if(null===t)return 1;switch(typeof t){case"undefined":return 0;case"boolean":return 2;case"string":return 3;case"symbol":return 4;case"number":return 5;case"object":return null===t?1:6;default:return 6}}function C(t){return void 0===t}function w(t){return null===t}function A(t){return"object"==typeof t?null!==t:"function"==typeof t}function x(t){var e=function(t,e){switch(E(t)){case 0:case 1:case 2:case 3:case 4:case 5:return t}var n=3===e?"string":5===e?"number":"default",r=R(t,i);if(void 0!==r){var o=r.call(t,n);if(A(o))throw new TypeError;return o}return function(t,e){if("string"===e){var n=t.toString;if(T(n)){var r=n.call(t);if(!A(r))return r}var i=t.valueOf;if(T(i)){var r=i.call(t);if(!A(r))return r}}else{var i=t.valueOf;if(T(i)){var r=i.call(t);if(!A(r))return r}var o=t.toString;if(T(o)){var r=o.call(t);if(!A(r))return r}}throw new TypeError}(t,"default"===n?"number":n)}(t,3);return function(t){return""+t}(e)}function N(t){return Array.isArray?Array.isArray(t):t instanceof Object?t instanceof Array:"[object Array]"===Object.prototype.toString.call(t)}function T(t){return"function"==typeof t}function O(t){return"function"==typeof t}function R(t,e){var n=t[e];if(void 0!==n&&null!==n){if(!T(n))throw new TypeError;return n}}function M(t){return t.value}function D(t){var e=t.next();return!e.done&&e}function _(t){var e=t.return;e&&e.call(t)}function L(t){var e=Object.getPrototypeOf(t);if("function"!=typeof t||t===l)return e;if(e!==l)return e;var n=t.prototype,r=n&&Object.getPrototypeOf(n);if(null==r||r===Object.prototype)return e;var i=r.constructor;return"function"!=typeof i?e:i===t?e:i}function S(t){return t.__=void 0,delete t.__,t}t("decorate",function(t,e,n,r){if(C(n)){if(!N(t))throw new TypeError;if(!O(e))throw new TypeError;return function(t,e){for(var n=t.length-1;n>=0;--n){var r=t[n],i=r(e);if(!C(i)&&!w(i)){if(!O(i))throw new TypeError;e=i}}return e}(t,e)}if(!N(t))throw new TypeError;if(!A(e))throw new TypeError;if(!A(r)&&!C(r)&&!w(r))throw new TypeError;return w(r)&&(r=void 0),n=x(n),function(t,e,n,r){for(var i=t.length-1;i>=0;--i){var o=t[i],a=o(e,n,r);if(!C(a)&&!w(a)){if(!A(a))throw new TypeError;r=a}}return r}(t,e,n,r)}),t("metadata",function(t,e){return function(n,r){if(!A(n))throw new TypeError;if(!C(r)&&!function(t){switch(E(t)){case 3:case 4:return!0;default:return!1}}(r))throw new TypeError;y(t,e,n,r)}}),t("defineMetadata",function(t,e,n,r){if(!A(n))throw new TypeError;return C(r)||(r=x(r)),y(t,e,n,r)}),t("hasMetadata",function(t,e,n){if(!A(e))throw new TypeError;return C(n)||(n=x(n)),function t(e,n,r){var i=m(e,n,r);if(i)return!0;var o=L(n);return!w(o)&&t(e,o,r)}(t,e,n)}),t("hasOwnMetadata",function(t,e,n){if(!A(e))throw new TypeError;return C(n)||(n=x(n)),m(t,e,n)}),t("getMetadata",function(t,e,n){if(!A(e))throw new TypeError;return C(n)||(n=x(n)),function t(e,n,r){var i=m(e,n,r);if(i)return b(e,n,r);var o=L(n);return w(o)?void 0:t(e,o,r)}(t,e,n)}),t("getOwnMetadata",function(t,e,n){if(!A(e))throw new TypeError;return C(n)||(n=x(n)),b(t,e,n)}),t("getMetadataKeys",function(t,e){if(!A(t))throw new TypeError;return C(e)||(e=x(e)),function t(e,n){var r=v(e,n),i=L(e);if(null===i)return r;var o=t(i,n);if(o.length<=0)return r;if(r.length<=0)return o;for(var a=new f,u=[],s=0,c=r;s<c.length;s++){var l=c[s],d=a.has(l);d||(a.add(l),u.push(l))}for(var p=0,h=o;p<h.length;p++){var l=h[p],d=a.has(l);d||(a.add(l),u.push(l))}return u}(t,e)}),t("getOwnMetadataKeys",function(t,e){if(!A(t))throw new TypeError;return C(e)||(e=x(e)),v(t,e)}),t("deleteMetadata",function(t,e,n){if(!A(e))throw new TypeError;C(n)||(n=x(n));var r=g(e,n,!1);if(C(r))return!1;if(!r.delete(t))return!1;if(r.size>0)return!0;var i=h.get(e);return i.delete(n),i.size>0||(h.delete(e),!0)})}(o)}()}(i||(i={}))},function(t,e){},,,,,,,,,,,,,function(t,e,n){"use strict";n.r(e),e.default="// removed by extract-text-webpack-plugin"}])}).call(this,n(7),n(8))},function(t,e){t.exports=function(t){var e=[];return e.toString=function(){return this.map(function(e){var n=function(t,e){var n=t[1]||"",r=t[3];if(!r)return n;if(e&&"function"==typeof btoa){var i=function(t){return"/*# sourceMappingURL=data:application/json;charset=utf-8;base64,"+btoa(unescape(encodeURIComponent(JSON.stringify(t))))+" */"}(r),o=r.sources.map(function(t){return"/*# sourceURL="+r.sourceRoot+t+" */"});return[n].concat(o).concat([i]).join("\n")}return[n].join("\n")}(e,t);return e[2]?"@media "+e[2]+"{"+n+"}":n}).join("")},e.i=function(t,n){"string"==typeof t&&(t=[[null,t,""]]);for(var r={},i=0;i<this.length;i++){var o=this[i][0];"number"==typeof o&&(r[o]=!0)}for(i=0;i<t.length;i++){var a=t[i];"number"==typeof a[0]&&r[a[0]]||(n&&!a[2]?a[2]=n:n&&(a[2]="("+a[2]+") and ("+n+")"),e.push(a))}},e}},function(t,e,n){t.exports=n.p+"29334f20971b9251d88c2b7c1a082917.eot"},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.JUPYTER_EXTENSION_VERSION="1.0.7"},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),function(t){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}(n(5));var r=n(3);e.version=r.JUPYTER_EXTENSION_VERSION},function(t,e,n){"use strict";var r=this&&this.__extends||function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),i=this&&this.__assign||function(){return(i=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)};Object.defineProperty(e,"__esModule",{value:!0});var o=n(6),a=n(0);n(9);var u=n(17),s=n(3),c={_data:[],_columns:[],options:{},rankings:[],value:[]},l=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r(e,t),e.prototype.defaults=function(){return i({},t.prototype.defaults.call(this),{_model_name:e.model_name,_model_module:e.model_module,_model_module_version:e.model_module_version,_view_name:e.view_name,_view_module:e.view_module,_view_module_version:e.view_module_version},c)},e.serializers=i({},o.DOMWidgetModel.serializers),e.model_name="LineUpModel",e.model_module="lineup_widget",e.model_module_version=s.JUPYTER_EXTENSION_VERSION,e.view_name="LineUpView",e.view_module="lineup_widget",e.view_module_version=s.JUPYTER_EXTENSION_VERSION,e}(o.DOMWidgetModel);e.LineUpModel=l;var d=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r(e,t),e.prototype.defaults=function(){return i({},t.prototype.defaults.call(this),{_model_name:e.model_name,_model_module:e.model_module,_model_module_version:e.model_module_version,_view_name:e.view_name,_view_module:e.view_module,_view_module_version:e.view_module_version},c)},e.serializers=i({},o.DOMWidgetModel.serializers),e.model_name="TaggleModel",e.model_module=l.model_module,e.model_module_version=s.JUPYTER_EXTENSION_VERSION,e.view_name="TaggleView",e.view_module=l.view_module,e.view_module_version=s.JUPYTER_EXTENSION_VERSION,e}(o.DOMWidgetModel);e.TaggleModel=d;var p=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r(e,t),e.prototype.render=function(){var t=this;this.data=this.createData(),this.model.on("change:_data",this.dataChanged,this),this.data.setSelection(this.model.get("value")),this.model.on("change:value",this.selectionChanged,this),this.data.on("selectionChanged",function(e){t.model.set("value",e,{updated_view:t}),t.touch()}),this.createRankings();var e=this.model.get("options");this.lineup=this.createLineUp(i({},e,{panel:!1!==e.sidePanel,panelCollapsed:"collapsed"===e.sidePanel}))},e.prototype.createData=function(){var t=this.model.get("options"),e=this.model.get("_data"),n=this.model.get("_columns");return new a.LocalDataProvider(e,a.deriveColors(n),{filterGlobally:t.filterGlobally,multiSelection:!t.singleSelection,maxGroupColumns:t.noCriteriaLimits?1/0:1,maxNestedSortingCriteria:t.noCriteriaLimits?1/0:1})},e.prototype.dataChanged=function(){var t=this;this.data.clearColumns();var e=this.model.get("_data"),n=this.model.get("_columns");a.deriveColors(n).forEach(function(e){return t.data.pushDesc(e)}),this.data.setData(e),this.createRankings()},e.prototype.createRankings=function(){var t=this;this.data.clearRankings();var e=this.model.get("rankings");0===e.length?this.data.deriveDefault():e.forEach(function(e){return u.pushRanking(t.data,e)})},e.prototype.selectionChanged=function(){this.data.setSelection(this.model.get("value"))},e}(o.DOMWidgetView);e.ALineUpView=p;var f=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r(e,t),e.prototype.createLineUp=function(t){return new a.LineUp(this.el,this.data,t)},e}(p);e.LineUpView=f;var h=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r(e,t),e.prototype.createLineUp=function(t){return new a.Taggle(this.el,this.data,t)},e}(p);e.TaggleView=h},function(e,n){e.exports=t},function(t,e){var n;n=function(){return this}();try{n=n||Function("return this")()||(0,eval)("this")}catch(t){"object"==typeof window&&(n=window)}t.exports=n},function(t,e){var n,r,i=t.exports={};function o(){throw new Error("setTimeout has not been defined")}function a(){throw new Error("clearTimeout has not been defined")}function u(t){if(n===setTimeout)return setTimeout(t,0);if((n===o||!n)&&setTimeout)return n=setTimeout,setTimeout(t,0);try{return n(t,0)}catch(e){try{return n.call(null,t,0)}catch(e){return n.call(this,t,0)}}}!function(){try{n="function"==typeof setTimeout?setTimeout:o}catch(t){n=o}try{r="function"==typeof clearTimeout?clearTimeout:a}catch(t){r=a}}();var s,c=[],l=!1,d=-1;function p(){l&&s&&(l=!1,s.length?c=s.concat(c):d=-1,c.length&&f())}function f(){if(!l){var t=u(p);l=!0;for(var e=c.length;e;){for(s=c,c=[];++d<e;)s&&s[d].run();d=-1,e=c.length}s=null,l=!1,function(t){if(r===clearTimeout)return clearTimeout(t);if((r===a||!r)&&clearTimeout)return r=clearTimeout,clearTimeout(t);try{r(t)}catch(e){try{return r.call(null,t)}catch(e){return r.call(this,t)}}}(t)}}function h(t,e){this.fun=t,this.array=e}function g(){}i.nextTick=function(t){var e=new Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)e[n-1]=arguments[n];c.push(new h(t,e)),1!==c.length||l||u(f)},h.prototype.run=function(){this.fun.apply(null,this.array)},i.title="browser",i.browser=!0,i.env={},i.argv=[],i.version="",i.versions={},i.on=g,i.addListener=g,i.once=g,i.off=g,i.removeListener=g,i.removeAllListeners=g,i.emit=g,i.prependListener=g,i.prependOnceListener=g,i.listeners=function(t){return[]},i.binding=function(t){throw new Error("process.binding is not supported")},i.cwd=function(){return"/"},i.chdir=function(t){throw new Error("process.chdir is not supported")},i.umask=function(){return 0}},function(t,e,n){var r=n(10);"string"==typeof r&&(r=[[t.i,r,""]]);var i={hmr:!0,transform:void 0,insertInto:void 0};n(15)(r,i);r.locals&&(t.exports=r.locals)},function(t,e,n){(e=t.exports=n(1)(!1)).i(n(11),""),e.push([t.i,".lineup-engine > main > article {\n  /*\n   * override jupyter style\n   */\n  line-height: 100%;\n}\n",""])},function(t,e,n){var r=n(12);(t.exports=n(1)(!1)).push([t.i,"/*! lineupjs - v3.1.7 - 2018\n* https://lineup.js.org\n* Copyright (c) 2018 Samuel Gratzl; Licensed BSD-3-Clause*/\n\n@charset \"UTF-8\";\n.lineup-engine [aria-hidden=true],\n.lu [aria-hidden=true],\n.lu-header [aria-hidden=true],\n.lu-side-panel [aria-hidden=true],\n.lu-dialog [aria-hidden=true] {\n  display: none; }\n\n.lineup-engine [draggable],\n.lu [draggable],\n.lu-header [draggable],\n.lu-side-panel [draggable],\n.lu-dialog [draggable] {\n  cursor: pointer;\n  cursor: -moz-grab;\n  cursor: -webkit-grab; }\n\n@font-face {\n  font-family: 'lu-font';\n  src: url("+r(n(2))+");\n  src: url("+r(n(2))+') format("embedded-opentype"), url(data:application/font-woff;base64, ) format("woff2"), url(data:application/font-woff;base64, ) format("woff"), url('+r(n(13))+') format("truetype"), url('+r(n(14))+") format(\"svg\");\n  font-weight: normal;\n  font-style: normal; }\n\n.lineup-engine [data-icon]::before,\n.lu [data-icon]::before,\n.lu-header [data-icon]::before,\n.lu-side-panel [data-icon]::before,\n.lu-dialog [data-icon]::before {\n  display: inline-block;\n  font: normal normal normal 14px/1 lu-font, serif;\n  font-size: inherit;\n  text-rendering: auto;\n  -webkit-font-smoothing: antialiased;\n  -moz-osx-font-smoothing: grayscale;\n  text-align: center; }\n\n.lineup-engine [data-icon=checked]::before,\n.lu [data-icon=checked]::before,\n.lu-header [data-icon=checked]::before,\n.lu-side-panel [data-icon=checked]::before,\n.lu-dialog [data-icon=checked]::before {\n  content: \"\\E800\"; }\n\n.lineup-engine [data-icon=unchecked]::before,\n.lu [data-icon=unchecked]::before,\n.lu-header [data-icon=unchecked]::before,\n.lu-side-panel [data-icon=unchecked]::before,\n.lu-dialog [data-icon=unchecked]::before {\n  content: \"\\F096\"; }\n\n.lineup-engine [data-icon=caret-down]::before,\n.lu [data-icon=caret-down]::before,\n.lu-header [data-icon=caret-down]::before,\n.lu-side-panel [data-icon=caret-down]::before,\n.lu-dialog [data-icon=caret-down]::before {\n  content: \"\\E801\"; }\n\n.lineup-engine [data-icon=caret-right]::before,\n.lu [data-icon=caret-right]::before,\n.lu-header [data-icon=caret-right]::before,\n.lu-side-panel [data-icon=caret-right]::before,\n.lu-dialog [data-icon=caret-right]::before {\n  content: \"\\E802\"; }\n\n.lu-dragover {\n  pointer-events: all !important; }\n  .lu-dragover::before, .lu-dragover::after {\n    pointer-events: none !important; }\n  .lu-dragover * {\n    pointer-events: none !important; }\n    .lu-dragover *::before, .lu-dragover *::after {\n      pointer-events: none !important; }\n\n.lu-filter-missing {\n  font-size: 13px; }\n\n.lu-checkbox > input {\n  display: inline; }\n\n.lu-checkbox > label {\n  display: inline-flex; }\n\n.lu-color-line {\n  display: flex; }\n\n.lu-checkbox-color {\n  width: 1.2em;\n  height: 1.2em;\n  margin: 0.1em;\n  line-height: 0;\n  display: flex; }\n  .lu-checkbox-color input:checked + label {\n    outline: 2px solid black; }\n  .lu-checkbox-color > input {\n    display: none; }\n  .lu-checkbox-color > label {\n    flex: 1 1 0; }\n\n.lu-color-gradient {\n  display: flex; }\n  .lu-color-gradient input:checked + label {\n    outline: 2px solid black; }\n  .lu-color-gradient > label {\n    margin: 0.1em;\n    flex: 1 1 auto; }\n\n.lu-summary .lu-checkbox label {\n  display: inline-flex; }\n\n.lu-gradient-text {\n  -moz-user-select: none;\n  -webkit-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n  position: absolute;\n  overflow: hidden;\n  left: 0;\n  top: 0;\n  bottom: 0;\n  right: 0;\n  pointer-events: none;\n  z-index: 1; }\n\n/**\r\n  popups, e.g., rename dialogs,...\r\n */\n@keyframes lu-dialogs-fade-in {\n  from {\n    opacity: 0; }\n  to {\n    opacity: 1; } }\n\n.lu-backdrop {\n  display: none;\n  position: absolute;\n  top: 0;\n  left: 0;\n  z-index: 8;\n  right: 0;\n  bottom: 0; }\n  .lu-backdrop > * {\n    position: absolute;\n    top: 0;\n    left: 0;\n    right: 0;\n    bottom: 0;\n    background: rgba(0, 0, 0, 0.1);\n    animation: lu-dialogs-fade-in 0.2s; }\n\n.lu-dialog,\n.lu-tooltip {\n  background-color: white;\n  border: 1px solid black;\n  display: flex;\n  flex-direction: column;\n  position: absolute;\n  padding: 5px;\n  z-index: 9;\n  font-size: 11pt;\n  clear: right;\n  animation: lu-dialogs-fade-in 0.4s; }\n  .lu-dialog strong,\n  .lu-tooltip strong {\n    margin: 1em 0.2em 0.4em;\n    display: inline-block; }\n\n.lu-dialog label {\n  font-weight: normal; }\n\n.lu-dialog input {\n  font-size: 8pt; }\n\n.lu-dialog button {\n  border: none;\n  background: none;\n  color: #c1c1c1;\n  transition: all 0.5s ease;\n  cursor: pointer; }\n  .lu-dialog button:hover, .lu-dialog button:active, .lu-dialog button:focus {\n    color: black; }\n  .lu-dialog button::before {\n    display: inline-block;\n    font: normal normal normal 14px/1 lu-font, serif;\n    font-size: inherit;\n    text-rendering: auto;\n    -webkit-font-smoothing: antialiased;\n    -moz-osx-font-smoothing: grayscale; }\n\n.lu-dialog button[type=submit],\n.lu-dialog button[type=reset],\n.lu-dialog button[title=cancel] {\n  margin-top: 1em; }\n\n.lu-dialog button[type=submit]::before {\n  content: \"\\E804\"; }\n\n.lu-dialog button[title=Cancel]::before {\n  content: \"\\E80C\"; }\n\n.lu-dialog button[type=reset]::before {\n  content: \"\\E80B\"; }\n\n.lu-dialog button[type=submit] {\n  float: right;\n  color: #8e8e8e; }\n\n.lu-dialog.lu-sub-nested {\n  width: 150px; }\n\n.lu-dialog.lu-filter-table.lu-weights-table input {\n  width: 6em; }\n\n.lu-dialog.lu-filter-table.lu-weights-table div > span {\n  width: 100px !important;\n  position: relative; }\n  .lu-dialog.lu-filter-table.lu-weights-table div > span > span {\n    top: 1px;\n    left: 0;\n    bottom: 1px;\n    position: absolute;\n    margin: 0; }\n\n.lu-dialog.lu-filter-table > div:first-of-type {\n  overflow-y: auto;\n  max-height: 50vh;\n  display: flex;\n  flex-direction: column; }\n  .lu-dialog.lu-filter-table > div:first-of-type * {\n    transition: all 0.5s ease; }\n  .lu-dialog.lu-filter-table > div:first-of-type span {\n    width: 1.2em;\n    margin: 0 0.5em; }\n  .lu-dialog.lu-filter-table > div:first-of-type input[type=number] + div {\n    width: 3em;\n    position: relative; }\n    .lu-dialog.lu-filter-table > div:first-of-type input[type=number] + div > * {\n      height: 100%; }\n  .lu-dialog.lu-filter-table > div:first-of-type input[type=checkbox]:not(:checked) ~ * {\n    opacity: 0.5; }\n  .lu-dialog.lu-filter-table > div:first-of-type > * {\n    display: flex;\n    flex: 0 0 auto; }\n\n.lu-dialog .lu_filter_missing {\n  margin-top: 1em; }\n\n.lu-dialog .script-description {\n  font-size: 120%;\n  max-width: 50em; }\n  .lu-dialog .script-description dd {\n    padding-left: 1em; }\n  .lu-dialog .script-description dl {\n    padding: 1em 0.2em; }\n\n.lu-dialog-color {\n  width: 15em; }\n  .lu-dialog-color > strong[data-toggle] {\n    cursor: pointer; }\n    .lu-dialog-color > strong[data-toggle]::before {\n      display: inline-block;\n      font: normal normal normal 14px/1 lu-font, serif;\n      font-size: inherit;\n      text-rendering: auto;\n      -webkit-font-smoothing: antialiased;\n      -moz-osx-font-smoothing: grayscale;\n      content: \"\\E802\";\n      width: 0.8em; }\n    .lu-dialog-color > strong[data-toggle][data-toggle=open]::before {\n      content: \"\\E801\"; }\n  .lu-dialog-color input[disabled] {\n    cursor: not-allowed;\n    opacity: 0.2; }\n  .lu-dialog-color [data-toggle=\"\"] + div {\n    display: none; }\n  .lu-dialog-color label.lu-color-custom {\n    display: inline-flex;\n    justify-content: space-between; }\n    .lu-dialog-color label.lu-color-custom > input {\n      width: unset;\n      flex: 1 1 0; }\n\n.lu-tooltip {\n  pointer-events: none;\n  min-width: 15em;\n  max-width: 50em; }\n  .lu-tooltip div[x-arrow] {\n    position: absolute;\n    width: 0;\n    height: 0; }\n  .lu-tooltip[x-placement=bottom] div[x-arrow] {\n    border-left: 5px solid transparent;\n    border-right: 5px solid transparent;\n    border-bottom: 5px solid black;\n    top: -5px; }\n  .lu-tooltip[x-placement=left] div[x-arrow] {\n    border-top: 5px solid transparent;\n    border-bottom: 5px solid transparent;\n    border-left: 5px solid black;\n    right: -5px; }\n  .lu-tooltip[x-placement=right] div[x-arrow] {\n    border-top: 5px solid transparent;\n    border-bottom: 5px solid transparent;\n    border-right: 5px solid black;\n    left: -5px; }\n  .lu-tooltip[x-placement=top] div[x-arrow] {\n    border-left: 5px solid transparent;\n    border-right: 5px solid transparent;\n    border-top: 5px solid black;\n    bottom: -5px; }\n\n.lu-dialog-mapper {\n  width: 25em; }\n  .lu-dialog-mapper > .lu-summary {\n    height: 5em;\n    margin: 0.5em 3px 0; }\n  .lu-dialog-mapper input[type=number] {\n    width: 5em; }\n  .lu-dialog-mapper div.lu-script {\n    margin-top: 1em;\n    display: none; }\n    .lu-dialog-mapper div.lu-script textarea {\n      height: 5em;\n      width: 98%;\n      font-size: 9pt;\n      display: block; }\n  .lu-dialog-mapper[data-scale=script] div.lu-script {\n    display: block; }\n  .lu-dialog-mapper svg {\n    shape-rendering: geometricprecision;\n    height: 15em; }\n    .lu-dialog-mapper svg rect {\n      fill: transparent;\n      stroke: none; }\n    .lu-dialog-mapper svg line {\n      stroke: black;\n      stroke-width: 1;\n      stroke-linejoin: round;\n      stroke-linecap: round; }\n    .lu-dialog-mapper svg line:not([x1]) {\n      stroke-width: 1.5; }\n    .lu-dialog-mapper svg line[data-v] {\n      stroke-opacity: 0.5;\n      stroke-width: 0.5;\n      stroke: #999;\n      pointer-events: none; }\n    .lu-dialog-mapper svg circle {\n      fill: #999; }\n    .lu-dialog-mapper svg g.lu-mapping {\n      cursor: ew-resize; }\n      .lu-dialog-mapper svg g.lu-mapping > line:first-child {\n        stroke-width: 5;\n        stroke: transparent !important; }\n      .lu-dialog-mapper svg g.lu-mapping.lu-frozen circle:first-of-type {\n        cursor: not-allowed;\n        fill: #999 !important; }\n      .lu-dialog-mapper svg g.lu-mapping:hover circle {\n        fill: #ffa500; }\n      .lu-dialog-mapper svg g.lu-mapping:hover line {\n        stroke: #ffa500; }\n\n.lu-header,\n.lu-side-panel-entry {\n  -moz-user-select: none;\n  -webkit-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n  position: relative;\n  display: flex;\n  flex-direction: column;\n  justify-content: flex-start;\n  background: white; }\n  .lu-header .lu-label,\n  .lu-side-panel-entry .lu-label {\n    white-space: nowrap;\n    overflow-x: hidden;\n    text-overflow: ellipsis; }\n  .lu-header.lu-dragging,\n  .lu-side-panel-entry.lu-dragging {\n    opacity: 0.5; }\n  .lu-header.lu-dragover::after,\n  .lu-side-panel-entry.lu-dragover::after {\n    -moz-user-select: none;\n    -webkit-user-select: none;\n    -ms-user-select: none;\n    user-select: none;\n    content: attr(data-draginfo);\n    position: absolute;\n    top: 0;\n    left: 0;\n    width: 100%;\n    height: 100%;\n    z-index: 10;\n    display: flex;\n    align-items: center;\n    justify-content: center;\n    pointer-events: none;\n    background: rgba(240, 240, 240, 0.8);\n    border: 2px dashed rgba(151, 151, 151, 0.8); }\n  .lu-header .lu-spacing,\n  .lu-side-panel-entry .lu-spacing {\n    flex: 1; }\n  .lu-header .lu-nested,\n  .lu-side-panel-entry .lu-nested {\n    position: relative; }\n  .lu-header .lu-summary,\n  .lu-side-panel-entry .lu-summary {\n    position: relative;\n    font-size: x-small; }\n\n.lu-summary label.lu-disabled {\n  -moz-user-select: none;\n  -webkit-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n  color: gray;\n  cursor: not-allowed; }\n\n.lu-summary input[type=text],\n.lu-summary select {\n  border: 1px solid #999;\n  border-radius: 3px;\n  padding: 3px 5px;\n  box-sizing: border-box; }\n\n.lu-summary select {\n  outline: none;\n  cursor: pointer; }\n\n.lu-summary label {\n  font-weight: normal;\n  display: flex;\n  align-items: center;\n  margin: 5px 0;\n  font-size: 13px; }\n\n.lu-toolbar {\n  position: relative;\n  font-size: 11pt;\n  display: flex;\n  justify-content: center;\n  padding: 3px; }\n  .lu-toolbar > i {\n    padding: 2px 4px;\n    cursor: pointer;\n    position: relative;\n    display: flex; }\n    .lu-toolbar > i:hover {\n      color: black; }\n    .lu-toolbar > i.hidden {\n      display: none; }\n    .lu-toolbar > i[title^='Group'][data-priority], .lu-toolbar > i[title^='Sort'][data-priority] {\n      padding-left: 10px; }\n      .lu-toolbar > i[title^='Group'][data-priority]::after, .lu-toolbar > i[title^='Sort'][data-priority]::after {\n        content: attr(data-priority) \".\";\n        position: absolute;\n        right: 0;\n        z-index: 1;\n        margin-top: 2px;\n        margin-right: 65%;\n        text-align: right;\n        color: #ffa500;\n        font-style: normal;\n        font-size: 10px;\n        font-weight: bold;\n        line-height: 1;\n        pointer-events: none; }\n    .lu-toolbar > i[title^='Group'][data-priority] {\n      padding-left: 15px; }\n      .lu-toolbar > i[title^='Group'][data-priority]::after {\n        margin-right: 60%; }\n\n.lu-action::before {\n  display: inline-block;\n  font: normal normal normal 14px/1 lu-font, serif;\n  font-size: inherit;\n  text-rendering: auto;\n  -webkit-font-smoothing: antialiased;\n  -moz-osx-font-smoothing: grayscale; }\n\n.lu-action[title^='Rename']::before {\n  content: \"\\E80A\"; }\n\n.lu-action[title='Clone']::before {\n  content: \"\\F126\"; }\n\n.lu-action[title^='Sort']::before {\n  content: \"\\F0DC\"; }\n\n.lu-action[title^='Visualization']::before {\n  content: \"\\F0EC\"; }\n\n.lu-action[title^='Color Mapping']::before {\n  content: \"\\F1FC\"; }\n\n.lu-action[title^='Data Mapping']::before {\n  content: \"\\F1DE\"; }\n\n.lu-action[title^='Edit Pattern']::before {\n  content: \"\\F08E\"; }\n\n.lu-action[title^='Reduce by']::before, .lu-action[title^='Edit Combine Script']::before, .lu-action[title^='Customize']::before {\n  content: \"\\E809\"; }\n\n.lu-action[title='Split Combined Column']::before {\n  content: \"\\F2D2\"; }\n\n.lu-action[title^='Filter']::before {\n  content: \"\\F0B0\"; }\n\n.lu-action[title^='Filter'][data-active]::before {\n  color: #ffa500; }\n\n.lu-action[title^='Group']::before {\n  content: \"\\F0DB\";\n  transform: rotate(-90deg);\n  height: 14px; }\n\n.lu-action[title='Sort']::before, .lu-action[title='Sort Group']::before {\n  content: \"\\F160\"; }\n\n.lu-action[title='Sort'][data-sort='desc']::before, .lu-action[title='Sort Group'][data-sort='desc']::before {\n  content: \"\\F161\"; }\n\n.lu-action[title='Sort'][data-sort$='sc']::before, .lu-action[title='Sort Group'][data-sort$='sc']::before, .lu-action[title^='Group'][data-group='true']::before {\n  color: #ffa500; }\n\n.lu-action[title^='Set Cut Off']::before {\n  content: \"\\E808\"; }\n\n.lu-action[title^='Search']::before {\n  content: \"\\E807\"; }\n\n.lu-action[title^='Edit Weights']::before {\n  content: \"\\F0AE\"; }\n\n.lu-action[title='UnCollapse']::before {\n  content: \"\\F152\"; }\n\n.lu-action[title='Collapse']::before {\n  content: \"\\F191\"; }\n\n.lu-action[title='Expand']::before {\n  content: \"\\E805\"; }\n\n.lu-action[title='Compress']::before {\n  content: \"\\E806\"; }\n\n.lu-action[title='Invert Selection']::before {\n  content: \"\\E800\"; }\n\n.lu-action[title^='Remove']::before {\n  content: \"\\F1F8\"; }\n\n.lu-action[title^='Choose']::before {\n  content: \"\\E801\"; }\n\n.lu-action[title^='More']::before {\n  content: \"\\F141\"; }\n\n.lu-action[title^='Contained Columns']::before {\n  content: \"\\F0E8\"; }\n\n.lu-action[title='Move Up']::before {\n  content: \"\\E81D\"; }\n\n.lu-action[title='Move Down']::before {\n  content: \"\\E81A\"; }\n\n.lu [data-col-id] {\n  /* Override alphabetical sort icons */\n  /* Override numerical sort icons */ }\n  .lu [data-col-id][data-type='string'] .lu-action[title='Sort']::before, .lu [data-col-id][data-type='string'] .lu-action[title='Sort Group']::before, .lu [data-col-id][data-type='categorical'] .lu-action[title='Sort']::before, .lu [data-col-id][data-type='categorical'] .lu-action[title='Sort Group']::before, .lu [data-col-id][data-type='link'] .lu-action[title='Sort']::before, .lu [data-col-id][data-type='link'] .lu-action[title='Sort Group']::before, .lu [data-col-id][data-type='annotate'] .lu-action[title='Sort']::before, .lu [data-col-id][data-type='annotate'] .lu-action[title='Sort Group']::before, .lu [data-col-id][data-type='hierarchy'] .lu-action[title='Sort']::before, .lu [data-col-id][data-type='hierarchy'] .lu-action[title='Sort Group']::before {\n    content: \"\\F15D\"; }\n  .lu [data-col-id][data-type='string'] .lu-action[title='Sort'][data-sort='desc']::before, .lu [data-col-id][data-type='string'] .lu-action[title='Sort Group'][data-sort='desc']::before, .lu [data-col-id][data-type='categorical'] .lu-action[title='Sort'][data-sort='desc']::before, .lu [data-col-id][data-type='categorical'] .lu-action[title='Sort Group'][data-sort='desc']::before, .lu [data-col-id][data-type='link'] .lu-action[title='Sort'][data-sort='desc']::before, .lu [data-col-id][data-type='link'] .lu-action[title='Sort Group'][data-sort='desc']::before, .lu [data-col-id][data-type='annotate'] .lu-action[title='Sort'][data-sort='desc']::before, .lu [data-col-id][data-type='annotate'] .lu-action[title='Sort Group'][data-sort='desc']::before, .lu [data-col-id][data-type='hierarchy'] .lu-action[title='Sort'][data-sort='desc']::before, .lu [data-col-id][data-type='hierarchy'] .lu-action[title='Sort Group'][data-sort='desc']::before {\n    content: \"\\F15E\"; }\n  .lu [data-col-id][data-type='rank'] .lu-action[title='Sort']::before, .lu [data-col-id][data-type='rank'] .lu-action[title='Sort Group']::before, .lu [data-col-id][data-type='number'] .lu-action[title='Sort']::before, .lu [data-col-id][data-type='number'] .lu-action[title='Sort Group']::before, .lu [data-col-id][data-type='numbers'] .lu-action[title='Sort']::before, .lu [data-col-id][data-type='numbers'] .lu-action[title='Sort Group']::before, .lu [data-col-id][data-type='ordinal'] .lu-action[title='Sort']::before, .lu [data-col-id][data-type='ordinal'] .lu-action[title='Sort Group']::before, .lu [data-col-id][data-type='boxplot'] .lu-action[title='Sort']::before, .lu [data-col-id][data-type='boxplot'] .lu-action[title='Sort Group']::before, .lu [data-col-id][data-type='script'] .lu-action[title='Sort']::before, .lu [data-col-id][data-type='script'] .lu-action[title='Sort Group']::before, .lu [data-col-id][data-type='reduce'] .lu-action[title='Sort']::before, .lu [data-col-id][data-type='reduce'] .lu-action[title='Sort Group']::before, .lu [data-col-id][data-type='stack'] .lu-action[title='Sort']::before, .lu [data-col-id][data-type='stack'] .lu-action[title='Sort Group']::before {\n    content: \"\\F162\"; }\n  .lu [data-col-id][data-type='rank'] .lu-action[title='Sort'][data-sort='desc']::before, .lu [data-col-id][data-type='rank'] .lu-action[title='Sort Group'][data-sort='desc']::before, .lu [data-col-id][data-type='number'] .lu-action[title='Sort'][data-sort='desc']::before, .lu [data-col-id][data-type='number'] .lu-action[title='Sort Group'][data-sort='desc']::before, .lu [data-col-id][data-type='numbers'] .lu-action[title='Sort'][data-sort='desc']::before, .lu [data-col-id][data-type='numbers'] .lu-action[title='Sort Group'][data-sort='desc']::before, .lu [data-col-id][data-type='ordinal'] .lu-action[title='Sort'][data-sort='desc']::before, .lu [data-col-id][data-type='ordinal'] .lu-action[title='Sort Group'][data-sort='desc']::before, .lu [data-col-id][data-type='boxplot'] .lu-action[title='Sort'][data-sort='desc']::before, .lu [data-col-id][data-type='boxplot'] .lu-action[title='Sort Group'][data-sort='desc']::before, .lu [data-col-id][data-type='script'] .lu-action[title='Sort'][data-sort='desc']::before, .lu [data-col-id][data-type='script'] .lu-action[title='Sort Group'][data-sort='desc']::before, .lu [data-col-id][data-type='reduce'] .lu-action[title='Sort'][data-sort='desc']::before, .lu [data-col-id][data-type='reduce'] .lu-action[title='Sort Group'][data-sort='desc']::before, .lu [data-col-id][data-type='stack'] .lu-action[title='Sort'][data-sort='desc']::before, .lu [data-col-id][data-type='stack'] .lu-action[title='Sort Group'][data-sort='desc']::before {\n    content: \"\\F163\"; }\n\n.lu-more-options {\n  font-size: 11pt;\n  padding: 0; }\n  .lu-more-options > i {\n    display: flex;\n    padding: 5px 10px 5px 5px;\n    align-items: center;\n    cursor: pointer; }\n    .lu-more-options > i:hover {\n      background: #e5e5e5; }\n    .lu-more-options > i::before {\n      width: 19px;\n      text-align: center; }\n    .lu-more-options > i > span {\n      font-style: normal;\n      margin-left: 3px; }\n\n/**\r\n see also animation/index.ts MAX_ANIMATION_TIME\r\n */\n/**\r\n see also animation/index.ts MAX_ANIMATION_TIME\r\n */\n/**\r\n * https://github.com/wilsonpage/sass-import-once/blob/master/_sass-import-once.scss\r\n */\n/* Module export mixin\r\n * This mixin helps making sure a module is imported once and only once.\r\n * @access public\r\n * @param {String} $name - Name of exported module\r\n * @param {Bool} $warn [true] - Warn when module has been already imported\r\n * @require $imported-modules\r\n */\n/**\r\n see also animation/index.ts MAX_ANIMATION_TIME\r\n */\n/**\r\n * https://github.com/wilsonpage/sass-import-once/blob/master/_sass-import-once.scss\r\n */\n/* Module export mixin\r\n * This mixin helps making sure a module is imported once and only once.\r\n * @access public\r\n * @param {String} $name - Name of exported module\r\n * @param {Bool} $warn [true] - Warn when module has been already imported\r\n * @require $imported-modules\r\n */\n.lineup-engine .loading {\n  background: url(data:image/svg+xml;base64,PHN2ZyB2ZXJzaW9uPSIxLjEiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyIgd2lkdGg9IjEyNCIgaGVpZ2h0PSIzMCIgdmlld0JveD0iMCAwIDEyNCAzMCIgeG1sOnNwYWNlPSJwcmVzZXJ2ZSI+DQogIDxzdHlsZT4vKiA8IVtDREFUQVsgKi8NCiAgICByZWN0IHsNCiAgICAgIGZpbGw6ICNjMWMxYzE7DQogICAgfS8qIF1dPiAqLw0KICA8L3N0eWxlPg0KICA8cmVjdCB4PSI1MCIgeT0iMTMiIHdpZHRoPSI0IiBoZWlnaHQ9IjUiIC8+DQogIDxyZWN0IHg9IjYwIiB5PSI1IiB3aWR0aD0iNCIgaGVpZ2h0PSIyMSIgLz4NCiAgPHJlY3QgeD0iNzAiIHk9IjkiIHdpZHRoPSI0IiBoZWlnaHQ9IjEzIiAvPg0KPC9zdmc+DQo=) center contain repeat no-repeat;\n  background-blend-mode: overlay; }\n  .lineup-engine .loading > * {\n    opacity: 0; }\n\n.lineup-engine.animated > header > article > *,\n.lineup-engine.animated > main > article > div > * {\n  transition: opacity 0.5s;\n  opacity: 1; }\n\n.lineup-engine.animated .loading {\n  background-image: url(data:image/svg+xml;base64,PHN2ZyB2ZXJzaW9uPSIxLjEiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyIgd2lkdGg9IjEyNCIgaGVpZ2h0PSIzMCIgdmlld0JveD0iMCAwIDEyNCAzMCIgeG1sOnNwYWNlPSJwcmVzZXJ2ZSI+DQogIDxzdHlsZT4vKiA8IVtDREFUQVsgKi8NCiAgICBAa2V5ZnJhbWVzIGxvYWQgew0KICAgICAgMCUgICB7IHRyYW5zZm9ybTogdHJhbnNsYXRlKDAsMjFweClzY2FsZSgxLDUpOyB9DQogICAgICAxMDAlIHsgdHJhbnNmb3JtOiB0cmFuc2xhdGUoMCwxM3B4KXNjYWxlKDEsMjEpOyB9DQogICAgfQ0KICAgIHJlY3Qgew0KICAgICAgZmlsbDogI2MxYzFjMTsNCiAgICAgIGFuaW1hdGlvbjogbG9hZCAwLjZzIGxpbmVhciBhbHRlcm5hdGUgaW5maW5pdGU7DQogICAgfS8qIF1dPiAqLw0KICA8L3N0eWxlPg0KICA8cmVjdCB4PSI1MCIgd2lkdGg9IjQiIGhlaWdodD0iMSIvPg0KICA8cmVjdCB4PSI2MCIgd2lkdGg9IjQiIGhlaWdodD0iMSIgc3R5bGU9ImFuaW1hdGlvbi1kZWxheTogMC4xNXMiLz4NCiAgPHJlY3QgeD0iNzAiIHdpZHRoPSI0IiBoZWlnaHQ9IjEiIHN0eWxlPSJhbmltYXRpb24tZGVsYXk6IDAuM3MiLz4NCjwvc3ZnPg0K); }\n\n.lineup-engine > main.le-scrolling {\n  background-image: url(data:image/svg+xml;base64,PHN2ZyB2ZXJzaW9uPSIxLjEiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyIgd2lkdGg9IjEyNCIgaGVpZ2h0PSIzMCIgdmlld0JveD0iMCAwIDEyNCAzMCIgeG1sOnNwYWNlPSJwcmVzZXJ2ZSI+DQogIDxzdHlsZT4vKiA8IVtDREFUQVsgKi8NCiAgICByZWN0IHsNCiAgICAgIGZpbGw6ICNjMWMxYzE7DQogICAgfS8qIF1dPiAqLw0KICA8L3N0eWxlPg0KICA8cmVjdCB4PSI1MCIgeT0iMTMiIHdpZHRoPSI0IiBoZWlnaHQ9IjUiIC8+DQogIDxyZWN0IHg9IjYwIiB5PSI1IiB3aWR0aD0iNCIgaGVpZ2h0PSIyMSIgLz4NCiAgPHJlY3QgeD0iNzAiIHk9IjkiIHdpZHRoPSI0IiBoZWlnaHQ9IjEzIiAvPg0KPC9zdmc+DQo=), url(data:image/svg+xml;base64,PHN2ZyB2ZXJzaW9uPSIxLjEiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyIgd2lkdGg9IjEyNCIgaGVpZ2h0PSIzMCIgdmlld0JveD0iMCAwIDEyNCAzMCIgeG1sOnNwYWNlPSJwcmVzZXJ2ZSI+DQogIDxzdHlsZT4vKiA8IVtDREFUQVsgKi8NCiAgICByZWN0IHsNCiAgICAgIGZpbGw6ICNjMWMxYzE7DQogICAgfS8qIF1dPiAqLw0KICA8L3N0eWxlPg0KICA8cmVjdCB4PSI1MCIgeT0iMTMiIHdpZHRoPSI0IiBoZWlnaHQ9IjUiIC8+DQogIDxyZWN0IHg9IjYwIiB5PSI1IiB3aWR0aD0iNCIgaGVpZ2h0PSIyMSIgLz4NCiAgPHJlY3QgeD0iNzAiIHk9IjkiIHdpZHRoPSI0IiBoZWlnaHQ9IjEzIiAvPg0KPC9zdmc+DQo=);\n  background-repeat: repeat-x;\n  background-position: top 10px left, bottom 30px left;\n  position: relative; }\n\n.lineup-engine article.le-row-animation {\n  /**\r\n      defaults\r\n       */\n  /**\r\n      in case of hiding shift update and show\r\n       */\n  /**\r\n      in case of update shift show\r\n       */\n  /**\r\n      in case of both update and hide shift show twice\r\n       */ }\n  .lineup-engine article.le-row-animation > * {\n    overflow: hidden !important;\n    transition: all 0.5s linear;\n    will-change: transform, opacity, height; }\n    .lineup-engine article.le-row-animation > *[data-animation=show] {\n      transition: all 0.25s linear; }\n    .lineup-engine article.le-row-animation > *[data-animation=hide] {\n      transition: all 0.25s linear; }\n    .lineup-engine article.le-row-animation > * * {\n      overflow: hidden !important; }\n  .lineup-engine article.le-row-animation.le-hide-animation > * {\n    transition: all 0.5s linear 0.3s; }\n    .lineup-engine article.le-row-animation.le-hide-animation > *[data-animation=hide] {\n      transition: all 0.25s linear; }\n    .lineup-engine article.le-row-animation.le-hide-animation > *[data-animation=show] {\n      transition: all 0.25s linear 0.3s; }\n  .lineup-engine article.le-row-animation.le-update-animation > *[data-animation=show] {\n    transition: all 0.25s linear 0.55s; }\n  .lineup-engine article.le-row-animation.le-update-animation.le-hide-animation > *[data-animation=show] {\n    transition: all 0.25s linear 0.85s; }\n\n.lineup-engine {\n  position: relative;\n  display: flex;\n  flex-direction: column;\n  align-items: stretch;\n  align-content: stretch;\n  box-sizing: border-box; }\n  .lineup-engine *::before,\n  .lineup-engine *::after {\n    box-sizing: inherit; }\n  .lineup-engine > header {\n    flex: 0 0 auto;\n    overflow-x: hidden;\n    margin-bottom: 3px;\n    padding-bottom: 1px;\n    transform: translate3d(0, 0, 2px); }\n    @supports (-moz-appearance: meterbar) {\n      .lineup-engine > header {\n        overflow-y: hidden; } }\n  .lineup-engine > main {\n    flex: 1 1 auto;\n    overflow: auto;\n    position: relative; }\n  .lineup-engine > header,\n  .lineup-engine > main {\n    display: flex;\n    flex-direction: row;\n    align-items: stretch;\n    justify-content: stretch;\n    scroll-behavior: auto; }\n    .lineup-engine > header > *,\n    .lineup-engine > main > * {\n      flex: 0 0 auto; }\n  .lineup-engine > header > article,\n  .lineup-engine > main > article > div {\n    display: flex; }\n  .lineup-engine.le-shifted > header > article > .frozen,\n  .lineup-engine.le-shifted > main > article > div > .frozen {\n    position: sticky;\n    left: 0;\n    background-color: white;\n    z-index: 1; }\n  .lineup-engine > main > article > div {\n    position: relative;\n    background: white;\n    background-clip: content-box;\n    box-sizing: content-box; }\n  .lineup-engine > main:not(.scrolling) > article > div:hover, .lineup-engine > main:not(.scrolling) > article > div.le-highlighted {\n    /* use box-shadow to create collapsing border (@see https://stackoverflow.com/a/28807765) */\n    box-shadow: 2px 0 0 0 #ffa500, 0 2px 0 0 #ffa500, 2px 2px 0 0 #ffa500, 2px 0 0 0 #ffa500 inset, 0 2px 0 0 #ffa500 inset; }\n  .lineup-engine > header > footer,\n  .lineup-engine > main > footer {\n    transition: none !important;\n    height: 1px !important;\n    width: 1px;\n    left: -1px;\n    visibility: hidden; }\n  .lineup-engine > main > article {\n    position: absolute; }\n  .lineup-engine.lineup-engine-striped > main > article:not(.odd) > div:nth-child(even),\n  .lineup-engine.lineup-engine-striped > main > article.odd > div:nth-child(odd),\n  .lineup-engine.lineup-engine-striped > main > article:not(.odd) > div:nth-child(even) > .frozen,\n  .lineup-engine.lineup-engine-striped > main > article.odd > div:nth-child(odd) > .frozen {\n    background-color: #f7f7f7; }\n\n.lineup-engine-scrollbar-tester {\n  position: absolute;\n  top: -10000px;\n  left: -10000px;\n  overflow: auto;\n  width: 100px;\n  height: 100px; }\n  .lineup-engine-scrollbar-tester > div {\n    width: 200px;\n    height: 200px;\n    overflow: auto; }\n\n.lineup-engine > header .lu-header,\n.lineup-engine .lu-row > * {\n  margin-right: 5px; }\n\nsection.lu-header {\n  color: black;\n  margin-bottom: 15px;\n  border-bottom: 1px solid black; }\n  section.lu-header.frozen::before {\n    /**\r\n    add a dummy white blocker for the margin-bottom\r\n     */\n    content: '';\n    position: absolute;\n    left: 0;\n    width: 100%;\n    height: 15px;\n    bottom: -16px;\n    background: white;\n    pointer-events: none; }\n  section.lu-header:hover {\n    background-color: #f3f3f3; }\n    section.lu-header:hover > .lu-toolbar {\n      color: #999; }\n  section.lu-header > .lu-toolbar {\n    color: #c1c1c1; }\n  section.lu-header > .lu-label {\n    text-align: center;\n    padding-left: 0.8em;\n    margin-right: 5px;\n    font-weight: 500; }\n  section.lu-header > .lu-sort {\n    position: absolute;\n    left: 1px;\n    top: 1px;\n    font-size: 11pt; }\n  section.lu-header > .lu-summary {\n    height: 2em; }\n    section.lu-header > .lu-summary:empty {\n      display: none; }\n    section.lu-header > .lu-summary[data-renderer=aggregate], section.lu-header > .lu-summary[data-renderer=selection] {\n      display: block; }\n  section.lu-header > .lu-nested > section {\n    border-bottom: none; }\n  section.lu-header > .lu-handle {\n    position: absolute;\n    right: -5px;\n    width: 5px;\n    height: 100%;\n    background-color: white;\n    cursor: col-resize;\n    border-bottom: 1px solid white;\n    text-align: center;\n    bottom: -1px; }\n    section.lu-header > .lu-handle.lu-dragover {\n      -moz-user-select: none;\n      -webkit-user-select: none;\n      -ms-user-select: none;\n      user-select: none;\n      transition: width 0.2s ease, right 0.2s ease;\n      color: black;\n      display: flex;\n      align-items: center;\n      justify-content: center;\n      border: 2px dashed rgba(151, 151, 151, 0.8);\n      background-color: rgba(240, 240, 240, 0.8);\n      width: 50px;\n      right: -25px;\n      z-index: 10;\n      font-size: small; }\n      section.lu-header > .lu-handle.lu-dragover::before {\n        content: 'Place here';\n        overflow: hidden; }\n    section.lu-header > .lu-handle:hover {\n      background-color: #e5e5e5; }\n\n.lu-sub-nested > section {\n  margin-bottom: 20px; }\n  .lu-sub-nested > section > .lu-handle {\n    height: 5px;\n    width: 100%;\n    bottom: -21px;\n    left: 0;\n    right: 0;\n    cursor: inherit; }\n    .lu-sub-nested > section > .lu-handle.lu-dragover {\n      transition: bottom 0.2s ease, height 0.2s ease;\n      height: 50px;\n      bottom: -46px;\n      width: 100%;\n      right: 0; }\n\n.lineup-engine.lu-rotated-label section.lu-header {\n  margin-top: 10em; }\n  .lineup-engine.lu-rotated-label section.lu-header > .lu-label.lu-rotated {\n    overflow: visible;\n    text-align: left;\n    transform-origin: top left; }\n\n.lineup-engine > hr {\n  position: absolute;\n  left: 0;\n  top: 0;\n  width: 1px;\n  height: 2px;\n  margin: 0;\n  background-color: #ffa500;\n  border: none;\n  transform-origin: left center;\n  display: none;\n  pointer-events: none;\n  -moz-user-select: none;\n  -webkit-user-select: none;\n  -ms-user-select: none;\n  user-select: none; }\n  .lineup-engine > hr.lu-selection-active {\n    display: block;\n    z-index: 1000; }\n\n.lineup-engine article.lu-selection-active * {\n  -moz-user-select: none;\n  -webkit-user-select: none;\n  -ms-user-select: none;\n  user-select: none; }\n\n.lu-row {\n  /**\r\n    selected state extends hover state\r\n   */ }\n  .lu-row .lu-hover-only,\n  .lu-row .lu-selection-only {\n    visibility: hidden;\n    padding-left: 4px; }\n    .lu-row .lu-hover-only > .lu-gradient-text,\n    .lu-row .lu-selection-only > .lu-gradient-text {\n      left: 4px; }\n  .lu-row:hover .lu-not-hover, .lu-row.lu-selected .lu-not-hover, .lu-row.le-highlighted .lu-not-hover {\n    visibility: visible; }\n  .lu-row:hover .lu-hover-only, .lu-row.lu-selected .lu-hover-only, .lu-row.le-highlighted .lu-hover-only {\n    visibility: visible; }\n  .lu-row.lu-selected {\n    /* use box-shadow to create collapsing border (@see https://stackoverflow.com/a/28807765) */\n    box-shadow: 2px 0 0 0 #ffa500, 0 2px 0 0 #ffa500, 2px 2px 0 0 #ffa500, 2px 0 0 0 #ffa500 inset, 0 2px 0 0 #ffa500 inset; }\n    .lu-row.lu-selected .lu-selection-only {\n      visibility: visible; }\n  .lu-row [data-group=d] {\n    white-space: nowrap;\n    overflow-x: hidden;\n    text-overflow: ellipsis; }\n  .lu-row > canvas {\n    width: 100%;\n    height: 100%; }\n\n.lu-whole-hover .lu-row:hover {\n  height: auto !important;\n  white-space: unset; }\n\n.lu {\n  /**\r\n see also animation/index.ts MAX_ANIMATION_TIME\r\n */ }\n  .lu .lu-row-body [data-renderer=aggregate] {\n    text-align: center;\n    position: relative;\n    overflow-x: unset; }\n    .lu .lu-row-body [data-renderer=aggregate]::after {\n      content: '';\n      position: absolute;\n      pointer-events: none;\n      top: 0;\n      width: 4px;\n      right: 0;\n      bottom: 0;\n      border-left: 1px solid #000; }\n  .lu .lu-row-body [data-meta~=first] [data-renderer=aggregate] {\n    cursor: pointer; }\n    .lu .lu-row-body [data-meta~=first] [data-renderer=aggregate]::before {\n      display: inline-block;\n      font: normal normal normal 14px/1 lu-font, serif;\n      font-size: inherit;\n      text-rendering: auto;\n      -webkit-font-smoothing: antialiased;\n      -moz-osx-font-smoothing: grayscale;\n      content: \"\\E801\";\n      padding-right: 4px; }\n    .lu .lu-row-body [data-meta~=first] [data-renderer=aggregate]::after {\n      border-top: 1px solid #000; }\n  .lu .lu-row-body [data-meta=last] [data-renderer=aggregate]::after {\n    border-bottom: 1px solid #000; }\n  .lu .lu-row-body [data-meta='first last'] [data-renderer=aggregate],\n  .lu .lu-row-body [data-agg=group] [data-renderer=aggregate] {\n    padding-right: 4px;\n    cursor: pointer; }\n    .lu .lu-row-body [data-meta='first last'] [data-renderer=aggregate]::after,\n    .lu .lu-row-body [data-agg=group] [data-renderer=aggregate]::after {\n      border-top: 1px solid #000;\n      border-bottom: 1px solid #000; }\n  .lu .lu-row-body [data-agg=group] [data-renderer=aggregate]::before {\n    display: inline-block;\n    font: normal normal normal 14px/1 lu-font, serif;\n    font-size: inherit;\n    text-rendering: auto;\n    -webkit-font-smoothing: antialiased;\n    -moz-osx-font-smoothing: grayscale;\n    content: \"\\E802\"; }\n  .lu .lu-summary[data-renderer=aggregate] {\n    padding-right: 4px;\n    cursor: pointer;\n    font-size: medium; }\n  .lu [data-renderer=number],\n  .lu [data-renderer=circle] {\n    position: relative;\n    white-space: nowrap; }\n    .lu [data-renderer=number] div,\n    .lu [data-renderer=circle] div {\n      position: absolute;\n      width: 100%; }\n    .lu [data-renderer=number] > div,\n    .lu [data-renderer=circle] > div {\n      top: 0;\n      bottom: 0;\n      display: flex;\n      justify-content: center;\n      flex-direction: column; }\n      .lu [data-renderer=number] > div > div,\n      .lu [data-renderer=circle] > div > div {\n        height: 100%; }\n      .lu [data-renderer=number] > div > span.lu-hover-only,\n      .lu [data-renderer=circle] > div > span.lu-hover-only {\n        position: relative; }\n  .lu [data-agg=group] [data-renderer$=number] > div > div {\n    display: flex;\n    justify-content: center;\n    flex-direction: column; }\n  .lu [data-renderer=stack] [data-renderer$=number] > div {\n    overflow-x: hidden;\n    text-overflow: ellipsis; }\n  .lu [data-renderer=boxplot] {\n    display: flex;\n    flex-direction: row;\n    align-items: center;\n    position: relative;\n    /**\r\n  outlier\r\n   */ }\n    .lu [data-renderer=boxplot] > div:first-of-type {\n      position: relative;\n      border-left: 1px solid black;\n      border-right: 1px solid black;\n      height: 90%;\n      max-height: 30px; }\n      .lu [data-renderer=boxplot] > div:first-of-type::before {\n        content: '';\n        position: absolute;\n        top: 0;\n        left: 0;\n        height: 50%;\n        width: 100%;\n        border-bottom: 1px solid black; }\n      .lu [data-renderer=boxplot] > div:first-of-type > div:first-child {\n        position: absolute;\n        top: 10%;\n        height: 80%;\n        background: #e0e0e0;\n        outline: 1px solid black; }\n      .lu [data-renderer=boxplot] > div:first-of-type > div:last-child {\n        position: absolute;\n        top: 10%;\n        height: 80%;\n        width: 1px;\n        background: black; }\n      .lu [data-renderer=boxplot] > div:first-of-type[data-sort=min] {\n        border-left-color: #ffa500; }\n      .lu [data-renderer=boxplot] > div:first-of-type[data-sort=max] {\n        border-right-color: #ffa500; }\n      .lu [data-renderer=boxplot] > div:first-of-type[data-sort=q1] > div:first-child::after, .lu [data-renderer=boxplot] > div:first-of-type[data-sort=q3] > div:first-child::after {\n        content: '';\n        position: absolute;\n        width: 1px;\n        height: 100%; }\n      .lu [data-renderer=boxplot] > div:first-of-type[data-sort=q1] > div:first-child::after {\n        left: -1px;\n        border-left: 1px solid #ffa500; }\n      .lu [data-renderer=boxplot] > div:first-of-type[data-sort=q3] > div:first-child::after {\n        right: -1px;\n        border-right: 1px solid #ffa500; }\n      .lu [data-renderer=boxplot] > div:first-of-type[data-sort=median] > div:last-child {\n        background-color: #ffa500; }\n    .lu [data-renderer=boxplot] > div:nth-of-type(n + 2) {\n      position: absolute;\n      top: 50%;\n      width: 5px;\n      height: 5px;\n      margin: -2.5px;\n      background: #e0e0e0;\n      border-radius: 50%; }\n      .lu [data-renderer=boxplot] > div:nth-of-type(n + 2)[data-sort] {\n        background: #ffa500; }\n  .lu [data-renderer=catdistributionbar] {\n    display: flex; }\n    .lu [data-renderer=catdistributionbar] > * {\n      overflow: hidden;\n      width: 0;\n      display: flex;\n      flex-direction: row;\n      align-items: center;\n      position: relative; }\n      .lu [data-renderer=catdistributionbar] > * > span {\n        position: absolute;\n        left: 5px;\n        right: 3px;\n        overflow: hidden;\n        text-overflow: ellipsis;\n        white-space: nowrap; }\n  .lu [data-agg=detail] [data-renderer=categorical],\n  .lu [data-agg=detail] [data-renderer=brightness],\n  .lu [data-agg=group] [data-renderer=brightness] > div {\n    display: flex;\n    white-space: nowrap; }\n    .lu [data-agg=detail] [data-renderer=categorical] :first-child,\n    .lu [data-agg=detail] [data-renderer=brightness] :first-child,\n    .lu [data-agg=group] [data-renderer=brightness] > div :first-child {\n      flex: 0 0 16px;\n      line-height: 100%;\n      margin-right: 0.5em;\n      /* border-radius: 8px; */ }\n    .lu [data-agg=detail] [data-renderer=categorical] :last-child,\n    .lu [data-agg=detail] [data-renderer=brightness] :last-child,\n    .lu [data-agg=group] [data-renderer=brightness] > div :last-child {\n      width: 0;\n      flex: 1 1 auto;\n      overflow: hidden;\n      text-overflow: ellipsis;\n      align-self: center; }\n  .lu [data-agg=group] [data-renderer=brightness] {\n    display: flex;\n    align-items: center; }\n    .lu [data-agg=group] [data-renderer=brightness] > div {\n      width: 100%; }\n  .lu [data-renderer=dot] {\n    position: relative; }\n    .lu [data-renderer=dot] > div {\n      background: gray;\n      opacity: 0.7;\n      border-radius: 2.5px;\n      width: 5px;\n      height: 5px;\n      margin-top: -2.5px;\n      margin-left: -2.5px;\n      top: 50%;\n      position: absolute; }\n  .lu [data-renderer=heatmap],\n  .lu [data-renderer=catheatmap],\n  .lu [data-renderer=verticalbar] {\n    display: flex;\n    flex-direction: row;\n    align-content: stretch;\n    align-items: stretch; }\n    .lu [data-renderer=heatmap] div,\n    .lu [data-renderer=catheatmap] div,\n    .lu [data-renderer=verticalbar] div {\n      flex: 1 1 auto;\n      position: relative; }\n    .lu [data-renderer=heatmap] .lu-missing,\n    .lu [data-renderer=catheatmap] .lu-missing,\n    .lu [data-renderer=verticalbar] .lu-missing {\n      background-size: 3px 3px; }\n  [data-renderer=heatmap] .lu:not(.lu-dense) > div[data-title]:first-of-type::before {\n    text-align: left; }\n  [data-renderer=heatmap] .lu:not(.lu-dense) > div[data-title]:last-of-type::before {\n    text-align: right; }\n  .lu [data-agg=group] [data-renderer=categorical],\n  .lu .lu-summary[data-renderer=categorical],\n  .lu .lu-summary[data-renderer=interleaving],\n  .lu [data-renderer=histogram] {\n    display: flex;\n    flex-direction: row; }\n    .lu [data-agg=group] [data-renderer=categorical] > div,\n    .lu .lu-summary[data-renderer=categorical] > div,\n    .lu .lu-summary[data-renderer=interleaving] > div,\n    .lu [data-renderer=histogram] > div {\n      flex: 1 1 auto;\n      position: relative;\n      margin: 0 1px;\n      font-size: x-small; }\n      .lu [data-agg=group] [data-renderer=categorical] > div[data-filtered=filtered],\n      .lu .lu-summary[data-renderer=categorical] > div[data-filtered=filtered],\n      .lu .lu-summary[data-renderer=interleaving] > div[data-filtered=filtered],\n      .lu [data-renderer=histogram] > div[data-filtered=filtered] {\n        opacity: 0.2; }\n      .lu [data-agg=group] [data-renderer=categorical] > div > div,\n      .lu .lu-summary[data-renderer=categorical] > div > div,\n      .lu .lu-summary[data-renderer=interleaving] > div > div,\n      .lu [data-renderer=histogram] > div > div {\n        position: absolute;\n        left: 0;\n        bottom: 0;\n        height: 0;\n        width: 100%; }\n      .lu [data-agg=group] [data-renderer=categorical] > div > .lu-selected,\n      .lu .lu-summary[data-renderer=categorical] > div > .lu-selected,\n      .lu .lu-summary[data-renderer=interleaving] > div > .lu-selected,\n      .lu [data-renderer=histogram] > div > .lu-selected {\n        background-color: #ffa500; }\n    .lu [data-agg=group] [data-renderer=categorical][data-interactive] > div,\n    .lu .lu-summary[data-renderer=categorical][data-interactive] > div,\n    .lu .lu-summary[data-renderer=interleaving][data-interactive] > div,\n    .lu [data-renderer=histogram][data-interactive] > div {\n      cursor: pointer; }\n    .lu [data-agg=group] [data-renderer=categorical] .lu-checkbox,\n    .lu .lu-summary[data-renderer=categorical] .lu-checkbox,\n    .lu .lu-summary[data-renderer=interleaving] .lu-checkbox,\n    .lu [data-renderer=histogram] .lu-checkbox {\n      position: absolute;\n      bottom: -5em; }\n    .lu [data-agg=group] [data-renderer=categorical].lu-dense > div,\n    .lu .lu-summary[data-renderer=categorical].lu-dense > div,\n    .lu .lu-summary[data-renderer=interleaving].lu-dense > div,\n    .lu [data-renderer=histogram].lu-dense > div {\n      margin: 0; }\n  .lu [data-agg=group] [data-renderer=categorical] > span,\n  .lu .lu-summary[data-renderer=categorical] > span,\n  .lu .lu-summary[data-renderer=interleaving] > span,\n  .lu .lu-summary[data-renderer=boxplot] > span,\n  .lu [data-renderer=histogram] > span {\n    -moz-user-select: none;\n    -webkit-user-select: none;\n    -ms-user-select: none;\n    user-select: none;\n    position: absolute;\n    bottom: -15px;\n    pointer-events: none;\n    line-height: 100%;\n    /*\r\n     * create axis tick as background image and position it left or right\r\n     */\n    background-image: linear-gradient(to bottom, #000 3px, transparent);\n    background-repeat: no-repeat;\n    background-size: 1px 3px;\n    padding-top: 5px; }\n    .lu [data-agg=group] [data-renderer=categorical] > span:first-of-type,\n    .lu .lu-summary[data-renderer=categorical] > span:first-of-type,\n    .lu .lu-summary[data-renderer=interleaving] > span:first-of-type,\n    .lu .lu-summary[data-renderer=boxplot] > span:first-of-type,\n    .lu [data-renderer=histogram] > span:first-of-type {\n      left: 0;\n      background-position: left top; }\n    .lu [data-agg=group] [data-renderer=categorical] > span:last-of-type,\n    .lu .lu-summary[data-renderer=categorical] > span:last-of-type,\n    .lu .lu-summary[data-renderer=interleaving] > span:last-of-type,\n    .lu .lu-summary[data-renderer=boxplot] > span:last-of-type,\n    .lu [data-renderer=histogram] > span:last-of-type {\n      right: 0;\n      text-align: right;\n      background-position: right top; }\n  .lu [data-renderer=categorical]:not(.lu-dense) > div[data-title]::before,\n  .lu [data-renderer=histogram]:not(.lu-dense) > div[data-title]::before,\n  .lu [data-renderer=interleaving]:not(.lu-dense) > div[data-title]::before,\n  .lu [data-renderer=heatmap]:not(.lu-dense) > div[data-title]::before,\n  .lu [data-renderer=catheatmap]:not(.lu-dense) > div[data-title]::before {\n    content: attr(data-title);\n    position: absolute;\n    bottom: -15px;\n    left: 0;\n    width: 100%;\n    overflow: hidden;\n    white-space: nowrap;\n    text-overflow: ellipsis;\n    text-align: center; }\n  .lu [data-interactive][data-renderer=histogram] {\n    margin-bottom: 2.75em;\n    position: relative; }\n    .lu [data-interactive][data-renderer=histogram] > [data-handle=min],\n    .lu [data-interactive][data-renderer=histogram] > [data-handle=max] {\n      position: absolute;\n      top: 0;\n      height: 100%;\n      border-right: 1px solid black;\n      transition: border-width 0.2s ease;\n      width: 1px;\n      cursor: ew-resize; }\n      .lu [data-interactive][data-renderer=histogram] > [data-handle=min]::before,\n      .lu [data-interactive][data-renderer=histogram] > [data-handle=max]::before {\n        content: attr(data-value);\n        position: absolute;\n        bottom: -15px;\n        width: 5em;\n        overflow: hidden;\n        white-space: nowrap;\n        text-overflow: ellipsis; }\n    .lu [data-interactive][data-renderer=histogram] > [data-handle=max] {\n      right: 0;\n      border-right: none;\n      border-left: 1px solid black; }\n      .lu [data-interactive][data-renderer=histogram] > [data-handle=max]::before {\n        left: -5em;\n        text-align: right; }\n    .lu [data-interactive][data-renderer=histogram]:hover > [data-handle=min] {\n      border-right-width: 5px; }\n    .lu [data-interactive][data-renderer=histogram]:hover > [data-handle=max] {\n      border-left-width: 5px; }\n    .lu [data-interactive][data-renderer=histogram] > [data-handle$=hint] {\n      pointer-events: none;\n      position: absolute;\n      top: 0;\n      height: 100%;\n      background: repeating-linear-gradient(-45deg, rgba(255, 255, 255, 0.7), rgba(255, 255, 255, 0.7) 7px, #bababa 7px, #bababa 10px); }\n      .lu [data-interactive][data-renderer=histogram] > [data-handle$=hint][data-handle=max-hint] {\n        right: 0; }\n    .lu [data-interactive][data-renderer=histogram] .lu-checkbox {\n      position: absolute;\n      bottom: -5em; }\n  .lu [data-renderer=image] {\n    background-size: contain;\n    background-position: center;\n    background-repeat: no-repeat;\n    position: relative; }\n    .lu [data-renderer=image]:not(.lu-missing)::before {\n      content: '';\n      position: absolute;\n      width: 100%;\n      height: 100%;\n      background: url(data:image/svg+xml;base64,PHN2ZyB2ZXJzaW9uPSIxLjEiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyIgd2lkdGg9IjEyNCIgaGVpZ2h0PSIzMCIgdmlld0JveD0iMCAwIDEyNCAzMCIgeG1sOnNwYWNlPSJwcmVzZXJ2ZSI+DQogIDxzdHlsZT4vKiA8IVtDREFUQVsgKi8NCiAgICBAa2V5ZnJhbWVzIGxvYWQgew0KICAgICAgMCUgICB7IHRyYW5zZm9ybTogdHJhbnNsYXRlKDAsMjFweClzY2FsZSgxLDUpOyB9DQogICAgICAxMDAlIHsgdHJhbnNmb3JtOiB0cmFuc2xhdGUoMCwxM3B4KXNjYWxlKDEsMjEpOyB9DQogICAgfQ0KICAgIHJlY3Qgew0KICAgICAgZmlsbDogI2MxYzFjMTsNCiAgICAgIGFuaW1hdGlvbjogbG9hZCAwLjZzIGxpbmVhciBhbHRlcm5hdGUgaW5maW5pdGU7DQogICAgfS8qIF1dPiAqLw0KICA8L3N0eWxlPg0KICA8cmVjdCB4PSI1MCIgd2lkdGg9IjQiIGhlaWdodD0iMSIvPg0KICA8cmVjdCB4PSI2MCIgd2lkdGg9IjQiIGhlaWdodD0iMSIgc3R5bGU9ImFuaW1hdGlvbi1kZWxheTogMC4xNXMiLz4NCiAgPHJlY3QgeD0iNzAiIHdpZHRoPSI0IiBoZWlnaHQ9IjEiIHN0eWxlPSJhbmltYXRpb24tZGVsYXk6IDAuM3MiLz4NCjwvc3ZnPg0K);\n      background-position: center;\n      background-repeat: no-repeat;\n      background-blend-mode: overlay;\n      z-index: -1; }\n  .lu [data-renderer=interleaving] {\n    display: flex;\n    flex-direction: column; }\n    .lu [data-renderer=interleaving] > div {\n      flex: 1 1 0;\n      overflow-y: hidden; }\n  .lu .lu-missing {\n    background-image: linear-gradient(180deg, #c1c1c1 3px, transparent 3px);\n    background-position: center center;\n    background-size: 10px 3px;\n    background-repeat: no-repeat; }\n    .lu .lu-missing > * {\n      display: none; }\n  .lu [data-renderer=rank] {\n    text-align: right; }\n  .lu [data-agg=detail] [data-renderer=rank] {\n    display: flex;\n    align-items: center;\n    justify-content: flex-end; }\n  .lu [data-agg=group] [data-renderer=rank] {\n    display: flex;\n    flex-direction: column;\n    justify-content: space-between; }\n  .lu .lu-row [data-renderer=selection] {\n    display: flex;\n    align-items: center;\n    text-align: center; }\n    .lu .lu-row [data-renderer=selection]::before {\n      display: inline-block;\n      font: normal normal normal 14px/1 lu-font, serif;\n      font-size: inherit;\n      text-rendering: auto;\n      -webkit-font-smoothing: antialiased;\n      -moz-osx-font-smoothing: grayscale;\n      display: inline;\n      content: \"\\F096\"; }\n  .lu .lu-selected [data-renderer=selection]::before,\n  .lu .selected [data-renderer=selection]::before,\n  .lu .lu-group-selected[data-renderer=selection]::before {\n    content: \"\\E800\";\n    padding-left: 2px; }\n  .lu .lu-summary[data-renderer=selection] {\n    cursor: pointer;\n    font-size: medium; }\n  .lu [data-renderer=sparkline] path {\n    stroke: black;\n    fill: none;\n    vector-effect: non-scaling-stroke; }\n  .lu [data-renderer=sparkline] line {\n    stroke: #c1c1c1;\n    stroke-width: 1;\n    vector-effect: non-scaling-stroke; }\n  .lu [data-renderer=stack] > *,\n  .lu [data-renderer=nested] > * {\n    height: 100%; }\n  .lu [data-agg=detail] [data-renderer=string] {\n    display: flex;\n    align-items: center; }\n  .lu [data-agg=group] [data-renderer=string] {\n    overflow: hidden;\n    text-overflow: ellipsis;\n    white-space: normal; }\n  .lu .lu-right {\n    justify-content: flex-end;\n    text-align: right; }\n  .lu .lu-center {\n    justify-content: center;\n    text-align: left; }\n  .lu [data-renderer=table],\n  .lu [data-renderer=map],\n  .lu [data-renderer=mapbars] {\n    font-size: 60%;\n    padding: 0;\n    margin: 0;\n    overflow: hidden;\n    display: grid;\n    grid-template-columns: auto auto;\n    grid-gap: 0.2em; }\n    .lu [data-renderer=table] > div,\n    .lu [data-renderer=map] > div,\n    .lu [data-renderer=mapbars] > div {\n      overflow: hidden;\n      min-height: 1.2em;\n      text-overflow: ellipsis; }\n  .lu [data-renderer=mapbars] > div[title] {\n    position: relative; }\n    .lu [data-renderer=mapbars] > div[title] > div {\n      position: absolute;\n      top: 0;\n      bottom: 0; }\n  .lu [data-renderer=threshold] {\n    align-items: flex-start; }\n    .lu [data-renderer=threshold] > div {\n      height: 50%; }\n      .lu [data-renderer=threshold] > div.down {\n        align-self: flex-end; }\n  .lu [data-renderer=upset] {\n    position: relative;\n    display: flex;\n    align-items: center;\n    justify-content: space-around; }\n    .lu [data-renderer=upset] > div:first-child {\n      position: absolute;\n      top: 0;\n      height: 50%;\n      border-bottom: 1px solid black; }\n    .lu [data-renderer=upset] > div:nth-child(n + 2) {\n      border-radius: 50%;\n      background-color: #676767;\n      opacity: 0.1;\n      width: 15px;\n      height: 15px; }\n      .lu [data-renderer=upset] > div:nth-child(n + 2).enabled {\n        opacity: 1; }\n    .lu [data-renderer=upset] path {\n      stroke: black;\n      fill: none; }\n  .lu [data-renderer=verticalbar] {\n    align-items: flex-end; }\n    .lu [data-renderer=verticalbar] > div {\n      position: relative; }\n\nsection.lu-slopegraph {\n  width: 200px;\n  position: relative; }\n\nsection.lu-slopegraph svg {\n  shape-rendering: geometricprecision;\n  height: 100%;\n  width: 100%;\n  background: white; }\n  section.lu-slopegraph svg path.lu-slope {\n    fill: none;\n    stroke: darkgray;\n    stroke-width: 1;\n    stroke-linecap: round; }\n    section.lu-slopegraph svg path.lu-slope:hover, section.lu-slopegraph svg path.lu-slope.lu-selected, section.lu-slopegraph svg path.lu-slope.le-highlighted {\n      z-index: 10;\n      stroke: #ffa500; }\n  section.lu-slopegraph svg path.lu-group-slope {\n    fill: rgba(169, 169, 169, 0.5);\n    stroke: darkgray;\n    stroke-linejoin: round; }\n    section.lu-slopegraph svg path.lu-group-slope:hover, section.lu-slopegraph svg path.lu-group-slope.lu-selected, section.lu-slopegraph svg path.lu-group-slope.le-highlighted {\n      z-index: 10;\n      stroke: #ffa500;\n      fill: rgba(255, 165, 0, 0.5); }\n\nsection.lu-slopegraph-header {\n  display: flex;\n  padding-top: 1.4em;\n  justify-content: center;\n  width: 200px; }\n  section.lu-slopegraph-header i {\n    padding: 2px 4px;\n    transition: all 0.5s ease;\n    cursor: pointer;\n    color: #999; }\n    section.lu-slopegraph-header i:hover {\n      color: black; }\n    section.lu-slopegraph-header i.active {\n      color: #ffa500;\n      cursor: not-allowed; }\n    section.lu-slopegraph-header i::before {\n      display: inline-block;\n      font: normal normal normal 14px/1 lu-font, serif;\n      font-size: inherit;\n      text-rendering: auto;\n      -webkit-font-smoothing: antialiased;\n      -moz-osx-font-smoothing: grayscale; }\n    section.lu-slopegraph-header i[title=Item]::before {\n      content: \"\\E815\"; }\n    section.lu-slopegraph-header i[title=Band]::before {\n      content: \"\\E816\"; }\n\n.lu-side-panel {\n  display: flex;\n  flex-direction: column;\n  width: 20em;\n  transform: translate3d(0, 0, 1px); }\n  .lu-side-panel .lu-label {\n    padding: 0 0 0 0.2em; }\n  .lu-side-panel > aside.lu-stats {\n    font-size: small;\n    padding: 1em 0.5em; }\n    .lu-side-panel > aside.lu-stats span {\n      padding: 2px;\n      border: 1px solid #ffa500; }\n  .lu-side-panel .lu-collapser {\n    position: absolute;\n    right: 0;\n    top: 0;\n    z-index: 20;\n    padding: 0.7em 0.2em;\n    cursor: pointer;\n    border-radius: 0.4em 0 0 0.4em;\n    background: #c1c1c1;\n    color: white; }\n    .lu-side-panel .lu-collapser:hover {\n      background: #a8a8a8; }\n    .lu-side-panel .lu-collapser::before {\n      display: inline-block;\n      font: normal normal normal 14px/1 lu-font, serif;\n      font-size: inherit;\n      text-rendering: auto;\n      -webkit-font-smoothing: antialiased;\n      -moz-osx-font-smoothing: grayscale;\n      content: \"\\E802\";\n      width: 0.8em;\n      text-align: center; }\n  .lu-side-panel.lu-collapsed {\n    width: 0;\n    transform: unset; }\n    .lu-side-panel.lu-collapsed > :not(.lu-collapser) {\n      display: none; }\n    .lu-side-panel.lu-collapsed .lu-collapser {\n      border-right: 1px solid #6d6c6c; }\n      .lu-side-panel.lu-collapsed .lu-collapser::before {\n        transform: scale(-1, 1); }\n\n.lu-search {\n  position: relative;\n  display: flex; }\n  .lu-search::before {\n    content: '+';\n    border-radius: 4px 0 0 4px;\n    width: 1.2em;\n    text-align: center;\n    border: 1px solid #999;\n    box-sizing: border-box;\n    border-right: none; }\n  .lu-search > input {\n    flex: 1 1 0;\n    height: 22px;\n    border: 1px solid #999;\n    box-sizing: border-box;\n    border-radius: 0 4px 4px 0;\n    padding: 3px 5px;\n    outline: none; }\n  .lu-search > ul {\n    display: none;\n    position: absolute;\n    overflow: auto;\n    z-index: 1000;\n    left: 1.2em;\n    top: 22px;\n    max-height: 0;\n    background: white;\n    text-align: left;\n    padding: 0;\n    margin-top: -1px;\n    transition: all linear 0.5s;\n    list-style: none;\n    box-sizing: border-box;\n    border: 1px solid #999;\n    border-radius: 0 0 3px 3px; }\n    .lu-search > ul.lu-search-empty::before {\n      content: 'No results found'; }\n    .lu-search > ul ul,\n    .lu-search > ul li {\n      list-style: none;\n      margin: 0;\n      padding: 0; }\n    .lu-search > ul li > span {\n      padding: 0.2em;\n      display: block; }\n    .lu-search > ul li.lu-search-group {\n      margin-bottom: 0.2em; }\n      .lu-search > ul li.lu-search-group > span {\n        background: #f0f0f0;\n        text-transform: capitalize;\n        padding-left: 0.4em;\n        font-weight: bold; }\n    .lu-search > ul li.lu-search-item > span {\n      cursor: pointer;\n      padding-left: 0.4em;\n      pointer-events: auto; }\n      .lu-search > ul li.lu-search-item > span:hover {\n        background-color: #f8f8f8; }\n    .lu-search > ul li.lu-search-highlighted > span {\n      background-color: #f8f8f8; }\n    .lu-search > ul li.hidden {\n      display: none; }\n  .lu-search.lu-search-open > input {\n    border-radius: 3px 3px 0 0; }\n  .lu-search.lu-search-open > ul {\n    display: block;\n    max-height: 70vh; }\n\narticle.lu-side-panel-entry {\n  background-color: #fff; }\n  article.lu-side-panel-entry > header {\n    display: flex;\n    align-content: space-between;\n    align-items: center;\n    background: #f0f0f0;\n    padding: 0.4em 0.5em;\n    position: relative;\n    font-size: 14px;\n    font-weight: 500; }\n  article.lu-side-panel-entry .lu-label {\n    margin: 0 1em 0 0;\n    padding: 0 0 0 0.4em;\n    overflow: hidden !important;\n    cursor: pointer;\n    flex: 1; }\n  article.lu-side-panel-entry .lu-toolbar {\n    color: #999;\n    padding: 0; }\n  article.lu-side-panel-entry .lu-summary {\n    margin: 0.5em;\n    font-size: small;\n    height: 50px; }\n    article.lu-side-panel-entry .lu-summary:empty {\n      display: none; }\n    article.lu-side-panel-entry .lu-summary[data-renderer=histogram], article.lu-side-panel-entry .lu-summary[data-renderer=categorical] {\n      margin-bottom: 4.5em; }\n    article.lu-side-panel-entry .lu-summary[data-renderer=string] {\n      display: flex;\n      flex-direction: column;\n      height: auto; }\n\n.lu-hierarchy {\n  position: relative;\n  order: 2; }\n  .lu-hierarchy > section {\n    counter-reset: hierarchy-counter;\n    margin-bottom: 0.5em; }\n    .lu-hierarchy > section::before {\n      content: '';\n      display: block;\n      font-size: small;\n      color: #999;\n      padding: 0.5em; }\n    .lu-hierarchy > section:empty {\n      display: none; }\n    .lu-hierarchy > section > div,\n    .lu-hierarchy > section > footer {\n      padding: 6px 0.5em 0 0.5em;\n      display: flex;\n      color: #999;\n      border-top: 1px solid #dedede;\n      font-size: 11pt;\n      font-weight: 500;\n      counter-increment: hierarchy-counter; }\n      .lu-hierarchy > section > div > div,\n      .lu-hierarchy > section > footer > div {\n        color: inherit;\n        flex: 1 1 0; }\n      .lu-hierarchy > section > div::before,\n      .lu-hierarchy > section > footer::before {\n        color: #999;\n        padding-right: 0.2em;\n        content: counter(hierarchy-counter); }\n    .lu-hierarchy > section > div i {\n      opacity: 0;\n      transition: opacity 0.1s linear; }\n    .lu-hierarchy > section > div:hover i {\n      opacity: 1; }\n    .lu-hierarchy > section > div i:first-of-type {\n      padding: 0 4px !important;\n      opacity: 1; }\n      .lu-hierarchy > section > div i:first-of-type::after {\n        display: none; }\n    .lu-hierarchy > section > div:first-of-type i[title=\"Move Up\"],\n    .lu-hierarchy > section > div:last-of-type i[title=\"Move Down\"] {\n      visibility: hidden; }\n  .lu-hierarchy > section.lu-hierarchy-group::before {\n    content: 'Grouping hierarchy'; }\n  .lu-hierarchy > section.lu-hierarchy-group i[title=Group] {\n    pointer-events: none; }\n  .lu-hierarchy > section:empty + hr {\n    display: none; }\n  .lu-hierarchy > section.lu-hierarchy-sort::before {\n    content: 'Sorting hierarchy'; }\n  .lu-hierarchy > section.lu-hierarchy-sortgroup::before {\n    content: 'Group Sorting hierarchy'; }\n  .lu-hierarchy .lu-search > input {\n    height: 19px;\n    font-size: x-small; }\n  .lu-hierarchy .lu-search > ul {\n    top: 19px; }\n  .lu-hierarchy ul {\n    color: black;\n    background: white; }\n\n.lu-side-panel > header[data-count] {\n  border-bottom: 2px solid #f0f0f0;\n  display: flex;\n  overflow: hidden;\n  position: relative;\n  padding-right: 1em; }\n  .lu-side-panel > header[data-count] > i {\n    position: absolute;\n    right: 0;\n    top: 0;\n    padding: 0.3em 0.3em 0.25em 0.2em;\n    z-index: 2;\n    background: white;\n    color: #999;\n    cursor: pointer;\n    font-size: 11pt; }\n    .lu-side-panel > header[data-count] > i:hover {\n      color: black; }\n  .lu-side-panel > header[data-count][data-count='1'], .lu-side-panel > header[data-count][data-count='0'] {\n    display: none; }\n  .lu-side-panel > header[data-count][data-count='2'] > i {\n    display: none; }\n  .lu-side-panel > header[data-count] > div {\n    margin-right: 0.1em;\n    border: 1px solid #f0f0f0;\n    border-top-left-radius: 5px;\n    border-top-right-radius: 5px; }\n\n.lu-side-panel > main {\n  flex: 1 1 0;\n  display: flex;\n  flex-direction: column; }\n  .lu-side-panel > main > section {\n    display: none;\n    flex: 1 1 0;\n    flex-direction: column; }\n    .lu-side-panel > main > section.lu-active {\n      display: flex; }\n    .lu-side-panel > main > section > header {\n      text-align: center;\n      padding: 1em 0.5em;\n      order: 1; }\n      .lu-side-panel > main > section > header form {\n        position: relative; }\n        .lu-side-panel > main > section > header form select {\n          width: 100%;\n          font-size: 120%; }\n    .lu-side-panel > main > section::before {\n      content: 'Column summaries';\n      font-size: small;\n      display: block;\n      color: #999;\n      padding: 0.5em;\n      order: 3; }\n    .lu-side-panel > main > section > div {\n      order: 4;\n      flex: 1 1 0;\n      position: relative;\n      overflow: auto; }\n\n.lu-side-panel > header[data-count] > div,\n.lu-choose-options > div {\n  padding: 0.2em 0.5em;\n  cursor: pointer; }\n  .lu-side-panel > header[data-count] > div > span,\n  .lu-choose-options > div > span {\n    white-space: nowrap; }\n  .lu-side-panel > header[data-count] > div:hover, .lu-side-panel > header[data-count] > div.lu-active,\n  .lu-choose-options > div:hover,\n  .lu-choose-options > div.lu-active {\n    background: #f0f0f0; }\n  .lu-side-panel > header[data-count] > div > i,\n  .lu-choose-options > div > i {\n    color: #999;\n    padding-left: 4px;\n    cursor: pointer;\n    font-size: 11pt; }\n    .lu-side-panel > header[data-count] > div > i:hover,\n    .lu-choose-options > div > i:hover {\n      color: black; }\n\n.lu span[data-type-cat]::before,\n.lu [data-type-cat] .lu-label::before {\n  display: inline-block;\n  font: normal normal normal 14px/1 lu-font, serif;\n  font-size: inherit;\n  text-rendering: auto;\n  -webkit-font-smoothing: antialiased;\n  -moz-osx-font-smoothing: grayscale;\n  color: #999;\n  width: 1.2em;\n  content: ''; }\n\n.lu span[data-type-cat='composite']::before,\n.lu [data-type-cat='composite'] .lu-label::before {\n  content: \"\\E813\"; }\n\n.lu span[data-type-cat='map']::before,\n.lu [data-type-cat='map'] .lu-label::before {\n  content: \"\\E814\"; }\n\n.lu span[data-type-cat='string']::before,\n.lu [data-type-cat='string'] .lu-label::before {\n  content: \"\\E80F\"; }\n\n.lu span[data-type-cat='categorical']::before,\n.lu [data-type-cat='categorical'] .lu-label::before {\n  content: \"\\E810\"; }\n\n.lu span[data-type-cat='number']::before,\n.lu [data-type='number'] .lu-label::before {\n  content: \"\\E811\"; }\n\n.lu span[data-type-cat='date']::before,\n.lu [data-type='date'] .lu-label::before {\n  content: \"\\E812\"; }\n\n.lu span[data-type-cat='array']::before,\n.lu [data-type='array'] .lu-label::before {\n  content: \"\\E80D\"; }\n\n.lu [data-type-cat='support'] .lu-label::before {\n  display: none; }\n\n.lu [data-type-cat='other'] .lu-label::before {\n  display: none; }\n\n/**\r\n * https://github.com/wilsonpage/sass-import-once/blob/master/_sass-import-once.scss\r\n */\n/* Module export mixin\r\n * This mixin helps making sure a module is imported once and only once.\r\n * @access public\r\n * @param {String} $name - Name of exported module\r\n * @param {Bool} $warn [true] - Warn when module has been already imported\r\n * @require $imported-modules\r\n */\n/**\r\n * https://github.com/wilsonpage/sass-import-once/blob/master/_sass-import-once.scss\r\n */\n/* Module export mixin\r\n * This mixin helps making sure a module is imported once and only once.\r\n * @access public\r\n * @param {String} $name - Name of exported module\r\n * @param {Bool} $warn [true] - Warn when module has been already imported\r\n * @require $imported-modules\r\n */\n.lineup-engine {\n  /* since the row height should be configurable -> dynamic rule, see TaggleRenderer */ }\n  .lineup-engine [data-lod=low][data-agg=detail] > * {\n    overflow: hidden; }\n  .lineup-engine [data-lod=low][data-agg=detail]:not(:hover)[data-meta~=first] {\n    /* change rendering of the first row for a group to show the collapse arrow of `[data-renderer=aggregate]` */\n    z-index: 11;\n    position: relative; }\n  .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) .lu-hover-only,\n  .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) .lu-selection-only,\n  .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer] {\n    visibility: hidden; }\n  .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=heatmap],\n  .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=catheatmap] {\n    visibility: visible !important; }\n  .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=aggregate] {\n    visibility: visible !important;\n    overflow: visible !important; }\n    .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=aggregate]::before {\n      position: absolute;\n      left: 0;\n      right: 0; }\n  .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=group] {\n    visibility: visible !important;\n    overflow: visible !important;\n    position: relative; }\n    .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=group] > * {\n      position: absolute;\n      left: 0;\n      right: 0; }\n  .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=number] {\n    visibility: visible !important; }\n    .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=number] > * {\n      top: 0;\n      bottom: 0; }\n  .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=boxplot] {\n    visibility: visible !important; }\n    .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=boxplot] > div {\n      height: 100%; }\n      .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=boxplot] > div::before,\n      .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=boxplot] > div > div:last-child,\n      .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=boxplot] > div > div:first-child::after {\n        display: none; }\n      .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=boxplot] > div > div:first-child {\n        top: 0;\n        height: 100%;\n        background: #c1c1c1; }\n  .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=upset] {\n    visibility: visible !important;\n    align-items: stretch; }\n    .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=upset] > div:first-child {\n      display: none; }\n    .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=upset] > div:nth-child(n + 2) {\n      border-radius: 0;\n      height: 100%; }\n      .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=upset] > div:nth-child(n + 2):not(.enabled) {\n        visibility: hidden; }\n  .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=categorical],\n  .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=brightness] {\n    visibility: visible !important; }\n    .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=categorical] > div:first-of-type,\n    .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=brightness] > div:first-of-type {\n      margin-right: 0;\n      flex: 1 1 auto; }\n    .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=categorical] > div:last-of-type,\n    .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=brightness] > div:last-of-type {\n      display: none; }\n  .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=dot] {\n    visibility: visible !important; }\n    .lineup-engine [data-lod=low][data-agg=detail]:not(:hover) [data-renderer=dot] > div {\n      top: 0 !important;\n      height: 100%;\n      margin-top: 0; }\n\n/**\r\n * https://github.com/wilsonpage/sass-import-once/blob/master/_sass-import-once.scss\r\n */\n/* Module export mixin\r\n * This mixin helps making sure a module is imported once and only once.\r\n * @access public\r\n * @param {String} $name - Name of exported module\r\n * @param {Bool} $warn [true] - Warn when module has been already imported\r\n * @require $imported-modules\r\n */\n.lu-side-panel > div.lu-rule-button-chooser {\n  padding: 0.5em 0.5em 0 1em;\n  flex: 0 0 auto;\n  cursor: pointer;\n  overflow: inherit;\n  display: flex;\n  align-items: center;\n  position: relative; }\n  .lu-side-panel > div.lu-rule-button-chooser span {\n    padding-left: 5px; }\n  .lu-side-panel > div.lu-rule-button-chooser div {\n    display: none; }\n  .lu-side-panel > div.lu-rule-button-chooser:hover div:not(:empty) {\n    opacity: 1;\n    visibility: visible;\n    transition-delay: 0s, 0.3s; }\n  .lu-side-panel > div.lu-rule-button-chooser.violated div {\n    display: block;\n    position: absolute;\n    background: #fffbe6;\n    border: 1px solid black;\n    top: 110%;\n    left: 5px;\n    width: 96%;\n    hyphens: manual;\n    padding: 5px;\n    z-index: 100;\n    pointer-events: none;\n    opacity: 0;\n    visibility: hidden;\n    transition: visibility 0s ease 0.3s, opacity 0.3s ease 0s; }\n    .lu-side-panel > div.lu-rule-button-chooser.violated div::before {\n      content: '';\n      width: 0;\n      height: 0;\n      border-left: 5px solid transparent;\n      border-right: 5px solid transparent;\n      border-bottom: 5px solid black;\n      position: absolute;\n      top: -6px;\n      left: 29.4%; }\n  .lu-side-panel > div.lu-rule-button-chooser.violated::after {\n    display: inline-block;\n    font: normal normal normal 14px/1 lu-font, serif;\n    font-size: inherit;\n    text-rendering: auto;\n    -webkit-font-smoothing: antialiased;\n    -moz-osx-font-smoothing: grayscale;\n    content: \"\\E803\";\n    margin-top: -1px;\n    margin-left: 5px;\n    color: #ffd700; }\n\n.lu-unsupported-browser {\n  display: flex;\n  flex-direction: column;\n  align-items: center;\n  justify-content: center; }\n  .lu-unsupported-browser > * {\n    margin: 2em; }\n\n.lu-unsupported-browser-hint > a {\n  text-decoration: none !important;\n  position: relative;\n  padding: 0.5em 1em;\n  color: black; }\n  .lu-unsupported-browser-hint > a::before {\n    display: inline-block;\n    font: normal normal normal 14px/1 lu-font, serif;\n    font-size: inherit;\n    text-rendering: auto;\n    -webkit-font-smoothing: antialiased;\n    -moz-osx-font-smoothing: grayscale;\n    font-size: 200%;\n    transition: font-size 0.25s ease;\n    color: gray; }\n  .lu-unsupported-browser-hint > a:hover::before {\n    font-size: 250%; }\n  .lu-unsupported-browser-hint > a[data-browser=chrome]::before {\n    content: \"\\F268\";\n    color: black; }\n  .lu-unsupported-browser-hint > a[data-browser=firefox]::before {\n    content: \"\\F269\"; }\n  .lu-unsupported-browser-hint > a[data-browser=edge]::before {\n    content: \"\\F282\"; }\n  .lu-unsupported-browser-hint > a::after {\n    content: attr(data-version) \"+\";\n    position: absolute;\n    top: 100%;\n    text-align: center;\n    left: -1em;\n    right: -1em; }\n\n.lu {\n  position: relative; }\n  .lu > aside {\n    float: right;\n    display: flex;\n    flex-direction: column;\n    border-left: 1px solid #6d6c6c;\n    border-radius: 0;\n    height: 100%; }\n  .lu > main {\n    height: 100%; }\n  .lu::after {\n    content: '';\n    clear: both; }",""])},function(t,e){t.exports=function(t){return"string"!=typeof t?t:(/^['"].*['"]$/.test(t)&&(t=t.slice(1,-1)),/["'() \t\n]/.test(t)?'"'+t.replace(/"/g,'\\"').replace(/\n/g,"\\n")+'"':t)}},function(t,e,n){t.exports=n.p+"6dbea47babde65521717314b0f38d514.ttf"},function(t,e,n){t.exports=n.p+"ccda38f2ae7b5f07875dc4bba68b9586.svg"},function(t,e,n){var r={},i=function(t){var e;return function(){return void 0===e&&(e=t.apply(this,arguments)),e}}(function(){return window&&document&&document.all&&!window.atob}),o=function(t){var e={};return function(t,n){if("function"==typeof t)return t();if(void 0===e[t]){var r=function(t,e){return e?e.querySelector(t):document.querySelector(t)}.call(this,t,n);if(window.HTMLIFrameElement&&r instanceof window.HTMLIFrameElement)try{r=r.contentDocument.head}catch(t){r=null}e[t]=r}return e[t]}}(),a=null,u=0,s=[],c=n(16);function l(t,e){for(var n=0;n<t.length;n++){var i=t[n],o=r[i.id];if(o){o.refs++;for(var a=0;a<o.parts.length;a++)o.parts[a](i.parts[a]);for(;a<i.parts.length;a++)o.parts.push(m(i.parts[a],e))}else{var u=[];for(a=0;a<i.parts.length;a++)u.push(m(i.parts[a],e));r[i.id]={id:i.id,refs:1,parts:u}}}}function d(t,e){for(var n=[],r={},i=0;i<t.length;i++){var o=t[i],a=e.base?o[0]+e.base:o[0],u={css:o[1],media:o[2],sourceMap:o[3]};r[a]?r[a].parts.push(u):n.push(r[a]={id:a,parts:[u]})}return n}function p(t,e){var n=o(t.insertInto);if(!n)throw new Error("Couldn't find a style target. This probably means that the value for the 'insertInto' parameter is invalid.");var r=s[s.length-1];if("top"===t.insertAt)r?r.nextSibling?n.insertBefore(e,r.nextSibling):n.appendChild(e):n.insertBefore(e,n.firstChild),s.push(e);else if("bottom"===t.insertAt)n.appendChild(e);else{if("object"!=typeof t.insertAt||!t.insertAt.before)throw new Error("[Style Loader]\n\n Invalid value for parameter 'insertAt' ('options.insertAt') found.\n Must be 'top', 'bottom', or Object.\n (https://github.com/webpack-contrib/style-loader#insertat)\n");var i=o(t.insertAt.before,n);n.insertBefore(e,i)}}function f(t){if(null===t.parentNode)return!1;t.parentNode.removeChild(t);var e=s.indexOf(t);e>=0&&s.splice(e,1)}function h(t){var e=document.createElement("style");if(void 0===t.attrs.type&&(t.attrs.type="text/css"),void 0===t.attrs.nonce){var r=function(){0;return n.nc}();r&&(t.attrs.nonce=r)}return g(e,t.attrs),p(t,e),e}function g(t,e){Object.keys(e).forEach(function(n){t.setAttribute(n,e[n])})}function m(t,e){var n,r,i,o;if(e.transform&&t.css){if(!(o="function"==typeof e.transform?e.transform(t.css):e.transform.default(t.css)))return function(){};t.css=o}if(e.singleton){var s=u++;n=a||(a=h(e)),r=y.bind(null,n,s,!1),i=y.bind(null,n,s,!0)}else t.sourceMap&&"function"==typeof URL&&"function"==typeof URL.createObjectURL&&"function"==typeof URL.revokeObjectURL&&"function"==typeof Blob&&"function"==typeof btoa?(n=function(t){var e=document.createElement("link");return void 0===t.attrs.type&&(t.attrs.type="text/css"),t.attrs.rel="stylesheet",g(e,t.attrs),p(t,e),e}(e),r=function(t,e,n){var r=n.css,i=n.sourceMap,o=void 0===e.convertToAbsoluteUrls&&i;(e.convertToAbsoluteUrls||o)&&(r=c(r));i&&(r+="\n/*# sourceMappingURL=data:application/json;base64,"+btoa(unescape(encodeURIComponent(JSON.stringify(i))))+" */");var a=new Blob([r],{type:"text/css"}),u=t.href;t.href=URL.createObjectURL(a),u&&URL.revokeObjectURL(u)}.bind(null,n,e),i=function(){f(n),n.href&&URL.revokeObjectURL(n.href)}):(n=h(e),r=function(t,e){var n=e.css,r=e.media;r&&t.setAttribute("media",r);if(t.styleSheet)t.styleSheet.cssText=n;else{for(;t.firstChild;)t.removeChild(t.firstChild);t.appendChild(document.createTextNode(n))}}.bind(null,n),i=function(){f(n)});return r(t),function(e){if(e){if(e.css===t.css&&e.media===t.media&&e.sourceMap===t.sourceMap)return;r(t=e)}else i()}}t.exports=function(t,e){if("undefined"!=typeof DEBUG&&DEBUG&&"object"!=typeof document)throw new Error("The style-loader cannot be used in a non-browser environment");(e=e||{}).attrs="object"==typeof e.attrs?e.attrs:{},e.singleton||"boolean"==typeof e.singleton||(e.singleton=i()),e.insertInto||(e.insertInto="head"),e.insertAt||(e.insertAt="bottom");var n=d(t,e);return l(n,e),function(t){for(var i=[],o=0;o<n.length;o++){var a=n[o];(u=r[a.id]).refs--,i.push(u)}t&&l(d(t,e),e);for(o=0;o<i.length;o++){var u;if(0===(u=i[o]).refs){for(var s=0;s<u.parts.length;s++)u.parts[s]();delete r[u.id]}}}};var b=function(){var t=[];return function(e,n){return t[e]=n,t.filter(Boolean).join("\n")}}();function y(t,e,n,r){var i=n?"":r.css;if(t.styleSheet)t.styleSheet.cssText=b(e,i);else{var o=document.createTextNode(i),a=t.childNodes;a[e]&&t.removeChild(a[e]),a.length?t.insertBefore(o,a[e]):t.appendChild(o)}}},function(t,e){t.exports=function(t){var e="undefined"!=typeof window&&window.location;if(!e)throw new Error("fixUrls requires window.location");if(!t||"string"!=typeof t)return t;var n=e.protocol+"//"+e.host,r=n+e.pathname.replace(/\/[^\/]*$/,"/");return t.replace(/url\s*\(((?:[^)(]|\((?:[^)(]+|\([^)(]*\))*\))*)\)/gi,function(t,e){var i,o=e.trim().replace(/^"(.*)"$/,function(t,e){return e}).replace(/^'(.*)'$/,function(t,e){return e});return/^(#|data:|http:\/\/|https:\/\/|file:\/\/\/|\s*$)/i.test(o)?t:(i=0===o.indexOf("//")?o:0===o.indexOf("/")?n+o:r+o.replace(/^\.\//,""),"url("+JSON.stringify(i)+")")})}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(0);e.pushRanking=function(t,e){var n=r.buildRanking();return(e.columns||["_*","*"]).forEach(function(t){return n.column(t)}),("string"==typeof e.sort_by?[e.sort_by]:e.sort_by||[]).forEach(function(t){return n.sortBy(t)}),("string"==typeof e.group_by?[e.group_by]:e.group_by||[]).forEach(function(t){return n.groupBy(t)}),n.build(t)}}])});
//# sourceMappingURL=index.js.map